package org.objectweb.telosys.auth;

/**
 * @author Laurent GUERIN
 * 
 * This class contains static methods to managed http authentication.
 * 
 */

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysFactory;
import org.objectweb.telosys.screen.env.ScreenSessionManager;

public final class Auth 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(Auth.class);

    //---------------------------------------------------------------------------
    private static IUserValidator getApplicationUserValidator()
    {
        TelosysFactory factory = Telosys.getFactory();
        if (factory != null)
        {
            return factory.getApplicationUserValidator();
        }
        log.error("Cannot get Telosys factory ! ");
        return null;
    }

    //---------------------------------------------------------------------------
    private static IUserValidator getDefaultUserValidator()
    {
        return new DefaultUserValidator();
    }

    //---------------------------------------------------------------------------
    /**
     * Validate the login user (delegate to the application)
     * @param user :
     *            LoginUser to validate
     * @return : Application user if validate, null if not validate
     */
    /* package */ static IAppUser validUser(final LoginUser user)
    {
        IUserValidator applicationUserValidator = getApplicationUserValidator();
        if (applicationUserValidator != null)
        {
            return applicationUserValidator.validUser(user.getLogin(), user.getPassword());
        }
        else
        {
            IUserValidator defaultUserValidator = getDefaultUserValidator();
            if (defaultUserValidator != null)
            {
                return defaultUserValidator.validUser(user.getLogin(), user.getPassword());
            }
            else
            {
                return null;
            }
        }
    }
    
    //---------------------------------------------------------------------------
    /**
     * Check if the given http request contains an authenticated user
     * @param request
     * @return true if authenticated (else false)
     */
    /* package */ static boolean isAuthenticated(final HttpServletRequest request)
    {        
        //--- Is there an authenticated user in the ScreenSession ?
        LoginUser user = ScreenSessionManager.findScreenSessionUser(request);
        if (user == null)
        {
            // No User in the ScreenSession => Not Authenticated
            log.trace("isAuthenticated(request) : FALSE ( No user in the ScreenSession ).");
            return false ;
        }
        //--- OK ; User authenticated.
        log.trace("isAuthenticated(request) : TRUE.");
        return true ;        
    }
    
    //---------------------------------------------------------------------------
    /**
     * Try to find an authenticated user in the session
     * @param request
     * @return the authenticated user or null 
     */
    /* package */ static LoginUser getAuthenticatedUser (final HttpServletRequest request)
    {        
        //--- Is there an authenticated user in the ScreenSession ?
        LoginUser user = ScreenSessionManager.findScreenSessionUser(request);
        log.trace("getAuthenticatedUser(request) : " + ( user != null ? "FOUND" : "NOT FOUND") );            
        return user ;        
    }
    //---------------------------------------------------------------------------
    /* package */ static LoginUser createLoginUser(String sUserLogin, String sUserPassword, HttpServletRequest request)
    {
        String sIP   = "???" ;
        String sHost = "???" ;
        if ( request != null )
        {
            sIP   = request.getRemoteAddr();
            sHost = request.getRemoteHost();
        }
		LoginUser user = new LoginUser(sUserLogin, sUserPassword, sIP, sHost) ;        
        return user;
    }
    //---------------------------------------------------------------------------
    public static LoginUser createDummyUser(HttpServletRequest request)
    {
        return createLoginUser("NonAuthenticatedUser", "NoPassword", request);
    }
}