package org.objectweb.telosys.auth;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.util.StrUtil;

/**
 * Filter to restrict access only for authenticated users.
 * Authenticated users can pass through the filter
 * Authentication based on <br>
 * 1) "Basic Http Authentication" but with dynamic Realm Names and URLs
 * to avoid the "Login/password" in each request and to provide a "logout" mechanism  
 * 2) "Login Page Authentication"
 * 
 * @author Laurent GUERIN 
 * 
 */ 
public class AuthFilter extends TelosysObject implements Filter
{
	//----------------------------------------------------------------------------
	// Constants
	//----------------------------------------------------------------------------
	private final static String BASIC_AUTH_LOGIN_PATH  = "/l/o/g/i/n/" ;
	private final static String TOKEN_PREFIX = "T" ;

	private final static String USER_LOGIN    = "user_login" ;
	private final static String USER_PASSWORD = "user_password";
	
	//--- Auth properties ( see 'telosys.properties' file )
	private final static String PROP_LOGIN_PAGE         = "LoginPage" ; 
	private final static String PROP_LOGIN_ACTION       = "LoginAction" ;
	private final static String PROP_LOGIN_MAX_TRIES    = "LoginMaxTries" ;
	private final static String PROP_LOGIN_ERROR_PAGE   = "LoginErrorPage" ; 
	
	//--- Default properties ( if not found in 'telosys.properties' file )
	private final static String DEFAULT_LOGIN_ACTION    = "/login_action" ;
	private final static String DEFAULT_LOGIN_MAX_TRIES = "3" ;
	private final static int    MAX_TRIES = 3 ;	
	
	//----------------------------------------------------------------------------
	// Attributes
	//----------------------------------------------------------------------------
    private ServletContext _servletContext = null ;
    
	private String _sLoginPageParam      = null ; // The "login page" read from the parameters file
	private String _sLoginPageURI        = null ; // The "login page" built with the context root 
	
	private String _sLoginActionParam    = null ; // The "login action" read from the parameters file
	private String _sLoginActionURI      = null ; // The "login action" built with the context root 
	
	private String _sLoginErrorPageParam = null ; // The "login error page" read from the parameters file
	private String _sLoginErrorPageURI   = null ; // The "login error page" built with the context root 
	
	private LinkedList _noAuthPages      = new LinkedList(); // Trusted Pages list

	private int _iMaxTries               = MAX_TRIES ; // 
	
	//---------------------------------------------------------------------------------------------------------
    /**
     * Returns the full URI for the given resource, or null if the given resource is null
     * e.g. '/context/resource' for 'resource' or '/resource'
     * @param request
     * @param sResource
     * @return URI or null
     */
    private String getResourceURI( HttpServletRequest httpRequest, String sResource )
    {
		//String sURI = httpRequest.getRequestURI();
		String sContextPath = httpRequest.getContextPath() ; // "/context"
		if ( sContextPath != null && sResource != null )
		{
		    String s1 = sContextPath.trim();
		    String s2 = sResource.trim();
		    if ( s2.startsWith("/") )
		    {
		        return s1 + s2 ;
		    }
		    else
		    {
		        return s1 + "/" + s2 ;
		    }
		}
		return null ;
    }
    
	//---------------------------------------------------------------------------------------------------------
    /**
     * Returns the Login Page URI according with the 'LoginPage' parameter
     * @param httpRequest
     * @return URI ( '/context/resource' ) or null if no parameter set
     */
    private String getLoginPageURI( HttpServletRequest httpRequest )
    {
        if ( _sLoginPageURI == null ) // if not yet initialized
        {
            _sLoginPageURI = getResourceURI( httpRequest, _sLoginPageParam ) ;
            if ( _sLoginPageURI != null )
            {
                trace("init LoginPage URI : '" + _sLoginPageURI + "'" );
            }
            else
            {
                trace("cannot init LoginPage URI, LoginPage parameter = '" + _sLoginPageParam + "'" );
            }
        }
        return _sLoginPageURI ;
    }
    
	//---------------------------------------------------------------------------------------------------------
    /**
     * Returns the Login Action URI according with the 'LoginAction' parameter
     * @param httpRequest
     * @return URI ( '/context/resource' ) if no parameter the default action is used
     */
    private String getLoginActionURI( HttpServletRequest httpRequest )
    {
        if ( _sLoginActionURI == null ) // if not yet initialized
        {
            _sLoginActionURI = getResourceURI( httpRequest, _sLoginActionParam ) ;
            if ( _sLoginActionURI != null )
            {
                trace("init LoginAction URI : '" + _sLoginActionURI + "'" );
            }
            else
            {
                trace("cannot init LoginAction URI, LoginAction parameter = '" + _sLoginActionParam + "'" );
            }
        }
        return _sLoginActionURI ;
    }
    
	//---------------------------------------------------------------------------------------------------------
    /**
     * Returns the Login Error Page URI according with the 'LoginErrorPage' parameter
     * @param httpRequest
     * @return URI ( '/context/resource' ) if no parameter the default action is used
     */
    private String getLoginErrorPageURI ( HttpServletRequest httpRequest )
    {
        if ( _sLoginErrorPageURI == null ) // if not yet initialized
        {
            _sLoginErrorPageURI = getResourceURI( httpRequest, _sLoginErrorPageParam ) ;
            if ( _sLoginErrorPageURI != null )
            {
                trace("init LoginErrorPage URI : '" + _sLoginErrorPageURI + "'" );
            }
            else
            {
                trace("cannot init LoginPage URI, LoginPage parameter = '" + _sLoginErrorPageParam + "'" );
            }
        }
        return _sLoginErrorPageURI ;
    }
	//---------------------------------------------------------------------------------------------------------
	private void setTelosysRedirect(HttpServletRequest request, HttpServletResponse response) throws ServletException
	{
        String sURI = request.getRequestURI();
        String sQry = request.getQueryString();

        //--- Build the redirection URI with or without Query String
        String sRedirect = sURI;
        if ( sQry != null )
        {
            if ( sQry.trim().length() > 0 )
            {
                sRedirect = sURI + "?" + sQry.trim() ;
            }
        }
        //--- Set response header
	    trace("set response header : 'Telosys-Redirect' : '" + sRedirect + "'");
        response.setHeader(TelosysConst.TELOSYS_REDIRECT, sRedirect );
	}
	
	//---------------------------------------------------------------------------------------------------------
	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException
	{
	    _servletContext = config.getServletContext() ;
		info("init() : . ServletContext  = '" + _servletContext.getServletContextName() + "'" );
	    Properties p = Telosys.getProperties();
	    if ( p != null )
	    {
		    _sLoginErrorPageParam = p.getProperty(PROP_LOGIN_ERROR_PAGE); 
		    if ( _sLoginErrorPageParam != null ) _sLoginErrorPageParam = _sLoginErrorPageParam.trim();
		    
		    _sLoginPageParam = p.getProperty(PROP_LOGIN_PAGE); 
		    if ( _sLoginPageParam != null )
		    {
		        _sLoginPageParam = _sLoginPageParam.trim();
			    _sLoginActionParam = p.getProperty(PROP_LOGIN_ACTION, DEFAULT_LOGIN_ACTION);
			    _sLoginActionParam = _sLoginActionParam.trim();
		    }
			info("init() : . LoginPage      = '" + _sLoginPageParam + "'" );
			info("init() : . LoginAction    = '" + _sLoginActionParam + "'" );
			info("init() : . LoginErrorPage = '" + _sLoginErrorPageParam + "'" );
			
			String sMaxTries = p.getProperty(PROP_LOGIN_MAX_TRIES, DEFAULT_LOGIN_MAX_TRIES );
			_iMaxTries = StrUtil.getInt( sMaxTries, MAX_TRIES );
			info("init() : . MaxTries       = " + _iMaxTries );
			
		    //--- Populate the "trusted pages" list
			_noAuthPages.clear();
			for ( int i = 1 ; i < 9999 ; i++ )
			{
			    String s = p.getProperty( "NoAuthPage" + i );
			    if ( s != null )
			    {
			        _noAuthPages.add(s.trim()); // add the Trusted Page in the list
					info("init() : . No auth page [ " + i + " ] : '" + s.trim() + "'");
			    }
			    else
			    {
			        break; // stop now 
			    }
			}
			info("init() : Authentication filter initialized." );
	    }
	    else
	    {
	        error("init() : Cannot get Telosys properties");
	    }
	}

	//---------------------------------------------------------------------------------------------------------
	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy()
	{
		trace("destroy()");
	}

	//---------------------------------------------------------------------------------------------------------
	/**
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException
	{
		trace("----- doFilter()...");
		HttpServletRequest  httpRequest  = null ;
		HttpServletResponse httpResponse = null;
		if ( request instanceof HttpServletRequest && response instanceof HttpServletResponse )
		{
			httpRequest  = (HttpServletRequest) request;
			httpResponse = (HttpServletResponse) response;
		}
		else
		{
		    error("doFilter() : request and/or response not HTTP ");
		    return;
		}
		
		trace("Request : '" + httpRequest.getRequestURL() + 
		        ( httpRequest.getQueryString() != null ? "?" + httpRequest.getQueryString() : "" ) );

		trace("request.getAuthType()   = " + httpRequest.getAuthType() );		
		trace("request.getRemoteUser() = " + httpRequest.getRemoteUser() );		
		
		//--- Check if user is in session ( already authenticated )  
		LoginUser user = Auth.getAuthenticatedUser(httpRequest); // v 0.9.9
		if ( user != null )  // v 0.9.9
		{			
			//--- Authenticated user => access permitted => chain
			trace("User authenticated (found in session) : '" + user.getLogin() + "' : req. count = " + user.getRequestCount() );	

			//--- v 0.9.9
			// NB: the first response can be a 302 generated by the servlet container 
			// to redirect to the "welcome page" => do not increment if 302 
			if ( user.getRequestCount() == 0 ) // first request after authentication for this user ?
			{
			    //--- This the first request => Set the http header for AJAX redirection 
			    // If this request is a Telosys AJAX call, it will be transform in a standard Http page loading
			    setTelosysRedirect(httpRequest, httpResponse);
			}
			//--- Access permitted => chain to the requested resource
			chain.doFilter(request, response);

			//--- What kind of response ?
			if ( httpResponse.containsHeader("Location") ) // 301 or 302
			{
			    //trace("After chain : header 'Location' found => Status 302 " );
			}
			else
			{
			    // Increment only if not 301 or 302
				user.incrementRequestCount();			
			}
		}
		else
		{
			//--- Not authenticated user : => Force authentication or go to error page
			trace("User not yet authenticated (not found in session).");
			int iAuthStatus = getAuthStatusCookieValue( httpRequest );
			if ( iAuthStatus < 0 )
			{
			    //--- Set the header for Ajax caller redirection
			    setTelosysRedirect(httpRequest, httpResponse); 
			    printLoginErrorPage(httpRequest, httpResponse);
			}
			else if ( isFormAuthLoginPageURL(httpRequest) ) // try to access the Login Page
			{
			    // The requested URL is the LoginPage => chain to this URL
				trace("Request URL is for the Login Page : OK, let it pass ... ");
			    //--- Set the header for Ajax caller redirection
			    setTelosysRedirect(httpRequest, httpResponse); 
			    //--- Set the number of tries in the request
//			    int iCount = getCountCookieValue( httpRequest ) ; // Always ZERO ( due to cookie domain path )
//			    httpRequest.setAttribute("LoginCount",""+iCount);
			    httpRequest.setAttribute("LoginMaxTries",""+_iMaxTries);
			    //--- Continue ... chain 
				chain.doFilter(request, response);
			}
			else if ( isFormAuthLoginActionURL(httpRequest) ) // submit the Login Action from the Login Page			    
			{
				trace("Request URL is a Form Auth Login Action : try to authenticate the user... ");
				LoginUser loginUser = getUserFromLoginActionRequest(httpRequest);
	            int iCount = getCountCookieValue( httpRequest ) ;
		        if (loginUser != null) // "login/password" found in the Request Header 
		        {
		            IAppUser appUser = Auth.validUser(loginUser);
		            if ( appUser != null )
		            {
			            trace("User login/password is valid. ");
			            //--- OPEN a SCREEN SESSION for this user
			            trace("Init user in session... ");
			            ScreenSessionManager.openScreenSession(httpRequest, loginUser, appUser);
			            trace("Redirect to first page... ");
					    redirectToFirstPage(httpRequest, httpResponse, appUser );
		            }
		            else
		            {
			            trace("Invalid login/password => redirect to login page ");
			            //int iCount = getCountCookieValue( httpRequest ) ;
			            if ( iCount < _iMaxTries )
			            {
			                //--- Try again ...
			                iCount++ ;
				            redirectToLoginPage(httpRequest, httpResponse, iCount);
			            }
			            else
			            {
			                //--- "This is the end my friend" ...
			                setAuthStatusCookie( httpResponse, -1 );
			                printLoginErrorPage(httpRequest, httpResponse);
			            }
		            }
		        }
		        else  // never happens (?)
		        {
		            trace("login/password not found in request => redirect to login page ");
		            //int iCount = getCountCookieValue( httpRequest ) + 1 ;
	                //--- Try again ...
	                iCount++ ;
		            redirectToLoginPage(httpRequest, httpResponse, iCount);
		        }
			}
			else if ( isBasicAuthLoginURL(httpRequest) ) // special URL for BASIC authentication			    
			{
				trace("Request URL is a Basic Auth Login URL : try to authenticate the user... ");
			    LoginUser loginUser = getUserFromBasicAuthRequest(httpRequest);
	            int iCount = getCountCookieValue( httpRequest ) ;
		        if (loginUser != null) // "login/password" found in the Request Header 
		        {
		            //IAppUser appUser = checkUserAuthentication(loginUser);
		            IAppUser appUser = Auth.validUser(loginUser);
		            if ( appUser != null )
		            {
			            trace("User login/password is valid. ");
			            //--- OPEN a SCREEN SESSION for this user
			            trace("Init user in session... ");
			            ScreenSessionManager.openScreenSession(httpRequest, loginUser, appUser);
			            trace("Redirect to first page... ");
					    redirectToFirstPage(httpRequest, httpResponse, appUser );
		            }
		            else
		            {
			            trace("Invalid login/password => '401' (Unauthorized) ");
			            if ( iCount < _iMaxTries )
			            {
			                //--- Try again ...
			                iCount++ ;
				            //--- Response = Authentication ( "401-Unauthorized" )
				            buildAuthenticateResponse(httpRequest, httpResponse, iCount);
			            }
			            else
			            {
			                //--- "This is the end my friend" ...
			                setAuthStatusCookie( httpResponse, -1 );
						    //--- Set the header for Ajax caller redirection
						    setTelosysRedirect(httpRequest, httpResponse); 			                
						    printLoginErrorPage(httpRequest, httpResponse);
			            }
		            }
		        }
		        else // never happens (?)
		        {
		            trace("login/password not found in request => '401' (Unauthorized) "); 
	                //--- Try again ...
	                iCount++ ;
		            //--- Response = Authentication ( "401-Unauthorized" )
		            buildAuthenticateResponse(httpRequest, httpResponse, iCount);
		        }
			}
			else if ( isNoAuthPageURL(httpRequest) ) // URL accessible without authentication
			{
				trace("The requested URL is a 'Trusted Page' : OK, let it pass ... ");
			    //--- Set the header for Ajax caller redirection
			    setTelosysRedirect(httpRequest, httpResponse); 
			    //--- Continue ... chain 
				chain.doFilter(request, response);
			}
			else
			{
			    //--- Not authenticated and URI is not an Authentication URI => force to authenticate
			    if ( getLoginPageURI( httpRequest ) != null )
			    {
			        //--- A LoginPage is set => use it
		            redirectToLoginPage(httpRequest, httpResponse, 1);
			    }
			    else
			    {
			        //--- No LoginPage => Redirect to BASIC AUTH 
					String sAuthURI = httpRequest.getContextPath() + BASIC_AUTH_LOGIN_PATH ; // "/context/l/o/g/i/n"
					trace("URL doesn't start with '" + sAuthURI + "' => '302' Redirect to Authentication URL.");
				    //--- "302" : Redirect to login URL
					redirectToBasicAuthLoginURL(httpRequest, httpResponse );
			    }
			}
		}
	}
	
	//---------------------------------------------------------------------------------------------------------
	/**
	 * Returns TRUE if the given request URI match the hard coded URI for Basic Authentication
	 * @param httpRequest
	 * @return
	 */
	private boolean isBasicAuthLoginURL(HttpServletRequest httpRequest) 
	{
		trace("isBasicAuthLoginURL('" + httpRequest.getRequestURI() + "').");
		String sAuthURI = httpRequest.getContextPath() + BASIC_AUTH_LOGIN_PATH ; // "/context/l/o/g/i/n/"
		String sURI = httpRequest.getRequestURI(); // "/context/path..."
		return sURI.startsWith(sAuthURI) ;
	}
	//---------------------------------------------------------------------------------------------------------
	/**
	 * Returns TRUE if the given request URI is the expected Authentication Login Page
	 * according to "LoginPage" parameter
	 * @param httpRequest
	 * @return
	 */
	private boolean isFormAuthLoginPageURL(HttpServletRequest httpRequest) 
	{
		trace("isFormAuthLoginPageURL('" + httpRequest.getRequestURI() + "').");
	    String sLoginPageURI = getLoginPageURI( httpRequest ); // null if no LoginPage parameter
		if ( sLoginPageURI != null  )
		{
			String sURI = httpRequest.getRequestURI(); // "/context/path..."
			return sURI.equals( sLoginPageURI ) ;
		}
		return false ; 
	}
	//---------------------------------------------------------------------------------------------------------
	/**
	 * Returns TRUE if the given request URI is the expected Authentication Action
	 * according to "LoginAction" parameter
	 * @param httpRequest
	 * @return
	 */
	private boolean isFormAuthLoginActionURL(HttpServletRequest httpRequest) 
	{
		trace("isFormAuthLoginActionURL('" + httpRequest.getRequestURI() + "').");
	    String sLoginActionURI = getLoginActionURI( httpRequest ); // null if no LoginAction parameter
		if ( sLoginActionURI != null  )
		{
			String sURI = httpRequest.getRequestURI(); // "/context/path..."
			return sURI.equals( sLoginActionURI ) ;
		}
		return false ; 
	}
	//---------------------------------------------------------------------------------------------------------
	/**
	 * Returns TRUE if the given request URI is the Login Error Page
	 * according to "LoginErrorPage" parameter
	 * @param httpRequest
	 * @return
	 */
	private boolean isLoginErrorPageURL(HttpServletRequest httpRequest) 
	{
		trace("isLoginErrorPageURL('" + httpRequest.getRequestURI() + "').");
	    String sLoginErrorPageURI = getLoginErrorPageURI( httpRequest ); // null if no LoginAction parameter
		if ( sLoginErrorPageURI != null  )
		{
			String sURI = httpRequest.getRequestURI(); // "/context/path..."
			return sURI.equals( sLoginErrorPageURI ) ;
		}
		return false ; 
	}
	//---------------------------------------------------------------------------------------------------------
	private boolean isNoAuthPageURL(HttpServletRequest httpRequest) 
	{	    
		trace("isNoAuthPageURL('" + httpRequest.getRequestURI() + "').");
		if ( _noAuthPages.isEmpty() )
		{
			return false ; 
		}
		else
		{
			String sContextPath = httpRequest.getContextPath() ; // "/context"
			String sReqURI = httpRequest.getRequestURI() ; // "/context/resource"
		    String sNoAuthPage = null ;
		    Iterator iter = _noAuthPages.iterator();
		    while ( iter.hasNext() )
		    {
		        sNoAuthPage = (String) iter.next();
		        if ( sReqURI.equals(sNoAuthPage) )
		        {
		            return true ;
		        }
		        if ( sReqURI.equals(sContextPath + sNoAuthPage) )
		        {
		            return true ;
		        }
		    }
		}
		return false ; 
	}
	//---------------------------------------------------------------------------------------------------------
	private String getFullURL(HttpServletRequest request, String sPath) 
	{
	    String sProtocol = request.getScheme() ; // http or https : EVOL force https ? ( get info from properties )
	    String sHost = request.getServerName() ;
	    int iPort = request.getServerPort() ;
	    String sContextPath = request.getContextPath() ;
	    
	    return sProtocol + "://" + sHost + ":" + iPort + sContextPath + sPath ;
	}
	//---------------------------------------------------------------------------------------------------------
//	private int getCount(HttpServletRequest request )
//	{
//	    int iCount = 0 ;
//	    String sCount = request.getParameter(COUNT_PARAM);
//	    if ( sCount != null )
//	    {
//	        iCount = StrUtil.getInt(sCount, 0);
//	    }
//	    trace("getCount() : return " + iCount);
//	    return iCount ;
//	}
	//---------------------------------------------------------------------------------------------------------
	private String getToken(HttpServletRequest request ) 
	{
	    String sToken = "";
	    String sURI = request.getRequestURI(); // "/(CONTEXT)/l/o/g/i/n/Txxxxxx/login" ( without QueryString )
		//trace("getToken() : request URI = '" + sURI + "'");
		
		int i = sURI.indexOf(BASIC_AUTH_LOGIN_PATH);
		if ( i >= 0 ) // Minimum for "/(CONTEXT)/l/o/g/i/n/"
		{
		    String s = sURI.substring(i+BASIC_AUTH_LOGIN_PATH.length()); // "Txxxxxx/login"
			int j = s.indexOf('/');
			if ( j > 0 )
			{
			    sToken = s.substring(0, j); // "Txxxxxxx"
			}
		}
		
		//trace("getToken() : return '" + sToken + "'");
		return sToken ;
	}
	//---------------------------------------------------------------------------------------------------------
	private String getNewToken() 
	{
	    Date date = new Date();	    
	    String sToken = TOKEN_PREFIX + date.getTime() ;
		//trace("getNewToken() : return '" + sToken + "'");
		return sToken ;
	}
	//---------------------------------------------------------------------------------------------------------
	private void redirectToBasicAuthLoginURL(HttpServletRequest request, HttpServletResponse response ) throws IOException //, ServletException
	{
	    String sToken = getNewToken();
	    String s = getFullURL( request, BASIC_AUTH_LOGIN_PATH + sToken + "/login") ;
		trace("sendRedirect to Login URL -> " + s);
		response.sendRedirect(s);
	}
    //------------------------------------------------------------------------
    /**
     * Build a http response with error code "401-Unauthorized"<br>
     * and header "WWW-Authenticate"
     * @param request
     * @param response
     */
    private void buildAuthenticateResponse(HttpServletRequest httpRequest, HttpServletResponse httpResponse, int iCount)
    {
        trace("buildAuthenticateResponse()... ( count = " + iCount + " )");
        //--- Set the "real name" (printed in the navigator dialog-box)
        String sToken = getToken(httpRequest) ;
        String sRealmName = TelosysAUTH.getRealmName();
        if ( TelosysAUTH.getRealmName() != null )
        {
            sRealmName = TelosysAUTH.getRealmName() + " (" + sToken + ")";
        }
        else if ( Telosys.getWebAppName() != null )
        {
            sRealmName = Telosys.getWebAppName() + " (" + sToken + ")";
        }
        else
        {
            sRealmName = httpRequest.getContextPath() + " (" + sToken + ")";
        }
        
		setCountCookie(httpRequest, httpResponse, iCount );
		httpResponse.setHeader("WWW-Authenticate", "Basic realm=\"" + sRealmName + "\"");
        //--- Code Retour HTTP "401-Unauthorized"
		httpResponse.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
    }

	//---------------------------------------------------------------------------------------------------------
	private void redirectToFirstPage(HttpServletRequest request, HttpServletResponse response, IAppUser appUser ) throws IOException, ServletException
	{
        //--- Try to get a specific First Page for the user
        String sUrlPage = appUser.getFirstPage();
        if (sUrlPage == null)
        {
            //--- Try to get the Application's First Page
            sUrlPage = TelosysAUTH.getFirstPage();
            if (sUrlPage == null)
            {
                //--- Default first page : WebApp root
                //sUrlPage = WebUtil.getRootUrl(request);
                sUrlPage = getFullURL(request, "/") ;
        		trace("No default application first page, use context root '" + sUrlPage + "'");
            }
            else
            {
        		trace("Default application first page = '" + sUrlPage + "'");
            }
        }
        else
        {
    		trace("User first page = '" + sUrlPage + "'");
        }
        if ( sUrlPage.startsWith("http") != true ) // Relative page 
        {
            sUrlPage = getFullURL(request, sUrlPage) ;
        }
		trace("sendRedirect to first page -> " + sUrlPage );
		response.sendRedirect(sUrlPage);
	}
	//---------------------------------------------------------------------------------------------------------
	private void redirectToLoginPage(HttpServletRequest httpRequest, HttpServletResponse httpResponse, int iCount ) throws IOException, ServletException
	{
        String sURI = getLoginPageURI(httpRequest);
        if ( sURI != null )
        {
    		trace("sendRedirect to login page -> '" + sURI + "'  ( count = " + iCount + " )");
    		setCountCookie(httpRequest, httpResponse, iCount );
            httpResponse.sendRedirect( sURI );
        }
        else // never happens
        {
            error("cannot redirect to login page (cannot get login page URI)");
            redirectToBasicAuthLoginURL(httpRequest, httpResponse);
        }
	}
	//---------------------------------------------------------------------------------------------------------
	private void printLoginErrorPage(HttpServletRequest httpRequest, HttpServletResponse httpResponse ) throws IOException, ServletException
	{
//        String sURI = getLoginErrorPageURI(httpRequest);
//        if ( sURI != null )
//		trace("sendRedirect to login error page -> '" + sURI + "' )");
//      httpResponse.sendRedirect( sURI );
	    
	    if ( _sLoginErrorPageParam != null )
        {            
    		trace("forward to login error page -> '" + _sLoginErrorPageParam + "' )");
            if ( _servletContext != null )
            {                
                RequestDispatcher rd = _servletContext.getRequestDispatcher( _sLoginErrorPageParam );
                rd.forward(httpRequest, httpResponse);
            }
            else
            {
                error("ServletContext is null => cannot forward to error page");
            }
        }
        else
        {
    		trace("generate login error response ");
            PrintWriter out = httpResponse.getWriter();
            out.println("<html>");
            out.println("<head>");
            out.println(" <title>Access denied</title>");
            out.println("</head>");
            out.println("<body>");
            out.println(" <div style=\"text-align:center;\">");
            out.println(" <h1>Not Authenticated</h1>");
            out.println(" <h1>Access denied !</h1>");
            out.println(" </div>");
            out.println("</body>");
            out.println("</html>");
            out.close();
        }
	}
	//---------------------------------------------------------------------------------------------------------
    private LoginUser getUserFromBasicAuthRequest(HttpServletRequest request)
    {
        //--- Get the "Authorization" HTTP header
        String sAuthorization = request.getHeader("Authorization");
        if (sAuthorization == null)
        {
            return null;
        }
        //--- Check the authorization type : "scheme BASIC"
        if (!sAuthorization.toLowerCase().startsWith("basic "))
        {
            return null;
        }
        //--- Get the Base64 string
        String sEncoded = sAuthorization.substring(6).trim();
        //--- Decode the Base64 string and parse it 
        String sDecoded = new String(Base64.decode(sEncoded.getBytes()));
        int iPosiColon = sDecoded.indexOf(':');
        if (iPosiColon < 0)
        {
            return null;
        }
        //--- Get the user login and password
        String sUserLogin = sDecoded.substring(0, iPosiColon).trim();
        String sUserPassword = sDecoded.substring(iPosiColon + 1).trim();

        //--- Retun the LoginUser instance
        return Auth.createLoginUser(sUserLogin, sUserPassword, request);
    }
	//---------------------------------------------------------------------------------------------------------
    private LoginUser getUserFromLoginActionRequest(HttpServletRequest request)
    {
        String sUserLogin = (String) request.getParameter( USER_LOGIN );
        String sUserPassword = (String) request.getParameter( USER_PASSWORD );
        this.trace("getUserFromLoginPageRequest() : login = '" + sUserLogin + "'" );
        
        //--- Retun the LoginUser instance
		LoginUser user = null ;
		if ( sUserLogin != null && sUserPassword != null )
		{
			user = Auth.createLoginUser(sUserLogin, sUserPassword, request);
		}
        return user;
    }
	//---------------------------------------------------------------------------------------------------------
    private void setCountCookie(HttpServletRequest httpRequest, HttpServletResponse httpResponse, int iCount )
    {
		Cookie cookie = new Cookie("TelosysAuth", ""+iCount );
		//cookie.setMaxAge(600); // 10 min
		String sLoginActionURI = getLoginActionURI(httpRequest) ;
		if ( sLoginActionURI != null ) 
		{
		    cookie.setPath( sLoginActionURI ); // limit the cookie usage to the LoginAction
		}
		httpResponse.addCookie(cookie);
    }
	//---------------------------------------------------------------------------------------------------------
    private int getCountCookieValue(HttpServletRequest httpRequest )
    {
		Cookie[] cookies = httpRequest.getCookies();
		if(cookies != null) 
		{
			for(int i=0 ; i<cookies.length ; i++) 
			{
				if ( "TelosysAuth".equals( cookies[i].getName() ) )
				{
					String sValue = cookies[i].getValue() ;
					if ( sValue != null )
					{
					    return StrUtil.getInt(sValue, 0);
					}
				}
			}
		}
		return 0 ;
    }
	//---------------------------------------------------------------------------------------------------------
    private void setAuthStatusCookie( HttpServletResponse httpResponse, int iStatus )
    {
		Cookie cookie = new Cookie("TelosysAuthStatus", ""+iStatus );
		//cookie.setMaxAge(600); // 10 min
		httpResponse.addCookie(cookie);
    }
	//---------------------------------------------------------------------------------------------------------
    private int getAuthStatusCookieValue(HttpServletRequest httpRequest )
    {
		Cookie[] cookies = httpRequest.getCookies();
		if(cookies != null) 
		{
			for(int i=0 ; i<cookies.length ; i++) 
			{
				if ( "TelosysAuthStatus".equals( cookies[i].getName() ) )
				{
					String sValue = cookies[i].getValue() ;
					if ( sValue != null )
					{
					    return StrUtil.getInt(sValue, 0);
					}
				}
			}
		}
		return 0 ;
    }
	//---------------------------------------------------------------------------------------------------------
}
