package org.objectweb.telosys.common;

/**
 * Generic abstract class for request parameters management.
 * 
 * @author Laurent GUERIN
 * @since v 1.0.2
 * 
 */
public interface IRequestParameters 
{

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or NULL if not found
     * @param paramName
     * @return
     */
    public String getParameter(String paramName);

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or the default value if not found
     * @param paramName
     * @param defaultValue : the default value used if the parameter is not found
     * @return 
     */
    public String getParameter(String paramName, String defaultValue);

    //-----------------------------------------------------------------------------------------------
    // BOOLEAN
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the boolean value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return true if the parameter value is "1" or "true" ( ignore case ), else false
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public boolean getParamAsBoolean(String paramName); 

    /**
     * Returns the boolean value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return true if the parameter value is "1" or "true" ( ignore case ), else false
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public boolean getParamAsBoolean(String paramName, boolean defaultValue ); 

    //-----------------------------------------------------------------------------------------------
    // INT
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the int value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public int getParamAsInt(String paramName); 

    /**
     * Returns the int value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public int getParamAsInt(String paramName, int defaultValue ); 

    //-----------------------------------------------------------------------------------------------
    // LONG
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the long value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public long getParamAsLong(String paramName);
    
    /**
     * Returns the long value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public long getParamAsLong(String paramName, long defaultValue ); 
    
    //-----------------------------------------------------------------------------------------------
    // SHORT
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the short value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public short getParamAsShort(String paramName); 

    /**
     * Returns the short value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public short getParamAsShort(String paramName, short defaultValue ); 
    
    //-----------------------------------------------------------------------------------------------
    // BYTE
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the byte value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public byte getParamAsByte(String paramName); 

    /**
     * Returns the byte value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public byte getParamAsByte(String paramName, byte defaultValue ); 

    //-----------------------------------------------------------------------------------------------
    // CHAR
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the char value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public char getParamAsChar(String paramName); 

    /**
     * Returns the char value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public char getParamAsChar(String paramName, char defaultValue ); 


    //-----------------------------------------------------------------------------------------------
    // FLOAT
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the float value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public float getParamAsFloat(String paramName); 

    /**
     * Returns the float value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public float getParamAsFloat(String paramName, float defaultValue); 
    
    //-----------------------------------------------------------------------------------------------
    // DOUBLE
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the double value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public double getParamAsDouble(String paramName); 

    /**
     * Returns the double value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public double getParamAsDouble(String paramName, double defaultValue); 
    
    //-----------------------------------------------------------------------------------------------
    // DATE
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the date/time/datetime value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public java.util.Date getParamAsDate(String paramName); 

    /**
     * Returns the date/time/datetime value of the given parameter name.<br>
     * Returns the given default value if the parameter cannot be found.<br>
     * Throws an exception if the parameter cannot be converted.
     * @param paramName
     * @param defaultValue
     * @return
     * @throws TelosysRuntimeException if the parameter value cannot be converted
     * @since 1.0.2
     */
    public java.util.Date getParamAsDate(String paramName, java.util.Date defaultValue); 
    
}
