package org.objectweb.telosys.common;

/**
 * @author Laurent GUERIN
 * 
 */
import java.io.*;
import java.util.*;

public final class LogWriter
{
    private static int STACK_MAX_CALLERS = 10 ;
    
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------
    // No log for the LogWriter ( v 0.9.9 )
    //private static final TelosysClassLogger log             = new TelosysClassLogger(LogWriter.class);

    private final static String             CONSOLE         = "CONSOLE";

    private final static String             LINE            = "------------------------------------------------------------";

    private final static int                LF              = 1;

    private final static int                CRLF            = 2;

    //--- Type de fin de ligne
    private int                             _iEndOfLine     = LF;

    //--- Collection des flux de sortie utilisables
    private Hashtable                       _htOutputs      = new Hashtable(5);

    //--- Collection des flux de sortie pour chaque mthode
    private OutputStream                    _errorOutputs[] = null;

    private OutputStream                    _infoOutputs[]  = null;

    private OutputStream                    _traceOutputs[] = null;

    public final static int                 INFO            = 1;

    public final static int                 TRACE           = 2;

    public final static int                 ERROR           = 3;

    //-----------------------------------------------------------------------------
    //--- CONSTRUCTORS
    //-----------------------------------------------------------------------------
    public LogWriter(String sFileName)
    {
        init(sFileName);
    }

    public LogWriter(Properties prop)
    {
        init(prop);
    }

    public LogWriter()
    {
        Properties prop = new Properties();
        prop.put("method.trace", "CONSOLE");
        prop.put("method.info", "CONSOLE");
        prop.put("method.error", "CONSOLE");
        init(prop);
    }

    //-----------------------------------------------------------------------------
    //--- PRIVATE METHODS
    //-----------------------------------------------------------------------------
    synchronized private void init(String sFileName)
    {
        if (sFileName != null)
        {
            FileInputStream fis = null;
            try
            {
                fis = new FileInputStream(sFileName);
                init(fis);
            }
            catch (FileNotFoundException ex)
            {
                internalError("init(" + sFileName + ")", ex);
            } finally
            {
                Tool.closeInputStream(fis);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void init(InputStream in)
    {
        if (in != null)
        {
            Properties prop = new Properties();
            try
            {
                prop.load(in);
                //log.trace("Properties loaded : ");
                Tool.showProperties(prop);
                // Telosys.trace(CLASS_NAME, prop.toString());
                init(prop);
            } catch (IOException ex)
            {
                internalError("init(InputStream)", ex);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void init(Properties prop)
    {
        //--- initialisation du type de fin de ligne (dans les flux de log)
        String sEndOfLine = prop.getProperty("endofline");
        if (sEndOfLine != null)
        {
            if (sEndOfLine.toUpperCase().trim().equals("CRLF"))
                _iEndOfLine = CRLF;
            if (sEndOfLine.toUpperCase().trim().equals("LF"))
                _iEndOfLine = LF;
        }
        //--- initialisation des flux de sortie utilisables
        initOutputs(prop);
        //--- initialisation des flux de sortie de chaque mthode
        _infoOutputs = initMethodeOutputs("info", prop);
        _traceOutputs = initMethodeOutputs("trace", prop);
        _errorOutputs = initMethodeOutputs("error", prop);
    }

    //-----------------------------------------------------------------------------
    synchronized private void internalError(String sMsg)
    {
        internalError(sMsg, null);
    }

    //-----------------------------------------------------------------------------
    synchronized private void internalError(String sMsg, Exception ex)
    {
        System.out.println("LogWritter internal error : " + sMsg );
        System.out.flush();
        
        //log.error("** INTERNAL ERROR ** : " + sMsg);
        if (ex != null)
        {
            System.out.println("LogWritter Exception : " + ex.getMessage() );
            System.out.flush();
            //Console.println(" Exception : " + ex.getMessage());
            ex.printStackTrace(Console.out);
            System.out.flush();
        }
    }

    //-----------------------------------------------------------------------------
    //--- Initialisation de la collection de flux de sortie dclars dans les
    // properties
    //-----------------------------------------------------------------------------
    private void initOutputs(Properties prop)
    {
        //log.trace("initOutputs(prop)");
        //--- Stockage de la CONSOLE (systmatiquement)
        addOuput(CONSOLE, null);

        //--- Stockage de tous les autres flux de sortie dclars dans les
        // properties
        Enumeration enumKeys = prop.keys();
        String sKey = null;
        String sKeyMinus = null;
        while (enumKeys.hasMoreElements()) // Pour chaque "cl" dans les
        // properties
        {
            sKey = (String) enumKeys.nextElement();
            //sKeyMinus = sKey.trim().toLowerCase() ;
            // Si la cl de la proprit commence par "output", c'est un nom de
            // fichier
            if (sKey.trim().toLowerCase().startsWith("output"))
            {
                addOuput(sKey, prop);
            }
        }
    }

    //-----------------------------------------------------------------------------
    private void addOuput(String sKey, Properties prop)
    {
        //log.trace("addOuput(" + sKey + ")");
        if ( sKey == null ) 
        {
            internalError("addOutput(null,...) : key is null");
            return ;
        }
        Object obj = _htOutputs.get(sKey);
        if (obj == null) // Non trouv => A AJOUTER
        {
            if (sKey.equals(CONSOLE))
            {
                //--- Sortie CONSOLE
                // NB: NullPointerException if the key or value is null
                if ( Console.out != null )
                {
                    _htOutputs.put(sKey, Console.out);                    
                }
            }
            else
            {
                //--- Sortie FICHIER
                String sFileName = prop.getProperty(sKey);
                //log.trace("sFileName = " + sFileName);
                if (sFileName != null)
                {
                    FileOutputStream fos = null ;
                    try
                    {
                        fos = new FileOutputStream(sFileName);
                        println(fos, LINE);
                        println(fos, " LOG INITIALIZED ");
                        println(fos, LINE);
                    } catch (FileNotFoundException ex) // Si cration impossible
                    {
                        // ERREUR
                        internalError("addOutput(" + sKey + ")", ex);
                    }
                    // NB: NullPointerException if the key or value is null 
                    if ( fos != null )
                    {
                        _htOutputs.put(sKey, fos);
                    }
                }
            }
        }
    }

    //-----------------------------------------------------------------------------
    //--- Initialisation des flux  utiliser pour chaque mthode : info, trace,
    // ...
    //-----------------------------------------------------------------------------
    //private PrintStream[] initMethodeOutputs( String sMethode, Properties
    // prop )
    private OutputStream[] initMethodeOutputs(String sMethode, Properties prop)
    {
        //log.trace("initMethodeOutputs(" + sMethode + ", prop)");
        Vector v = new Vector(5);
        String sOutputName;
        String sList = prop.getProperty("method." + sMethode);
        /**
         * if ( sList == null ) // Rien de dfini pour cette mthode { v.add(0, Console.out ); // Un seul flux de sortie : la console } else
         */
        // Si cette mthode est dfinie dans le fichier de properties (sinon
        // aucune trace)
        if (sList != null)
        {
            //--- Liste des noms de flux de sortie ( spars par : BLANC,
            // VIRGULE, POINT-VIRGULE )
            StringTokenizer st = new StringTokenizer(sList, " ,;");
            int i = 0;
            while (st.hasMoreTokens())
            {
                sOutputName = st.nextToken();
                //--- Ce nom de flux est il dfini ?
                Object obj = _htOutputs.get(sOutputName);
                if (obj != null) // Si OUI => On le prend en compte
                {
                    //log.trace(" v.add(" + sOutputName + " stream )");
                    v.add(i++, obj);
                }
                else
                {
                    internalError("initMethodeOutputs(" + sMethode + ", prop) : Stream object is null !");
                }
            }
        }

        //--- CASTING du tableau pour tre conforme au type de retour

        /***********************************************************************************************************************************************************************************************
         * //--- Mthode 1 : Par recopie du tableau initial dans un tableau de retour avec le bon type Object [] objects = v.toArray(); OutputStream[] streams = new OutputStream[objects.length] ;
         * System.arraycopy( objects, 0, streams, 0, objects.length ); return streams ;
         **********************************************************************************************************************************************************************************************/

        //--- Mthode 2 : En passant un objet "modle" pour le casting dans la
        // mthode "toArray"
        OutputStream[] typeForCasting = new OutputStream[0];
        OutputStream[] returnValue = (OutputStream[]) v.toArray(typeForCasting);
        return returnValue;

    }

    //-----------------------------------------------------------------------------
    //--- Ecriture dans chacun des flux de sortie associs au type de trace
    //-----------------------------------------------------------------------------
    private void println(OutputStream outputs[], String msg)
    {
        if (outputs != null)
        {
            //--- Pour chaque flux de sortie
            for ( int i = 0 ; i < outputs.length ; i++ )
            {
                println(outputs[i], msg);
            }
        }
    }

    //-----------------------------------------------------------------------------
    synchronized private void println(OutputStream out, String msg)
    {
        if (out != null)
        {
            try
            {
                out.write(msg.getBytes());
                if (_iEndOfLine == CRLF)
                {
                    out.write('\r');
                    out.write('\n');
                }
                else
                {
                    out.write('\n');
                }
                out.flush();
            } catch (IOException ex)
            {
                internalError("println(outputs[], msg) ", ex);
            }
        }
    }

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    public void error(String sClassName, String sMsg)
    {
        println(_errorOutputs, "ERROR : [" + sClassName + "] " + sMsg);
    }

    public void error(String sMsg)
    {
        println(_errorOutputs, "ERROR : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    public void warn(String sClassName, String sMsg)
    {
        println(_infoOutputs, "WARN  : [" + sClassName + "] " + sMsg);
    }

    public void warn(String sMsg)
    {
        println(_infoOutputs, "WARN  : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    public void info(String sClassName, String sMsg)
    {
        println(_infoOutputs, "info  : [" + sClassName + "] " + sMsg);
    }

    public void info(String sMsg)
    {
        println(_infoOutputs, "info  : " + sMsg);
    }

    //-----------------------------------------------------------------------------
    public void trace(String sClassName, String sMsg)
    {
        println(_traceOutputs, "Trace : [" + sClassName + "] " + sMsg);
    }

    public void trace(String sMsg)
    {
        println(_traceOutputs, "Trace : " + sMsg);
    }

// Not used => removed ( v 0.9.9 )
//    /**
//     * Print a non formated message on one the 3 output types of the logger
//     * 
//     * @param iOutputType :
//     *            INFO, TRACE, ERROR
//     * @param sMsg :
//     *            Message to print
//     */
//    public void println(int iOutputType, String sMsg)
//    {
//        switch (iOutputType)
//        {
//        case INFO :
//            println(_infoOutputs, sMsg);
//            break;
//        case TRACE :
//            println(_traceOutputs, sMsg);
//            break;
//        case ERROR :
//            println(_errorOutputs, sMsg);
//            break;
//        }
//    }

    //-----------------------------------------------------------------------------
    public void showConfig(PrintStream out)
    {
        out.println("------------------------------");
        out.println("Logger configuration : ");
        out.println(" Outputs declared : ");
        Enumeration enumKeys = _htOutputs.keys();
        String sKey = null;
        while (enumKeys.hasMoreElements()) // Pour chaque "cl" dans les
        // properties
        {
            sKey = (String) enumKeys.nextElement();
            out.println("  . " + sKey);
        }
        out.println(" number of outputs for 'info'  : " + (_infoOutputs == null ? 0 : _infoOutputs.length));
        out.println(" number of outputs for 'trace' : " + (_traceOutputs == null ? 0 : _traceOutputs.length));
        out.println(" number of outputs for 'error' : " + (_errorOutputs == null ? 0 : _errorOutputs.length));
        out.println("------------------------------");
    }
    //-------------------------------------------------------------------------------------
    public void exeptionReport( Throwable ex )
    {
        exeptionReport( ex, false );
    }
    //-------------------------------------------------------------------------------------
    public void exeptionReport( Throwable ex, boolean bCause )
    {
        String sCause = bCause ? "CAUSE " : "" ;
        println(_errorOutputs, "EXCEPTION " + sCause + ": [" + ex.getClass().getName() + "] : \"" + ex.getMessage() + "\"" );
        StackTraceElement st[] = ex.getStackTrace();
        StackTraceElement loc = null;
        if ( st.length > 0 )
        {
            //--- Exception origin ( 0 ) 
	        loc = st[0];
            //println(" in '" + loc.getFileName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "'" );
            //println(" in '" + loc.getClassName() + "', line " + loc.getLineNumber() + ", method '" + loc.getMethodName() + "', callers :" );
            //--- Callers stack ( 1 to N )
	        for ( int i = 0 ; i < st.length && i < STACK_MAX_CALLERS+1 ; i++ )
	        {
	            loc = st[i];
	            println(_errorOutputs, " . " + loc.getClassName() + "." + loc.getMethodName() + " : " + loc.getLineNumber() );
	        }
        }
        if ( ex.getCause() != null )
        {
            exeptionReport( ex.getCause(), true );
        }
    }

}