package org.objectweb.telosys.common;

import org.objectweb.telosys.util.DateUtil;

/**
 * Generic abstract class for request parameters management.
 * 
 * @author Laurent GUERIN
 * @since v 1.0.2
 * 
 */
public abstract class RequestParameters extends TelosysObject implements IRequestParameters
{

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or NULL if not found
     * @param paramName
     * @return
     */
    public abstract String getParameter(String paramName);

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or the default value if not found
     * @param paramName
     * @param defaultValue : the default value used if the parameter is not found
     * @return 
     */
    public abstract String getParameter(String paramName, String defaultValue);

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, 
     * throws TelosysRuntimeException if the parameter is not found.
     * @since v 1.0.2
     * @param sParamName
     * @return
     */
    private String getParamOrThrowException(String sParamName) 
    {
        String s = getParameter(sParamName);
        if ( s != null )
        {
            return s ;
        }
        else
        {
            throw new TelosysRuntimeException("No '" + sParamName + "' parameter");
        }
    }

    //-----------------------------------------------------------------------------------------------
    // BOOLEAN
    //-----------------------------------------------------------------------------------------------
    private boolean parseBoolean(String name, String value) 
    {
        String s = value.trim() ;
        if ( s.equals("1") ) return true ;
        if ( s.equalsIgnoreCase("true") ) return true ;
        return false ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsBoolean(java.lang.String)
     */
    public boolean getParamAsBoolean(String paramName) 
    {
        return parseBoolean(paramName, getParamOrThrowException(paramName) ) ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsBoolean(java.lang.String, boolean)
     */
    public boolean getParamAsBoolean(String paramName, boolean defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseBoolean(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // INT
    //-----------------------------------------------------------------------------------------------
    private int parseInt(String name, String value) 
    {
        try
        {
            return Integer.parseInt( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to int (value='"+value+"')");
        }        
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsInt(java.lang.String)
     */
    public int getParamAsInt(String paramName) 
    {
        return parseInt(paramName, getParamOrThrowException(paramName) );
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsInt(java.lang.String, int)
     */
    public int getParamAsInt(String paramName, int defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseInt(paramName, s ) : defaultValue ;
    }

    //-----------------------------------------------------------------------------------------------
    // LONG
    //-----------------------------------------------------------------------------------------------
    private long parseLong(String name, String value) 
    {
        try
        {
            return Long.parseLong( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to long (value='"+value+"')");
        }        
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsLong(java.lang.String)
     */
    public long getParamAsLong(String paramName) 
    {
        return parseLong(paramName, getParamOrThrowException(paramName) ) ;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsLong(java.lang.String, long)
     */
    public long getParamAsLong(String paramName, long defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseLong(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // SHORT
    //-----------------------------------------------------------------------------------------------
    private short parseShort(String name, String value) 
    {
        try
        {
            return Short.parseShort( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to short (value='"+value+"')");
        }        
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsShort(java.lang.String)
     */
    public short getParamAsShort(String paramName) 
    {
        return parseShort(paramName, getParamOrThrowException(paramName) ) ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsShort(java.lang.String, short)
     */
    public short getParamAsShort(String paramName, short defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseShort(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // BYTE
    //-----------------------------------------------------------------------------------------------
    private byte parseByte(String name, String value) 
    {
        try
        {
            return Byte.parseByte( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to byte (value='"+value+"')");
        }        
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsByte(java.lang.String)
     */
    public byte getParamAsByte(String paramName) 
    {
        return parseByte(paramName, getParamOrThrowException(paramName) ) ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsByte(java.lang.String, byte)
     */
    public byte getParamAsByte(String paramName, byte defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseByte(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // CHAR
    //-----------------------------------------------------------------------------------------------
    private char parseChar(String name, String value) 
    {
        String s = value ;        
        if ( s.length() > 1 )
        {
            s = s.trim();
        }
        if ( s.length() != 1 )
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to char (value='"+value+"')");
        }
        else
        {
            return s.charAt(0);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsChar(java.lang.String)
     */
    public char getParamAsChar(String paramName) 
    {
        return parseChar(paramName, getParamOrThrowException(paramName) ) ;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsChar(java.lang.String, char)
     */
    public char getParamAsChar(String paramName, char defaultValue ) 
    {
        String s = getParameter(paramName);
        return s != null ? parseChar(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // FLOAT
    //-----------------------------------------------------------------------------------------------
    private float parseFloat(String name, String value) 
    {
        try
        {
            return Float.parseFloat( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to float (value='"+value+"')");
        }        
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsFloat(java.lang.String)
     */
    public float getParamAsFloat(String paramName) 
    {
        return parseFloat(paramName, getParamOrThrowException(paramName));
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsFloat(java.lang.String, float)
     */
    public float getParamAsFloat(String paramName, float defaultValue) 
    {
        String s = getParameter(paramName);
        return s != null ? parseFloat(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // DOUBLE
    //-----------------------------------------------------------------------------------------------
    private double parseDouble(String name, String value) 
    {
        try
        {
            return Double.parseDouble( value.trim() );
        }
        catch (NumberFormatException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to double (value='"+value+"')");
        }        
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDouble(java.lang.String)
     */
    public double getParamAsDouble(String paramName) 
    {
        return parseDouble( paramName, getParamOrThrowException(paramName) ) ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDouble(java.lang.String, double)
     */
    public double getParamAsDouble(String paramName, double defaultValue) 
    {
        String s = getParameter(paramName);
        return s != null ? parseDouble(paramName, s ) : defaultValue ;
    }
    
    //-----------------------------------------------------------------------------------------------
    // DATE
    //-----------------------------------------------------------------------------------------------
    private java.util.Date parseDate(String name, String value) 
    {
        //java.util.Date date = null ;
        try
        {
            String sTrim = value.trim();
            //--- Use the LENGTH to determine the type 
            int length = sTrim.length() ;
            if ( length == 0 ) // ""
            {
                return null ;
            }
            else if ( length == 10 ) // "YYYY-MM-DD"
            {
                return DateUtil.parseDate(sTrim);
            }
            else if ( length == 8 ) // "HH:MM:SS"
            {
                return DateUtil.parseTime(sTrim);
            }
            else if ( length == 19 ) // "YYYY-MM-DD HH:MM:SS"
            {
                return DateUtil.parseDateTime(sTrim);
            }
            else
            {
                throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                        + "' to Date (value='"+value+"') : invalid length");
            }
        }
        catch (TelosysRuntimeException ex)
        {
            throw new TelosysRuntimeException("Cannot convert parameter '" + name 
                    + "' to Date (value='"+value+"') : " + ex.getMessage() );
        }        
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDouble(java.lang.String)
     */
    public java.util.Date getParamAsDate(String paramName) 
    {
        return parseDate( paramName, getParamOrThrowException(paramName) ) ;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDouble(java.lang.String, double)
     */
    public java.util.Date getParamAsDate(String paramName, java.util.Date defaultValue) 
    {
        String s = getParameter(paramName);
        return s != null ? parseDate(paramName, s ) : defaultValue ;
    }
    
}
