package org.objectweb.telosys.common; 

import java.util.Date;
import java.util.Properties;

/**
 * @author Laurent GUERIN
 *
 */
public class TelosysProperties
{
    //--- Origin of the properties
    public final static int UNKNOWN    = 0 ;
    public final static int FILESYSTEM = 1 ;
    public final static int WEBAPP     = 2 ;
    public final static int CLASSPATH  = 3 ;
    
    private Properties _prop = null ;

    private String     _sFile = null ; // Properties file 
    
    private int        _iOrigin = UNKNOWN ; // Origin of the properties ( WEBAPP, CLASSPATH, FILESYSTEM )
    
    private Date       _initDate = null ; // Date & time of instance construction
    
    //-----------------------------------------------------------------------------------------
    /**
     * @param prop
     * @param file
     * @param origin
     */
    protected TelosysProperties(Properties prop, String file, int origin)
    {
        super();
        _prop = prop;
        _sFile = file;
        if ( origin >= 1 && origin <= 3 )
        {
            _iOrigin = origin;
        }
        _initDate = new Date();
    }
    
    //-----------------------------------------------------------------------------------------
    public int getOrigin()
    {
        return _iOrigin;
    }
    
    //-----------------------------------------------------------------------------------------
    public Properties getProperties()
    {
        return _prop;
    }
    
    //-----------------------------------------------------------------------------------------
    public String getFile()
    {
        return _sFile;
    }

    //-----------------------------------------------------------------------------------------
    public Date getInitDate()
    {
        return _initDate;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Returns the value of the given property name
     * @param sPropertyName
     * @return the property value or null
     */
    public String getProperty(String sPropertyName) // v 1.0.0
    {
        if ( _prop != null )
        {
            return _prop.getProperty(sPropertyName);
        }
        return null;
    }
    
    //-----------------------------------------------------------------------------------------
    private String getLocation()
    {
        if ( _sFile != null )
        {            
	        int i1 = _sFile.lastIndexOf('/');
	        int i2 = _sFile.lastIndexOf('\\');
	        int i = ( i1 > i2 ? i1 : i2 ) ; // Take the Max
	        if ( i >= 0 )
	        {
	            return _sFile.substring(0,i); // Root without ending '/' or '\'
	        }
	        else
	        {
	            return "" ;
	        }
        }
        return "";
    }
    //-----------------------------------------------------------------------------------------
    private String transformFileName(String sFile)
    {
        if ( sFile == null )
        {
            return null ;
        }
        
        if ( sFile.length() == 0 )
        {
            return sFile ;
        }
        
        //--- If the file name starts with "." => it's a relative path 
        if ( sFile.charAt(0) == '.' )
        {
            // ie : "./foo" => "Dir/foo"
            return getLocation() + sFile.substring(1) ;
        }
        //--- Not special path => return "as is"
        return sFile ;
    }
    
    //-----------------------------------------------------------------------------------------
    /**
     * Returns the config file name associated with the given property name <br>
     * NB : if the value starts with ".", the "." is replaced by the "telosys.properties" location <br>
     * e.g. : "./myfile" --> "location/myfile"
     * @param sPropertyName : e.g. "ScreensConfFile", "DbConfFile", ... ( see telosys.properties )
     * @return the value 
     */
    public String getConfigFile(String sPropertyName)
    {
        if ( _prop != null )
        {
            String s = _prop.getProperty(sPropertyName);
            if (s != null)
            {
                return transformFileName( s.trim() );
            }
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------
    // Other configuration files 
    //-----------------------------------------------------------------------------------------
//    public String getTracePropertiesFile()
//    {
//        return getConfigFile("TraceConfFile");
//    }
//    //-----------------------------------------------------------------------------------------
//    public String getTagsPropertiesFile()
//    {
//        return getConfigFile("TagsConfFile");
//    }
//    //-----------------------------------------------------------------------------------------
//    public String getDbConfFile()
//    {
//        return getConfigFile("DbConfFile");
//    }
//    //-----------------------------------------------------------------------------------------
//    public String getScreensConfFile()
//    {
//        return getConfigFile("ScreensConfFile");
//    }
    
    //-----------------------------------------------------------------------------------------
    public String toString()
    {
        String s = "unknown" ;
        switch(_iOrigin)
        {
            case FILESYSTEM : s = "file system" ; break ;
            case WEBAPP : s = "web application" ; break ;
            case CLASSPATH : s = "class path" ; break ;
        }
        return "Telosys properties loaded from '"+_sFile+"' ( origin : " + s + " ) at " + _initDate ; 
    }
    //-----------------------------------------------------------------------------------------
}
