package org.objectweb.telosys.common;

/**
 * 
 * @author Laurent GUERIN
 *  
 */

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import javax.servlet.ServletContext;

import org.objectweb.telosys.common.TelosysClassLogger;

public class TelosysPropertiesLoader
{
    private static final TelosysClassLogger log                = new TelosysClassLogger(TelosysPropertiesLoader.class);

    private static final String             TELOSYS_PROPERTIES_PARAM = "TelosysProperties" ;
    
    private static final String             TELOSYS_PROPERTIES = "telosys.properties";
    
    private static final String             WEB_INF_CONF_TELOSYS_PROPERTIES = "/WEB-INF/conf/telosys.properties" ;

    //-----------------------------------------------------------------------------------------
    /**
     * Try to load the main TELOSYS properties file <br>
     * Step 1 - Find init parameter of the "web.xml" (file or resource name) <br>
     * Step 1.1 - try to load it from the filesystem  <br>
     * Step 1.2 - try to load it from the WebApp context <br>
     * Step 2 - Try to load the standard "/WEB-INF/conf/telosys.properties"  <br>
     * Step 3 - Try to load the standard "telosys.properties" via the CLASSPATH <br>
     * 
     * @param servletContext : the ServletContext to use for 'web.xml' parameters and 'WEB-INF/conf/telosys.properties'( can be null )
     * @return
     */
    public static TelosysProperties load( ServletContext servletContext )
    {
        TelosysProperties prop = null ;
        if ( servletContext != null )
        {
            //---------------------------------------------------------------------------------------
            //--- (1) Try to find the properties file name in the "web.xml" ( in servlet init parameters )
            //---------------------------------------------------------------------------------------
            log.info("[1] Try to find properties name with 'context-param' from the 'web.xml' ... ");
            String sTelosysPropFile = servletContext.getInitParameter( TELOSYS_PROPERTIES_PARAM );
            if (sTelosysPropFile != null)
            {
                log.info("[1] Parameter '" + TELOSYS_PROPERTIES_PARAM + "' found in web.xml : '" + sTelosysPropFile + "'");

                //--- Try to find as FILE ...
                log.info("[1.1] Try to load '" + sTelosysPropFile + "' from filesystem ...");
                prop = loadPropertiesFromFileSystem ( sTelosysPropFile );
                if ( prop != null )
                {
                    log.info("[1.1] OK : File '" + sTelosysPropFile + "' found." );
                    return prop ;
                }
                else
                {
                    log.info("[1.1] File '" + sTelosysPropFile + "' not found." );
                }
                
                //--- Try to find as WAR RESOURCE ...
                log.info("[1.2] Try to load '" + sTelosysPropFile + "' from WebApp context ...");
                prop = loadPropertiesFromWebAppResource(sTelosysPropFile, servletContext );
                if ( prop != null )
                {
                    log.info("[1.2] OK : Resource '" + sTelosysPropFile + "' found.");
                    return prop ;
                }
                else
                {
                    log.info("[1.2] Resource '" + sTelosysPropFile + "' not found.");
                }
            }
            else
            {
                log.info("[1] No parameter '" + TELOSYS_PROPERTIES_PARAM + "' in web.xml ");
            }

            //---------------------------------------------------------------------------------------
            //--- (2) Try to find the Prop File in the "/WEB-INF/conf" directory
            //---------------------------------------------------------------------------------------
            log.info("[2] Try to load standard '" + WEB_INF_CONF_TELOSYS_PROPERTIES + "' from WebApp context ... ");
            //--- Try to find as WAR RESOURCE ...
            prop = loadPropertiesFromWebAppResource( WEB_INF_CONF_TELOSYS_PROPERTIES, servletContext );
            if ( prop != null )
            {
                log.info("[2] OK : Resource '" + WEB_INF_CONF_TELOSYS_PROPERTIES + "' found.");
                return prop ;
            }
            else
            {
                log.info("[2] Resource '" + WEB_INF_CONF_TELOSYS_PROPERTIES + "' not found.");
            }
        }
        else
        {
            log.info("No ServletContext ( not in WebApp ) => Skip steps [1] and [2]. ");
        }

        //---------------------------------------------------------------------------------------
        //--- (3) Try to find the Prop File using the classpath
        //---------------------------------------------------------------------------------------
        log.info("[3] Try to load '" + TELOSYS_PROPERTIES + "' via CLASSPATH... ");
        prop = loadPropertiesFromClassPath( TELOSYS_PROPERTIES );
        if ( prop != null )
        {
            log.info("[3] OK : '" + TELOSYS_PROPERTIES + "' found with ClassPath " );
            return prop ;
        }
        else
        {
            log.info("[3] Resource '" + TELOSYS_PROPERTIES + "' not found with ClassPath.");
        }

        return null ;
    }
    
    //-----------------------------------------------------------------------------------------
    /**
     * Try to load 'telosys.properties' via CLASSPATH
     * @return
     */
    public static TelosysProperties load()
    {
        log.info("[1] Try to load '" + TELOSYS_PROPERTIES + "' via CLASSPATH... ");
        TelosysProperties prop = loadPropertiesFromClassPath( TELOSYS_PROPERTIES );
        if ( prop != null )
        {
            log.info("[1] OK : '" + TELOSYS_PROPERTIES + "' found with ClassPath " );
            return prop ;
        }
        else
        {
            log.info("[1] Resource '" + TELOSYS_PROPERTIES + "' not found with ClassPath.");
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------------
    /**
     * Try to load a specific properties file from the filesystem 
     * @param sFile
     * @return
     */
    public static TelosysProperties load( String sFile )
    {
        log.info("[1] Try to load '" + sFile + "' from filesystem ...");
        TelosysProperties prop = loadPropertiesFromFileSystem ( sFile );
        if ( prop != null )
        {
            log.info("[1] OK : File '" + sFile + "' found." );
            return prop ;
        }
        else
        {
            log.info("[1] File '" + sFile + "' not found." );
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------
    protected static TelosysProperties loadProperties ( String sName, int iOrigin, ServletContext servletContext )
    {
        switch(iOrigin)
        {
            case TelosysProperties.FILESYSTEM : 
                return loadPropertiesFromFileSystem ( sName ) ; 
            case TelosysProperties.WEBAPP : 
                return loadPropertiesFromWebAppResource( sName, servletContext ) ; 
            case TelosysProperties.CLASSPATH : 
                return loadPropertiesFromClassPath( sName ) ; 
        }
        log.error("loadProperties("+sName+","+iOrigin+",..) : invalid origin parameter " + iOrigin );
        return null ;
    }
    
    /**
     * Loads a properties file located in the file system
     * @param sFileName
     * @return
     */
    private static TelosysProperties loadPropertiesFromFileSystem ( String sFileName )
    {
        TelosysProperties tp = null ;
        FileInputStream fis = null ;
        try
        {
            fis = new FileInputStream(sFileName);
        } catch (FileNotFoundException e)
        {
            log.trace("loadPropertiesFromFileSystem() : file '" + sFileName + "' not found " );
            return null ;
        }
        Properties prop = loadPropertiesInputStream(fis);
        tp = buildFileSystemProperties ( prop, sFileName );
        return tp;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Loads properties via the ClassLoader (using the ClassPath )
     * The properties file can be located in a folder or a JAR file accessible via the CLASSPATH
     * ( the WebApp WAR file can be exploded or not )
     * @param sResource
     * @return
     */
    private static TelosysProperties loadPropertiesFromClassPath ( String sResource )
    {
        TelosysProperties tp = null ;
        //Properties prop = null;
        Thread t = Thread.currentThread() ;
        if ( t != null )
        {
            ClassLoader cl = t.getContextClassLoader() ;
            if ( cl != null )
            {
                InputStream is = cl.getResourceAsStream(sResource);
                if ( is != null )
                {
                    Properties prop = loadPropertiesInputStream(is);
                    tp = buildClassPathProperties ( prop, sResource, cl );                    
                }
                else
                {
                    log.trace("loadPropertiesFromClassPath() : resource '" + sResource + "' not found " );
                }
            }
            else
            {
                log.error("loadPropertiesFromClassPath() : cannot get ClassLoader from current thread ! " );
            }
        }
        else
        {
            log.error("loadPropertiesFromClassPath() : cannot get current thread ! " );
        }
        return tp ;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Loads properties as a "WebApp resource" 
     * The properties file can be located in the WebApp folder (exploded) 
     * or in the WebApp WAR file (non exploded)
     * @param sResource
     * @param servletContext
     * @return
     */
    private static TelosysProperties loadPropertiesFromWebAppResource ( String sResource, ServletContext servletContext )
    {
        if ( servletContext == null )
        {
            log.error("loadPropertiesFromWebAppResource('" + sResource + "', servletContext ) : ServletContext is null !");
            return null ;
        }
        TelosysProperties tp = null ;
        InputStream is = servletContext.getResourceAsStream(sResource);
        if ( is != null )
        {
            Properties prop = loadPropertiesInputStream(is);
            tp = buildWebAppProperties ( prop, sResource, servletContext );
        }
        else
        {
            log.trace("loadPropertiesFromWebAppResource() : resource '" + sResource + "' not found " );
        }
        return tp;
    }
    //-----------------------------------------------------------------------------------------
    private static Properties loadPropertiesInputStream ( InputStream is  )
    {
        Properties prop = null;
        if ( is != null )
        {            
            prop = new Properties();
            try
            {
                prop.load(is);
            } catch (IOException ex)
            {
                log.error("loadPropertiesInputStream() : IOException : " + ex.getMessage());
                prop = null;
	        } finally
	        {
                try {
                       is.close();
                } catch (IOException ex) {}
	        }
        }
        else
        {
            log.error("loadPropertiesInputStream() : InputStream is NULL " );
        }
        return prop ;
    }
    
    //-----------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------
    private static TelosysProperties buildFileSystemProperties ( Properties prop, String sFilePath )
    {
        if ( prop != null && sFilePath != null )
        {
            return new TelosysProperties(prop, sFilePath, TelosysProperties.FILESYSTEM );
        }
        return null ;
    }    
    //-----------------------------------------------------------------------------------------
    private static TelosysProperties buildWebAppProperties ( Properties prop, String sResource, ServletContext servletContext )
    {
        if ( prop != null && sResource != null && servletContext != null )
        {
            try
            {
                URL url = servletContext.getResource(sResource);
                if ( url != null )
                {
                    //return new TelosysProperties(prop, url.getPath(), TelosysProperties.WEBAPP );
                    return new TelosysProperties(prop, sResource, TelosysProperties.WEBAPP );
                }
            } catch (MalformedURLException e)
            {
                log.error("buildWebAppProperties( '" + sResource + "' ) : MalformedURLException!");
            }
        }
        return null ;
    }    
    //-----------------------------------------------------------------------------------------
    private static TelosysProperties buildClassPathProperties ( Properties prop, String sResource, ClassLoader cl )
    {
        if ( prop != null && sResource != null && cl != null )
        {
            URL url = cl.getResource(sResource);
            if ( url != null )
            {
                //return new TelosysProperties(prop, url.getPath(), TelosysProperties.CLASSPATH );
                return new TelosysProperties(prop, sResource, TelosysProperties.CLASSPATH );
            }
        }
        return null ;
    }
    //-----------------------------------------------------------------------------------------
    
    //-----------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------
    protected static InputStream getInputStream ( String sName, int iOrigin, ServletContext servletContext )
    {
        switch(iOrigin)
        {
            case TelosysProperties.FILESYSTEM : 
                return getInputStreamFromFileSystem ( sName ) ; 
            case TelosysProperties.WEBAPP : 
                return getInputStreamFromWebAppResource( sName, servletContext ) ; 
            case TelosysProperties.CLASSPATH : 
                return getInputStreamFromClassPath( sName ) ; 
        }
        log.error("getInputStream("+sName+","+iOrigin+",..) : invalid origin parameter " + iOrigin );
        return null ;
    }

    //-----------------------------------------------------------------------------------------
    private static InputStream getInputStreamFromFileSystem ( String sFileName )
    {
        FileInputStream fis = null ;
        try
        {
            fis = new FileInputStream(sFileName);
        } catch (FileNotFoundException e)
        {
            log.trace("getInputStreamFromFileSystem() : file '" + sFileName + "' not found " );
            return null ;
        }
        return fis;
    }
    //-----------------------------------------------------------------------------------------
    private static InputStream getInputStreamFromWebAppResource ( String sResource, ServletContext servletContext )
    {
        if ( servletContext == null )
        {
            log.error("getInputStreamFromWebAppResource('" + sResource + "', servletContext ) : ServletContext is null !");
            return null ;
        }
        InputStream is = servletContext.getResourceAsStream(sResource);
        if ( is == null )
        {
            log.trace("getInputStreamFromWebAppResource() : resource '" + sResource + "' not found " );
        }
        return is;
    }
    //-----------------------------------------------------------------------------------------
    private static InputStream getInputStreamFromClassPath ( String sResource )
    {
        Thread t = Thread.currentThread() ;
        if ( t != null )
        {
            ClassLoader cl = t.getContextClassLoader() ;
            if ( cl != null )
            {
                return cl.getResourceAsStream(sResource);
            }
            else
            {
                log.error("getInputStreamFromClassPath() : cannot get ClassLoader from current thread ! " );
            }
        }
        else
        {
            log.error("getInputStreamFromClassPath() : cannot get current thread ! " );
        }
        return null ;
    }
    //-----------------------------------------------------------------------------------------
}