package org.objectweb.telosys.common.vo;

import java.io.Serializable;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Generic abstract class to build Value Objects List
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GenericVOList extends TelosysObject implements Serializable
{
    private final static int UNSELECTED = -1 ;
    
    //-----------------------------------------------------------------------------
    // ATTRIBUTES(S)
    //-----------------------------------------------------------------------------

    private LinkedList _list           = null;

    private Class      _elementClass   = null;

    private int        _iCurrentIndex  = 0; // Used by getFirstElement & getNextElement

    private boolean    _bSorted        = false;

    private String     _sSortedByField = "";

    private boolean    _bSortOrder     = FieldComparer.ASC;

    private int        _iSelectedIndex = UNSELECTED;  
    
    //-----------------------------------------------------------------------------
    // CONSTRUCTOR(S)
    //-----------------------------------------------------------------------------
    protected GenericVOList(Class eltClass)
    {
        if (eltClass == null)
        {
            throw new TelosysRuntimeException("GenericVOList constructor : element class argument is null ");
        }
        _elementClass = eltClass;
        _list = new LinkedList();
        _iCurrentIndex = 0;
    }

    //-----------------------------------------------------------------------------
    // PRIVATE METHOD(S)
    //-----------------------------------------------------------------------------
    /**
     * Returns the boolean order flag according to the given order in string format
     * @param sOrder
     * @return
     */
    private boolean getSortOrderFlag(String sOrder)
    {
        if (sOrder == null)
        {
            throw new TelosysRuntimeException("VOList.sort : order parameter is 'null' ");
        }
        String sOrderUC = sOrder.trim().toUpperCase();
        if (sOrderUC.startsWith("ASC"))
        {
            return FieldComparer.ASC;
        }
        else if (sOrderUC.startsWith("DESC"))
        {
            return FieldComparer.DESC;
        }
        else
        {
            throw new TelosysRuntimeException("VOList.sort : invalid order parameter '" + sOrder + "' ( 'ASC' or 'DESC' expected )");
        }
    }

    /**
     * Sorts the list according to the given parameters
     * @param sFieldName : field to use as sort criteria
     * @param bOrder : sort order flag ( ASC / DESC )
     * @param bFlagIgnoreCase : ignore case flag 
     */
    private void sortList(String sFieldName, boolean bOrder, boolean bFlagIgnoreCase)
    {
        if (sFieldName == null)
        {
            throw new TelosysRuntimeException("VOList.sort : field name is 'null' ");
        }
        String sField = sFieldName.trim();
        Collections.sort(_list, new FieldComparer(sField, bOrder, bFlagIgnoreCase));
        _bSorted = true;
        _sSortedByField = sField;
        _bSortOrder = bOrder;
    }

    
    //-----------------------------------------------------------------------------
    // PROTECTED METHOD(S)
    //-----------------------------------------------------------------------------
    /**
     * Appends the given element to the end of the list
     * 
     * @param newElement
     */
    protected void addElement(Object newElement)
    {
        _list.addLast(newElement);
    }

    //-----------------------------------------------------------------------------
    // Inserts the specified element at the specified position in the list.
    //-----------------------------------------------------------------------------
    protected void insertElement(int index, Object newElement)
    {
        _list.add(index, newElement);
    }

    //-----------------------------------------------------------------------------
    // Replaces the element at the specified position in the list with the
    // specified element.
    //-----------------------------------------------------------------------------
    protected Object replaceElement(int index, Object newElement)
    {
        return (Object) _list.set(index, newElement);
    }

    //-----------------------------------------------------------------------------
    // Removes the first occurrence of the specified element in the list.
    //-----------------------------------------------------------------------------
    protected boolean removeElement(Object element)
    {
        return _list.remove(element);
    }

    //-----------------------------------------------------------------------------
    // Removes the element at the specified position in the list.
    //-----------------------------------------------------------------------------
    protected Object removeElement(int index)
    {
        return (Object) _list.remove(index);
    }

    //-----------------------------------------------------------------------------
    // Returns the element at the specified position in this list.
    //-----------------------------------------------------------------------------
    protected Object getElement(int index)
    {
        if (_list.isEmpty())
        {
            return null;
        }
        else
        {
            return (Object) _list.get(index);
        }
    }

    //-----------------------------------------------------------------------------
    // Returns the first element in the list.
    //-----------------------------------------------------------------------------
    protected Object getFirstElement()
    {
        if (_list.isEmpty())
        {
            _iCurrentIndex = 0;
            return null;
        }
        else
        {
            _iCurrentIndex = 0;
            return (Object) _list.get(_iCurrentIndex);
        }
    }

    //-----------------------------------------------------------------------------
    // Returns the next element in the list.
    //-----------------------------------------------------------------------------
    protected Object getNextElement()
    {
        if (_list.isEmpty())
        {
            _iCurrentIndex = 0;
            return null;
        }
        else
        {
            if (_iCurrentIndex + 1 < _list.size())
            {
                _iCurrentIndex++;
                return (Object) _list.get(_iCurrentIndex);
            }
            else
            {
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHOD(S)
    //-----------------------------------------------------------------------------
    /**
     * Returns the type (class) of the elements stored in the list
     * 
     * @return
     */
    public Class getElementType()
    {
        return _elementClass;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the encapsulated List object
     * 
     * @return
     */
    public List getList()
    {
        return _list;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns an iterator over the elements in this list in proper sequence
     * 
     * @return the iterator
     */
    public Iterator iterator()
    {
        return _list.iterator();
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns a list iterator of the elements in this list (in proper sequence)
     * 
     * @return the list iterator
     */
    public ListIterator listIterator()
    {
        return _list.listIterator();
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the number of elements in this list.
     * 
     * @return
     */
    public int size()
    {
        return _list.size();
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns 'true' if this list contains no elements
     * 
     * @return
     */
    public boolean isEmpty()
    {
        return _list.isEmpty();
    }

    //-----------------------------------------------------------------------------
    /**
     * Removes all of the elements from this list
     */
    public void clear()
    {
        _list.clear();
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Select the given index   
     * @param iNewSelectedIndex
     * @return the previous selected index
     */
    public int select( int iNewSelectedIndex )
    {
        int iRet = _iSelectedIndex ;
        if (_list.isEmpty())
        {
            throw new TelosysRuntimeException("VOList.select() : the list is empty");
        }
        else
        {
            if ( iNewSelectedIndex >= 0 && iNewSelectedIndex < _list.size() )
            {
                _iSelectedIndex = iNewSelectedIndex ;                
            }
            else
            {
                throw new TelosysRuntimeException("VOList.select("+iNewSelectedIndex+") : out of bounds (list size =" + _list.size() + ")" );                
            }
        }
        return iRet ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Set the list in a state where there's no selected element (no selected index)  
     * @return the previous selected index
     */
    public int unselect()
    {
        int iRet = _iSelectedIndex ;
        _iSelectedIndex = UNSELECTED ;
        return iRet ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the current selected index (or -1 if no selection)
     * @return
     */
    public int getSelectedIndex()
    {
        return _iSelectedIndex ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns true if the given index is the current selected index
     * @param iIndex
     * @return
     */
    public boolean isSelected( int iIndex )
    {
        return (  iIndex == _iSelectedIndex ? true : false ) ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the current selected element (or null if no element selected) 
     * @return
     */
    public Object getSelectedElement()
    {
        if ( _iSelectedIndex >= 0 && _iSelectedIndex < _list.size() )
        {
            return _list.get( _iSelectedIndex ) ;
        }
        return null ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Replaces the current selected element if any
     * @param newElement
     * @return true if done (if there is a valid current selected index), else false 
     */
    public boolean replaceSelectedElement(Object newElement)
    {
        if ( _iSelectedIndex >= 0 && _iSelectedIndex < _list.size() )
        {
            //return (Object) _list.set(_iSelectedIndex, newElement);
            _list.set(_iSelectedIndex, newElement);
            return true ;
        }
        return false ;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the field name used as current sort criteria
     * 
     * @return : field name, or "" if the list is not sorted
     */
    public String getSortField()
    {
        if (_bSorted && (_sSortedByField != null))
        {
            return _sSortedByField;
        }
        return "";
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted by the given field
     * 
     * @param sFieldName
     * @return true/false
     */
    public boolean isSortedBy(String sFieldName)
    {
        if (sFieldName == null)
        {
            return false;
        }
        if (_bSorted && (_sSortedByField != null))
        {
            if (_sSortedByField.equalsIgnoreCase(sFieldName.trim()))
            {
                return true;
            }
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted in ascending order
     * @return
     */
    public boolean isSortedInAscendingOrder()
    {
        if (_bSorted && (_bSortOrder == FieldComparer.ASC))
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns true if the list is currently sorted in descending order
     * @return
     */
    public boolean isSortedInDescendingOrder()
    {
        if (_bSorted && (_bSortOrder == FieldComparer.DESC))
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria (case sensitive) in ascending order
     * 
     * @param sFieldName :
     *            the name of the field to compare
     */
    public void sort(String sFieldName)
    {
        //String sField = getSortFieldName(sFieldName);
        //Collections.sort( _list, new FieldComparer(sField, FieldComparer.ASC, FieldComparer.DO_NOT_IGNORE_CASE ));
        sortList(sFieldName, FieldComparer.ASC, FieldComparer.DO_NOT_IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria (case sensitive) and the specified order ( ascending or descending )
     * 
     * @param sFieldName :
     *            the name of the field to compare
     * @param sOrder :
     *            'ASC' = ascending order , 'DESC' = descending order
     */
    public void sort(String sFieldName, String sOrder)
    {
        //String sField = getSortFieldName(sFieldName);
        boolean bOrder = getSortOrderFlag(sOrder);
        //Collections.sort( _list, new FieldComparer(sField, bOrder, FieldComparer.DO_NOT_IGNORE_CASE ));
        sortList(sFieldName, bOrder, FieldComparer.DO_NOT_IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria ( ignoring case ) in ascending order
     * 
     * @param sFieldName :
     *            the name of the field to compare
     */
    public void sortIgnoreCase(String sFieldName)
    {
        //String sField = getSortFieldName(sFieldName);
        //Collections.sort( _list, new FieldComparer(sField, FieldComparer.ASC, FieldComparer.IGNORE_CASE ));
        sortList(sFieldName, FieldComparer.ASC, FieldComparer.IGNORE_CASE);
    }

    //-----------------------------------------------------------------------------
    /**
     * Sort the list using the given field as criteria ( ignoring case ) and the specified order ( ascending or descending )
     * 
     * @param sFieldName :
     *            the name of the field to compare
     * @param sOrder :
     *            'ASC' = ascending order , 'DESC' = descending order
     */
    public void sortIgnoreCase(String sFieldName, String sOrder)
    {
        //String sField = getSortFieldName(sFieldName);
        boolean bOrder = getSortOrderFlag(sOrder);
        //Collections.sort( _list, new FieldComparer(sField, bOrder, FieldComparer.IGNORE_CASE ));
        sortList(sFieldName, bOrder, FieldComparer.IGNORE_CASE);
    }
    
    //-----------------------------------------------------------------------------
    public String toString()
    {
        StringBuffer sb = new StringBuffer(512);
        //sb.append(_list.getClass().getName() + " : \n");
        sb.append(this.getClass().getName() + " : \n");
        Object obj = getFirstElement();
        while (obj != null)
        {
            sb.append(" . " + obj.toString() + "\n");
            obj = getNextElement();
        }
        return sb.toString();
    }
}