package org.objectweb.telosys.dal.csv;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

/**
 * A "SAX like" parser to process CSV files. <br>
 * As the content is parsed by the underlying parser, methods of the given Handler are called.
 * 
 * @author GUERIN
 *  
 */
public class CSVReader
{
    private static final int  BUFFER_SIZE  = 2048;

    private static final char NO_SEPARATOR = '\0';

    private char              _cSepar      = NO_SEPARATOR; // Fields separator

    // character

    private String            _sComment    = null;        // String for comments

    private ICSVHandler       _handler     = null;

    //-----------------------------------------------------------------------------------
    // CONSTRUCTOR(S)
    //-----------------------------------------------------------------------------------
    /**
     * Construct a CSVReader object initialized with a separator and a handler
     * 
     * @param cSepar -
     *            the field separator
     * @param handler -
     *            the CSV handler to use
     */
    public CSVReader(char cSepar, ICSVHandler handler)
    {
        super();
        _cSepar = cSepar;
        _handler = handler;
    }

    //-----------------------------------------------------------------------------------
    // PUBLIC METHOD(S)
    //-----------------------------------------------------------------------------------
    /**
     * Construct a CSVReader object initialized with a separator and a handler
     * 
     * @param sSepar -
     *            the field separator (use only the first character of the string)
     * @param handler -
     *            the CSV handler to use
     */
    public CSVReader(String sSepar, ICSVHandler handler)
    {
        super();
        _cSepar = NO_SEPARATOR;
        if (sSepar != null)
        {
            if (sSepar.length() > 0)
            {
                _cSepar = sSepar.charAt(0);
            }
        }
        _handler = handler;
    }

    //-----------------------------------------------------------------------------------
    /**
     * Set the comment string
     * 
     * @param sComment -
     *            the string which identify a comment
     */
    public void setComment(String sComment)
    {
        _sComment = sComment;
    }

    //-----------------------------------------------------------------------------------
    /**
     * Parse the content of the file specified, using the current Handler.
     * 
     * @param sFileName -
     *            the name of the file to parse
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void parse(String sFileName) throws FileNotFoundException, IOException
    {
        FileReader fr = new FileReader(sFileName);
        BufferedReader br = new BufferedReader(fr, BUFFER_SIZE);
        try
        {
            parse(br);
        } catch (IOException e)
        {
            throw e;
        } finally
        {
            br.close();
        }
        fr.close();
    }

    //-----------------------------------------------------------------------------------
    /**
     * Parse the input specified, using the current Handler.
     * 
     * @param br -
     *            the BufferedReader to parse
     * @throws FileNotFoundException
     * @throws IOException
     */
    public void parse(BufferedReader br) throws IOException
    {
        int iRowNum = 0;
        String sLine;
        String[] fields = null;
        _handler.start();
        while ((sLine = br.readLine()) != null)
        {
            iRowNum++;
            if (sLine.length() == 0)
            {
                _handler.voidLine(iRowNum);
            }
            else
            {
                if (isComment(sLine))
                {
                    _handler.commentLine(iRowNum, sLine);
                }
                else
                {
                    fields = split(sLine);
                    _handler.dataLine(iRowNum, sLine, fields);
                    //printFields(fields);
                }
            }
        }
        _handler.end();
        br.close();
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param sLine
     *            the line to check
     * @return true if the line is a comment, false else
     */
    private boolean isComment(String sLine)
    {
        if (_sComment != null)
        {
            if (sLine.startsWith(_sComment))
            {
                return true;
            }
        }
        return false;
    }

    //-----------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------
    // PRIVATE
    //-----------------------------------------------------------------------------------
    private String[] split(String sLine)
    {
        //System.out.println("split [" + sLine + "]");
        int len = sLine.length();
        //--- Ligne sous forme de tableau de caractres
        char chars[] = new char[len];
        sLine.getChars(0, len, chars, 0);

        int iFieldCount = getFieldCount(chars);
        String fields[] = new String[iFieldCount];

        String sField = null;
        int iFieldIndex = 0;
        int iOffset = -1;
        int iCount = 0;
        for ( int i = 0 ; i < len ; i++ )
        {
            if (chars[i] == _cSepar)
            {
                fields[iFieldIndex++] = getField(chars, iOffset, iCount);
                iOffset = -1;
                iCount = 0;
            }
            else
            {
                if (iOffset == -1)
                    iOffset = i;
                iCount++;
            }
        }
        fields[iFieldIndex++] = getField(chars, iOffset, iCount);
        return fields;
    }

    //-----------------------------------------------------------------------------------
    private int getFieldCount(char chars[])
    {
        //--- Nb separ + 1
        int len = chars.length;
        int iCount = 0;
        for ( int i = 0 ; i < len ; i++ )
        {
            if (chars[i] == _cSepar)
            {
                iCount++;
            }
        }
        return iCount + 1;
    }

    //-----------------------------------------------------------------------------------
    private String getField(char chars[], int iOffset, int iCount)
    {
        if (iOffset < 0)
        {
            return "";
        }
        else
        {
            //            String sField = new String(chars,iOffset,iCount);
            //            //System.out.println(" . field [" + sField + "] offset = " +
            // iOffset + " count = " + iCount );
            //            return sField ;
            return new String(chars, iOffset, iCount);
        }
    }
    //-----------------------------------------------------------------------------------
    //    private void printFields(String fields[])
    //    {
    //        for ( int i = 0 ; i < fields.length ; i++ )
    //        {
    //            System.out.println(" . field " + i + " = [" + fields[i] + "] ");
    //        }
    //    }
    //-----------------------------------------------------------------------------------
}