package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * This class gives access to the current DatabaseSession associated with the current thread <br>
 * The DatabaseSession is stored as a "thread-local variable" ( see ThreadLocal for more details )
 * 
 * @author Laurent Guerin
 * @since v 1.0.2
 *
 */
public class CurrentDatabaseSession {

    private static final TelosysClassLogger log = new TelosysClassLogger(CurrentDatabaseSession.class);

	//--- Thread-local variable
    private static final ThreadLocal threadLocalVar = new ThreadLocal();
    
    //--------------------------------------------------------------------------------------
    /**
     * Returns the DatabaseSession instance associated with the current thread.<br>
     * @return
     * @throws TelosysRuntimeException if no current DatabaseSession
     */
    public static final DatabaseSession get() 
    {
        log.trace("get()");
    	DatabaseSession dbSession = (DatabaseSession) threadLocalVar.get();
    	if ( dbSession != null )
    	{
    		return dbSession ;
    	}
    	else
    	{
    		throw new TelosysRuntimeException("No DatabaseSession for the current thread");
    	}
    }

    //--------------------------------------------------------------------------------------
    /**
     * Returns the DatabaseSession instance associated with the current thread.<br>
     * or null if there's no current DatabaseSession for the thread.
     * Method reserved for framework internal use.
     * @return 
     */
    protected static final DatabaseSession getInternal() 
    {
        log.trace("getNoException()");
    	return (DatabaseSession) threadLocalVar.get();
    }
    
    //--------------------------------------------------------------------------------------
    /**
     * Sets the DatabaseSession instance associated with the current thread.
     * Method reserved for framework internal use.
     * @param dbSession
     */
    protected static final void set(DatabaseSession dbSession)
    {
        log.trace("set(" + dbSession + ")");
    	threadLocalVar.set(dbSession);
    }

    //--------------------------------------------------------------------------------------
    /**
     * Removes the DatabaseSession instance associated with the current thread
     * if it's the same instance as the given DatabaseSession.
     * Method reserved for framework internal use.
     * @param dbSession
     * @return
     */
    protected static final boolean removeIfStored(DatabaseSession dbSession)
    {
        log.trace("remove(" + dbSession + ")");
    	DatabaseSession currentDbSession = (DatabaseSession) threadLocalVar.get();
    	if ( currentDbSession != null )
    	{
    	    //--- Is it the current thread-local instance ?
            if ( currentDbSession == dbSession )  
            {
                //--- Yes => Remove it !
            	threadLocalVar.set(null);
        		return true ; // removed
            }
    	}
    	return false ;
    }

    //--------------------------------------------------------------------------------------
    /**
     * Removes the DatabaseSession instance associated with the current thread.
     * Method reserved for framework internal use.
     */
    protected static final void remove() 
    {
        log.trace("remove()");
    	threadLocalVar.set(null);
    }

    //--------------------------------------------------------------------------------------
}
