package org.objectweb.telosys.dal.dao;

import java.util.Iterator;
import java.util.List;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.sql.Database;

/**
 * Set of DAO registries (one registry for each database)
 * 
 * @since 1.0.0
 * @author Laurent GUERIN
 *
 */
public class DAORegistries extends TelosysObject
{
    private DAORegistry[] _registries = new DAORegistry[0] ;
    
    private int createRegistriesArray(List list) 
    {
        int iMaxId = 0 ;
        int id = 0 ;
        Iterator it = list.iterator();
        while ( it.hasNext() )
        {
            Database db = (Database) it.next();
            id = db.getId() ; // v 1.0.2 ( bug fix )
            trace("createRegistriesArray() : . DB " + id + " : " + db.getName()  );
            if ( id > iMaxId ) iMaxId = id ;
        }        
        
        trace("createRegistriesArray() : new DAORegistry[" + (iMaxId+1) + "]");
        _registries = new DAORegistry[iMaxId+1];
        return _registries.length ;
    }

    private DAORegistry createStandardRegistry( String sClass, int dbId ) 
    {
        //--- Try to load the class
        Class cl = null ;
        try
        {
            cl = Class.forName(sClass);
        } catch (ClassNotFoundException e)
        {
            throw new TelosysRuntimeException("Cannot load class '" + sClass + "'", e);
        }

        //--- Try to instanciate the class            
        Object instance = null ;
        try
        {
            instance = cl.newInstance();
        } catch (InstantiationException e1)
        {
            throw new TelosysRuntimeException("Cannot create instance for class '" + sClass + "'", e1);
        } catch (IllegalAccessException e2)
        {
            throw new TelosysRuntimeException("Cannot create instance for class '" + sClass + "'", e2);
        }
        
        //--- Return the instance of DAORegistry
        if ( ! ( instance instanceof DAORegistry ) )
        {
            throw new TelosysRuntimeException("Class '" + sClass + "' is not a DAORegistry");
        }
        DAORegistry reg = (DAORegistry) instance ;
        info( " . Database " + dbId + " : DAO registry created ( size = " + reg.getSize() + " )");
        return reg ;
    }

    private DAORegistry createDynamicRegistry( String sDAOClassPattern, int dbId ) 
    {
        DAORegistry reg = new DynamicDAORegistry( sDAOClassPattern ) ;
        info( " . Database " + dbId + " : Dynamic DAO registry created ( class pattern = " + sDAOClassPattern + " )" );
        return reg ;
    }
    
    /**
     * Try to create a DAO registry for the given database
     * @param db
     * @return the DAO registry created or null if no configuration for the database
     */
    private DAORegistry createDbRegistry(Database db) 
    {
        if ( db == null )
        {
            throw new TelosysRuntimeException("createDbRegistry(null) : no database parameter");            
        }

        //--- Is there a DAO registry class for this database ?
        String sDAORegistryClass = db.getDAORegistryClass() ;
        if ( sDAORegistryClass != null )
        {        
            return createStandardRegistry( sDAORegistryClass, db.getId() );
        }
        else
        {
            //--- Is there a DAO class pattern for this database ?
            String sDAOClassPattern = db.getDAOClassPattern() ;
            if ( sDAOClassPattern != null )
            {        
                return createDynamicRegistry( sDAOClassPattern, db.getId() );
            }
            else
            {            
	            info( " . Database " + db.getId() + " : no DAO registry");
	            return null ;
            }
        }        
    }
    
    
    public DAORegistries()
    {
        //--- Try to create a DAO registry for each database
        List list = ConnectionManager.getDatabases();
        if ( list != null )
        {
            int size = createRegistriesArray(list) ;
            trace("DAO registries array created ( size = " + size + " )");
            
            Iterator it = list.iterator();
            while ( it.hasNext() )
            {
                Database db = (Database) it.next();
                int id = db.getId() ;
                if ( id < _registries.length )
                {
                    //DAORegistry r = createDbRegistry(db, sDefaultClassName );
                    DAORegistry r = createDbRegistry(db);
                    if ( r != null )
                    {
                        _registries[id] = r ; 
                        trace(" . Database " + id + " : DAO registry created ( size = " + r.getSize() +" )");
                    }
                    else
                    {
                        _registries[id] = null ; 
                        trace(" . Database " + id + " : No DAO registry.");
                    }
                }
            }        
        }
    }
    
    /**
     * Returns the DAO registry corresponding to the given database ID
     * @param databaseId ( 0 to N )
     * @return
     * @throws TelosysRuntimeException if the database id is invalid or unknown
     * 
     */
    protected DAORegistry getRegistry(int databaseId)
    {
        if ( databaseId < 0 )
        {
            throw new TelosysRuntimeException("Invalid database id " + databaseId );
        }
        if ( _registries.length == 0 )
        {
            throw new TelosysRuntimeException("No DAO registry");
        }        
        if ( databaseId >= _registries.length )
        {
            throw new TelosysRuntimeException("Cannot get registry for database id " + databaseId + " ( max id = " + (_registries.length-1) + " )");
        }
        return _registries[databaseId] ; 
    }
    
    /**
     * Return the number of registries defined.
     * @return
     */
    public int getNumberOfRegistries()
    {
        int n = 0 ;
        if ( _registries != null )
        {
            for ( int i = 0 ; i < _registries.length ; i++ )
            {
                if ( _registries[i] != null ) n++ ;
            }
        }
        return n ;
    }
    
}
