package org.objectweb.telosys.dal.dao;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.dal.dao.StandardDAO;

/**
 * DAO registry, used to provide DAO by bean class or by table name
 * If the application uses multiple databases, there's a DAO registry for each database. 
 * 
 * @since 1.0.0
 * @author Laurent GUERIN
 *  
 */
public class DAORegistry extends TelosysObject 
{
    //--- Wrapper instances accessible by bean class name
    private final HashMap            _hmRegistryByBeanClass    = new HashMap(128);
    
    private final HashMap            _hmRegistryByTableName    = new HashMap(128);

    //-----------------------------------------------------------------------------------
    protected void register(Class daoClass)
    {
        if ( daoClass == null )
        {
            throw new TelosysRuntimeException("register(null) : class argument is null ");
        }
        
//        if (StandardDAO.class.isAssignableFrom(daoClass) != true)
//        {
//            throw new TelosysRuntimeException("register("+daoClass+") : is not a StandardDAO !");
//        }
//
        //--- Create the DAO instance 
        StandardDAO instance = createDAOInstance(daoClass);
        
        //--- Register the DAO instance 
        registerInstance(instance);
    }

    //-----------------------------------------------------------------------------------
    private StandardDAO createDAOInstance(Class daoClass)
    {
        try
        {
            //--- Create the instance with the given class
            Object instance = daoClass.newInstance();
            if (instance instanceof StandardDAO)
            {
                return (StandardDAO) instance;
            }
            else
            {
                throw new TelosysRuntimeException("createDAOInstance(" + daoClass + ") : not a DAO class ");
            }
        } catch (InstantiationException ex)
        {
            throw new TelosysRuntimeException("createDAOInstance(" + daoClass + ") : InstantiationException ", ex);
        } catch (IllegalAccessException ex)
        {
            throw new TelosysRuntimeException("createDAOInstance(" + daoClass + ") : IllegalAccessException ", ex);
        } catch (Throwable t)
        {
            throw new TelosysRuntimeException("createDAOInstance(" + daoClass + ") : Throwable " + t.toString(), t);
        }
    }

    //-----------------------------------------------------------------------------------
    private void registerInstance(StandardDAO daoInstance)
    {
        if (daoInstance == null)
        {
            throw new TelosysRuntimeException("registerInstance(null) : no parameter (null)");
        }
        
        //--- Try to get Bean Class and Table Name
        Class beanClass = daoInstance.getEntityBeanClass();        
        String sTableName = daoInstance.getTableName();
        if (beanClass == null)
        {
            throw new TelosysRuntimeException("registerInstance(" + daoInstance.getClass().getName() + ") : Cannot get bean class");
        }        
        if ( sTableName == null )
        {
            throw new TelosysRuntimeException("registerInstance(" + daoInstance.getClass().getName() + ") : Cannot get table name");
        }

        //--- Register by bean class
        _hmRegistryByBeanClass.put(beanClass.getName(), daoInstance);
        
        //--- Register by table name (in upper case)
        _hmRegistryByTableName.put(sTableName.toUpperCase(), daoInstance);

    }


    //-----------------------------------------------------------------------------------
    /**
     * Returns the DAO associated with the given Java class 
     * @param beanClass 
     * @return the DAO ( or null if not found )
     */
    protected StandardDAO getByBeanClass(Class beanClass)
    {
        if (beanClass == null)
        {
            throw new TelosysRuntimeException("getByBeanClass(null) : parameter is null");
        }
        String sClassName = beanClass.getName();
        trace("getByClass(" + sClassName + ")...");
        //--- Search a wrapper in the registry 
        Object obj = _hmRegistryByBeanClass.get(sClassName);
        
        if (obj != null)
        {
            trace("getByClass(" + sClassName + ") : FOUND");   
            if ( obj instanceof StandardDAO )
            {
                return (StandardDAO) obj ;
            }
            else
            {
                throw new TelosysRuntimeException("registry corrupted : entry '"+sClassName+"' is not a DAO ");
            }
        }
        else
        {
            trace("getByClass(" + sClassName + ") : NOT FOUND");   
            return null ;
        }
    }
    //-----------------------------------------------------------------------------------
    /**
     * Returns the DAO associated with the given table name
     * @param tableName
     * @return the DAO ( or null if not found )
     */
    protected StandardDAO getByTableName(String tableName)
    {
        if (tableName == null)
        {
            throw new TelosysRuntimeException("getByTableName(null) : parameter is null");
        }
        trace("getByTableName(" + tableName + ")...");
        //--- Search a wrapper in the registry (in upper case)
        Object obj = _hmRegistryByTableName.get(tableName.toUpperCase());
        
        if (obj != null)
        {
            trace("getByTableName(" + tableName + ") : FOUND");   
            if ( obj instanceof StandardDAO )
            {
                return (StandardDAO) obj ;
            }
            else
            {
                throw new TelosysRuntimeException("registry corrupted : entry '"+tableName+"' is not a DAO ");
            }
        }
        else
        {
            trace("getByTableName(" + tableName + ") : NOT FOUND");   
            return null ;
        }
    }
    //-----------------------------------------------------------------------------------
    /**
     * Returns the registry size 
     * @return
     */
    protected int getSize()
    {
        return _hmRegistryByBeanClass.size();
    }
}