package org.objectweb.telosys.dal.dao;

import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.util.StrUtil;

/**
 * Dynamic DAO Registry.<br>
 * This registry is initially empty and try to determine the DAO class dynamically using 
 * a DAO class name pattern. 
 *  
 * @since 1.0.0
 * @author Laurent GUERIN
 *
 */
public class DynamicDAORegistry extends DAORegistry
{

    private String _sDAOClassPattern = null ;
    
    /**
     * 
     */
    public DynamicDAORegistry(String sDAOClassPattern)
    {
        trace("DynamicDAORegistry constructor : pattern = '" + sDAOClassPattern + "'");
        if ( sDAOClassPattern != null )
        {
            _sDAOClassPattern = sDAOClassPattern.trim() ;
        }
        else
        {
	        throw new TelosysRuntimeException("Cannot create DynamicDAORegistry with null pattern");
        }
    }        
    
    private String getSimpleClassName(Class beanClass)
    {
        String s1 = beanClass.getName();
        // last part after split 
        String[] parts = StrUtil.split(s1, '.');
        if ( parts.length > 0 )
        {
            String s2 = parts[parts.length-1] ;
            return s2.trim() ;
        }
        else
        {
            return s1.trim();
        }
    }

    private Class loadClass(String sClass)
    {
	    //--- Try to load the class
	    Class cl = null ;
	    try
	    {
	        cl = Class.forName(sClass);
	    } catch (ClassNotFoundException e)
	    {
	        //throw new TelosysRuntimeException("Cannot load class '" + sClass + "'", e);
	        cl = null ;
	    }
	    return cl ;
	}

    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.DAORegistry#getByBeanClass(java.lang.Class)
     */
    protected StandardDAO getByBeanClass(Class beanClass)
    {
        StandardDAO dao = super.getByBeanClass(beanClass);
        if ( dao != null )
        {
            //--- FOUND => return it
            return dao ;
        }
        else
        {
            //--- NOT FOUND => try to determine de class dynamically and register
            if ( _sDAOClassPattern != null )
            {	             
                //--- Build the DAO class name dynamically
	            String sBeanClass = getSimpleClassName(beanClass);
	            if ( sBeanClass != null )
	            {
	                //String sDAOClass = StrUtil.replaceVar(_sDAOClassPattern , "${BEANCLASS}", sBeanClass );
	        	    String sDAOClass = Tool.buildClassName(_sDAOClassPattern, sBeanClass);
	                Class cl = loadClass(sDAOClass);
	                
	                if ( cl != null )
	                {
	                    trace("DynamicDAORegistry : register '" + cl.getName() + "'");
	                    
		                //--- Register the DAO for future use
		                register(cl);
		                
		                //--- Try to get again
		                return super.getByBeanClass(beanClass);
	                }
	                else
	                {
	                    trace("DynamicDAORegistry : class '" + sDAOClass + "' not found.");
	                }
	            }
            }
        }
        return null ; // Not found 
    }
}
