package org.objectweb.telosys.dal.dao;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 * 
 */
public class QueryContext extends TelosysObject //implements IConstantsDAO
{
    //============================================================================
    // Constants
    //============================================================================
    public  static final boolean  RETURN_GENERATED_KEYS  = true;

    private static final int      BLOB_BUFFER_SIZE = 1024 ; // 1 kb            

//    private static final int  STORE_BOOLEAN_DEFAULT   = 0;
//
//    private static final int  STORE_BOOLEAN_AS_INT    = 1;
//
//    private static final int  STORE_BOOLEAN_AS_STRING = 2;

    //============================================================================
    // Instance Attributes
    //============================================================================

    private PreparedStatement _ps                     = null;

    private ResultSet         _rs                     = null;

//    private int               _iBooleanStorage        = STORE_BOOLEAN_DEFAULT;
    private boolean           _bReturnGeneratedKeys   = false ;

    //============================================================================
    // Constructor(s)
    //============================================================================
    public QueryContext(Connection con, String sSql) throws SQLException
    {
        super();
        this._ps = con.prepareStatement(sSql);
        this._rs = null;
//        this._iBooleanStorage = STORE_BOOLEAN_DEFAULT;
    }

//    public QueryContext(Connection con, String sSql, int iBooleanStorage) throws SQLException
    public QueryContext(Connection con, String sSql, boolean bReturnGeneratedKeys) throws SQLException
    {
        super();
        _bReturnGeneratedKeys = bReturnGeneratedKeys ;
        if ( _bReturnGeneratedKeys )
        {
            //--- Special Prepared Statement to retrieve Generated Keys
            this._ps = con.prepareStatement(sSql, Statement.RETURN_GENERATED_KEYS);
        }
        else
        {
            //--- Standard Prepared Statement
            this._ps = con.prepareStatement(sSql);
        }
        this._rs = null;
//        this._iBooleanStorage = iBooleanStorage;
    }

    //============================================================================
    // Methods
    //============================================================================
    protected int executeUpdate() throws SQLException
    {
        // _rs = null ;
        return _ps.executeUpdate();
    }

    //----------------------------------------------------------------------------
    protected ResultSet executeQuery() throws SQLException
    {
        _rs = _ps.executeQuery();
        return _rs;
    }
    
    //----------------------------------------------------------------------------
    /**
     * Executes an INSERT with KEY GENERATION  ( Autoincrement Key )
     * @return
     * @throws SQLException
     */
    protected Long executeInsertKeyGen() throws SQLException
    {
        Long generatedKey = null;
        //long lGeneratedKey = 0L;
        //--- Execute the SQL INSERT via PreparedStatement  
        _ps.executeUpdate();
        //--- If the Prepared Statement is built to retrieve generated keys 
        if ( _bReturnGeneratedKeys )
        {            
            //--- Retrieve the generated key 
	        ResultSet rs = _ps.getGeneratedKeys();
	        if ( rs.next() )
	        {
	            long lGeneratedKey = rs.getLong(1);
	            generatedKey = new Long( lGeneratedKey );
	        }
	        rs.close();
        }
        return generatedKey ;
    }

//    //----------------------------------------------------------------------------
//    // Type of storage for Boolean
//    //----------------------------------------------------------------------------
//    public void storeBooleanAsInt()
//    {
//        _iBooleanStorage = STORE_BOOLEAN_AS_INT;
//    }
//
//    //----------------------------------------------------------------------------
//    public void storeBooleanAsString()
//    {
//        _iBooleanStorage = STORE_BOOLEAN_AS_STRING;
//    }

    //---------------------------------------------------------------------------
    // Close the ResultSet
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    private void closeResultSet()
    {
        if (_rs != null)
        {
            try
            {
                _rs.close();
            } catch (SQLException e)
            {
                error("Cannot close the ResultSet ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // Close the PreparedStatement
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    private void closePreparedStatement()
    {
        if (_ps != null)
        {
            try
            {
                _ps.close();
            } catch (SQLException e)
            {
                error("Cannot close the PreparedStatement ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // Close the PreparedStatement and the ResultSet
    // Doesn't throw Exception (just trace the error)
    //---------------------------------------------------------------------------
    protected void close()
    {
        closeResultSet();
        closePreparedStatement();
    }

    //============================================================================
    // METHODES DE VALORISATION DES PARAMETRES DU "PREPARED STATEMENT"
    // Les exceptions sont gres au niveau des mthodes "executeSqlXxxx"
    //============================================================================
    /* package */void setParam(int i, Object oParam) throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("setParamInt: _ps is null !");
        }
        else
        {
            try
            {
                if (oParam instanceof Boolean)
                {
                    setParamBoolean(i, ((Boolean) oParam).booleanValue());
                }
                //--- Integer Wrappers
                else if (oParam instanceof Integer)
                {
                    setParamInt(i, ((Integer) oParam).intValue());
                }
                else if (oParam instanceof Byte)
                {
                    setParamByte(i, ((Byte) oParam).byteValue());
                }
                else if (oParam instanceof Short)
                {
                    setParamShort(i, ((Short) oParam).shortValue());
                }
                else if (oParam instanceof Long)
                {
                    setParamLong(i, ((Long) oParam).longValue());
                }
                //--- Float/Double Wrappers
                else if (oParam instanceof Float)
                {
                    setParamFloat(i, ((Float) oParam).floatValue());
                }
                else if (oParam instanceof Double)
                {
                    setParamDouble(i, ((Double) oParam).doubleValue());
                }
                //--- Standards Objects
                else if (oParam instanceof String)
                {
                    setParamString(i, (String) oParam);
                }
                else if (oParam instanceof java.util.Date)
                {
                    setParamDate(i, (java.util.Date) oParam);
                }
                else if (oParam instanceof java.math.BigDecimal)
                {
                    setParamBigDecimal(i, (java.math.BigDecimal) oParam);
                }
                //--- Others
                else
                {
                    _ps.setObject(i, oParam);
                }
            } catch (SQLException e)
            {
                throw new TelosysException("setParam", e);
            }
        }
    }

    //-------------------------------------------------------------
    private void checkPreparedStatement() throws TelosysException
    {
        if (_ps == null)
        {
            throw new TelosysException("PreparedStatement '_ps' is null !");
        }
    }

    //-------------------------------------------------------------
    // INT
    //-------------------------------------------------------------
    public void setParamInt(int i, int iValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setInt(i, iValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamInt", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamInt(int i, java.lang.Integer obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setInt(i, obj.intValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.INTEGER );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamInt", e);
        }
    }

    //-------------------------------------------------------------
    // BYTE
    //-------------------------------------------------------------
    public void setParamByte(int i, byte value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setByte(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamByte", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamByte(int i, java.lang.Byte obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setByte(i, obj.byteValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.TINYINT ); // JDBC : "TINYINT" => getByte/setByte
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamByte", e);
        }
    }

    //-------------------------------------------------------------
    // SHORT
    //-------------------------------------------------------------
    public void setParamShort(int i, short value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setShort(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamShort", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamShort(int i, java.lang.Short obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setShort(i, obj.shortValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.SMALLINT ); // JDBC : "SMALLINT" => getShort/setShort
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamShort", e);
        }
    }

    //-------------------------------------------------------------
    // LONG
    //-------------------------------------------------------------
    public void setParamLong(int i, long value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setLong(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamLong", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamLong(int i, java.lang.Long obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setLong(i, obj.longValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.BIGINT ); // JDBC : "BIGINT" => getLong/setLong
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamLong", e);
        }
    }

    //-------------------------------------------------------------
    // FLOAT
    //-------------------------------------------------------------
    public void setParamFloat(int i, float fValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setFloat(i, fValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamFloat", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamFloat(int i, Float obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setFloat(i, obj.floatValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.FLOAT );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamFloat", e);
        }
    }

    //-------------------------------------------------------------
    // DOUBLE
    //-------------------------------------------------------------
    public void setParamDouble(int i, double dValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setDouble(i, dValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDouble", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamDouble(int i, Double obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setDouble(i, obj.doubleValue());
            }
            else
            {
                _ps.setNull(i, java.sql.Types.DOUBLE );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDouble", e);
        }
    }


    //-------------------------------------------------------------
    public void setParamBigDecimal(int i, BigDecimal value) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBigDecimal(i, value);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBigDecimal", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamString(int i, String sValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setString(i, sValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamString", e);
        }
    }

    //-------------------------------------------------------------
//    public void setParamDate(int i, java.sql.Date dateValue) throws TelosysException
//    {
//        try
//        {
//            checkPreparedStatement();
//            _ps.setDate(i, dateValue);
//        } catch (SQLException e)
//        {
//            throw new TelosysException("setParamDate", e);
//        }
//    }

    //-------------------------------------------------------------
    /**
     * Converts the given java.util.Date to java.sql.Date and set it as Date parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamDate(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setDate(i, new java.sql.Date(dateValue.getTime()));
            }
            else
            {
                _ps.setDate(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamDate", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Converts the given java.util.Date to java.sql.Timestamp and set it as Timestamp parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamTimestamp(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setTimestamp(i, new java.sql.Timestamp(dateValue.getTime()));
            }
            else
            {
                _ps.setTimestamp(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamTimestamp(int i, java.sql.Timestamp timestamp ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setTimestamp(i, timestamp);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    /**
     * Converts the given java.util.Date to java.sql.Time and set it as Time parameter
     * @param i
     * @param dateValue
     * @throws TelosysException
     */
    public void setParamTime(int i, java.util.Date dateValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if (dateValue != null)
            {
                _ps.setTime(i, new java.sql.Time(dateValue.getTime()));
            }
            else
            {
                _ps.setTime(i, null);
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTime", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamTime(int i, java.sql.Time time ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setTime(i, time);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamTime", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamBytes(int i, byte[] bytes ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBytes(i, bytes);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBytes", e);
        }
    }

//    //-------------------------------------------------------------
//    public void setParamBoolean(int i, boolean bValue) throws TelosysException
//    {
//        try
//        {
//            checkPreparedStatement();
//            switch (_iBooleanStorage)
//            {
//            case STORE_BOOLEAN_AS_INT :
//                _ps.setInt(i, (bValue ? 1 : 0));
//                break;
//            case STORE_BOOLEAN_AS_STRING :
//                _ps.setString(i, (bValue ? "1" : "0"));
//                break;
//            default :
//                _ps.setBoolean(i, bValue);
//                break;
//            }
//        } catch (SQLException e)
//        {
//            throw new TelosysException("setParamBoolean", e);
//        }
//    }
    //-------------------------------------------------------------
    // BOOLEAN
    //-------------------------------------------------------------
    public void setParamBoolean(int i, boolean bValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
			_ps.setBoolean(i, bValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setBoolean", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamBoolean(int i, java.lang.Boolean obj ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( obj != null )
            {
                _ps.setBoolean(i, obj.booleanValue() );
            }
            else
            {
                _ps.setNull(i, java.sql.Types.BOOLEAN );
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setBoolean/setNull", e);
        }
    }

    //-------------------------------------------------------------
    public void setParamBooleanAsString(int i, boolean bValue, String sTrueValue, String sFalseValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            String sValue = ( bValue ? sTrueValue : sFalseValue ) ;
            _ps.setString(i, sValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setString", e);
        }
    }
    //-------------------------------------------------------------
    public void setParamBooleanAsInt(int i, boolean bValue, int iTrueValue, int iFalseValue) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            int iValue = ( bValue ? iTrueValue : iFalseValue ) ;
            _ps.setInt(i, iValue);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBoolean -> ps.setInt", e);
        }
    }

    //-------------------------------------------------------------
    // CLOB 
    //-------------------------------------------------------------
    public void setParamClob(int i, Clob param ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setClob(i, param);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamClob", e);
        }
    }
    
    //-------------------------------------------------------------
    // BLOB
    //-------------------------------------------------------------
    //private static final ByteArrayInputStream VOID_STREAM = new ByteArrayInputStream(new byte[0]);
    public void setParamBlob(int i, byte[] bytes ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            if ( bytes != null )
            {
                _ps.setBinaryStream(i, new ByteArrayInputStream (bytes), bytes.length );
            }
            else
            {
                //_ps.setBinaryStream(i, null, 0 );
                _ps.setNull(i, java.sql.Types.LONGVARBINARY );                
            }
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBinaryStream", e);
        }
    }
    
    public void setParamBlob(int i, Blob param ) throws TelosysException
    {
        try
        {
            checkPreparedStatement();
            _ps.setBlob(i, param);
        } catch (SQLException e)
        {
            throw new TelosysException("setParamBlob", e);
        }
    }


    //============================================================================
    // Mthodes de rcupration des donnes  partir du "RESULT SET"
    //============================================================================
    private void checkResultSet() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("ResultSet '_rs' is null !");
        }
    }

    //-------------------------------------------------------------
    public String getResultString(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getString(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultString", e);
        }
    }

    //-------------------------------------------------------------
    // INT
    //-------------------------------------------------------------
    public int getResultInt(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getInt(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultInt", e);
        }
    }
    //-------------------------------------------------------------
    public Integer getResultIntObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Integer )
                {
                    return (Integer) obj;
                }
                else
                {
                    warn("getResultIntObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Integer( _rs.getInt(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultIntObj", e);
        }
    }
    
    //-------------------------------------------------------------
    // BYTE
    //-------------------------------------------------------------
    public byte getResultByte(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getByte(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultByte", e);
        }
    }
    //-------------------------------------------------------------
    public Byte getResultByteObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Byte )
                {
                    return (Byte) obj;
                }
                else
                {
                    warn("getResultByteObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Byte( _rs.getByte(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultByteObject", e);
        }
    }
        
    //-------------------------------------------------------------
    // SHORT
    //-------------------------------------------------------------
    public short getResultShort(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getShort(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultShort", e);
        }
    }
    //-------------------------------------------------------------
    public Short getResultShortObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Short )
                {
                    return (Short) obj;
                }
                else
                {
                    warn("getResultShortObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Short( _rs.getShort(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultShortObject", e);
        }
    }
        
    
    //-------------------------------------------------------------
    // LONG
    //-------------------------------------------------------------
    public long getResultLong(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getLong(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultLong", e);
        }
    }
    //-------------------------------------------------------------
    public Long getResultLongObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Long )
                {
                    return (Long) obj;
                }
                else
                {
                    warn("getResultLongObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Long( _rs.getLong(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultLongObject", e);
        }
    }
        

    //-------------------------------------------------------------
    // FLOAT
    //-------------------------------------------------------------
    public float getResultFloat(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getFloat(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultFloat", e);
        }
    }
    //-------------------------------------------------------------
    public Float getResultFloatObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Float )
                {
                    return (Float) obj;
                }
                else
                {
                    warn("getResultFloatObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Float( _rs.getFloat(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultFloatObject", e);
        }
    }
    
    //-------------------------------------------------------------
    // DOUBLE
    //-------------------------------------------------------------
    public double getResultDouble(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDouble(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDouble", e);
        }
    }
    //-------------------------------------------------------------
    public Double getResultDoubleObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition);
            if ( obj != null )
            {
                if ( obj instanceof Double )
                {
                    return (Double) obj;
                }
                else
                {
                    warn("getResultDoubleObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    return new Double( _rs.getDouble(iPosition) );
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDoubleObject", e);
        }
    }
    
    //-------------------------------------------------------------
    public BigDecimal getResultBigDecimal(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBigDecimal(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBigDecimal", e);
        }
    }

    //-------------------------------------------------------------
    public java.util.Date getResultDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Date date = _rs.getDate(iPosition);
            if ( date != null )
            {
                return new java.util.Date(date.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDate", e);
        }
    }
    //-------------------------------------------------------------
    public java.sql.Date getResultDateSql(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getDate(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultDateSql", e);
        }
    }    
    
    //-------------------------------------------------------------
    public java.sql.Time getResultTime(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getTime(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTime", e);
        }
    }
    //-------------------------------------------------------------
    public java.util.Date getResultTimeAsDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Time time = _rs.getTime(iPosition);
            if ( time != null )
            {
                return new java.util.Date(time.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimeAsDate", e);
        }
    }

    //-------------------------------------------------------------
    public java.sql.Timestamp getResultTimestamp(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getTimestamp(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimestamp", e);
        }
    }

    //-------------------------------------------------------------
    public java.util.Date getResultTimestampAsDate(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            java.sql.Timestamp ts = _rs.getTimestamp(iPosition);
            if ( ts != null )
            {
                return new java.util.Date(ts.getTime());
            }
            else
            {
                return null ;
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultTimestampAsDate", e);
        }
    }

    //-------------------------------------------------------------
    public byte[] getResultBytes(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBytes(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBytes", e);
        }
    }

    //-------------------------------------------------------------
//    public boolean getResultBoolean(int iPosition) throws TelosysException
//    {
//        boolean bRet = false;
//        try
//        {
//            checkResultSet();
//            switch (_iBooleanStorage)
//            {
//            case STORE_BOOLEAN_AS_INT :
//                bRet = (_rs.getInt(iPosition) != 0 ? true : false);
//                break;
//            case STORE_BOOLEAN_AS_STRING :
//                String s = _rs.getString(iPosition);
//                if (s == null)
//                {
//                    bRet = false;
//                }
//                else
//                {
//                    bRet = (s.trim().equals("0") ? true : false);
//                }
//                break;
//            default :
//                bRet = _rs.getBoolean(iPosition);
//                break;
//            }
//        } catch (SQLException e)
//        {
//            throw new TelosysException("getResultBoolean", e);
//        }
//        return bRet;
//    }

    //-------------------------------------------------------------
    // BOOLEAN
    //-------------------------------------------------------------
    public boolean getResultBoolean(int iPosition) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            bRet = _rs.getBoolean(iPosition); // return FALSE if "SQL NULL"
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBoolean --> rs.getBoolean", e);
        }
        return bRet;
    }
    //-------------------------------------------------------------
    public Boolean getResultBooleanObject(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            Object obj = _rs.getObject(iPosition); // if "SQL NULL" -> return null
            if ( obj != null )
            {
                if ( obj instanceof Boolean )
                {
                    return (Boolean) obj;
                }
                else
                {
                    warn("getResultBooleanObject("+iPosition+") : rs.getObject() return is " + obj.getClass().getName() );
                    //return new Boolean( _rs.getBoolean(iPosition) );
                    return Boolean.valueOf( _rs.getBoolean(iPosition) ) ; // better space and time perf.
                }
            }
            return null ;
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBooleanObject", e);
        }
    }
        
    //-------------------------------------------------------------
    public boolean getResultBooleanFromInt(int iPosition, int iTrueValue) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            int iValue = _rs.getInt(iPosition); // return 0 if NULL
            bRet = ( iValue == iTrueValue ? true : false);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBoolean --> rs.getInt", e);
        }
        return bRet;
    }
    //-------------------------------------------------------------
    public boolean getResultBooleanFromString(int iPosition, String sTrueValue) throws TelosysException
    {
        boolean bRet = false;
        try
        {
            checkResultSet();
            String s = _rs.getString(iPosition);
            if ( s != null && sTrueValue != null)
            {
                if ( s.equals(sTrueValue) )
                {
                    bRet = true ;
                }
            }
            else
            {
                if ( s == null && sTrueValue == null )
                {
                    bRet = true ;
                }
            }
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBoolean --> rs.getString", e);
        }
        return bRet;
    }

    //-------------------------------------------------------------
    // CLOB 
    //-------------------------------------------------------------
    public Clob getResultClob(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getClob(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultClob", e);
        }
    }
    
    //-------------------------------------------------------------
    // BLOB 
    //-------------------------------------------------------------
    public byte[] getResultBlobAsByteArray(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            //--- 1) Get the InputStream
            InputStream is = _rs.getBinaryStream(iPosition);
            
            //--- 2) Load the InputStream content in a buffer
            if (is != null)
            {
	            ByteArrayOutputStream os = new ByteArrayOutputStream(BLOB_BUFFER_SIZE);
	            byte [] buffer = new byte [BLOB_BUFFER_SIZE] ; 
	            int i = 0;
	            try
	            {
	                while ( ( i = is.read(buffer) ) > 0 )
	                {
	                    os.write(buffer, 0, i);                
	                }
	            } catch (IOException e)
	            {
	                error("getResultBlobAsByteArray ! IOException : " + e.getMessage());
	            }
	            return os.toByteArray();
            }
            else
            {
                //return new byte[0] ;
                return null ;
            }
            
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBinaryStream", e);
        }
    }
    //-------------------------------------------------------------
    public Blob getResultBlob(int iPosition) throws TelosysException
    {
        try
        {
            checkResultSet();
            return _rs.getBlob(iPosition);
        } catch (SQLException e)
        {
            throw new TelosysException("getResultBlob", e);
        }
    }
    //-------------------------------------------------------------


    //-------------------------------------------------------------
    //-------------------------------------------------------------
    protected boolean nextResult() throws TelosysException
    {
        if (_rs == null)
        {
            throw new TelosysException("nextResult: _rs is null !");
        }
        else
        {
            try
            {
                return _rs.next();
            } catch (SQLException e)
            {
                throw new TelosysException("nextResult", e);
            }
        }
    }

    //-------------------------------------------------------------
}