package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.dal.sql.ConnectionManager;

/**
 * @author Laurent GUERIN
 *  
 */
public class SequenceDAO extends TelosysObject
{
    // ============================================================================
    // Constants
    // ============================================================================

    public static final int     UNKNOWN                = 0;
    
    public static final int     ORACLE                 = 1;

    public static final int     POSTGRESQL             = 2;

    public static final int     MYSQL                  = 3;

    //---------------------------------------------------------------------------

    private static final String SQL_ORACLE_NEXTVAL     = "select SEQUENCE_NAME.nextval from dual";

    private static final String SQL_ORACLE_CURRVAL     = "select SEQUENCE_NAME.currval from dual";

    //---------------------------------------------------------------------------
    
    private static final String SQL_POSTGRESQL_NEXTVAL = "select nextval('SEQUENCE_NAME')";

    private static final String SQL_POSTGRESQL_CURRVAL = "select currval('SEQUENCE_NAME')";

    //---------------------------------------------------------------------------
    
    private static final String SQL_MYSQL_NEXTVAL = "update SEQUENCE_NAME set val = LAST_INSERT_ID(val+1)" ;

    //private static final String SQL_MYSQL_CURRVAL = "select val from SEQUENCE_NAME";

    // ============================================================================
    // Instance Attributes
    // ============================================================================

    private int                 _iDatabaseType         = UNKNOWN;

    private String              _sSequenceName         = null;

    private String              _sSqlRequestNextVal    = null;

    private String              _sSqlRequestCurrVal    = null;

    // ============================================================================
    // CONSTRUCTOR
    // ============================================================================

    public SequenceDAO(String sSequenceName, int iBaseType)
    {
        super();
        if ( sSequenceName == null )
        {
            throw new TelosysRuntimeException("SequenceDAO constructor : Sequence name is null");            
        }
        _sSequenceName = sSequenceName.trim() ;
        switch (iBaseType)
        {
        case ORACLE :
            _iDatabaseType = ORACLE ;
            _sSqlRequestNextVal = SQL_ORACLE_NEXTVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            _sSqlRequestCurrVal = SQL_ORACLE_CURRVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            break;
        case POSTGRESQL :
            _iDatabaseType = POSTGRESQL ;
            _sSqlRequestNextVal = SQL_POSTGRESQL_NEXTVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            _sSqlRequestCurrVal = SQL_POSTGRESQL_CURRVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            break;
        case MYSQL :
            _iDatabaseType = MYSQL ;
            _sSqlRequestNextVal = SQL_MYSQL_NEXTVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            //_sSqlRequestCurrVal = SQL_MYSQL_CURRVAL.replaceFirst("SEQUENCE_NAME", _sSequenceName);
            _sSqlRequestCurrVal = null ;
            break;
        default :
            throw new TelosysRuntimeException("SequenceDAO constructor : Invalid database type '" + iBaseType + "'");
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the name of the sequence
     * @return
     */
    public String getSequenceName()
    {
        return _sSequenceName;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the database type
     * @return
     */
    public int getDatabaseType()
    {
        return _iDatabaseType;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the SQL request used to get 'nextval'
     * @return
     */
    public String getSqlNextVal()
    {
        return _sSqlRequestNextVal;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the SQL request used to get 'currval'
     * @return
     */
    public String getSqlCurrVal()
    {
        return _sSqlRequestCurrVal;
    }

    //============================================================================
    // NEXTVAL
    //============================================================================
    /**
     * Get the 'nextval' of the sequence (using the default database)
     * @return : the Sequence 'NextVal'
     * @throws TelosysException
     */
    public long nextVal() throws TelosysException
    {
        return nextVal(ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    /**
     * Get the 'nextval' of the sequence (using the given database id)
     * @param iDatabaseId : the database id to use
     * @return : the Sequence 'NextVal'
     * @throws TelosysException
     */
    public long nextVal(int iDatabaseId) throws TelosysException
    {
        long lRet = 0 ;
        Connection con = getConnection(iDatabaseId);
        try
        {
            lRet = nextVal(con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return lRet ;
    }

    //---------------------------------------------------------------------------
    /**
     * Get the 'nextval' of the sequence (using the given database session)
     * @param session : the database session to use
     * @return : the Sequence 'NextVal'
     * @throws TelosysException
     */
    public long nextVal(DatabaseSession session) throws TelosysException
    {
        return nextVal(session.getConnection());
    }

    //---------------------------------------------------------------------------
    /**
     * Get the 'nextval' of the sequence (using the given database connection)
     * @param con : the connection to use
     * @return : the Sequence 'NextVal'
     * @throws TelosysException
     */
    public long nextVal(Connection con) throws TelosysException
    {
        if ( _iDatabaseType == MYSQL )
        {
            return getMySQLNextVal(con, _sSqlRequestNextVal);
        }
        else
        {
            return execSql(con, _sSqlRequestNextVal, "nextVal");            
        }
    }

    //============================================================================
    // CURRVAL ( NB : non sense if the connection is not the same )
    //============================================================================
//    /**
//     * Get the 'currval' of the sequence (using the default database)
//     * @return : the Sequence 'currval'
//     * @throws TelosysException
//     */
//    public long currVal() throws TelosysException
//    {
//        return currVal(ConnectionManager.getDefaultDatabase());
//    }

    //---------------------------------------------------------------------------
//    /**
//     * Get the 'currval' of the sequence (using the given database id)
//     * @param iDatabaseId : the database id to use
//     * @return : the Sequence 'currval'
//     * @throws TelosysException
//     */
//    public long currVal(int iDatabaseId) throws TelosysException
//    {
//        long lRet = 0 ;
//        Connection con = getConnection(iDatabaseId);
//        try
//        {
//            lRet = currVal(con);
//        }
//        // Don't catch exceptions (just force "releaseConnection" in "finally")
//        finally
//        {
//            releaseConnection(con);
//        }
//        return lRet ;
//    }

    //---------------------------------------------------------------------------
    /**
     * Get the 'currval' of the sequence (using the given database session)
     * @param session : the database session to use
     * @return : the Sequence 'currval'
     * @throws TelosysException
     */
    public long currVal(DatabaseSession session) throws TelosysException
    {
        return currVal(session.getConnection());
    }

    //---------------------------------------------------------------------------
    /**
     * Get the 'currval' of the sequence (using the given database connection)
     * @param con : the connection to use
     * @return : the Sequence 'currval'
     * @throws TelosysException
     */
    public long currVal(Connection con) throws TelosysException
    {
        if ( _iDatabaseType == MYSQL )
        {
            throw new TelosysException("currVal not supported with MySQL");
        }
        else
        {
            return execSql(con, _sSqlRequestCurrVal, "currVal");
        }
    }

    //============================================================================
    // PRIVATE METHODS
    //============================================================================
    /**
     * Returns a connection according to the given Database ID
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    private Connection getConnection(int iDatabaseId) throws TelosysException
    {
        trace("getConnection(" + iDatabaseId + ")...");
        //--- Get a connection from the connection provider
        Connection dbCon = null;
        try
        {
            dbCon = ConnectionManager.getConnection(iDatabaseId);
        } catch (Exception e)
        {
            throw new TelosysException("SequenceDAO.getConnection(" + iDatabaseId + ")", e);
        }
        return dbCon;
    }

    //---------------------------------------------------------------------------
    /**
     * Releases the given connection without exception propagation (just trace the error)
     * @param con
     */
    private void releaseConnection(Connection con)
    {
        trace("releaseConnection()...");
        if (con != null)
        {
            try
            {
                con.close();
            } catch (SQLException e)
            {
                error("Cannot close the connection ! SQLException : " + e.getMessage());
            }
        }
    }


    //---------------------------------------------------------------------------
    /**
     * Executes the given SQL request to get 'nextval' or 'currval' from the sequence
     * @param con
     * @param sSqlRequest
     * @param sOperation
     * @return
     * @throws TelosysException
     */
    private long execSql(Connection con, String sSqlRequest, String sOperation) throws TelosysException
    {
        String sMsg = "SequenceDAO - " + sOperation + "(" + sSqlRequest + ")";
        long lValue = 0;
        Statement stmt = null;
        ResultSet rs = null;

        if (con == null)
        {
            throw new TelosysException(sMsg + " : Connection is null");
        }
        try
        {
            stmt = con.createStatement();
            rs = stmt.executeQuery(sSqlRequest);
            rs.next();
            lValue = rs.getLong(1);
        } catch (SQLException ex)
        {
            throw new TelosysException(sMsg, ex);
        } catch (Exception ex)
        {
            throw new TelosysException(sMsg, ex);
        } finally
        {
            if (rs != null)
            {
                try
                {
                    rs.close();
                } catch (SQLException e)
                {
                    // Nothing to do
                    error("Cannot close the ResultSet ! SQLException : " + e.getMessage());
                }
            }
            if (stmt != null)
            {
                try
                {
                    stmt.close();
                } catch (SQLException e)
                {
                    // Nothing to do
                    error("Cannot close the Statement ! SQLException : " + e.getMessage());
                }
            }
        }
        return lValue;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Executes the specific SQL request to get 'nextval' from a 'pseudo-sequence' table
     * ( standard MySQL sequence simulation based on LAST_INSERT_ID )
     * @param con
     * @param sSqlRequest
     * @return
     * @throws TelosysException
     */
    private long getMySQLNextVal(Connection con,  String sSqlRequest) throws TelosysException
    {
        String sMsg = "SequenceDAO - NextVal / MySQL (" + sSqlRequest + ") : ";        
        String sError = null ;
        long lNewSequenceValue = 0;
        if (con == null)
        {
            throw new TelosysException(sMsg + "Connection is null !");
        }
        try
        {
            Statement stmt = null ;
            
            stmt = con.createStatement();
            if (stmt != null)
            {
                //--- 1) Update the new sequence value and set the LAST_INSERT_ID
	            stmt.executeUpdate( sSqlRequest );
	            
	            //--- 2) Retreive the LAST_INSERT_ID ( the new sequence value )
                ResultSet rs = stmt.executeQuery( "select LAST_INSERT_ID()" );
                if (rs.first())
                {
                    lNewSequenceValue = rs.getLong(1);
                }
                else
                {
                    sError = "Cannot get first row in result set !" ;
                }
                rs.close();
                
	            //--- 3) close
	            stmt.close();
            }
            else
            {
                sError = "Cannot create statement !";
            }
        } catch (SQLException sqlex)
        {
            throw new TelosysException(sMsg, sqlex);            
        } catch (Exception ex)
        {
            throw new TelosysException(sMsg, ex);            
        }
        if ( sError != null )
        {
            throw new TelosysException(sMsg + sError );                
        }
        return lNewSequenceValue;        
    }    
}