package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.sql.Criterion;
import org.objectweb.telosys.dal.sql.Criteria;
import org.objectweb.telosys.util.StrUtil;

/**
 * Standard DAO ancestor
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class StandardDAO extends TelosysObject
{
    //============================================================================
    // Constants
    //============================================================================

    private static final int     ACTION_SQL_INSERT       = 1;

    private static final int     ACTION_SQL_UPDATE       = 2;

    private static final int     ACTION_SQL_DELETE       = 3;

    private static final boolean COMMIT                  = true;

    private static final boolean DO_NOT_COMMIT           = false;

    private static final String  LOAD                    = "load";

    private static final String  SAVE                    = "save";

    private static final String  INSERT                  = "insert";

    private static final String  INSERT_KEY_GEN          = "insertKeyGen";

    private static final String  UPDATE                  = "update";

    private static final String  DELETE                  = "delete";

    private static final String  EXISTS                  = "exists";

    private static final String  COUNT                   = "count";

    private static final String  LOAD_LIST               = "loadList";

    private static final String  SAVE_LIST               = "saveList";

    private static final String  DELETE_LIST             = "deleteList";

    private static final String  INSERT_LIST             = "insertList";

    private static final String  UPDATE_LIST             = "updateList";

    //---

    protected static final int   STORE_BOOLEAN_DEFAULT   = 0;

    protected static final int   STORE_BOOLEAN_AS_INT    = 1;

    protected static final int   STORE_BOOLEAN_AS_STRING = 2;

    //============================================================================
    // Instance Attributes
    //============================================================================

    /**
     * Boolean storage type : DEFAULT("boolean" supported by database) or INT or STRING Used by QueryContext
     */
    private int                  _iBooleanStorage        = STORE_BOOLEAN_DEFAULT;

    private Class                _voClass                = null;

    private SqlRequests          _sqlRequests            = null;

    private boolean              _bSaveByDeleteInsert    = false;

    //============================================================================
    // Abstract methods (to be implemented in each DAO)
    //============================================================================

    /**
     * Set the table Primary Key columns values from the Java Bean attributes
     * Must be implemented in the "concrete DAO"
     * @param context
     * @param entity
     * @throws TelosysException
     */
    protected abstract void setPrimaryKey(QueryContext context, Object entity) throws TelosysException;

    /**
     * Set the Java Bean attributes values from the table columns
     * Must be implemented in the "concrete DAO"
     * @param context
     * @param entity
     * @return
     * @throws TelosysException
     */
    protected abstract int tableToBean(QueryContext context, Object entity) throws TelosysException;

    /**
     * Set the table Data columns values from the Java Bean attributes ( except the Primary Key ) 
     * Must be implemented in the "concrete DAO"
     * @param context
     * @param entity
     * @return
     * @throws TelosysException
     */
    protected abstract int beanToTable(QueryContext context, Object entity) throws TelosysException;

    /**
     * Set the table columns values from the Java Bean attributes, except the Auto-Incremented Key column.
     * It is dedicated to SQL INSERT operations on a table with an "Auto-Incremented Key".
     * This method is designed to be overriden in the "concrete DAO" only if necessary.
     * It is required only if the table has an Auto-Incremented Key ( it's default behavior throws an Exception )
     * @param context
     * @param entity
     * @return
     * @throws TelosysException
     */
    protected int beanToTableWithoutAutoInc(QueryContext context, Object entity) throws TelosysException // v 0.9.9
    {
        //--- No default behavior : Error if call without sub-class implementation
        throw new TelosysException("Insert with Key Generation not supported ( 'beanToTableWithoutAutoInc' not implemented )");
    }

    //============================================================================
    // CONSTRUCTOR
    //============================================================================

    public StandardDAO(Class voClass, SqlRequests sqlRequests)
    {
        super();
        //--- Check the parameters
        if (voClass == null)
        {
            throw new TelosysRuntimeException("DAO constructor (" + getClass().getName() + ") : VO class parameter is null");
        }
        if (sqlRequests == null)
        {
            throw new TelosysRuntimeException("DAO constructor (" + getClass().getName() + ") : SQL requests parameter is null");
        }
        //--- Set the parameters
        _voClass = voClass;
        _sqlRequests = sqlRequests;
    }

    /**
     * Returns the class of the Value Object managed by the current DAO
     * 
     * @return
     */
    public Class getEntityBeanClass()
    {
        return _voClass;
    }

    /**
     * Returns the name of the table managed by the current DAO
     * 
     * @return
     */
    public String getTableName()
    {
        return _sqlRequests.getTableName();
    }

    /**
     * Returns the SQL Requests built for the current DAO
     * 
     * @return
     */
    public SqlRequests getSqlRequests()
    {
        return _sqlRequests;
    }

    //----------------------------------------------------------------------------
    // Type of storage for Boolean
    //----------------------------------------------------------------------------
    /**
     * Set the boolean storage type to "int"
     */
    protected void storeBooleanAsInt()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_INT;
    }

    //----------------------------------------------------------------------------
    /**
     * Set the boolean storage type to "String"
     */
    protected void storeBooleanAsString()
    {
        _iBooleanStorage = STORE_BOOLEAN_AS_STRING;
    }

    //----------------------------------------------------------------------------
    /**
     * Return the boolean storage type
     * 
     * @return
     */
    protected int getBooleanStorage()
    {
        return _iBooleanStorage;
    }

    //---------------------------------------------------------------------------
    protected void setSaveByDeleteInsert(boolean bFlag)
    {
        _bSaveByDeleteInsert = bFlag;
    }

    //---------------------------------------------------------------------------
    protected boolean getSaveByDeleteInsert()
    {
        return _bSaveByDeleteInsert;
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table to get all the records
     * 
     * @return
     * @throws PdcException
     */
    public ListQuery createQueryAll() throws TelosysException
    {
        return _sqlRequests.createQuery(0, null, null);
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table to get all the records with an additional clause
     * 
     * @param sQueryEndOfSelect :
     *            the additional SQL clause ( ie : order by )
     * @return
     * @throws PdcException
     */
    public ListQuery createQueryAll(String sQueryEndOfSelect) throws TelosysException
    {
        return _sqlRequests.createQuery(0, null, sQueryEndOfSelect);
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given criteria <BR>
     * the number of SQL parameters is the number of '?' in the string
     * 
     * @param sQueryCriteria :
     *            SQL criteria for the WHERE clause
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(String sQueryCriteria) throws TelosysException
    {
        int iParamCount = StrUtil.countChar(sQueryCriteria, '?');
        return _sqlRequests.createQuery(iParamCount, sQueryCriteria, null);
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given criteria <br>
     * ( force the number of criteria parameters )
     * 
     * @param iParamCount :
     *            number of parameters in the criteria
     * @param sQueryCriteria :
     *            SQL criteria for the WHERE clause
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(int iParamCount, String sQueryCriteria) throws TelosysException
    {
        return _sqlRequests.createQuery(iParamCount, sQueryCriteria, null);
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given criteria and an additional clause <br>
     * ( the number of SQL parameters is the number of '?' in the string )
     * 
     * @param sQueryCriteria
     *            the SQL criteria for the WHERE clause
     * @param sQueryEndOfSelect :
     *            the additional SQL clause ( e.g. : order by )
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery( String sQueryCriteria, String sQueryEndOfSelect) throws TelosysException
    {
        int iParamCount = StrUtil.countChar(sQueryCriteria, '?');
        return _sqlRequests.createQuery(iParamCount, sQueryCriteria, sQueryEndOfSelect);
    }

    //----------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given criteria and an additional clause <br>
     * ( force the number of criteria parameters )
     * 
     * @param iParamCount
     *            the number of SQL parameters in the criteria
     * @param sQueryCriteria
     *            the SQL criteria for the WHERE clause
     * @param sQueryEndOfSelect :
     *            the additional SQL clause ( e.g. : order by )
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(int iParamCount, String sQueryCriteria, String sQueryEndOfSelect) throws TelosysException
    {
        return _sqlRequests.createQuery(iParamCount, sQueryCriteria, sQueryEndOfSelect);
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given dynamic criterias and an additional clause, <br>
     * using only the useful criterias
     * 
     * @param criterias
     * @param sQueryEndOfSelect :
     *            the additional SQL clause ( ie : order by )
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(Criteria criterias, String sQueryEndOfSelect) throws TelosysException
    {
        if (criterias == null)
        {
            throw new TelosysException("createQuery : SQLCriterias parameter is null ");
        }
        //--- Creates the query
        ListQuery query = this.createQuery(criterias.getUsefulParametersCount(), criterias.getDynamicWhere(), sQueryEndOfSelect);
        //--- Init the query parameters with the useful criterias (if any)
        Criterion[] usefulCriterias = criterias.getUsefulCriteria();
        for ( int i = 0 ; i < usefulCriterias.length ; i++ )
        {
            Criterion c = usefulCriterias[i];
            query.setParameter(i + 1, c.getParamValueAsObject());
        }
        return query;
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a query on a single table with the given dynamic criterias, <br>
     * using only the useful criterias
     * 
     * @param criterias
     * @return
     * @throws TelosysException
     */
    public ListQuery createQuery(Criteria criterias) throws TelosysException
    {
        return this.createQuery(criterias, null);
    }

    //---------------------------------------------------------------------------
    /**
     * Change the SQL SELECT by adding a JOIN
     * 
     * @param sJoinTables
     * @param sJoinColumns
     * @param sJoinCriteria
     */
    protected void alterSqlSelectWithJoin(String sJoinTables, String sJoinColumns, String sJoinCriteria)
    {
        _sqlRequests.alterSqlSelectWithJoin(sJoinTables, sJoinColumns, sJoinCriteria);
    }

    //---------------------------------------------------------------------------
    /**
     * Change the class of the bean associated with the current DAO
     * 
     * @param newClass
     */
    protected void alterEntityBeanClass(Class newClass)
    {
        _voClass = newClass;
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a new instance of the bean class associated with the current DAO
     * 
     * @return
     * @throws TelosysException
     */
    private Object createEntityBean() throws TelosysException
    {
        if (_voClass != null)
        {
            try
            {
                return _voClass.newInstance();
            } catch (Exception e)
            {
                String msg = "createEntityBean() : class = " + _voClass.getName() + " : Exception = " + e.getMessage();
                throw new TelosysException(msg, e);
            }
        }
        return null;
    }

    //---------------------------------------------------------------------------
    /**
     * Return a connection according to the given Database ID
     * 
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    private Connection getConnection(int iDatabaseId) throws TelosysException
    {
        //--- Rcuprer une connexion dans le pool
        trace("getConnection(" + iDatabaseId + ")...");
        //--- Get a connection from the connection provider
        Connection dbCon = null;
        try
        {
            dbCon = ConnectionManager.getConnection(iDatabaseId);
        } catch (Exception e)
        {
            throw new TelosysException("EntityDAO.getConnection(" + iDatabaseId + ")", e);
        }
        return dbCon;
    }

    //---------------------------------------------------------------------------
    /**
     * Releases the given connection without exception propagation (just trace the error)
     * 
     * @param con
     */
    private void releaseConnection(Connection con)
    {
        trace("releaseConnection()...");
        if (con != null)
        {
            try
            {
                con.close();
            } catch (SQLException e)
            {
                error("Cannot close the connection ! SQLException : " + e.getMessage());
            }
        }
    }

    //---------------------------------------------------------------------------
    // "AutoCommit" management
    //---------------------------------------------------------------------------
    /**
     * Returns the current auto-commit mode of the given connection
     * 
     * @param con
     * @return
     */
    private boolean getAutoCommit(Connection con)
    {
        try
        {
            return con.getAutoCommit();
        } catch (SQLException ex)
        {
            error("Cannot get the AutoCommit mode : getAutoCommit() : Exception " + ex.toString());
            return false;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Sets the auto-commit mode of the given connection
     * 
     * @param con
     * @param bMode
     */
    private void setAutoCommit(Connection con, boolean bMode)
    {
        try
        {
            con.setAutoCommit(bMode);
        } catch (SQLException ex)
        {
            error("Cannot set AutoCommit mode : setAutoCommit() : Exception " + ex.toString());
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Rollbacks all changes in the current transaction
     * 
     * @param con
     */
    private void rollback(Connection con)
    {
        try
        {
            con.rollback();
        } catch (SQLException ex)
        {
            error("Cannot rollback : rollback() : Exception " + ex.toString());
        }
    }

    //============================================================================
    //                             ENTITY DAO
    //============================================================================

    //============================================================================
    // Abstract methods calls
    //============================================================================
    private void callSetPrimaryKey(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call setPrimaryKey : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call setPrimaryKey : Entity Object is null ");
        }
        setPrimaryKey(context, entity);
    }

    private void callTableToBean(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call tableToBean : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call tableToBean : Entity Object is null ");
        }
        tableToBean(context, entity); //--- Implemented in the sub-class 
    }

    private void callBeanToTable(QueryContext context, Object entity) throws TelosysException
    {
        if (context == null)
        {
            throw new TelosysException("DAO call beanToTable : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call beanToTable : Entity Object is null ");
        }
        beanToTable(context, entity); //--- Implemented in the sub-class 
    }

    private void callBeanToTableWithoutAutoInc(QueryContext context, Object entity) throws TelosysException // v 0.9.9
    {
        if (context == null)
        {
            throw new TelosysException("DAO call beanToTableWithoutAutoInc : Query Context is null ");
        }
        if (entity == null)
        {
            throw new TelosysException("DAO call beanToTableWithoutAutoInc : Entity Object is null ");
        }
        beanToTableWithoutAutoInc(context, entity);
    }

    /**
     * Mapping from Java Bean object to Table columns befor an SQL INSERT.
     * Since v 0.9.9 there's to kind of mapping : AUTO-INCREMENTED KEY or STANDARD KEY  
     * @param context
     * @param entity
     * @throws TelosysException
     */
    private void callBeanToTableForInsert(QueryContext context, Object entity) throws TelosysException // v 0.9.9
    {
        if ( _sqlRequests.hasAutoIncColumn() ) // The table has an "AUTO-INC" column
        {
            //--- Call the special "bean to table" for INSERT without AUTO-INC column
            callBeanToTableWithoutAutoInc(context, entity);
        }
        else // The table doesn't have an "AUTO-INC" column => Standard behavior
        {
            //--- Call the STANDARD "bean to table" for INSERT with ALL the columns
            callBeanToTable(context, entity);
        }
    }

    //============================================================================
    //                             S E L E C T ( JDBC / SQL )
    // Return Code :
    // . 1 : Found and loaded in the bean
    // . 0 : Not found
    //============================================================================
    //---------------------------------------------------------------------------
    private int executeSqlSelect(Object bean, String sOptionalClause, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSelect(bean, sOptionalClause, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    private int executeSqlSelect(Object bean, String sOptionalClause, Connection con) throws TelosysException
    {
        int iReturnCode = 0;
        //String sSql = _sqlRequests.getSqlSelect();
        //--- Get the SQL Select clause ( with an optional clause at the end if necessary )
        String sSql = null;
        if (sOptionalClause != null)
        {
            sSql = _sqlRequests.getSqlSelect(sOptionalClause);
        }
        else
        {
            sSql = _sqlRequests.getSqlSelect();
        }

        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("executeSqlSelect(" + sSql + "): DB Connection is null");
        }
        try
        {
            //--- Create a "query context" for the SQL request
            trace("executeSqlSelect: create QueryContext...");
//            queryContext = new QueryContext(con, sSql, getBooleanStorage());
            queryContext = new QueryContext(con, sSql);

            //--- Set the primary key parameters
            callSetPrimaryKey(queryContext, bean); //-- in sub-class

            //--- Execute the SQL request
            trace("executeSqlSelect: executeQuery()...");
            queryContext.executeQuery();

            //--- Only one row to process ...
            trace("executeSqlSelect: rs.next()...");
            if (!queryContext.nextResult())
            {
                //--- Normal termination, but not found
                iReturnCode = 0;
                trace("executeSqlSelect: NOT FOUND ");
            }
            else
            {
                //--- Normal termination : found => load bean attributes
                callTableToBean(queryContext, bean);
                iReturnCode = 1;
                trace("executeSqlSelect: FOUND ");
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("EntityDAO.executeSqlSelect(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlSelect(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        return iReturnCode;
    }

    //---------------------------------------------------------------------------
    private boolean executeSqlExists(Object bean, int iDatabaseId) throws TelosysException
    {
        boolean bRet = false;
        Connection con = getConnection(iDatabaseId);
        try
        {
            //bRet = executeSqlExists(sSql, bean, con);
            bRet = executeSqlExists(bean, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return bRet;
    }

    //---------------------------------------------------------------------------
    private boolean executeSqlExists(Object bean, Connection con) throws TelosysException
    {
        String sSql = _sqlRequests.getSqlExists();
        int iCount = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("executeSqlExists(" + sSql + "): DB Connection is null");
        }
        try
        {
            trace("executeSqlExists: create QueryContext...");
//            queryContext = new QueryContext(con, sSql, getBooleanStorage());
            queryContext = new QueryContext(con, sSql);
            callSetPrimaryKey(queryContext, bean); //-- in sub-class

            //--- Execute the SQL count(*) request
            trace("executeSqlExists: executeQuery()...");
            queryContext.executeQuery();

            //--- Get the first and unique row of the ResultSet
            if (queryContext.nextResult())
            {
                iCount = queryContext.getResultInt(1);
            }
            else
            {
                iCount = -1;
                error("executeSqlCount: nextResult() return false (ResultSet is void)");
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlExists(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("EntityDAO.executeSqlExists(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("EntityDAO.executeSqlExists(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlExists(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        if (iCount < 0)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ") : result set is void ( 0 row ) ");
        }
        if (iCount > 0)
        {
            return true; // This entity exists
        }
        else
        {
            return false; // This entity doesn't exist
        }
    }

    //============================================================================
    //  SQL for SAVE operation
    //============================================================================
    private int executeSqlSaveList(ListQuery query, List listData, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSaveList(query, listData, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //============================================================================
    // 
    //============================================================================
    private int executeSql(int iAction, Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSql(iAction, bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    private int executeSql(int iAction, Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        String sSql = null;
        String sAction = null;
        switch (iAction)
        {
        case ACTION_SQL_INSERT :
            sAction = "Insert";
            sSql = _sqlRequests.getSqlInsert();
            break;
        case ACTION_SQL_UPDATE :
            sAction = "Update";
            sSql = _sqlRequests.getSqlUpdate();
            break;
        case ACTION_SQL_DELETE :
            sAction = "Delete";
            sSql = _sqlRequests.getSqlDelete();
            break;
        default :
            throw new TelosysException("EntityDAO.executeSql( action = " + iAction + " ): invalid action code ");
        }

        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContext = null;
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("EntityDAO.executeSql(" + sSql + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }
            //--- Cration de la requte SQL
            trace("executeSql(" + sAction + "): create QueryContext...");
//            queryContext = new QueryContext(con, sSql, getBooleanStorage());
            queryContext = new QueryContext(con, sSql);

            //--- Valorisation des paramtres de la requte SQL
            switch (iAction)
            {
            case ACTION_SQL_INSERT :
                trace("executeSql(" + sAction + "): beanToTable()...");
                //callBeanToTable(queryContext, bean); //--- in sub-class
                callBeanToTableForInsert(queryContext, bean); //--- in sub-class // v 0.9.9
                break;
            case ACTION_SQL_UPDATE :
                trace("executeSql(" + sAction + "): beanToTable()...");
                callBeanToTable(queryContext, bean); //--- in sub-class
                break;
            case ACTION_SQL_DELETE :
                trace("executeSql(" + sAction + "): setPrimaryKey()...");
                callSetPrimaryKey(queryContext, bean); //--- in sub-class
                break;
            default :
                error("executeSql(" + sAction + "): Unknown ACTION !");
                break;
            }
            //--- Excution de la requte SQL
            trace("executeSql(" + sAction + "): executeUpdate()...");
            iReturnCode = queryContext.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSql(" + sAction + "): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSql(" + sSql + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSql(" + sSql + ") : ", ex);
        } finally
        {
            trace("executeSql(" + sAction + "): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSql(" + sAction + "): rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }
    
    //============================================================================
    //============================================================================

    //---------------------------------------------------------------------------
    private Long executeSqlInsertKeyGen( Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        Long ret = null ;
        Connection con = getConnection(iDatabaseId);
        try
        {
            ret = executeSqlInsertKeyGen( bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return ret;
    }
    //---------------------------------------------------------------------------
    /**
     * Executes an INSERT with Key Generation ( AUTO_INCREMENT column ) 
     * and returns the generated key  
     * @param bean
     * @param bCommitMode
     * @param con
     * @return : the value of the generated key ( or null if none )
     * @throws TelosysException
     */
    private Long executeSqlInsertKeyGen( Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        Long generatedKey = null;
        String sSql = _sqlRequests.getSqlInsert();

        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContext = null;
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("EntityDAO.executeSqlInsertKeyGen(" + sSql + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }
            //--- Create the SQL request context
            trace("executeSqlInsertKeyGen : create QueryContext...");
            //queryContext = new QueryContext(con, sSql);
            queryContext = new QueryContext(con, sSql, QueryContext.RETURN_GENERATED_KEYS );

            //--- Valorisation des paramtres de la requte SQL
            trace("executeSqlInsertKeyGen : beanToTable()...");
            //callBeanToTable(queryContext, bean); //--- in sub-class
            callBeanToTableForInsert(queryContext, bean); //--- in sub-class  // v 0.9.9
            
            //--- Execute the SQL request
            trace("executeSqlInsertKeyGen : executeUpdate()...");
            //iReturnCode = queryContext.executeUpdate();
            generatedKey = queryContext.executeInsertKeyGen();
            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlInsertKeyGen : con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlInsertKeyGen(" + sSql + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlInsertKeyGen(" + sSql + ") : ", ex);
        } finally
        {
            trace("executeSqlInsertKeyGen : finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlInsertKeyGen : con.rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return generatedKey;
    }

    
    //============================================================================
    // S A V E : DELETE + INSERT
    // Code Retour :
    // Nombre de lignes insres, modifies ou supprimes ( 0  N )
    //============================================================================
    /**
     * First SAVE methode : execute SQL requests DELETE + INSERT
     * 
     * @param bean
     * @param bCommitMode
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    private int executeSqlSave1(Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            //iRet = executeSqlSave(sSqlDelete, sSqlInsert, bean, bCommitMode, con);
            iRet = executeSqlSave1(bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    /**
     * First SAVE methode : execute SQL requests DELETE + INSERT
     * 
     * @param bean
     * @param bCommitMode
     * @param con
     * @return
     * @throws TelosysException
     */
    private int executeSqlSave1(Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        String sSqlDelete = _sqlRequests.getSqlDelete();
        String sSqlInsert = _sqlRequests.getSqlInsert();
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContextDelete = null;
        QueryContext queryContextInsert = null;
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("executeSqlSave(): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            //--- SQL DELETE
            trace("executeSqlSave() : DELETE - create QueryContext...");
            queryContextDelete = new QueryContext(con, sSqlDelete);
            callSetPrimaryKey(queryContextDelete, bean); //--- in sub-class
            trace("executeSqlSave() : DELETE - executeUpdate()...");
            iReturnCode = queryContextDelete.executeUpdate();

            //--- SQL INSERT
            trace("executeSqlSave() : INSERT - create QueryContext...");
            queryContextInsert = new QueryContext(con, sSqlInsert);
            // callBeanToTable(queryContextInsert, bean); //--- in sub-class
            callBeanToTableForInsert(queryContextInsert, bean); //--- in sub-class  // v 0.9.9
            trace("executeSqlSave() : INSERT - executeUpdate()...");
            iReturnCode = queryContextInsert.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlSave(): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } finally
        {
            trace("executeSqlSave(): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlSave(): rollback()...");
                    rollback(con);
                }
            }
            if (queryContextDelete != null)
            {
                queryContextDelete.close();
            }
            if (queryContextInsert != null)
            {
                queryContextInsert.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //---------------------------------------------------------------------------
    /**
     * Second SAVE methode : execute SQL requests SELECT COUNT + INSERT or UPDATE
     * 
     * @param bean
     * @param bCommitMode
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    private int executeSqlSave2(Object bean, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlSave2(bean, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //---------------------------------------------------------------------------
    /**
     * Second SAVE methode : execute SQL requests SELECT COUNT + INSERT or UPDATE
     * 
     * @param bean
     * @param bCommitMode
     * @param con
     * @return
     * @throws TelosysException
     */
    private int executeSqlSave2(Object bean, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        QueryContext queryContext = null;
        int iReturnCode = 0;

        if (con == null)
        {
            throw new TelosysException("executeSqlSave(): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            //--- TEST IF EXISTS : choose between INSERT and UPDATE
            boolean bSqlUpdate = false ; // v 0.9.9
            //String sSql = null; // v 0.9.9
            if (executeSqlExists(bean, con))
            {
                //--- The key exists => UPDATE
                bSqlUpdate = true ; // v 0.9.9
                //sSql = _sqlRequests.getSqlUpdate(); // v 0.9.9
            }
            else
            {
                //--- The key doesn't exist => INSERT
                bSqlUpdate = false ; // v 0.9.9
                //sSql = _sqlRequests.getSqlInsert(); // v 0.9.9
            }

            //--- EXECUTE THE SQL ( INSERT or UPDATE )
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            trace("executeSqlSave() : create QueryContext...");
// removed in v 0.9.9
//            queryContext = new QueryContext(con, sSql);
//            callBeanToTable(queryContext, bean); //--- in sub-class
// replaced by ...
            if ( bSqlUpdate )
            {
                //--- UPDATE
                queryContext = new QueryContext(con, _sqlRequests.getSqlUpdate() );
                callBeanToTable(queryContext, bean); //--- in sub-class
            }
            else
            {
                //--- INSERT
                queryContext = new QueryContext(con, _sqlRequests.getSqlInsert() );
                callBeanToTableForInsert(queryContext, bean); //--- in sub-class
            }
// end of v 0.9.9
            
            trace("executeSqlSave() : executeUpdate()...");
            iReturnCode = queryContext.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlSave(): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("EntityDAO.executeSqlSave() : ", ex);
        } finally
        {
            trace("executeSqlSave(): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlSave(): rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //    //---------------------------------------------------------------------------
    //    private void checkParams(String sMethode, Object bean ) throws TelosysException
    //    {
    //        if (bean == null)
    //        {
    //            throw new TelosysException("DAO " + sMethode + " : bean/list parameter is null ");
    //        }
    //    }

    //---------------------------------------------------------------------------
    private void checkBeanOrList(String sMethode, Object beanOrList) throws TelosysException
    {
        if (beanOrList == null)
        {
            throw new TelosysException("DAO " + sMethode + " : bean/list parameter is null ");
        }
    }

    //---------------------------------------------------------------------------
    private void checkDbId(String sMethode, int iBaseId) throws TelosysException
    {
        if (iBaseId < 0)
        {
            throw new TelosysException("DAO " + sMethode + " : Database ID < 0  (" + iBaseId + ")");
        }
    }

    //---------------------------------------------------------------------------
    private void checkDbSession(String sMethode, DatabaseSession dbSession) throws TelosysException
    {
        if (dbSession == null)
        {
            throw new TelosysException("DAO " + sMethode + " : database session is null ");
        }
        else
        {
            if (dbSession.getConnection() == null)
            {
                throw new TelosysException("DAO " + sMethode + " : connection in database session is null ");
            }
        }
    }

    //---------------------------------------------------------------------------
    private void checkQuery(String sMethode, ListQuery query) throws TelosysException
    {
        if (query == null)
        {
            throw new TelosysException("DAO " + sMethode + " : query parameter is null ");
        }
    }

    //---------------------------------------------------------------------------
    private void checkConnection(String sMethode, Connection conn) throws TelosysException
    {
        if (conn == null)
        {
            throw new TelosysException("DAO " + sMethode + " : connection is null ");
        }
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object beanOrList, int iBaseId) throws TelosysException
    {
        checkBeanOrList(sMethode, beanOrList);
        checkDbId(sMethode, iBaseId);
    }

    private void checkParams(String sMethode, ListQuery query, Object beanOrList, int iBaseId) throws TelosysException
    {
        checkQuery(sMethode, query);
        checkBeanOrList(sMethode, beanOrList);
        checkDbId(sMethode, iBaseId);
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object beanOrList, DatabaseSession session) throws TelosysException
    {
        checkBeanOrList(sMethode, beanOrList);
        checkDbSession(sMethode, session);
    }

    private void checkParams(String sMethode, ListQuery query, Object beanOrList, DatabaseSession session) throws TelosysException
    {
        checkQuery(sMethode, query);
        checkBeanOrList(sMethode, beanOrList);
        checkDbSession(sMethode, session);
    }

    //---------------------------------------------------------------------------
    private void checkParams(String sMethode, Object beanOrList, Connection conn) throws TelosysException
    {
        checkBeanOrList(sMethode, beanOrList);
        checkConnection(sMethode, conn);
    }

    private void checkParams(String sMethode, ListQuery query, Object beanOrList, Connection conn) throws TelosysException
    {
        checkQuery(sMethode, query);
        checkBeanOrList(sMethode, beanOrList);
        checkConnection(sMethode, conn);
    }

    //---------------------------------------------------------------------------
    private String getSqlAction(int iSqlAction) throws TelosysException
    {
        switch (iSqlAction)
        {
        case ACTION_SQL_INSERT :
            return "Insert";
        case ACTION_SQL_UPDATE :
            return "Update";
        case ACTION_SQL_DELETE :
            return "Delete";
        default :
            throw new TelosysException("DAO invalid action code ( action = " + iSqlAction + " ) ");
        }
    }
    //---------------------------------------------------------------------------
//    private List getList(int iSqlAction, GenericVOList voList) throws TelosysException
    private List getList( GenericVOList voList ) throws TelosysException
    {
        if ( voList != null )
        {
            return voList.getList();
        }
//        String s = getSqlAction(iSqlAction);
//        throw new TelosysException("DAO " + s + " : GenericVOList parameter is null ");
        throw new TelosysException("DAO : GenericVOList parameter is null ");
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     L O A D
    //============================================================================
    protected int doLoad(Object bean) throws TelosysException
    {
        return doLoad(bean, ConnectionManager.getDefaultDatabase());
    }

    protected int doLoad(Object bean, String sOptionalClause) throws TelosysException
    {
        return doLoad(bean, sOptionalClause, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(LOAD, bean, iDatabaseId);
        return executeSqlSelect(bean, null, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, String sOptionalClause, int iDatabaseId) throws TelosysException
    {
        checkParams(LOAD, bean, iDatabaseId);
        return executeSqlSelect(bean, sOptionalClause, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(LOAD, bean, session);
        return executeSqlSelect(bean, null, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, String sOptionalClause, DatabaseSession session) throws TelosysException
    {
        checkParams(LOAD, bean, session);
        return executeSqlSelect(bean, sOptionalClause, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, Connection con) throws TelosysException
    {
        checkParams(LOAD, bean, con);
        return executeSqlSelect(bean, null, con);
    }

    //---------------------------------------------------------------------------
    protected int doLoad(Object bean, String sOptionalClause, Connection con) throws TelosysException
    {
        checkParams(LOAD, bean, con);
        return executeSqlSelect(bean, sOptionalClause, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     E X I S T S
    //============================================================================
    protected boolean doExists(Object bean) throws TelosysException
    {
        return doExists(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected boolean doExists(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(EXISTS, bean, iDatabaseId);
        return executeSqlExists(bean, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected boolean doExists(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(EXISTS, bean, session);
        return executeSqlExists(bean, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected boolean doExists(Object bean, Connection con) throws TelosysException
    {
        checkParams(EXISTS, bean, con);
        return executeSqlExists(bean, con);
    }

    //============================================================================
    //     S A V E ( DELETE + INSERT )
    //============================================================================
    protected int doSave(Object bean) throws TelosysException
    {
        return doSave(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(SAVE, bean, iDatabaseId);
        if (_bSaveByDeleteInsert)
        {
            return executeSqlSave1(bean, COMMIT, iDatabaseId);
        }
        else
        {
            return executeSqlSave2(bean, COMMIT, iDatabaseId);
        }
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(SAVE, bean, session);
        if (_bSaveByDeleteInsert)
        {
            return executeSqlSave1(bean, DO_NOT_COMMIT, session.getConnection());
        }
        else
        {
            return executeSqlSave2(bean, DO_NOT_COMMIT, session.getConnection());
        }
    }

    //---------------------------------------------------------------------------
    protected int doSave(Object bean, Connection con) throws TelosysException
    {
        checkParams(SAVE, bean, con);
        if (_bSaveByDeleteInsert)
        {
            return executeSqlSave1(bean, DO_NOT_COMMIT, con);
        }
        else
        {
            return executeSqlSave2(bean, DO_NOT_COMMIT, con);
        }
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     I N S E R T
    //============================================================================
    protected int doInsert(Object bean) throws TelosysException
    {
        return doInsert(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(INSERT, bean, iDatabaseId);
        return executeSql(ACTION_SQL_INSERT, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(INSERT, bean, session);
        return executeSql(ACTION_SQL_INSERT, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doInsert(Object bean, Connection con) throws TelosysException
    {
        checkParams(INSERT, bean, con);
        return executeSql(ACTION_SQL_INSERT, bean, DO_NOT_COMMIT, con);
    }

    //============================================================================
    //     I N S E R T     WITH  KEY GENERATION
    //============================================================================
    protected Long doInsertKeyGen(Object bean) throws TelosysException
    {
        return doInsertKeyGen(bean, ConnectionManager.getDefaultDatabase());
    }
    //---------------------------------------------------------------------------
    protected Long doInsertKeyGen(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(INSERT, bean, iDatabaseId);
        return executeSqlInsertKeyGen( bean, COMMIT, iDatabaseId);
    }
    //---------------------------------------------------------------------------
    protected Long doInsertKeyGen(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(INSERT_KEY_GEN, bean, session);
        return executeSqlInsertKeyGen( bean, DO_NOT_COMMIT, session.getConnection());
    }
    //---------------------------------------------------------------------------
    protected Long doInsertKeyGen(Object bean, Connection con) throws TelosysException
    {
        checkParams(INSERT_KEY_GEN, bean, con);
        return executeSqlInsertKeyGen( bean, DO_NOT_COMMIT, con);
    }
    //---------------------------------------------------------------------------

    //============================================================================
    //     U P D A T E
    //============================================================================
    protected int doUpdate(Object bean) throws TelosysException
    {
        return doUpdate(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(UPDATE, bean, iDatabaseId);
        return executeSql(ACTION_SQL_UPDATE, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(UPDATE, bean, session);
        return executeSql(ACTION_SQL_UPDATE, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doUpdate(Object bean, Connection con) throws TelosysException
    {
        checkParams(UPDATE, bean, con);
        return executeSql(ACTION_SQL_UPDATE, bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //     D E L E T E
    //============================================================================
    protected int doDelete(Object bean) throws TelosysException
    {
        return doDelete(bean, ConnectionManager.getDefaultDatabase());
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, int iDatabaseId) throws TelosysException
    {
        checkParams(DELETE, bean, iDatabaseId);
        return executeSql(ACTION_SQL_DELETE, bean, COMMIT, iDatabaseId);
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, DatabaseSession session) throws TelosysException
    {
        checkParams(DELETE, bean, session);
        return executeSql(ACTION_SQL_DELETE, bean, DO_NOT_COMMIT, session.getConnection());
    }

    //---------------------------------------------------------------------------
    protected int doDelete(Object bean, Connection con) throws TelosysException
    {
        checkParams(DELETE, bean, con);
        return executeSql(ACTION_SQL_DELETE, bean, DO_NOT_COMMIT, con);
    }

    //---------------------------------------------------------------------------

    //============================================================================
    //                              LIST DAO
    //============================================================================

    //============================================================================
    // KEY PARAMETERS ( FOR SQL WHERE CLAUSE )
    //============================================================================
    //    private void setKeyParameters(QueryContext context, Object[] key) throws TelosysException
    //    {
    //        if (key != null)
    //        {
    //// Object oKeyElement = null;
    //            int iNbKeyElements = key.length;
    //// int i = 0;
    //// int j = 0;
    //            for ( int i = 1 ; i < iNbKeyElements ; i++ ) // index : from 1 to N
    //            {
    //// oKeyElement = key[i];
    //// j = i + 1; // PreparedStatement : 1 to N
    //// context.setParam(j, oKeyElement);
    //                context.setParam(i, key[i]);
    //            }
    //        }
    //    }

    private void setKeyParameters(QueryContext context, ListQuery query) throws TelosysException
    {
        if (query != null)
        {
            int iNbKeyElements = query.getParamCount();
            for ( int i = 1 ; i <= iNbKeyElements ; i++ ) // index : from 1 to N
            {
                context.setParam(i, query.getParameter(i));
            }
        }
    }

    //============================================================================
    // LOAD : Chargement de la liste
    //============================================================================
    private int executeSqlSelectList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        int iCount = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iCount = executeSqlSelectList(query, listData, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iCount;
    }

    //============================================================================
    private int executeSqlSelectList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        int iCount = 0;
        String sSql = query.getSqlSelect();
        QueryContext queryContext = null;
        Object entityBean = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeLoad(" + sSql + "): DB Connection is null");
        }
        try
        {
            trace("executeSqlSelect: create QueryContext ...");
//            queryContext = new QueryContext(con, sSql, getBooleanStorage());
            queryContext = new QueryContext(con, sSql);

            //--- Set the SQL request parameters (from the query object)
            setKeyParameters(queryContext, query);

            //--- Execute the SQL request
            trace("executeSqlSelect: executeQuery()...");
            queryContext.executeQuery();

            //--- 0 .. N elements in the ResultSet
            while (queryContext.nextResult())
            {
                trace("executeSqlSelect: nextResult()...");
                //--- Create a new Entity Bean
                entityBean = this.createEntityBean();
                //--- Populate the Entity Bean with SQL Data
                this.tableToBean(queryContext, entityBean);
                //--- Add the new Entity Bean in the list
                listData.add(entityBean); // Idem addLast() for LinkedList
                iCount++;
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("ListDAO.executeSqlSelect(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlSelect(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        return iCount;
    }

    //============================================================================
    // COUNT : Select count(*) from ... where
    //============================================================================
    private int executeSqlCount(ListQuery query, int iDatabaseId) throws TelosysException
    {
        int iCount = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iCount = executeSqlCount(query, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iCount;
    }

    //============================================================================
    private int executeSqlCount(ListQuery query, Connection con) throws TelosysException
    {
        int iCount = 0;
        String sSql = query.getSqlCount();
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + "): DB Connection is null");
        }
        try
        {
            trace("executeSqlCount: create QueryContext ...");
            queryContext = new QueryContext(con, sSql);

            //--- Set the SQL request parameters (from the query object)
            setKeyParameters(queryContext, query);

            //--- Execute the SQL request
            trace("executeSqlCount: executeQuery()...");
            queryContext.executeQuery();

            //--- Get the first and unique row of the ResultSet
            if (queryContext.nextResult())
            {
                trace("executeSqlCount: nextResult()...");
                iCount = queryContext.getResultInt(1);
            }
            else
            {
                error("executeSqlCount: nextResult() return false (ResultSet is void)");
                iCount = -1;
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } catch (NumberFormatException ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } catch (Exception ex)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ")", ex);
        } finally
        {
            trace("executeSqlCount(): finally.");
            if (queryContext != null)
            {
                queryContext.close();
            }
        }
        if (iCount < 0)
        {
            throw new TelosysException("ListDAO.executeSqlCount(" + sSql + ") : result set is void ( 0 row ) ");
        }
        return iCount;
    }

    //============================================================================
    //                         D E L E T E L I S T
    //============================================================================
    private int executeSqlDeleteList(ListQuery query, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlDeleteList(query, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }

    //============================================================================
    private int executeSqlDeleteList(ListQuery query, boolean bCommitMode, Connection con) throws TelosysException
    {
        String sSql = query.getSqlDelete();
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        int iReturnCode = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            //--- Prparation de la requte SQL
            trace("executeSqlDelete: create QueryContext ...");
//            queryContext = new QueryContext(con, sSql, getBooleanStorage());
            queryContext = new QueryContext(con, sSql);

            //--- Valorisation des paramtres de la requte SQL
            trace("executeSqlDelete(): setSqlxxxxParameters()...");
            //---------------------------------------------------
            // Mthode de la classe DAO drive
            // Valorisation des paramtres de la clause WHERE
            //---------------------------------------------------
            setKeyParameters(queryContext, query);

            //--- Excution de la requte SQL
            trace("executeSqlDelete(): executeUpdate()...");
            iReturnCode = queryContext.executeUpdate();

            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlDelete(): con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            //error(ex);
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            //error(ex);
            throw new TelosysException("ListDAO.executeSqlDelete(" + sSql + ") : ", ex);
        } finally
        {
            trace("executeSqlDelete(): finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlDelete(): rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //============================================================================
    //                         I N S E R T L I S T
    //============================================================================
//    private int executeSqlInsertList(List listData, boolean bCommitMode, int iDatabaseId) throws TelosysException
//    {
//        int iRet = 0;
//        Connection con = getConnection(iDatabaseId);
//        try
//        {
//            iRet = executeSqlInsertList(listData, bCommitMode, con);
//        }
//        // Don't catch exceptions (just force "releaseConnection" in "finally")
//        finally
//        {
//            releaseConnection(con);
//        }
//        return iRet;
//    }

    //============================================================================
//    private int executeSqlInsertList(List listData, boolean bCommitMode, Connection con) throws TelosysException
//    {
//        boolean bMemoAutoCommitMode = false;
//        boolean bError = false;
//        int iReturnCode = 0;
//
//        if (con == null)
//        {
//            throw new TelosysException("ListDAO.executeSqlInsertList(): DB Connection is null");
//        }
//
//        if (bCommitMode)
//        {
//            //--- Store the original AutoCommit mode
//            bMemoAutoCommitMode = getAutoCommit(con);
//        }
//
//        try
//        {
//            if (bCommitMode)
//            {
//                //--- Programmatic Transaction control
//                con.setAutoCommit(false);
//            }
//
//            //================================================================
//            //   INSERT LINES
//            //================================================================
//            Object element = null;
//            int i = 0;
//            iReturnCode = 0;
//            int iSize = listData.size();
//            for ( i = 0 ; i < iSize ; i++ )
//            {
//                element = listData.get(i);
//                //--- SQL INSERT (via the Entity DAO)
//                this.doInsert(element, con);
//                iReturnCode++;
//            }
//            //================================================================
//            if (bCommitMode)
//            {
//                //--- COMMIT the operation
//                trace("executeSqlSave : con.commit()...");
//                con.commit();
//            }
//        } catch (SQLException ex)
//        {
//            bError = true;
//            throw new TelosysException("ListDAO.executeSqlInsertList() : ", ex);
//        } catch (Exception ex)
//        {
//            bError = true;
//            throw new TelosysException("ListDAO.executeSqlInsertList() : ", ex);
//        } finally
//        {
//            trace("executeSqlInsertList : finally.");
//            if (bError)
//            {
//                if (bCommitMode)
//                {
//                    //--- CANCEL the operation
//                    trace("executeSqlInsertList : rollback()...");
//                    rollback(con);
//                }
//            }
//            if (bCommitMode)
//            {
//                //--- Restore the original AutoCommit mode
//                setAutoCommit(con, bMemoAutoCommitMode);
//            }
//        }
//        return iReturnCode;
//    }
    
    //============================================================================
    // Execute an action ( INSERT, UPDATE, DELETE ) for each element of the list
    //============================================================================
    private int executeSqlListAction(int iAction, List listData, boolean bCommitMode, int iDatabaseId) throws TelosysException
    {
        int iRet = 0;
        Connection con = getConnection(iDatabaseId);
        try
        {
            iRet = executeSqlListAction(iAction, listData, bCommitMode, con);
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            releaseConnection(con);
        }
        return iRet;
    }
    
    private int executeSqlListAction(int iAction, List listData, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        int iReturnCode = 0;

        String sAction = getSqlAction(iAction);
        
        if (con == null)
        {
            throw new TelosysException("executeSqlListAction(" + sAction + "): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }

            //================================================================
            //   INSERT/UPDATE/DELETE EACH ELEMENT OF THE LIST
            //================================================================
            Object element = null;
            int iRet = 0 ;
            int i = 0;
            iReturnCode = 0;
            int iSize = listData.size();
            for ( i = 0 ; i < iSize ; i++ )
            {
                element = listData.get(i);
                if ( element != null )
                {
                    //--- Execute the action for the current element ( INSERT, UPDATE or DELETE )
                    iRet = executeSql(iAction, element, DO_NOT_COMMIT, con);
                    iReturnCode++;
                }
            }
            //================================================================
            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlListAction(" + sAction + ") : con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("executeSqlListAction(" + sAction + ") : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("executeSqlListAction(" + sAction + ") : ", ex);
        } finally
        {
            trace("executeSqlListAction(" + sAction + ") : finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlListAction(" + sAction + ") : rollback()...");
                    rollback(con);
                }
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }


    //============================================================================
    // Return : number of lines saved
    private int executeSqlSaveList(ListQuery query, List listData, boolean bCommitMode, Connection con) throws TelosysException
    {
        boolean bMemoAutoCommitMode = false;
        boolean bError = false;
        int iReturnCode = 0;
        QueryContext queryContext = null;

        if (con == null)
        {
            throw new TelosysException("ListDAO.executeSqlSave(): DB Connection is null");
        }

        if (bCommitMode)
        {
            //--- Store the original AutoCommit mode
            bMemoAutoCommitMode = getAutoCommit(con);
        }

        try
        {
            if (bCommitMode)
            {
                //--- Programmatic Transaction control
                con.setAutoCommit(false);
            }
            //================================================================
            //   DELETE LINES
            //================================================================
            //--- Prparation de la requte SQL
            trace("executeSqlDelete: create QueryContext ...");
//            queryContext = new QueryContext(con, query.getSqlDelete(), getBooleanStorage());
            queryContext = new QueryContext(con, query.getSqlDelete());
            //--- Valorisation des paramtres de la requte SQL
            trace("executeSqlSave [Delete] : setSqlxxxxParameters()...");
            //---------------------------------------------------
            setKeyParameters(queryContext, query);

            //---------------------------------------------------
            //--- Excution de la requte SQL
            trace("executeSqlSave [Delete] : executeUpdate()...");
            queryContext.executeUpdate();

            //================================================================
            //   INSERT LINES
            //================================================================
            Object element = null;
            int i = 0;
            iReturnCode = 0;
            int iSize = listData.size();
            for ( i = 0 ; i < iSize ; i++ )
            {
                element = listData.get(i);
                //--- SQL INSERT (via the Entity DAO)
                this.doInsert(element, con);
                iReturnCode++;
            }
            //================================================================
            if (bCommitMode)
            {
                //--- COMMIT the operation
                trace("executeSqlSave : con.commit()...");
                con.commit();
            }
        } catch (SQLException ex)
        {
            bError = true;
            throw new TelosysException("ListDAO.executeSqlSave() : ", ex);
        } catch (Exception ex)
        {
            bError = true;
            throw new TelosysException("ListDAO.executeSqlSave() : ", ex);
        } finally
        {
            trace("executeSqlSave : finally.");
            if (bError)
            {
                if (bCommitMode)
                {
                    //--- CANCEL the operation
                    trace("executeSqlSave : rollback()...");
                    rollback(con);
                }
            }
            if (queryContext != null)
            {
                queryContext.close();
            }
            if (bCommitMode)
            {
                //--- Restore the original AutoCommit mode
                setAutoCommit(con, bMemoAutoCommitMode);
            }
        }
        return iReturnCode;
    }

    //============================================================================
    //     L O A D L I S T
    //============================================================================
    /**
     * Loads a standard list using a query
     * 
     * @param query
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, List listData) throws TelosysException
    {
        return doLoadList(query, listData, ConnectionManager.getDefaultDatabase());
    }

    /**
     * Loads a VO list using a query
     * 
     * @param query
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, GenericVOList listData) throws TelosysException
    {
        return doLoadList(query, listData, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    /**
     * Loads a standard list using a query
     * 
     * @param query
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, iDatabaseId);
        return executeSqlSelectList(query, listData, iDatabaseId);
    }

    /**
     * Loads a VO list using a query
     * 
     * @param query
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, iDatabaseId);
        return executeSqlSelectList(query, listData.getList(), iDatabaseId);
    }

    //----------------------------------------------------------------------------
    /**
     * Loads a standard list using a query
     * 
     * @param query
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, session);
        return executeSqlSelectList(query, listData, session.getConnection());
    }

    /**
     * Loads a VO list using a query
     * 
     * @param query
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, session);
        return executeSqlSelectList(query, listData.getList(), session.getConnection());
    }

    //----------------------------------------------------------------------------
    /**
     * Loads a standard list using a query
     * 
     * @param query
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, con);
        return executeSqlSelectList(query, listData, con);
    }

    /**
     * Loads a VO list using a query
     * 
     * @param query
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doLoadList(ListQuery query, GenericVOList listData, Connection con) throws TelosysException
    {
        checkParams(LOAD_LIST, query, listData, con);
        return executeSqlSelectList(query, listData.getList(), con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     S A V E L I S T
    //============================================================================
    /**
     * Saves a standard list using a query criterias
     * 
     * @param query
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, List listData) throws TelosysException
    {
        return doSaveList(query, listData, ConnectionManager.getDefaultDatabase());
    }

    /**
     * Saves a VO list using a query criterias
     * 
     * @param query
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, GenericVOList listData) throws TelosysException
    {
        return doSaveList(query, listData.getList(), ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    /**
     * Saves a standard list using a query criterias
     * 
     * @param query
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, List listData, int iDatabaseId) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, iDatabaseId);
        return executeSqlSaveList(query, listData, COMMIT, iDatabaseId);
    }

    /**
     * Saves a VO list using a query criterias
     * 
     * @param query
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, iDatabaseId);
        return executeSqlSaveList(query, listData.getList(), COMMIT, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    /**
     * Saves a standard list using a query criterias
     * 
     * @param query
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, List listData, DatabaseSession session) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, session);
        return executeSqlSaveList(query, listData, DO_NOT_COMMIT, session.getConnection());
    }

    /**
     * Saves a VO list using a query criterias
     * 
     * @param query
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, session);
        return executeSqlSaveList(query, listData.getList(), DO_NOT_COMMIT, session.getConnection());
    }

    //----------------------------------------------------------------------------
    /**
     * Saves a standard list using a query criterias
     * 
     * @param query
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, List listData, Connection con) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, con);
        return executeSqlSaveList(query, listData, DO_NOT_COMMIT, con);
    }

    /**
     * Saves a VO list using a query criterias
     * 
     * @param query
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doSaveList(ListQuery query, GenericVOList listData, Connection con) throws TelosysException
    {
        checkParams(SAVE_LIST, query, listData, con);
        return executeSqlSaveList(query, listData.getList(), DO_NOT_COMMIT, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     D E L E T E L I S T
    //============================================================================
    /**
     * Deletes list using a query
     * 
     * @param query
     * @return
     * @throws TelosysException
     */
    protected int doDeleteList(ListQuery query) throws TelosysException
    {
        return doDeleteList(query, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    /**
     * Deletes list using a query
     * 
     * @param query
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doDeleteList(ListQuery query, int iDatabaseId) throws TelosysException
    {
        checkParams(DELETE_LIST, query, iDatabaseId);
        return executeSqlDeleteList(query, COMMIT, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    /**
     * Deletes list using a query
     * 
     * @param query
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doDeleteList(ListQuery query, DatabaseSession session) throws TelosysException
    {
        checkParams(DELETE_LIST, query, session);
        return executeSqlDeleteList(query, DO_NOT_COMMIT, session.getConnection());
    }

    //----------------------------------------------------------------------------
    /**
     * Deletes list using a query
     * 
     * @param query
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doDeleteList(ListQuery query, Connection con) throws TelosysException
    {
        checkParams(DELETE_LIST, query, con);
        return executeSqlDeleteList(query, DO_NOT_COMMIT, con);
    }

    //============================================================================
    //     INSERT LIST    ( GenericVOList )
    //============================================================================
    /**
     * Inserts all items of the given VO list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(GenericVOList listData) throws TelosysException
    {
        return doInsertListItems(getList(listData));
    }
    
    /**
     * Inserts all items of the given VO list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return doInsertListItems(getList(listData), iDatabaseId);
    }

    /**
     * Inserts all items of the given VO list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return doInsertListItems(getList(listData), session);
    }

    /**
     * Inserts all items of the given VO list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return doInsertListItems(getList(listData), con);
    }

    //============================================================================
    //     INSERT LIST    ( List )
    //============================================================================
    /**
     * Inserts all items of the given standard list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(List listData) throws TelosysException
    {
        return doInsertListItems(listData, ConnectionManager.getDefaultDatabase());
    }

    /**
     * Inserts all items of the given standard list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(List listData, int iDatabaseId) throws TelosysException
    {
        checkParams(INSERT_LIST, listData, iDatabaseId);
        return executeSqlListAction(ACTION_SQL_INSERT, listData, COMMIT, iDatabaseId);
    }

    /**
     * Inserts all items of the given standard list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(List listData, DatabaseSession session) throws TelosysException
    {
        checkParams(INSERT_LIST, listData, session);
        return executeSqlListAction(ACTION_SQL_INSERT, listData, DO_NOT_COMMIT, session.getConnection());
    }

    /**
     * Inserts all items of the given standard list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doInsertListItems(List listData, Connection con) throws TelosysException
    {
        checkParams(INSERT_LIST, listData, con);
        return executeSqlListAction(ACTION_SQL_INSERT, listData, DO_NOT_COMMIT, con);
    }
    
    //============================================================================
    //     UPDATE LIST    ( GenericVOList )
    //============================================================================
    /**
     * Updates all items of the given VO list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(GenericVOList listData) throws TelosysException
    {
        return doUpdateListItems(getList(listData));
    }
    
    /**
     * Updates all items of the given VO list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return doUpdateListItems(getList(listData), iDatabaseId);
    }

    /**
     * Updates all items of the given VO list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return doUpdateListItems(getList(listData), session);
    }

    /**
     * Updates all items of the given VO list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return doUpdateListItems(getList(listData), con);
    }

    //============================================================================
    //     UPDATE LIST    ( List )
    //============================================================================
    /**
     * Updates all items of the given standard list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(List listData) throws TelosysException
    {
        return doUpdateListItems(listData, ConnectionManager.getDefaultDatabase());
    }

    /**
     * Updates all items of the given standard list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(List listData, int iDatabaseId) throws TelosysException
    {
        checkParams(UPDATE_LIST, listData, iDatabaseId);
        return executeSqlListAction(ACTION_SQL_UPDATE, listData, COMMIT, iDatabaseId);
    }

    /**
     * Updates all items of the given standard list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(List listData, DatabaseSession session) throws TelosysException
    {
        checkParams(UPDATE_LIST, listData, session);
        return executeSqlListAction(ACTION_SQL_UPDATE, listData, DO_NOT_COMMIT, session.getConnection());
    }

    /**
     * Updates all items of the given standard list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doUpdateListItems(List listData, Connection con) throws TelosysException
    {
        checkParams(UPDATE_LIST, listData, con);
        return executeSqlListAction(ACTION_SQL_UPDATE, listData, DO_NOT_COMMIT, con);
    }

    //============================================================================
    //     DELETE LIST    ( GenericVOList )
    //============================================================================
    /**
     * Deletes all items of the given VO list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(GenericVOList listData) throws TelosysException
    {
        return doDeleteListItems(getList(listData));
    }
    
    /**
     * Deletes all items of the given VO list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(GenericVOList listData, int iDatabaseId) throws TelosysException
    {
        return doDeleteListItems(getList(listData), iDatabaseId);
    }

    /**
     * Deletes all items of the given VO list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(GenericVOList listData, DatabaseSession session) throws TelosysException
    {
        return doDeleteListItems(getList(listData), session);
    }

    /**
     * Deletes all items of the given VO list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(GenericVOList listData, Connection con) throws TelosysException
    {
        return doDeleteListItems(getList(listData), con);
    }

    
    //============================================================================
    //     DELETE LIST    ( List )
    //============================================================================
    /**
     * Deletes all items of the given standard list
     * 
     * @param listData
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(List listData) throws TelosysException
    {
        return doDeleteListItems(listData, ConnectionManager.getDefaultDatabase());
    }

    /**
     * Deletes all items of the given standard list
     * 
     * @param listData
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(List listData, int iDatabaseId) throws TelosysException
    {
        checkParams(DELETE_LIST, listData, iDatabaseId);
        return executeSqlListAction(ACTION_SQL_DELETE, listData, COMMIT, iDatabaseId);
    }

    /**
     * Deletes all items of the given standard list
     * 
     * @param listData
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(List listData, DatabaseSession session) throws TelosysException
    {
        checkParams(DELETE_LIST, listData, session);
        return executeSqlListAction(ACTION_SQL_DELETE, listData, DO_NOT_COMMIT, session.getConnection());
    }

    /**
     * Deletes all items of the given standard list
     * 
     * @param listData
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doDeleteListItems(List listData, Connection con) throws TelosysException
    {
        checkParams(DELETE_LIST, listData, con);
        return executeSqlListAction(ACTION_SQL_DELETE, listData, DO_NOT_COMMIT, con);
    }

    //============================================================================
    //     C O U N T
    //============================================================================
    /**
     * Count operation using a query
     * 
     * @param query
     * @return
     * @throws TelosysException
     */
    protected int doCount(ListQuery query) throws TelosysException
    {
        return doCount(query, ConnectionManager.getDefaultDatabase());
    }

    //----------------------------------------------------------------------------
    /**
     * Count operation using a query
     * 
     * @param query
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    protected int doCount(ListQuery query, int iDatabaseId) throws TelosysException
    {
        checkParams(COUNT, query, iDatabaseId);
        return executeSqlCount(query, iDatabaseId);
    }

    //----------------------------------------------------------------------------
    /**
     * Count operation using a query
     * 
     * @param query
     * @param session
     * @return
     * @throws TelosysException
     */
    protected int doCount(ListQuery query, DatabaseSession session) throws TelosysException
    {
        checkParams(COUNT, query, session);
        return executeSqlCount(query, session.getConnection());
    }

    //----------------------------------------------------------------------------
    /**
     * Count operation using a query
     * 
     * @param query
     * @param con
     * @return
     * @throws TelosysException
     */
    protected int doCount(ListQuery query, Connection con) throws TelosysException
    {
        checkParams(COUNT, query, con);
        return executeSqlCount(query, con);
    }

    //----------------------------------------------------------------------------

    //============================================================================
    //     D E S C R I P T I O N
    //============================================================================
    /**
     * Describes the DAO
     * 
     * @return the description
     */
    public String describe()
    {
        return "DAO description : " + "\n . Java class : " + this.getClass().getName() + "\n . SQL table  : " + _sqlRequests.getTableName() // this.getMainTable()
                + "\n . select : " + _sqlRequests.getSqlSelect() + "\n . insert : " + _sqlRequests.getSqlInsert() + "\n . update : " + _sqlRequests.getSqlUpdate() + "\n . delete : " + _sqlRequests.getSqlDelete() + "\n . boolean storage type : " + this._iBooleanStorage;
    }

}