package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * Standard implementation of DatabaseSession <br>
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardDatabaseSession extends TelosysObject implements DatabaseSession
{
	private static long DATABASE_SESSION_ID = 0 ; // since v 1.0.2
    
    long       _lDatabaseSessionId  = 0; // since v 1.0.2
    
    int        _iDatabaseId         = 0;

    Connection _con                 = null;

    boolean    _bOriginalAutoCommit = false;

    /**
     * Constructor for a real database session <br>
     * Since v 1.0.2, the instance created is stored as a thread-local variables<br>
     * accessible via the class CurrentDatabaseSession.
     * 
     * @param iDatabaseId
     * @param con
     * @throws TelosysException
     */
    public StandardDatabaseSession(int iDatabaseId, Connection con) throws TelosysException
    {
        super();
        
        trace("constructor(" + iDatabaseId + ",...)");

        if (iDatabaseId < 0)
        {
            throw new TelosysException("DatabaseSession constructor : database id < 0 ( " + iDatabaseId + ") ");
        }
        if (con == null)
        {
            throw new TelosysException("DatabaseSession constructor : connection is null ");
        }
        // In a session, the connection must not be in "AutoCommit mode"
        try
        {
            //--- Keep the original AutoCommit state
            _bOriginalAutoCommit = con.getAutoCommit();
            //--- Set the Programmatic Transaction control
            con.setAutoCommit(false);
        } catch (SQLException ex)
        {
            throw new TelosysException("DatabaseSession constructor : cannot get/set AutoCommit", ex);
        }
        _iDatabaseId = iDatabaseId;
        _con = con;
        
        //--- Set a unique ID for this new DatabaseSession // v 1.0.2
        DATABASE_SESSION_ID++ ; // next id
        _lDatabaseSessionId = DATABASE_SESSION_ID ; // v 1.0.2
        
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#getId()
     */
    public long getId() // since v 1.0.2
    {
        return _lDatabaseSessionId ;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#isDummy()
     */
    public boolean isDummy()
    {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#getDatabaseId()
     */
    public int getDatabaseId()
    {
        return _iDatabaseId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#getConnection()
     */
    public Connection getConnection()
    {
        return _con;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#commit()
     */
    public void commit() throws TelosysException
    {
        trace("#" + _lDatabaseSessionId + ".commit()");
        
        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.commit() : connection is null");
        }
        else
        {
            try
            {
                _con.commit();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.commit() : Cannot commit", e);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#rollback()
     */
    public void rollback() throws TelosysException
    {
        trace("#" + _lDatabaseSessionId +".rollback()");
        
        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.rollback() : connection is null");
        }
        else
        {
            try
            {
                _con.rollback();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.rollback() : Cannot rollback", e);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.dal.dao.DatabaseSession#close()
     */
    public void close() throws TelosysException
    {
        //--- Remove this DatabaseSession from the Thread-local variable if it's it  // v 1.0.2
        boolean b = CurrentDatabaseSession.removeIfStored(this);
        
        trace("#" + _lDatabaseSessionId + ".close() : removed from current thread ? " + b );

        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.close() : connection is null");
        }
        else
        {
            try
            {
                //--- Reset the original AutoCommit state
                _con.setAutoCommit(_bOriginalAutoCommit);
                //--- Release the connection
                _con.close();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.close() : connection error on setAutoCommit or close ", e);
            }
        }
    }
    
    public String toString() 
    {
        return "StandardDatabaseSession : id = "+_lDatabaseSessionId+", DB id = "+_iDatabaseId ;
    }
    
}