package org.objectweb.telosys.dal.dataset;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.data.DataRow;
import org.objectweb.telosys.common.data.UpdatableDataSet;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.sql.Criteria;
import org.objectweb.telosys.dal.sql.SqlTool;

/**
 * @author Laurent GUERIN 
 *  
 */
public class SQLDataSet extends UpdatableDataSet
{
    private static final int RELEASE_CONNECTION        = 1;

    private static final int DO_NOT_RELEASE_CONNECTION = 0;
    
    //--- Request definition for this DataSet
    private SQLDataSetDefinition _definition     = null;

    //--- SQL parameters for this DataSet
    private Object[]             _parameters     = null;

    //--- Flag : is DataSet META DATA initialized ?
    private boolean              _bMetaDataInit  = false;
    
    //--- Number of columns in each DataRow 
    private int   				 _iColCount      = 0;

    //--- JDBC type for each column in the ResultSet
    private int                  _ColJdbcTypes[] = null;

    //--- Values for each column of a ROW in the ResultSet
    private Object[]             _oValues = null;


    //--- Flag : all rows loaded by the last load call ?
    private boolean              _bEndOfResultSet = false;

    //-----------------------------------------------------------------------------
    //--- CONSTRUCTORS
    //-----------------------------------------------------------------------------
    private SQLDataSet() // No default constructor
    {
    }

    private void init(SQLDataSetDefinition def, String[] parameters) throws TelosysException
    {
        if (def == null)
        {
            throw new TelosysException("Cannot create SQLDataSet : No DataSetDefinition ( def == null )");
        }
        if (parameters == null)
        {
            throw new TelosysException("Cannot create SQLDataSet : No parameters ( parameters == null )");
        }
        if (parameters.length < def.getNbParams())
        {
            throw new TelosysException("Cannot create SQLDataSet : " + def.getNbParams() + " SQL parameters expected (" + parameters.length + " received)");
        }
        //--- Store the SQL parameters
        _parameters = def.castParameters(parameters);
        //--- Store the request definition
        _definition = def;

    }

    /**
     * Creates a new SQLDataSet using the given definition
     * @param def the DataSet definition without parameter
     * @throws TelosysException
     */
    public SQLDataSet(SQLDataSetDefinition def) throws TelosysException
    {
        super();
        String[] noParameters = new String[0];
        init(def, noParameters);
    }

    /**
     * Creates a new SQLDataSet using the given definition and the given parameters values
     * @param def the DataSet definition with parameters
     * @param paramValues array of parameters values 
     * @throws TelosysException
     */
    public SQLDataSet(SQLDataSetDefinition def, String[] paramValues) throws TelosysException
    {
        super();
        init(def, paramValues);
    }

    /**
     * Creates a new SQLDataSet using the given definition and the given dynamic criterias
     * @param def the DataSet definition without parameter
     * @param crit the dynamic criterias 
     * @throws TelosysException
     */
    public SQLDataSet(SQLDataSetDefinition def, Criteria crit) throws TelosysException
    {
        super();
        //--- Set the where clause dynamically
      	def.alterWhere(crit);       	
        //--- Set the parameters dynamically
      	String paramValues[] = crit.getDynamicParameters(); 
        //--- Init the DataSet with the dynamic definition
        init(def, paramValues);
    }

    //-----------------------------------------------------------------------------
    //--- PRIVATE/PROTECTED METHODS
    //-----------------------------------------------------------------------------
    /**
     * Release the given connection <br>
     * Doesn't throw Exception (just trace the error)
     * 
     * @param con
     */
    private void releaseConnection(Connection con)
    {
        trace("releaseConnection()...");
        if (con != null)
        {
            try
            {
                con.close();
            } catch (SQLException e)
            {
                error("Cannot close the connection ! SQLException : " + e.getMessage());
            }
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Execute a SELECT COUNT request with WHERE parameters <br>
     * release all the JDBC resources ( result set and connection ) after use.
     * 
     * @param con
     * @param aParameters
     * @return the count value
     * @throws TelosysException 
     */
    private int executeCount(Connection con, Object aParameters[]) throws TelosysException
    {
        int iNbRows = 0;
        //--- Get the full SQL "select count(1) from ... where ..."
        String sSql = _definition.getFullSqlCount();
        ResultSet rs = null;
        trace("ExecuteCount()...");
        try
        {
            //--- Execute the SQL request with parameters
            rs = SqlTool.executeSelect(con, sSql, aParameters);
            if (rs.next())
            {
                iNbRows = rs.getInt(1);
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("SQLException : " + ex.getMessage(), ex);
        } catch (Exception ex)
        {
            throw new TelosysException("Exception : " + ex.getMessage(), ex);
        } finally
        {
            SqlTool.close(rs);
            releaseConnection(con);
        }
        return iNbRows;
    }

    //-------------------------------------------------------------
    /**
     * Executes a SELECT COLUMNS request with WHERE parameters <br>
     * release the JDBC result set after use.
     * 
     * @param con
     * @param aParameters
     * @param iPage
     * @throws TelosysException
     */
    private void executeSelect(Connection con, Object aParameters[], int iPage) throws TelosysException
    {
        //--- Get the full SQL "select col1, col2 from ... where ..."
        String sSql = _definition.getFullSqlSelect();
        int iMaxRows = _definition.getMaxRows();
        ResultSet rs = null;

        trace("ExecuteSelect()...");
        //_iRowCount = 0;
        try
        {
            //--- Execute the SQL request with parameters
            rs = SqlTool.executeSelect(con, sSql, aParameters);
            if (rs != null)
            {
                if (iPage < 0)
                {
                    //--- No page number => load the iMaxRows first lines
                    for ( int i = 1 ; i <= iMaxRows ; i++ )
                    {
                        if (rs.next() != true)
                        {
                            // end of rs
                            break;
                        }
                        //--- Store this row and increment row count
                        storeDataRow(rs);
                    }
                }
                else
                {
                    //--- Get first and last row for the given page ( from 1 to N )
                    int iFirstRow = _definition.getFirstRowForPage(iPage);
                    int iLastRow = _definition.getLastRowForPage(iPage);

                    //--- Store only lines between the first and the last row for this page
                    for ( int iRowNum = 1 ; iRowNum <= iLastRow ; iRowNum++ )
                    {
                        if (rs.next() != true)
                        {
                            // end of rs
                            break;
                        }

                        if (iRowNum >= iFirstRow && iRowNum <= iLastRow)
                        {
                            //--- Store this row and increment row count
                            storeDataRow(rs);
                        }
                    }
                }
                //--- Is there more rows after the current position ?
                _bEndOfResultSet = false;
                if (!rs.next())
                {
                    // We are at the end of the ResultSet
                    _bEndOfResultSet = true;
                }
            }
        } catch (SQLException ex)
        {
            throw new TelosysException("SQLException : " + ex.getMessage(), ex);
        } catch (Exception ex)
        {
            throw new TelosysException("Exception : " + ex.getMessage(), ex);
        } finally
        {
            SqlTool.close(rs);
            //releaseConnection(con);// not systematic
        }
        
//        //--- Return the number of rows in the DataSet
//        //return _iRowCount;
//        return getRowCount();
    }

    /**
     * Store the curent row of the ResultSet in a new DataRow
     * 
     * @param rs
     * @return
     */
    private DataRow storeDataRow(ResultSet rs) throws TelosysException
    {
        DataRow dr = createDataRow(rs);
        super.addRow(dr);
//        _vDataRows.add(dr);
//        _iRowCount++;
        return dr;
    }

    private int getColCount(ResultSet rs) throws TelosysException
    {
        if (!_bMetaDataInit)
        {
            try
            {
                ResultSetMetaData rsmd = rs.getMetaData();
                _iColCount = rsmd.getColumnCount();
                //--- Create the types array
                _ColJdbcTypes = new int[_iColCount + 1];
                for ( int iCol = 1 ; iCol <= _iColCount ; iCol++ ) // 1  N
                {
                    _ColJdbcTypes[iCol] = rsmd.getColumnType(iCol);
                }
                //--- Create the values array
                _oValues = new Object[_iColCount + 1];
                _bMetaDataInit = true;
            } catch (SQLException ex)
            {
                throw new TelosysException("SQLException : " + ex.getMessage(), ex);
            }
        }
        return _iColCount ;
    }
    
    /**
     * Create a new DataRow object for the curent ResultSet row
     * 
     * @param rs
     * @return : the DataRow created
     */
    private DataRow createDataRow(ResultSet rs) throws TelosysException
    {
//        if (!_bMetaDataInit)
//        {
//            try
//            {
//                ResultSetMetaData rsmd = rs.getMetaData();
//                _iColCount = rsmd.getColumnCount();
//                //--- Create the types array
//                _ColJdbcTypes = new int[_iColCount + 1];
//                for ( int iCol = 1 ; iCol <= _iColCount ; iCol++ ) // 1  N
//                {
//                    _ColJdbcTypes[iCol] = rsmd.getColumnType(iCol);
//                }
//                //--- Create the values array
//                _oValues = new Object[_iColCount + 1];
//                _bMetaDataInit = true;
//            } catch (SQLException SqlEx)
//            {
//                error("getResponseObject: SQLException: " + SqlEx.getMessage());
//                return null;
//            }
//        }
        int iColCount = getColCount(rs);
        
        trace ( "createDataRow(rs) : _iColCount = " + iColCount );
        //--- Populate the values array  
        Object oValue = null;
        for ( int iCol = 0 ; iCol < iColCount ; iCol++ ) // 0 to N-1 ( ARRAY )
        {
            //--- Column Values from the ResultSet
            try
            {
                //--- NB : Array 0 to N-1 / ResultSet 1 to N => "+1"
                oValue = SqlTool.getColValue(rs, iCol+1, _ColJdbcTypes[iCol+1]);
            } catch (SQLException SqlEx)
            {
                error("createDataRow: SQLException: " + SqlEx.getMessage());
                oValue = null;
            } catch ( Throwable t)
            {
                error("createDataRow: Throwable: " + t.getMessage());
                oValue = null;                
            }
            if ( iCol < _oValues.length )
            {
                _oValues[iCol] = oValue ;                
            }
        }
        //--- Create a new DataRow with the result set values and return it 
        return new DataRow(iColCount, _oValues);
    }

    /**
     * Generic LOAD operation ( from SQL to DATASET )
     * @param con
     * @param aParameters
     * @param iPage
     * @return
     * @throws TelosysException
     */
    private int genericLoad(Connection con, int iFlag, Object aParameters[], int iPage) throws TelosysException
    {
        //--- Initialize the DataSet ( prepare to load )
        initBeforeLoad();
        //--- Load the DataSet
        try
        {
            executeSelect( con, aParameters, iPage) ;
        }
        // Don't catch exceptions (just force "releaseConnection" in "finally")
        finally
        {
            if ( iFlag == RELEASE_CONNECTION )
            {
                releaseConnection(con);
            }
        }
        //--- Return the number of rows in the DataSet
        return getRowCount();

    }
    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------

    /**
     * Returns the SQL request used to load the DataSet
     * @return
     */
    public String getSqlRequest() 
    {
        return _definition.getFullSqlSelect();
    }
    
    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.data.DataSet#load()
     */
    public int load() throws TelosysException
    {
        Connection con = ConnectionManager.getConnection(_definition.getDatabaseId());
//        return executeSelect(con, _parameters, -1);
        return genericLoad(con, RELEASE_CONNECTION, _parameters, -1);
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.data.DataSet#load(java.lang.Object)
     */
    public int load(Object obj) throws TelosysException
    {
        if ( obj == null )
        {
            throw new TelosysException("SQLDataSet load(obj) : parameter is null");
        }
        
        int iFlag = RELEASE_CONNECTION ;
        Connection con = null ;
        if ( obj instanceof Connection )
        {
            con = (Connection) obj;
            iFlag = DO_NOT_RELEASE_CONNECTION ;
        }
        else  if ( obj instanceof DatabaseSession )
        {
            DatabaseSession dbs = (DatabaseSession) obj;
            con = dbs.getConnection();
            iFlag = DO_NOT_RELEASE_CONNECTION ;
        }
        else  if ( obj instanceof Integer )
        {
            Integer integer = (Integer) obj;
            int iDbId = integer.intValue();
            con = ConnectionManager.getConnection( iDbId );
            iFlag = RELEASE_CONNECTION ;
        }
        else
        {
            throw new TelosysException("SQLDataSet(obj) : unusable parameter " + obj.getClass().getName() );
        }
//        return executeSelect(con, _parameters, -1);
        return genericLoad(con, iFlag, _parameters, -1);
    }

    //-----------------------------------------------------------------------------
    /**
     * Load a given page (set of rows) using the LinesPerPage parameter of the definition <br>
     * IE : with 10 lines per page ... <br>
     * loadPage(1) will load lines 1 to 10 <br>
     * loadPage(2) will load lines 11 to 20 <br>
     * 
     * @param iPage the page number (from 1 to N) 
     * @return the number of lines loaded ( 0 to LinesPerPage )
     * @throws TelosysException ( if the page number is < 0 )
     */
    public int loadPage(int iPage) throws TelosysException
    {
        Connection con = ConnectionManager.getConnection(_definition.getDatabaseId());
        if ( iPage <= 0 )
        {
            throw new TelosysException("loadPage("+iPage+") : Invalid page");
        }
//        return executeSelect(con, _parameters, iPage);
        return genericLoad(con, RELEASE_CONNECTION, _parameters, iPage);
    }

    //-----------------------------------------------------------------------------
    /**
     * Indicates if the last line of the ResultSet is loaded in the DataSet <br>
     * This method is significant only after a load() or loadPage()
     * 
     * @return true if the end of the ResultSet was reached, else false
     */
    public boolean endOfResultSet()
    {
        return _bEndOfResultSet;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the number of lines corresponding to the WHERE parameters <br>
     * (just count with SELECT COUNT, the lines are not loaded in the DataSet)
     * 
     * @return the count value
     * @throws TelosysException
     */
    public int count() throws TelosysException
    {
        Connection con = ConnectionManager.getConnection(_definition.getDatabaseId());
        return executeCount(con, _parameters);
    }
    //-----------------------------------------------------------------------------
}