package org.objectweb.telosys.dal.dataset;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.dal.sql.Cast;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.sql.Criteria;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLDataSetDefinition extends TelosysObject
{
    private final static int[] VOID_PARAM_TYPES = new int[0];
    
    private String  _sFullSqlSelectRequest = null ;
    
    private boolean _bStaticDefinition  = false ;    
    
    private String _sSqlSelect        = null;

    private String _sSqlFrom          = null;

    private String _sSqlWhere         = null;

    private String _sOriginalSqlWhere = null;

    private String _sSqlOption        = "";

    private int[]  _parametersTypes   = null;

    private int    _iMaxRows          = 1000;

    private int    _iLinesPerPage     = 20;  // 20 lines per page (by default)

    private int    _iDatabaseId       = 0;

    /**
     *  
     */
    private void init(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption, int[] paramTypes)
    {
        _iDatabaseId = ConnectionManager.getDefaultDatabase(); // v 0.9.8 ( 29/08/2007 )
        
        _sSqlSelect = (sSqlSelect != null ? sSqlSelect : "ERR_no_select");
        _sSqlFrom = (sSqlFrom != null ? sSqlFrom : "ERR_no_from");
        _sSqlWhere = (sSqlWhere != null ? sSqlWhere : "");
        _sOriginalSqlWhere = _sSqlWhere;
        _sSqlOption = (sSqlOption != null ? sSqlOption : "");
        if (paramTypes != null)
        {
            //--- Clone to preserve "private"
            _parametersTypes = (int[]) paramTypes.clone();
        }
        else
        {
            _parametersTypes = VOID_PARAM_TYPES ; // new int[0];
        }
    }
    
    private void init(String sSqlSelectRequest, int[] paramTypes)
    {
        _iDatabaseId = ConnectionManager.getDefaultDatabase(); 
        
        _sFullSqlSelectRequest = sSqlSelectRequest ;
        if (paramTypes != null)
        {
            //--- Clone to preserve "private"
            _parametersTypes = (int[]) paramTypes.clone();
        }
        else
        {
            _parametersTypes = VOID_PARAM_TYPES ; // new int[0];
        }
    }

    //------------------------------------------------------------------------------------------
    // CONSTRUCTORS
    //------------------------------------------------------------------------------------------
    /**
     * SQLDataSet definition constructor for full SQL request in a single string.
     * NB: With this constructor, the WHERE clause cannot be dynamic ( alterWhere cannot be used ) 
     * @param sSqlSelectRequest
     */
    public SQLDataSetDefinition(String sSqlSelectRequest)
    {
        if ( sSqlSelectRequest == null )
        {
            throw new TelosysRuntimeException ( "SQLDataSetDefinition constructor : SQL request is null" );
        }
        _bStaticDefinition = true ;
        init(sSqlSelectRequest, null);
    }
    
    /**
     * SQLDataSet definition constructor for full SQL request in a single string, with parameters.
     * NB: With this constructor, the WHERE clause cannot be dynamic ( alterWhere cannot be used ) 
     * @param sSqlSelectRequest
     * @param paramTypes
     */
    public SQLDataSetDefinition(String sSqlSelectRequest, int[] paramTypes)
    {
        if ( sSqlSelectRequest == null )
        {
            throw new TelosysRuntimeException ( "SQLDataSetDefinition constructor : SQL request is null" );
        }
        _bStaticDefinition = true ;
        init(sSqlSelectRequest, paramTypes);
    }
    
    /**
     * SQLDataSet definition constructor.
     * The WHERE clause is potentially dynamic ( alterWhere can be used ) 
     * @param sSqlSelect
     * @param sSqlFrom
     * @param sSqlWhere
     * @param sSqlOption
     */
    public SQLDataSetDefinition(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption)
    {
        init(sSqlSelect, sSqlFrom, sSqlWhere, sSqlOption, null);
    }

    /**
     * SQLDataSet definition constructor.
     * The WHERE clause is potentially dynamic ( alterWhere can be used ) 
     * @param sSqlSelect
     * @param sSqlFrom
     * @param sSqlWhere
     * @param sSqlOption
     * @param paramTypes
     */
    public SQLDataSetDefinition(String sSqlSelect, String sSqlFrom, String sSqlWhere, String sSqlOption, int[] paramTypes)
    {
        init(sSqlSelect, sSqlFrom, sSqlWhere, sSqlOption, paramTypes);
    }

    //------------------------------------------------------------------------------------------
    /**
     * Change the initial SQL where clause using a criteria set to build the new WHERE clause dynamicaly
     * NB: if the SQLDataSet definition is static, this method cannot be used ( throws a TelosysRuntimeException )
     * @param criterias
     */
    public void alterWhere(Criteria criterias)
    {
        if ( _bStaticDefinition )
        {
            throw new TelosysRuntimeException ( "alterWhere cannot be used with a static request definition." );            
        }
        if (_sOriginalSqlWhere.trim().toLowerCase().startsWith("where"))
        {
            //--- Default = original static WHERE
            _sSqlWhere = _sOriginalSqlWhere.trim() ;
            //--- Static WHERE clause => add the dynamic WHERE clause if it exists
            String sDynamic = criterias.getDynamicWhere();
            if ( sDynamic != null )
            {
                if ( sDynamic.trim().length() > 0 )
                {
                    _sSqlWhere = _sOriginalSqlWhere.trim() + " and " + sDynamic ;                 
                }
            }
        }
        else
        {
            //--- No static WHERE clause => rebuild all the WHERE clause
            String sDynamic = criterias.getDynamicWhere();
            if ( sDynamic != null )
            {
                if ( sDynamic.trim().length() > 0 )
                {
                    _sSqlWhere = " where " + sDynamic ;
                }
            }
        }
        //--- Set the new parameters types
        _parametersTypes = criterias.getDynamicTypes();
        trace("alterWhere() : " + _sSqlWhere);

        //_criterias = criterias ;
    }

    //------------------------------------------------------------------------------------------
    // SETTERS & GETTERS
    //------------------------------------------------------------------------------------------
    //--- Setters
    public void setMaxRows(int v)
    {
        _iMaxRows = v;
    }

    public void setLinesPerPage(int v)
    {
        if (v <= _iMaxRows)
        {
            _iLinesPerPage = v;
        }
    }

    public void setDatabaseId(int v)
    {
        _iDatabaseId = v;
    }

    //--- Getters

    public String getSqlSelect()
    {
        return _sSqlSelect;
    }

    public String getSqlWhere()
    {
        return _sSqlWhere;
    }

    public String getSqlFrom()
    {
        return _sSqlFrom;
    }

    public String getSqlOption()
    {
        return _sSqlOption;
    }

    /**
     * Return the number of parameters expected to execute the SQL request
     * 
     * @return : number of parameters expected
     */
    public int getNbParams()
    {
        return _parametersTypes.length;
    }

    public int getLinesPerPage()
    {
        return _iLinesPerPage;
    }

    public int getMaxRows()
    {
        return _iMaxRows;
    }

    public int getDatabaseId()
    {
        return _iDatabaseId;
    }

    public String getParametersTypes()
    {
        StringBuffer sb = new StringBuffer(100);
        for ( int i = 0 ; i < _parametersTypes.length ; i++ )
        {
            if (i > 0)
            {
                sb.append("|");
            }
            sb.append("" + _parametersTypes[i]);
        }
        return sb.toString();
    }

    /**
     * Return the full SQL request for count(*)
     * 
     * @return : SQL request
     */
    public String getFullSqlCount()
    {
        String sSelectCount = "SELECT COUNT(1) " ;
        String sSQL = null ;
        if ( _bStaticDefinition )
        {
            String s = _sFullSqlSelectRequest.toUpperCase();
            int i = s.indexOf("FROM") ;
            if ( i > 0 )
            {
                sSQL = sSelectCount + _sFullSqlSelectRequest.substring(i);
            }
        }
        else
        {
	        sSQL = sSelectCount + _sSqlFrom + " " + _sSqlWhere;
        }
        trace("getFullSqlCount() : " + sSQL);
        if ( sSQL == null )
        {
            throw new TelosysRuntimeException ( "Cannot get SQL COUNT request." );            
        }
        return sSQL ;
    }

    /**
     * Return the full SQL request to retrieve data
     * 
     * @return : SQL request
     */
    public String getFullSqlSelect()
    {
        if ( _bStaticDefinition )
        {
            return _sFullSqlSelectRequest ;
        }
        else
        {
            String sSQL = _sSqlSelect + " " + _sSqlFrom + " " + _sSqlWhere + " " + _sSqlOption;
            trace("getFullSqlSelect() : " + sSQL);
            return sSQL;
        }
    }

    /**
     * Return the first row of the given page (calculated with "lines per page")
     * 
     * @param iPage :
     *            page number (from 1 to N )
     * @return : the row number ( from 1 to N )
     */
    public int getFirstRowForPage(int iPage)
    {
        return ((iPage - 1) * _iLinesPerPage) + 1;
    }

    /**
     * Return the last row of the given page (calculated with "lines per page")
     * 
     * @param iPage :
     *            page number (from 1 to N )
     * @return : the row number ( from 1 to N )
     */
    public int getLastRowForPage(int iPage)
    {
        return (iPage * _iLinesPerPage);
    }

    public Object[] castParameters(String strParams[]) throws TelosysException
    {
        if (strParams == null)
        {
            return new Object[0];
        }
        int iParamLength = _parametersTypes.length;
        if (strParams.length < iParamLength)
        {
            throw new TelosysException("Invalid parameters length (" + strParams.length + " < " + iParamLength + ")");
        }

        Object[] objParams = new Object[iParamLength];

        int i = 0;
        String sParam = "(null)";
        try
        {
            for ( i = 0 ; i < iParamLength ; i++ )
            {
                sParam = strParams[i];
                objParams[i] = Cast.toObject(strParams[i], _parametersTypes[i]);
            }
        } catch (Exception e)
        {
            throw new TelosysException("Cannot cast parameter " + (i + 1) + " : '" + sParam + "' ", e);
        } catch (Throwable t)
        {
            throw new TelosysException("Cannot cast parameter " + (i + 1) + " : '" + sParam + "' ", t);
        }

        return objParams;
    }

}