package org.objectweb.telosys.dal.rest;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.common.vo.LooselyTypedVOList;
import org.objectweb.telosys.dal.TelosysDAL;
import org.objectweb.telosys.dal.dao.DAOProvider;
import org.objectweb.telosys.dal.dao.ListQuery;
import org.objectweb.telosys.dal.dao.StandardDAO;
import org.objectweb.telosys.dal.dao.StandardDAOAdapter;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.dal.sql.Database;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * Servlet for DAO invocation in REST mode<br>
 * 
 * URL Syntax examples : <br>
 * http://localhost:8080/myapp/db/0/Author/load?Id=1  <br>
 * http://localhost:8080/myapp/db/0/Author/load.xml?Id=1  <br>
 * http://localhost:8080/myapp/db/0/Author/update?Id=1&FirstName=Mike&LastName=BUSH  <br>
 * 
 * NB : the bean name is case sensitive ( it is used to build the full class name )
 * 
 * @author Olivier LAURENDEAU, Laurent GUERIN
 * @since 1.0.0
 */
public class DAOServlet extends TelosysServlet //extends HttpServlet
{
	static final long serialVersionUID = 1L;
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException {
	    trace("doGet");
		process(request, response);
	}
	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException {
	    trace("doPost");
		process(request, response);
	}

	/**
	 * Main process method, called by doPost & doGet
	 * 
	 * @param request
	 * @param response
	 */
	private void process(HttpServletRequest httpRequest, HttpServletResponse httpResponse) 
	{
	    
	    trace("process : URL = " + httpRequest.getRequestURL() );
	    trace("process : URI = " + httpRequest.getRequestURI() );
		//TelosysDAL.init();
	    
	    WebUtil.noCache(httpResponse);
	    
		PrintWriter pw = null ;
		try
        {
            pw = httpResponse.getWriter();
        } catch (IOException e1)
        {
            error("Cannot get PrintWriter");
            return ;
        }

        //--- Parse the request URI
		DAORequest req = null ;
        try
        {
    		req = parseURI( httpRequest.getRequestURI() ) ;
        } catch (Throwable e) // Catch all exceptions
        {
            int iType = getTypeFromURI( httpRequest.getRequestURI() );
            DAORenderer renderer = getRenderer( iType, pw ) ;
            renderer.printError(e);
            return ;
        }
        
		//--- Get the renderer for the URI type extension ( ".xml", ".html", ... )
		DAORenderer renderer = getRenderer( req.getType(), pw ) ;
		
		//--- Process the DAO request
        try
        {
            DAOResponse response = processRequest( req, httpRequest, pw ) ;
            renderer.printResponse(response) ;
        } catch (Throwable e) // Catch all exceptions
        {
            renderer.printError(e);
        }
	}
    
	private DAORenderer getRenderer( int iType, PrintWriter pw ) 
    {
		DAORenderer renderer = null ;
		switch ( iType )
		{
			case DAORequest.TYPE_HTML : 
			    renderer = new DAORendererHTML(pw);
				break;
			case DAORequest.TYPE_TEXT : 
			    renderer = new DAORendererText(pw);
				break;
			case DAORequest.TYPE_XML : 
			    renderer = new DAORendererXML(pw);
				break;
			default :
			    renderer = new DAORendererHTML(pw);
		}
		return renderer ;
    }
    
    private DAOResponse processRequest(DAORequest req, HttpServletRequest httpRequest, PrintWriter pw ) throws TelosysException
    {
        DAOResponse response = null ;
        
	    trace("processRequest : " + req );
		
	    //--- Get the specific DAO ( DAO dedicated to the given bean name and database )
		StandardDAO stdDAO = getDAO( req.getRecordBean(), req.getDatabaseId() );
		if ( stdDAO != null ) {
		
		    //--- Get a generic DAO adapter 
		    StandardDAOAdapter dao = new StandardDAOAdapter(stdDAO);
		    
			//--- Create the record bean
			Object bean = createBean ( dao );
			if ( bean == null )
			{
		        throw new RuntimeException("Cannot create bean for class " + dao.getEntityBeanClass().getName() );
			}

			if ( req.isListAction() ) // List action with SQL WHERE + SQL OPTIONAL END OF REQUEST ( "order by", .. )
			{
			    String sWhere = httpRequest.getParameter("where") ;
			    String sOpt   = httpRequest.getParameter("opt") ;
			    if ( sOpt == null )
			    {
				    sOpt = httpRequest.getParameter("option") ;
			    }
			    response = execDAOListAction(dao, sWhere, sOpt, req.getDatabaseId(), req.getAction(), pw);
			}
			else // Entity action
			{			    
				//--- Populate the record bean with the request attributes
				populateBean(bean, httpRequest);			    
				//--- Execute the DAO action
				response = execDAOAction(dao, bean, req.getDatabaseId(), req.getAction(), pw);
			}			
		}
		else
		{
	        throw new RuntimeException("DAO not found ( req = " + req + " )");
		}
		
		return response ;
	}
	
	/**
	 * Cut the string at the 3rd '/' character<br>
	 * Examples : <br>
	 *   "/appli/dao/0/Bean/load" -> "0/Bean/load"<br>
	 *   "/appli/dao/Bean/load"   -> "Bean/load"<br>
	 * @param s
	 * @return
	 */
	private String removeBeginning( String s ) {
	    int length = s.length();
	    int  c = 0 ;
	    for ( int i = 0 ; i < length ; i++ )
	    {
		    if ( s.charAt(i) == '/' ) c++ ;
	        if ( c == 3 )
	        {
	            return s.substring(i+1);
	        }
	    }
        throw new RuntimeException("Invalid beginning of URI '" + s + "' ");
	}
	
	/**
	 * Parse the given URI, and store the parameters in the given instance
	 * @param pURI
	 * @param servletParameters
	 */
	private DAORequest parseURI( String pURI ) {
	    
	    trace ("parseURI : URI = '" + pURI + "' ");
	    if ( pURI == null )
	    {
	        throw new RuntimeException("URI is null ");
	    }
	    
	    //String sDefaultDbId   = "" ;
	    String sDefaultDbId   = "" + ConnectionManager.getDefaultDatabase(); // v 1.0.2
	    String sDefaultAction = "load" ;
	    String sType = null ;
	        
	    String[] parts = null ;
	    String s = removeBeginning( pURI );
	    trace ("parseURI : URI without beginning = '" + s + "' ");
//	    String[] parts = pURI.split("/db/");
//		if ( parts.length > 1 ) 
//		{
//		    String s = parts[1];

		    //--- Get and remove trailing type ( ".xml", ".html", ... )
		    parts =  StrUtil.split(s,'.');
		    if ( parts.length > 1 )
		    {
		        sType = parts[parts.length-1]; // take last part as type
			    int i = s.lastIndexOf(".");
			    s = s.substring(0,i); // keep the left part ( before ".type" )
		    }
		    
		    //--- Parse the URI without the type extension
		    String sDbId   = sDefaultDbId ;
		    String sBean   = "xxxxx" ;
		    String sAction = sDefaultAction ;
		    trace ("parseURI : right part = '" + s + "' ");
		    // "0/Author/load?Id=1" 
		    // "Author/load?Id=1" 
			if ( s != null && ! s.equals("") ) 
			{
			    //--- Remove parameters if any
			    parts =  StrUtil.split(s,'?');
				if ( parts.length > 1 ) 
				{
				    s = parts[0];
				}
				
			    // "0/Author/load", "Author/load", "0/Author", "Author" 
			    parts =  StrUtil.split(s,'/');
			    if ( parts.length == 3 )
			    {
				    // "0/author/load" 
			        sDbId = parts[0] ;
			        sBean = parts[1] ;
			        sAction = parts[2] ;
			    }
			    else if ( parts.length == 2 )
			    {
				    // "Author/load" or "0/Author"
			        int i = StrUtil.getInt(parts[0], -1 );
			        if ( i != -1 ) {
			            sDbId   = parts[0];
				        sBean   = parts[1] ;
				        sAction = sDefaultAction ;
			        }
			        else {
				        sDbId   = sDefaultDbId ;
				        sBean   = parts[0] ;
				        sAction = parts[1] ;
			        }
			    }
			    else if ( parts.length == 1 )
			    {
				    // "Author" 
			        sDbId = sDefaultDbId ;
			        sBean = parts[0] ;
			        sAction = sDefaultAction ;
			    }
			    
			    //--- Database ID 
			    int iDbId = StrUtil.getInt(sDbId, -1);
			    if ( iDbId < 0 ) // v 1.0.2
			    {
			        int iMax = ConnectionManager.getNumberOfDatabases() - 1 ;
			        throw new RuntimeException("Invalid DB id '" + sDbId + "' ( 0 to " + iMax + " expected )");
			    }
			    
			    // Bean name is case sensitive
			    sAction = sAction.toLowerCase(); 
			    DAORequest req = new DAORequest(iDbId, sBean, sAction, sType );
			    return req ;
			}
			else
			{
		        throw new RuntimeException("Invalid URI '" + pURI + "' ");
			}
//		}
//		else
//		{
//	        throw new RuntimeException("Invalid URI '" + pURI + "' ");
//		}		
	}
	
	private int getTypeFromURI( String pURI ) {
	    	    
	    trace ("getReturnTypeFromURI : URI = '" + pURI + "' ");
	    if ( pURI != null )
	    {
		    int i = pURI.lastIndexOf(".");
		    if ( i >= 0 )
		    {
			    String s = pURI.substring(i+1); // keep the right part ( "xxxx.text"  --> "text" )
			    return DAORequest.getIntType(s);
		    }
		    else
		    {
		        return DAORequest.DEFAULT_TYPE ;
		    }
	    }
	    else
	    {
	        return DAORequest.DEFAULT_TYPE ;
	    }	    
	}
	
	private Class getRecordBeanClass(String beanName, int dbId) {
	    trace("getRecordBeanClass("+ beanName+","+dbId+")" );
		if ( beanName == null )
		{
		    throw new RuntimeException("Bean name is null");
		}
		beanName = beanName.trim();
		if ( beanName.length() == 0 )
		{
		    throw new RuntimeException("Bean name is void");
		}
		
	    Database db = ConnectionManager.getDatabase(dbId) ; 
	    if ( db == null )
	    {
		    throw new RuntimeException("No database for id = " + dbId );
	    }
	    
	    //--- Try to get the record class name pattern 
	    String s = db.getRecordClassPattern();
	    if ( s == null )
	    {
		    throw new RuntimeException("No class pattern for database " + dbId );
	    }
	    trace("getRecordBeanClass("+ beanName+","+dbId+") : Class pattern = " + s );
	    
	    //--- Build class name
	    //String className = StrUtil.replaceVar(s, "${SHORTNAME}", beanName);
	    String className = Tool.buildClassName(s, beanName);
	    trace("getRecordBeanClass("+ beanName+","+dbId+") :className = " + className );
	    
	    //--- Try to load the class
		Class beanClass = null ;
        try
        {
            beanClass = Class.forName( className );
        } catch (ClassNotFoundException e1)
        {
		    throw new RuntimeException("Cannot load class '" + className + "' (ClassNotFoundException)" );
        }
        return beanClass ;
	}

	/**
	 * Returns the DAO for the given bean name in the given database
	 * @param beanName
	 * @param dbId
	 * @return
	 */
	private StandardDAO getDAO(String beanName, int dbId) {
	    //trace("getDAO("+ beanName+","+dbId+")" );
	    Class beanClass = getRecordBeanClass(beanName, dbId);
		if ( beanClass != null )
		{
		    trace("getDAO("+ beanName+","+dbId+") : bean class = " + beanClass.getName() );
		    DAOProvider p = TelosysDAL.getDAOProvider();
		    // Get DAO from bean class
		    StandardDAO dao = p.getDAO(beanClass, dbId);
		    return dao ;
		}
		else
		{
		    throw new RuntimeException("No class for bean '" + beanName + "' in database " + dbId );
		}
	}

	/**
	 * @param dao
	 * @return
	 */
	private Object createBean ( StandardDAOAdapter dao ) {
	    Object bean = null ;
		Class beanClass = dao.getEntityBeanClass();
		if ( beanClass != null )
		{
			try
            {
                bean = beanClass.newInstance();
            } catch (InstantiationException e)
            {
                throw new RuntimeException("Cannot create bean", e);
            } catch (IllegalAccessException e)
            {
                throw new RuntimeException("Cannot create bean", e);
            }
		}
		else
		{
            throw new RuntimeException("No bean class for DAO " + dao.getDAOClass().getName() );
		}
		return bean ;
	}
	
	private DAOResponse execDAOAction(StandardDAOAdapter dao, Object bean, int id, String sAction, PrintWriter pw) throws TelosysException
	{
	    Class daoClass = dao.getDAOClass();
		trace("execDAOAction : db = " + id + ", action = '" + sAction + "', bean = " + bean  );
		
		//--- Call the DAO method according with the given action (supposed to be in lower case )
		if ( sAction.equals("exists") ) {
		    boolean b = dao.exists(bean, id ); 
		    return new DAOResponse(daoClass, "exists", b);
		}
		if ( sAction.equals("load") ) {
		    int i = dao.load(bean, id );
		    boolean bFound = ( i > 0 ) ;
		    return new DAOResponse(daoClass, "load", i, bean, bFound);
		}
		if ( sAction.equals("insert") ) {
		    int i = dao.insert(bean, id );
		    return new DAOResponse(daoClass, "insert", i);
		}
		if ( sAction.equals("insertkeygen") ) {
		    Long l = dao.insertKeyGen(bean, id );
		    return new DAOResponse(daoClass, "insertkeygen", l);
		}
		if ( sAction.equals("delete") ) {
		    int i = dao.delete(bean, id );
		    return new DAOResponse(daoClass, "delete", i);
		}
		if ( sAction.equals("update") ) {
		    int i = dao.update(bean, id );
		    return new DAOResponse(daoClass, "update", i);
		}
		if ( sAction.equals("save") ) {
		    int i = dao.save(bean, id );
		    return new DAOResponse(daoClass, "save", i);
		}
	    throw new RuntimeException("Invalid action '" + sAction + "'");
	}

	private DAOResponse execDAOListAction(StandardDAOAdapter dao, String sWhere, String sOpt, int id, String sAction, PrintWriter pw) throws TelosysException
	{
	    Class daoClass = dao.getDAOClass();
		trace("execDAOListAction : db = " + id + ", action = '" + sAction + "', where = " + sWhere  );

		//--- Create the query
	    ListQuery qry = null ;
	    if ( sWhere != null )
	    {
	        if ( StrUtil.nullOrVoid( sOpt ) )
	        {
			    qry = dao.createQuery(sWhere) ;
	        }
	        else
	        {
			    qry = dao.createQuery(sWhere, sOpt) ;
	        }
	    }
	    else
	    {
	        if ( StrUtil.nullOrVoid( sOpt ) )
	        {
			    qry = dao.createQueryAll() ;
	        }
	        else
	        {
			    qry = dao.createQueryAll(sOpt) ;
	        }
	    }

		//--- Create the special "Loosely Typed VOList"
	    GenericVOList list = null ;
	    Class beanClass = dao.getEntityBeanClass();
	    if ( beanClass != null )
	    {
			list = new LooselyTypedVOList( beanClass );
	    }
	    else
	    {
	        throw new RuntimeException("Cannot get entity class from DAO " + daoClass );
	    }
		    
		//--- Call the DAO method according with the given action (supposed to be in lower case )
		if ( sAction.equals("loadlist") ) {
		    int i = dao.loadList(qry, list, id);
		    return new DAOResponse(daoClass, "loadlist", i, list );
		}
	    throw new RuntimeException("Invalid action '" + sAction + "'");
	}
	
//	private void populateBean (Object vo, DALServletParam servletParameters) {
//		// Chargement des donnes dans le VO
//	    String sName = null ;
//	    String sValue = null ;
//		for (int i = 0; i < servletParameters.beanDataAttributes.length; i++) {
//		    sName  = servletParameters.beanDataAttributes[i] ;
//		    sValue = (String)servletParameters.htBeanContents.get( servletParameters.beanDataAttributes[i] );
//			try {
//				Util.setFieldValue(vo, sName, sValue );
//			} catch (Exception e) {
//				error("Echec chargement des donnes dans le VO");
//				e.printStackTrace();
//			}
//		}
//	}
	
	private void populateBean (Object vo, HttpServletRequest request) {
	    String sName = null ;
	    String sValue = null ; 
	    
	    Enumeration e = request.getParameterNames();
	    while ( e.hasMoreElements() )
	    {
	        sName = (String) e.nextElement();
	        sValue = request.getParameter(sName);
			try {
			    Util.setFieldValue(vo, sName, sValue );
			} catch (Exception ex) {
				error("Echec chargement des donnes dans le VO");
				ex.printStackTrace();
			}
	    }
	}

}