package org.objectweb.telosys.dal.sql;

import java.io.InputStream;
import java.sql.Connection;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.dao.DummyDatabaseSession;
import org.objectweb.telosys.dal.dao.StandardDatabaseSession;
import org.objectweb.telosys.util.StrUtil;
import org.objectweb.telosys.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Unique (static) access point to get databases connections or databases sessions <br>
 * This class manages 1 to N databases connection providers
 * 
 * 
 * @author Laurent GUERIN
 *  
 */
public final class ConnectionManager
{
    private static final String             DAOREGISTRY       = "daoregistry" ;
    
    private static final String             DAOCLASS          = "daoclass" ;
    
    private static final String             RECORDCLASS       = "recordclass" ;
    
    private static final TelosysClassLogger log               = new TelosysClassLogger(ConnectionManager.class);

    private static String                   $sConfigFileName  = null;

    private static ConnectionProvider[]     $conProviders     = null;

    private static Database[]               $databases        = null;

    private static int                      $iDefaultDatabase = 0;

    /**
     * <code>$dummyDbSession</code> : the single instance of "Dummy" DatabaseSession
     */
    private final static DummyDatabaseSession $dummyDbSession = new DummyDatabaseSession(); // v 1.0.2

    //---------------------------------------------------------------------------
    /**
     * Try to get an element attribute as is or in lower case if not found as is
     * @param element
     * @param attributeName
     * @return
     * @since v 1.0.1
     */
    private static String getAttribute(Element element, String attributeName )
    {
        log.trace("getAttribute(..,'" + attributeName + "') ... ");
        if ( element == null || attributeName == null ) 
        {
            log.error("getAttribute(.,.) null parameter");
            return null ;
        }
        String name = attributeName ;
        log.trace("getAttribute(..,'" + name + "') ... ");
        //--- Try to find it as is ( eg : "defaultId" )
        String s = element.getAttribute(name); // NB : return '' if not found 
        if ( notVoid(s) )
        {
            log.trace("getAttribute(..,'" + name + "') --> '" + s + "'");
            return s ; // Found as is
        }
        else
        {
            //--- Try to find it in lower case ( eg : "defaultid" )
            name = name.toLowerCase();
            s = element.getAttribute(name);
            log.trace("getAttribute(..,'" + name + "') --> '" + s + "'");
            return s ;
        }
    }
    //---------------------------------------------------------------------------
    private static Properties getProperties(Element elem)
    {
        NodeList nodes = elem.getElementsByTagName("property");
        Properties prop = null;
        if (nodes.getLength() > 0)
        {
            prop = new Properties();
            for ( int iProp = 0 ; iProp < nodes.getLength() ; iProp++ )
            {
                Element eltProperty = (Element) nodes.item(iProp);
                String sPropName = eltProperty.getAttribute("name");
                String sPropValue = eltProperty.getAttribute("value");
                prop.put(sPropName, sPropValue);
                log.trace("     " + iProp + " -> property : " + sPropName + "=" + sPropValue);
            }
        }
        return prop;
    }

    //---------------------------------------------------------------------------
    private static boolean isValidDataSource(String sDataSource)
    {
        if (sDataSource != null)
        {
            if (sDataSource.trim().length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //---------------------------------------------------------------------------
    /**
     * Creates a Connection Provider according with the "db" configuration tag parameters
     * 
     * @param sDbId
     * @param sDbName
     * @param sDataSource
     * @param sDbDriver
     * @param sDbUrl
     * @param props
     * @param sDbIsolationLevel
     * @param sDbPoolSize
     * @return the ConnectionProvider created
     */
    private static ConnectionProvider createConnectionProvider(String sDbId, String sDbName, String sDataSource, String sDbDriver, String sDbUrl, Properties props, String sDbIsolationLevel, String sDbPoolSize)
    {
        log.trace("createConnectionProvider : id=" + sDbId + " name=" + sDbName + " datasource=" + sDataSource + " driver=" + sDbDriver + " url=" + sDbUrl + " poolSize=" + sDbPoolSize + " isolation level=" + sDbIsolationLevel);

        ConnectionProvider cp = null;
        if (isValidDataSource(sDataSource))
        {
            //--- Connection Provider = standard JDBC DataSource
            log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : DataSource '" + sDataSource + "' ...");
            try
            {
                cp = new SqlDataSource(sDbName, sDataSource);
            } catch (TelosysException e)
            {
                log.error("Cannot create SqlDataSource : TelosysException : " + e.getMessage());
                return null;
            }
        }
        else
        {
            log.trace("DB " + sDbId + " (" + sDbName + ") driver    = " + sDbDriver);
            log.trace("DB " + sDbId + " (" + sDbName + ") url       = " + sDbUrl);

            int iDbPoolSize = StrUtil.getInt(sDbPoolSize, 0);
            log.trace("DB " + sDbId + " (" + sDbName + ") pool size = " + sDbPoolSize + "(" + iDbPoolSize + ")");

            int iDbIsolationLevel = SqlTool.getIsolationLevel(sDbIsolationLevel, Connection.TRANSACTION_READ_COMMITTED);
            log.trace("DB " + sDbId + " (" + sDbName + ") isolation level = " + sDbIsolationLevel + "(" + iDbIsolationLevel + ")");

            if (iDbPoolSize > 1)
            {
                //--- Connection Pool
                log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : Telosys Connections Pool (size=" + iDbPoolSize + ") ...");
                cp = new SqlConnectionPool(sDbName, sDbDriver, sDbUrl, props, iDbIsolationLevel, iDbPoolSize);
            }
            else
            {
                //--- No pool ( simple Connection Factory )
                log.info("DB " + sDbId + " (" + sDbName + ") : Create ConnectionProvider : Telosys simple Connection Factory ...");
                cp = new SqlConnectionFactory(sDbName, sDbDriver, sDbUrl, props, iDbIsolationLevel);
            }
        }
        return cp;
    }
    
    //---------------------------------------------------------------------------
    /**
     * Creates a Connection Provider based on a specific custom class, according with the "db" configuration tag parameters
     * 
     * @param sDbId
     * @param sDbName
     * @param sCustomClass
     * @return
     * @since v 1.0.1
     */
    private static ConnectionProvider createCustomClassConnectionProvider(String sDbId, String sDbName, String sCustomClass)
    {
        log.trace("createCustomClassConnectionProvider : id=" + sDbId + " name=" + sDbName + " customclass=" + sCustomClass );

        //--- Try to load the custom class 
        Class clazz = null ;
        try
        {
            clazz = Class.forName(sCustomClass);
        } catch (ClassNotFoundException e)
        {
            log.error("Cannot load ConnectionProvider custom class : '"+sCustomClass+"' (ClassNotFoundException)" ); 
            return null ;
        }
        
        //--- Try to create an instance from the custom class 
        Object obj = null ;
        try
        {
            obj = clazz.newInstance();
        } catch (InstantiationException e1)
        {
            log.error("Cannot create instance from class : '"+sCustomClass+"' (InstantiationException)" ); 
            return null ;
        } catch (IllegalAccessException e1)
        {
            log.error("Cannot create instance from class : '"+sCustomClass+"' (IllegalAccessException)" ); 
            return null ;
        }
        
        //--- Check interface implementation
        if ( obj instanceof ConnectionProvider )
        {
            // OK : return it
            return (ConnectionProvider) obj; 
        }
        else
        {
            log.error("Custom class '"+sCustomClass+"' does not implement ConnectionProvider interface");
            return null ;
        }
    }
    
    //---------------------------------------------------------------------------------------------
    /**
     * Connection manager initialization <br>
     * Load the 'dbcongif' XML file and create a connection provider for each database definition
     * 
     * @param sFileName :
     *            dbconfig file name
     * @return
     */
    public static boolean init(String sFileName)
    {
        log.info("init(" + sFileName + ")");        
        if (sFileName == null)
        {
            log.error("init(" + sFileName + ") file name parameter is null" );
            return false ;
        }
        //--- Keep the dbconfig file name
        $sConfigFileName = sFileName;
        
        //Document document = XmlUtil.loadFile(sFileName);
        Document document = null;
        try
        {
            document = XmlUtil.parse(sFileName);
        } catch (TelosysException e)
        {
            log.error("init(" + sFileName + ") cannot parse. TelosysException : " + e.getMessage() );
            return false ;
        }
        
        return init( document );
    }
    
    //---------------------------------------------------------------------------------------------
    public static boolean init(InputStream is)
    {
        log.info("init(InputStream)");        
        if (is == null)
        {
            log.error("init(InputStream) InputStream parameter is null" );
            return false ;
        }
        //--- Keep the dbconfig file name
        $sConfigFileName = null ;
        
        //Document document = XmlUtil.loadFile(sFileName);
        Document document = null;
        try
        {
            document = XmlUtil.parse(is);
        } catch (TelosysException e)
        {
            log.error("init(InputStream) cannot parse. TelosysException : " + e.getMessage() );
            return false ;
        }
        
        return init( document );
    }
    
    //---------------------------------------------------------------------------------------------
    private static boolean notVoid( String s  ) // v 1.0.0
    {
        if ( s != null )
        {
            if ( s.trim().length() > 0 )
            {
                return true ;
            }
        }
        return false ;
    }
    
    //---------------------------------------------------------------------------------------------
    /**
     * Try to get a class name pattern from "dbconfig.xml" and "telosys.properties"
     * @param sParamName
     * @param root
     * @param db
     * @param dbId
     * @return
     */
    private static String getClassPattern( String sParamName, Element root, Element db, String dbId  ) // v 1.0.0
    {
        //--- 1) Try to find a specific pattern from database element in "dbconfig.xml"
        String s = db.getAttribute(sParamName);
        if ( notVoid(s) ) 
        {
            return s.trim() ; // Return it "as is"
        }

        //--- 2) Try to find a generic pattern from root element in "dbconfig.xml" 
        s = root.getAttribute(sParamName);
        if ( notVoid(s) && dbId != null )
        {
            return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
        }

        //--- 3) Try to find a generic pattern in "telosys.properties" 
        Properties prop = Telosys.getProperties();
        if ( prop != null ) // Can be null if Telosys DAL is initialized in standalone mode (without Telosys properties)
        {
            s = prop.getProperty(sParamName);
            if ( notVoid(s) && dbId != null )
            {
                return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
            }
        }
        
        //--- 4) Not found => no pattern for this database
        return null ;
    }
    //---------------------------------------------------------------------------------------------
    /**
     * Try to get a DAO registry class name, from the more specific configuration to the more general
     * @param root
     * @param db
     * @param dbId
     * @return
     */
    private static String getDAORegistryClass( Element root, Element db, String dbId  ) // v 1.0.0
    {
        return getClassPattern( DAOREGISTRY, root, db, dbId );
        
//        //--- 1) Try to find a DAO registry specific to the DataBase if any ( from "dbconfig.xml" )
//        String s = db.getAttribute(DAOREGISTRY);
//        if ( notVoid(s) ) 
//        {
//            return s.trim() ; // Return it "as is"
//        }
//
//        //--- 2) Try to find a generic DAO registry ( from "dbconfig.xml" )
//        s = root.getAttribute(DAOREGISTRY);
//        if ( notVoid(s) && dbId != null )
//        {
//            return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
//        }
//
//        //--- 3) Try to find a generic DAO registry from "telosys.properties" 
//        Properties prop = Telosys.getProperties();
//        if ( prop != null ) // Can be null if Telosys DAL is initialized in standalone mode (without Telosys properties)
//        {
//            s = prop.getProperty(DAOREGISTRY);
//            if ( notVoid(s) && dbId != null )
//            {
//                return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
//            }
//        }
//        
//        //--- 4) Not found => no DAO registry for this database
//        return null ;
    }
    
    //---------------------------------------------------------------------------------------------
    /**
     * Try to get a DAO class name pattern, from the more specific configuration to the more general
     * @param root
     * @param db
     * @param dbId
     * @return
     */
    private static String getDAOClassPattern( Element root, Element db, String dbId  ) // v 1.0.0
    {
        return getClassPattern( DAOCLASS, root, db, dbId );
        
//        //--- 1) Try to find a DAO class name pattern for the DataBase if any ( from "dbconfig.xml" )
//        String s = db.getAttribute(DAOCLASS);
//        if ( notVoid(s) ) 
//        {
//            return s.trim() ; // Return it "as is"
//        }
//        //--- 2) Try to find a generic DAO class name pattern ( from "dbconfig.xml" )
//        s = root.getAttribute(DAOCLASS);
//        if ( notVoid(s) && dbId != null )
//        {
//            return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
//        }
//        //--- 3) Try to find a generic DAO class name pattern from "telosys.properties" 
//        Properties prop = Telosys.getProperties();
//        if ( prop != null ) // Can be null if Telosys DAL is initialized in standalone mode (without Telosys properties)
//        {
//            s = prop.getProperty(DAOCLASS);
//            if ( notVoid(s) && dbId != null )
//            {
//                return StrUtil.replaceVar(s.trim(), "${DBID}", dbId.trim() );
//            }
//        }
//        //--- 4) Not found => no DAO registry for this database
//        return null ;
    }
    
    private static String getRecordClassPattern( Element root, Element db, String dbId  ) // v 1.0.0
    {
        return getClassPattern( RECORDCLASS, root, db, dbId );
    }
    
    //---------------------------------------------------------------------------------------------
    private static boolean init( Document document )
    {
        log.trace("init(Document)");        
        if (document == null)
        {
            return false ;
        }
        
        Element root = document.getDocumentElement();

        //--- Number of Databases = Maximum Database ID + 1 ( 10 if not set, cannot exceed 100)
        //String sMaxId = root.getAttribute("maxId");
        String sMaxId = getAttribute(root, "maxId"); // v 1.0.1
        int iMaxId = StrUtil.getInt(sMaxId, 10);
        if (iMaxId > 100)
        {
            log.error("maxId = " + iMaxId + " : maxId > 100 => reduce to 100 !");
            iMaxId = 100;
        }
        int iNbDatabases = iMaxId + 1;
        log.trace("Maximum database Id = '" + sMaxId + "' ( int : " + iMaxId + ")");
        log.trace("Number of database  = " + iNbDatabases + " ( maxId + 1 ) ");

        //--- Default Database ID ( 0 if not set )
        //String sDefaultId = root.getAttribute("defaultId");
        String sDefaultId = getAttribute(root, "defaultId"); // v 1.0.1
        $iDefaultDatabase = StrUtil.getInt(sDefaultId, 0);
        if ($iDefaultDatabase < 0 || $iDefaultDatabase >= iNbDatabases)
        {
            log.error("Invalid default database (" + $iDefaultDatabase + ") : set to 0 ");
            $iDefaultDatabase = 0;
        }
        log.trace("Default database Id = '" + $iDefaultDatabase + "' ( int : " + $iDefaultDatabase + ")");

        //--- Array of 'N' databases
        $databases = new Database[iNbDatabases];
        
        //--- Array of 'N' providers
        $conProviders = new ConnectionProvider[iNbDatabases];
        log.info("Ready to store " + iNbDatabases + " connection providers ( ID from 0 to " + iMaxId + ", default = " + $iDefaultDatabase +" )");

        //--- Databases list
        NodeList nodes = document.getElementsByTagName("db");
        int nbDbNodes = nodes.getLength();

        //--- For each database defined in the XML file ...
        for ( int i = 0 ; i < nbDbNodes ; i++ )
        {
            //--- Tag "bd" ( get the attributes )
            Element eltBase = (Element) nodes.item(i);
            if (eltBase == null)
            {
                log.error("XML 'db' node is null ( Node " + i + " ) !");
            }
            else
            {
                String sDbId = eltBase.getAttribute("id");
                int iDbId = StrUtil.getInt(sDbId, -1);
                String sDbName = eltBase.getAttribute("name");

                //--- Database managed by a DataSource
                //String sDataSource = eltBase.getAttribute("datasource");
                String sDataSource = getAttribute(eltBase, "dataSource"); // v 1.0.1

                //--- Database managed by a specific custom Class
                String sCustomClass = getAttribute(eltBase, "customClass"); // v 1.0.1
                
                //--- Database managed by Telosys-DAL
                String sDbDriver = eltBase.getAttribute("driver");
                String sDbUrl = eltBase.getAttribute("url");
                //String sDbIsolationLevel = eltBase.getAttribute("isolationLevel");
                String sDbIsolationLevel = getAttribute(eltBase, "isolationLevel"); // v 1.0.1
                //int iDbIsolationLevel = SqlTool.getIsolationLevel(sDbIsolationLevel, Connection.TRANSACTION_READ_COMMITTED);
                //String sDbPoolSize = eltBase.getAttribute("poolSize");
                String sDbPoolSize = getAttribute(eltBase, "poolSize"); // v 1.0.1
                //int iDbPoolSize = StrUtil.getInt(sDbPoolSize, 0);

                //--- Tags "property"
                Properties props = getProperties(eltBase);

                //--- Try to get a DAO Registry class name for this database
                String sDAORegistryClass = getDAORegistryClass( root, eltBase, sDbId  ); // v 1.0.0
                //--- Try to get a DAO class name pattern for this database
                String sDAOClassPattern = getDAOClassPattern( root, eltBase, sDbId  ); // v 1.0.0                
                //--- Try to get a record class name pattern for this database
                String sRecordClassPattern = getRecordClassPattern( root, eltBase, sDbId  ); // v 1.0.0                
                
                //--- Connection Provider creation :
                // v 1.0.1 // ConnectionProvider cp = createConnectionProvider(sDbId, sDbName, sDataSource, sDbDriver, sDbUrl, props, sDbIsolationLevel, sDbPoolSize);
                ConnectionProvider cp = null ;
                if ( notVoid(sCustomClass) ) // v 1.0.1
                {
                    cp = createCustomClassConnectionProvider(sDbId, sDbName, sCustomClass); // v 1.0.1
                }
                else
                {
                    cp = createConnectionProvider(sDbId, sDbName, sDataSource, sDbDriver, sDbUrl, props, sDbIsolationLevel, sDbPoolSize);
                }
                if ( cp != null )
                {
                    log.info("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider created.");                        
                }
                
                //--- Store provider in the array
                if (iDbId >= 0 && iDbId < $conProviders.length)
                {
                    if ($conProviders[iDbId] == null)
                    {
                        if (cp != null)
                        {
                            //--- Register the database connection provider
                            $conProviders[iDbId] = cp;
                            log.info("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider ready.");
                            //--- Register the database definition
                            //$databases[iDbId] = new Database(iDbId, sDbName, sDbDriver, sDbUrl, sDbIsolationLevel, sDbPoolSize);                            
                            $databases[iDbId] = new Database(iDbId, sDbName, sDbDriver, 
                                    sDbUrl, sDbIsolationLevel, sDbPoolSize, 
                                    sDAORegistryClass,    // v 1.0.0 
                                    sDAOClassPattern,     // v 1.0.0
                                    sRecordClassPattern); // v 1.0.0                        
                        }
                        else
                        {
                            log.error("DB " + sDbId + " (" + sDbName + ") : ConnectionProvider is NULL !");
                        }
                    }
                    else
                    {
                        log.error("Database ID " + iDbId + " is already used : cannot register this database ! ");
                    }
                }
                else
                {
                    log.error("Invalid Database ID ( " + iDbId + " ) : cannot register this database ! ");
                }
            }
        }
        return true ;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a real database session for the default database id.
     * @return
     * @throws TelosysException
     */
    public static DatabaseSession getSession() throws TelosysException
    {
        return getSession($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a real database session for the given database id.
     * @param iDatabaseId
     * @return
     * @throws TelosysException
     */
    public static DatabaseSession getSession(int iDatabaseId) throws TelosysException
    {
        Connection con = getConnection(iDatabaseId);
        return new StandardDatabaseSession(iDatabaseId, con);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a "dummy" database session.
     * @return
     * @since v 1.0.2
     */
    public static DatabaseSession getDummySession() 
    {
        return $dummyDbSession ;
    }

    //---------------------------------------------------------------------------------------------
//    /**
//     * Returns the configuration file name used to initialized the Connection Manager
//     * 
//     * @return the file name (or null)
//     */
//    public static String getConfigFileName()
//    {
//        return $sConfigFileName;
//    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the default database id
     * 
     * @return
     */
    public static int getDefaultDatabase()
    {
        return $iDefaultDatabase;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the number of databases entries supported by the connection manager <br>
     * including void entries
     * 
     * @return
     */
    public static int getNumberOfDatabases()
    {
        if ($conProviders != null)
        {
            return $conProviders.length;
        }
        return 0;
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a connection for the default database
     * 
     * @return
     * @throws TelosysException
     */
    public static Connection getConnection() throws TelosysException
    {
        return getConnection($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns a connection for the given database id
     * 
     * @param iBaseId
     * @return
     * @throws TelosysException
     */
    public static Connection getConnection(int iBaseId) throws TelosysException
    {
        //SqlConnectionProvider cp = getConnectionProvider(iBaseId);
        ConnectionProvider cp = getConnectionProvider(iBaseId);
        if (cp != null)
        {
            try
            {
                return cp.getConnection();
            } catch (Exception ex)
            {
                throw new TelosysException("getConnection(" + iBaseId + ") : Exception ! " + ex);
            }
        }
        else
        {
            throw new TelosysException("getConnection(" + iBaseId + ") : No ConnectionProvider for this DB id !");
        }
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Returns the default database connection provider
     * 
     * @return
     */
    protected static ConnectionProvider getConnectionProvider()
    {
        return getConnectionProvider($iDefaultDatabase);
    }

    //---------------------------------------------------------------------------------------------
    /**
     * Return the connection provider for the given database id
     * 
     * @param iBaseId
     * @return the connection provider (or null if no provider for the given DB id)
     */
    protected static ConnectionProvider getConnectionProvider(int iBaseId)
    {
        if ($conProviders != null)
        {
            if (iBaseId >= 0 && iBaseId <= $conProviders.length)
            {
                if ($conProviders[iBaseId] != null)
                {
                    return $conProviders[iBaseId];
                }
                else
                {
                    log.error("getConnectionProvider(" + iBaseId + ") : no connection provider for this database ID ! ");
                    return null;
                }
            }
            else
            {
                log.error("getConnectionProvider(" + iBaseId + ") : invalid database ID ! ");
                return null;
            }
        }
        else
        {
            log.error("getConnectionProvider(" + iBaseId + ") : connection providers not initialized ! ");
            return null;
        }
    }
    //---------------------------------------------------------------------------------------------
    /**
     * Returns the list of all the databases managed by the current application  
     * @return list of databases descriptions 
     */
    public static List getDatabases()
    {
        LinkedList list = new LinkedList();
        if ( $databases != null )
        {
            Database db = null ;
            int len = $databases.length ;
            for ( int i = 0 ; i < len ; i++ )
            {
                db = $databases[i];
                if ( db != null )
                {
                    list.add(db);
                }
            }
        }
        return list ;
    }
    //---------------------------------------------------------------------------------------------
    /**
     * Returns the database description for the given database id
     * @param id the database id ( 0 to N )
     * @return the database description or null 
     * @since 1.0.0
     */
    public static Database getDatabase(int id)
    {
        if ( id >= 0 && id < $databases.length )
        {
            return $databases[id];
        }
        return null ;
    }
    
    //---------------------------------------------------------------------------------------------
    /**
     * Returns the pool size for the given database id <br>
     * or 0 if the database exists but doesn't use a Telosys pool ( ConnectionFactory or DataSource )<br>
     * or -1 if there's no database for the given id
     * @param id the database id ( 0 to N )
     * @return the size of the pool ( or 0 or -1  ) 
     * @since 1.0.0
     */
    public static int getPoolSize(int id)
    {
        ConnectionProvider cp = getConnectionProvider(id);
        if ( cp != null )
        {
            if ( cp instanceof SqlConnectionPool )
            {
                return ((SqlConnectionPool)cp).getCurrentSize();
            }
            else
            {
                return 0 ;
            }
//            else if ( cp instanceof SqlConnectionFactory )
//            {
//                return 0 ;
//            }
//            else if ( cp instanceof SqlDataSource )
//            {
//                return 0 ;
//            }            
        }
        return -1 ;
    }
    
    /**
     * Returns the ConnectionProvider's class for the given database id. 
     * @param id the database id ( 0 to N )
     * @return the class or null if there's no database for the given id
     * @since 1.0.0
     */
    public static Class getProviderClass(int id)
    {
        ConnectionProvider cp = getConnectionProvider(id);
        if ( cp != null )
        {
            return cp.getClass();
        }
        return null ;
    }

    /**
     * Returns the type of ConnectionProvider <br>
     * one of the ConnectionProvider types : FACTORY, POOL, DATASOURCE <br>
     * or NO_PROVIDER if there's no database for the given id 
     * @return
     * @since 1.0.0
     */
    public static int getProviderType(int id)
    {
        ConnectionProvider cp = getConnectionProvider(id);
        if ( cp != null )
        {
            return cp.getType();
        }
        return ConnectionProvider.NO_PROVIDER ;
    }

}