package org.objectweb.telosys.dal.sql;

import java.io.Serializable;

import org.objectweb.telosys.common.TelosysObject;

/**
 * A Criteria object is a collection of Criterion objects to be used together
 * to build dynamic SQL Queries
 * 
 * @author Laurent GUERIN
 *  
 */
public class Criteria extends TelosysObject implements Serializable
{
    private Criterion[]   _criteria  = null; // index : from 1 to N

    private int           _iCount    = 0;

    private String        _sOperator = "and";

    /**
     * Build a new criteria ( collection of criterion objects )
     * 
     * @param critArray
     *            array of criterion objects to be used in conjunction for dynamic request
     * @param sOperator
     *            where operator to use between each criterion ( "and" / "or" )
     */
    public Criteria(Criterion[] critArray, String sOperator)
    {
        // Duplicate the given array in a N+1 array
        if (critArray != null)
        {
            _criteria = new Criterion[critArray.length + 1];
            _criteria[0] = null;
            for ( int i = 0 ; i < critArray.length ; i++ )
            {
                _criteria[i + 1] = critArray[i];
            }
            _iCount = critArray.length;
        }
        else
        {
            _criteria = new Criterion[1];
            _criteria[0] = null;
            _iCount = 0;
        }
        _sOperator = sOperator;
    }

    public String getOperator()
    {
        return _sOperator;
    }

    public int getCount()
    {
        return _iCount;
    }

    /**
     * Return the SQL criterion expression ( ie "thecolumn = ?" )
     * 
     * @param i
     *            criterion index ( from 1 to N )
     * @return
     */
    public String getSqlExpression(int i)
    {
        if (_criteria != null)
        {
            if (i > 0 && i < _criteria.length)
            {
                return _criteria[i].getSqlExpression();
            }
        }
        return null;
    }

    /**
     * Return the parameter type expected for the given criterion index
     * 
     * @param i
     *            criterion index ( from 1 to N )
     * @return
     */
    public int getParamType(int i)
    {
        if (_criteria != null)
        {
            if (i > 0 && i < _criteria.length)
            {
                return _criteria[i].getParamType();
            }
        }
        return ParamType.UNKNOWN ;
    }

    /**
     * Fix the given criterion as "useful" and set its parameter value
     * 
     * @param i
     *            criterion index ( from 1 to N )
     * @param sParamVal
     *            criterion parameter value
     */
    public void useWithValue(int i, String sParamVal)
    {
        if (_criteria != null)
        {
            if (i > 0 && i < _criteria.length)
            {
                _criteria[i].useWithValue(sParamVal);
            }
        }
    }

    /**
     * Fix the given criterion as "not used"
     * 
     * @param i
     *            criterion index ( from 1 to N )
     */
    public void doNotUse(int i)
    {
        if (_criteria != null)
        {
            if (i > 0 && i < _criteria.length)
            {
                _criteria[i].doNotUse();
            }
        }
    }

    /**
     * Reset each criterion as "not used" 
     */
    public void reset()
    {
        if (_criteria != null)
        {
            for ( int i = 1 ; i < _criteria.length ; i++)
            {
                Criterion criterion = _criteria[i];
                if (criterion != null)
                {
                    criterion.doNotUse();
                }
            }
        }
    }

    /**
     * Build the SQL where clause dynamicaly with the useful criterion objects
     * 
     * @return the SQL where clause built
     */
    public String getDynamicWhere()
    {
        int iAddCount = 0;
        StringBuffer sb = new StringBuffer(100);

        for ( int i = 1 ; i < _criteria.length ; i++ )
        {
            Criterion crit = _criteria[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (iAddCount > 0)
                    {
                        sb.append(" " + _sOperator + " ");
                    }
                    sb.append(crit.getSqlExpression());
                    iAddCount++;
                }
            }
        }

        String sRet = "";
        if (iAddCount > 0)
        {
            sRet = sb.toString();
        }
        return sRet;
    }

    /**
     * Return the number of useful parameters
     * 
     * @return
     */
    public int getUsefulParametersCount()
    {
        int iSize = 0;
        for ( int i = 1 ; i < _criteria.length ; i++ )
        {
            Criterion crit = _criteria[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    iSize++;
                }
            }
        }
        return iSize;
    }

    /**
     * Build a int array containing the useful parameters types
     * 
     * @return
     */
    public int[] getDynamicTypes()
    {
        //--- Determine the array size
        int iSize = getUsefulParametersCount();
        //--- Build the String array
        int[] types = new int[iSize];
        int x = 0;
        for ( int i = 1 ; i < _criteria.length ; i++ )
        {
            Criterion crit = _criteria[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (x < types.length)
                    {
                        types[x++] = crit.getParamType();
                    }
                }
            }
        }
        return types;
    }

    /**
     * Build a String array containing the useful parameters values
     * 
     * @return
     */
    public String[] getDynamicParameters()
    {
        //--- Determine the array size
        int iSize = getUsefulParametersCount();
        //--- Build the String array
        String[] strParams = new String[iSize];
        int x = 0;
        for ( int i = 1 ; i < _criteria.length ; i++ )
        {
            Criterion crit = _criteria[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (x < strParams.length)
                    {
                        strParams[x++] = crit.getParamValueAsString();
                    }
                }
            }
        }
        return strParams;
    }

    /**
     * Build an array containing the useful Criterion objects
     * @return
     */
    public Criterion[] getUsefulCriteria()
    {
        //--- Determine the array size
        int iSize = getUsefulParametersCount();
        //--- Build the String array
        Criterion[] criterias = new Criterion[iSize];
        int x = 0;
        for ( int i = 1 ; i < _criteria.length ; i++ )
        {
            Criterion crit = _criteria[i];
            if (crit != null)
            {
                if (crit.isUsed())
                {
                    if (x < criterias.length)
                    {
                        criterias[x++] = crit;
                    }
                }
            }
        }
        return criterias;
    }
}