package org.objectweb.telosys.dal.sql;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public final class SqlTool
{
    //-----------------------------------------------------------------------------------------
    /**
     * Returns a description about the given connection ( autocomit + isolation level )
     * @param con 
     * @return the description string
     */
    public static String connectionStatus(Connection con)
    {
        String s1 = null;
        String s2 = null;
        String sStatus = null;
        if (con != null)
        {
            try
            {
                if (con.getAutoCommit())
                {
                    s1 = "true";
                }
                else
                {
                    s1 = "false";
                }

                switch (con.getTransactionIsolation())
                {
                case Connection.TRANSACTION_NONE :
                    s2 = "TRANSACTION_NONE";
                    break;
                case Connection.TRANSACTION_READ_COMMITTED :
                    s2 = "TRANSACTION_READ_COMMITTED";
                    break;
                case Connection.TRANSACTION_READ_UNCOMMITTED :
                    s2 = "TRANSACTION_READ_UNCOMMITTED";
                    break;
                case Connection.TRANSACTION_REPEATABLE_READ :
                    s2 = "TRANSACTION_REPEATABLE_READ";
                    break;
                case Connection.TRANSACTION_SERIALIZABLE :
                    s2 = "TRANSACTION_SERIALIZABLE";
                    break;
                default :
                    s2 = "???";
                    break;
                }
                sStatus = "AutoCommit = " + s1 + " / Transaction Isolation Level = " + s2;
            } catch (SQLException ex)
            {
                sStatus = "SQLException : " + ex.getMessage();
            }
        }
        else
        {
            sStatus = "Connection is null !";
        }
        return sStatus;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Returns the "isolation level" integer value according to the given string 
     * @param sIsolationLevel e.g. "TRANSACTION_NONE", "TRANSACTION_READ_COMMITTED", etc
     * @param iDefaultValue
     * @return
     */
    public static int getIsolationLevel(String sIsolationLevel, int iDefaultValue)
    {
        if (sIsolationLevel != null)
        {
            String s = sIsolationLevel.trim().toUpperCase();
            if (s.equals("TRANSACTION_NONE"))
            {
                return Connection.TRANSACTION_NONE;
            }
            else if (s.equals("TRANSACTION_READ_COMMITTED"))
            {
                return Connection.TRANSACTION_READ_COMMITTED;
            }
            else if (s.equals("TRANSACTION_READ_UNCOMMITTED"))
            {
                return Connection.TRANSACTION_READ_UNCOMMITTED;
            }
            else if (s.equals("TRANSACTION_REPEATABLE_READ"))
            {
                return Connection.TRANSACTION_REPEATABLE_READ;
            }
            else if (s.equals("TRANSACTION_SERIALIZABLE"))
            {
                return Connection.TRANSACTION_SERIALIZABLE;
            }
            else
            {
                return iDefaultValue;
            }
        }
        else
        {
            return iDefaultValue;
        }
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Execute a SQL "SELECT" with or without parameters and return a ResultSet
     * 
     * @param con
     *            the connection to use
     * @param sSql
     *            the SQL SELECT
     * @param aParameters
     * @return
     * @throws SQLException
     */
    public static ResultSet executeSelect(Connection con, String sSql, Object aParameters[]) throws SQLException
    {
        ResultSet rs = null;

        //trace("ExecuteSelect()...");
        if (aParameters != null) // if paramters => PreparedStatement
        {
            //--- Prepare the statement
            PreparedStatement pstmt = con.prepareStatement(sSql);

            //--- Set SQL statement parameters
            if (setSqlParameters(pstmt, aParameters))
            {
                //--- Execute SQL request
                rs = pstmt.executeQuery();
            }
        }
        else
        // no parameter => "normal" statement
        {
            Statement stmt = con.createStatement();
            if (stmt != null)
            {
                //--- Execute SQL request
                rs = stmt.executeQuery(sSql);
            }
        }

        //--- Return the ResultSet
        return rs;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Closes the given result set 
     * @param rs : the ResultSet to close 
     * @return
     */
    public static boolean close(ResultSet rs)
    {
        //trace("CloseStatement()...");
        try
        {
            if (rs != null)
            {
                Statement stmt = rs.getStatement(); // Statement ou
                // PreparedStatement ou
                // CallableStatement
                rs.close();
                if (stmt != null)
                {
                    stmt.close(); // Provoque galement la fermeture du
                    // ResultSet s'il existe
                }
            }
        } catch (SQLException SqlEx)
        {
            //error("closeStatement: SQLException: "+ SqlEx.getMessage());
            return false;
        }
        return true;
    }

    //-------------------------------------------------------------
    // Affectation des paramtres de la requte SQL
    //-------------------------------------------------------------
    private static boolean setSqlParameters(PreparedStatement ps, Object aParameters[])
    {
        //trace("setSqlParameters() ...");
        //--- Pour chaque paramtre ...
        for ( int i = 0 ; i < aParameters.length ; i++ )
        {
            Object oParam = aParameters[i];
            if (oParam != null)
            {
                //--- Dtermination du type du paramtre
                try
                {
                    if (oParam instanceof String)
                    {
                        ps.setString(i + 1, (String)oParam);
                    }
                    //----- Byte, Short, Integer, Long
                    else if (oParam instanceof Byte)
                    {
                        ps.setByte(i + 1, ((Byte)oParam).byteValue());
                    }
                    else if (oParam instanceof Short)
                    {
                        ps.setShort(i + 1, ((Short)oParam).shortValue());
                    }
                    else if (oParam instanceof Integer)
                    {
                        ps.setInt(i + 1, ((Integer)oParam).intValue());
                    }
                    else if (oParam instanceof Long)
                    {
                        ps.setLong(i + 1, ((Long)oParam).longValue());
                    }
                    //----- Float, Double, BigDecimal
                    else if (oParam instanceof Float)
                    {
                        ps.setFloat(i + 1, ((Float)oParam).floatValue());
                    }
                    else if (oParam instanceof Double)
                    {
                        ps.setDouble(i + 1, ((Double)oParam).doubleValue());
                    }
                    else if (oParam instanceof java.math.BigDecimal)
                    {
                        ps.setBigDecimal(i + 1, (java.math.BigDecimal)oParam);
                    }
                    //----- Date, Time, Timestamp
                    else if (oParam instanceof java.sql.Date)
                    {
                        ps.setDate(i + 1, (java.sql.Date)oParam);
                    }
                    else if (oParam instanceof java.sql.Time)
                    {
                        ps.setTime(i + 1, (java.sql.Time)oParam);
                    }
                    else if (oParam instanceof java.sql.Timestamp)
                    {
                        ps.setTimestamp(i + 1, (java.sql.Timestamp)oParam);
                    }
                    //----- Boolean
                    else if (oParam instanceof Boolean)
                    {
                        ps.setBoolean(i + 1, ((Boolean)oParam).booleanValue() );
                    }
                    //----- Other : unsupported
                    else
                    {
                        throw new TelosysRuntimeException("Unsupported SQL parameter type : " + oParam.getClass().getName());
                    }
                } catch (SQLException ex)
                {
                    throw new TelosysRuntimeException("SQLException", ex);
                } catch (Throwable ex)
                {
                    throw new TelosysRuntimeException("Exception", ex);
                }
            }
        }
        return true;
    }

    //-----------------------------------------------------------------------------------------
    /**
     * Returns a column value from a ResultSet ( value can be null )
     * 
     * @param rs
     * @param iCol
     * @param iColType
     * @return : the object containing the value ( String, BigDecimal, Boolean, ... )
     * @throws SQLException
     */
    public static Object getColValue(ResultSet rs, int iCol, int iColType) throws SQLException
    {
        switch (iColType)
        {
        //--- All "NUMERIC" JDBC TYPES ( can be NULL )
        case Types.TINYINT :
        case Types.SMALLINT :
        case Types.INTEGER :
        case Types.BIGINT :
        case Types.REAL :
        case Types.FLOAT :
        case Types.DOUBLE :
        case Types.DECIMAL : // My SQL
        case Types.NUMERIC : // Oracle
            return rs.getBigDecimal(iCol);
        // Remplacement de "NULL" par "ZERO" via "getDouble"
        // rec.setObject(iCol, new BigDecimal(rs.getDouble(iCol)));
        //break;

        //--- All "CHARACTER/STRING" JDBC TYPES ( can be NULL )
        case Types.CHAR :
        case Types.VARCHAR :
        case Types.LONGVARCHAR : // JDBC type value = -1 
        case Types.CLOB: // JDBC type value = 2005 ( used by Oracle ) : getString() is the simplest way to retrieve the clob value ( see also getClob() )
            return rs.getString(iCol);

        //--- Boolean JDBC TYPES ( can be NULL )
        case Types.BIT :
        case Types.BOOLEAN :
            Boolean boolObj = null ;
        	// NB: if the value is NULL, rs.getBoolean(..) return false ( never null )
        	// => use getObject(..) then getBoolean(..)
            if ( rs.getObject(iCol) != null )
            {
                boolObj = Boolean.valueOf(rs.getBoolean(iCol));
            }
            return boolObj ;

        //--- "DATE" JDBC ( can be NULL )
        case Types.DATE :
            return rs.getDate(iCol); // java.sql.Date

        //--- "TIMESTAMP" JDBC ( can be NULL )
        case Types.TIMESTAMP :
            return rs.getTimestamp(iCol); // java.sql.Timestamp

        //--- "TIME" JDBC ( can be NULL )
        case Types.TIME :
            return rs.getTime(iCol); // java.sql.Time

        //--- Binary JDBC TYPES
        case Types.BINARY :
        case Types.VARBINARY :
            return rs.getBytes(iCol); // byte[]

        }
        //--- For all other types : return NULL
        return null;
    }
}