package org.objectweb.telosys.rpl;

import java.util.ArrayList;
import java.util.Iterator;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.IRequestParameters;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.screen.core.ScreenDataElement;
import org.objectweb.telosys.screen.core.ScreenDataElements;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;

/**
 * Request object for a Screen Action Request<br>
 * 
 * It contains all informations sent by the client side,<br> 
 * the request is built by parsing the XML input flow. 
 * 
 * @author Laurent GUERIN
 *  
 */
public class ScreenRequest extends TelosysObject implements IRequestParameters
{
    //-----------------------------------------------------------------------------------------------
    // Constants
    //-----------------------------------------------------------------------------------------------
    private final static String[]   VOID_STRING_ARRAY      = new String[0];

    private final static ScreenRequestView[]   VOID_VIEWS_ARRAY      = new ScreenRequestView[0];
    
    //-----------------------------------------------------------------------------------------------
    // Instance variables
    //-----------------------------------------------------------------------------------------------
    private HttpServletRequest      _httpRequest    = null;

    private String                  _sScreenName    = null;

    private String                  _sScreenId      = null;

    private String                  _sAction        = null;

    private boolean                 _bFlagSet       = true;

    private boolean                 _bFlagGet       = true;

    private boolean                 _bFlagClear     = true;

    private String                  _sProcName      = null;

    private String                  _sElements      = null;

    private String[]                _sElementsArray = null;

    private ScreenDataElements      _dataElements   = null;         // Data elements from the XML <data> section

    private ScreenRequestParameters _parameters     = null;

    private ArrayList               _alViews        = null;

    /**
     *  
     */
    public ScreenRequest( HttpServletRequest httpRequest )
    {
        _httpRequest = httpRequest;

        _parameters = new ScreenRequestParameters();
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard HTTP servlet request 
     * 
     * @return
     */
    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard HTTP session
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public HttpSession getHttpSession() // new in v 0.9.9
    {
        if ( _httpRequest != null )
        {
            return _httpRequest.getSession(false); // the session always exists
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ID of the standard HTTP session <br>
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public String getHttpSessionId() // new in v 0.9.9
    {
        HttpSession httpSession = getHttpSession();
        if ( httpSession != null )
        {
            return httpSession.getId();
        }
        return null ;
    }

	//-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard web app ServletContext <br>
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public ServletContext getServletContext() // new in v 0.9.9
    {
        HttpSession httpSession = getHttpSession();
        if (httpSession != null)
        {
            return httpSession.getServletContext();
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the current ScreenSession associated with the request <br>
     * or null if the HttpSession doesn't exist or if it doesn't contain a ScreenSession attribute <br>
     * It cannot returns null, unless the HttpSession has been corrupted by an external way
     * 
     * @return
     */
    public ScreenSession getScreenSession() // new in v 0.9.9
    {
        return ScreenSessionManager.getCurrentScreenSession(_httpRequest);
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication object associated with the current web app<br>
     * or null if the ScreenApplication has not yet been created (opened)<br>
     * It cannot returns null, unless the ServletContext has been corrupted by an external way
     * @return
     */
    public ScreenApplication getScreenApplication() // new in v 0.9.9
    {
        return ScreenApplicationManager.getScreenApplication(); // v 1.0.0
        /***
        HttpSession httpSession = getHttpSession();
        if ( httpSession != null )
        {
            return ScreenApplicationManager.getScreenApplication(httpSession); 
        }
        return null ;
        ***/
    }

    //-----------------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the action attribute of the request
     * 
     * @return the action
     */
    public String getAction()
    {
        return _sAction;
    }

    /**
     * Set the action attribute of the request
     * 
     * @param action
     *            The action to set.
     */
    public void setAction(String action)
    {
        _sAction = action;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the procedure attribute of the request
     * 
     * @return the procedure
     */
    public String getProcName()
    {
        return _sProcName;
    }

    /**
     * Set the procedure attribute of the request
     * 
     * @param procName
     *            The _sProcName to set.
     */
    public void setProcName(String procName)
    {
        _sProcName = procName;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the Screen Id.
     */
    public String getScreenId()
    {
        return _sScreenId;
    }

    /**
     * @param screenId
     *            The Screen Id to set.
     */
    public void setScreenId(String screenId)
    {
        _sScreenId = screenId;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @param screenName
     *            The Screen Name to set.
     */
    public void setScreenName(String screenName)
    {
        _sScreenName = screenName;
    }

    /**
     * @return Returns the Screen Name.
     */
    public String getScreenName()
    {
        return _sScreenName;
    }

    //---------------------------------------------------------------------
    /**
     * Builds an array of String with the given items list and returns it
     * 
     * @return
     */
    private String[] buildArray(String sInputList)
    {
        if (sInputList != null)
        {
            if ( sInputList.trim().length() == 0 )
            {
                //--- The elements list is void "" => void array
                return VOID_STRING_ARRAY;                
            }
            else
            {                
                //--- The elements list is not void => split it 
	            String[] list1 = sInputList.split(",");
	            String[] list2 = new String[list1.length];
	            String sItem = null;
	            for ( int i = 0 ; i < list2.length ; i++ )
	            {
	                sItem = list1[i];
	                if (sItem != null)
	                {
	                    list2[i] = sItem.trim();
	                }
	                else
	                {
	                    list2[i] = "";
	                }
	            }
	            return list2;
            }
        }
        else
        {
            //--- No elements list => void array
            return VOID_STRING_ARRAY;
        }
    }

    //---------------------------------------------------------------------
    // ELEMENTS
    //---------------------------------------------------------------------
    /**
     * Set the "elements" attribute of the request
     * 
     * @param v
     */
    public void setElements(String v)
    {
        _sElements = v;
    }

    /**
     * Returns the "elements" attribute of the request
     * 
     * @return
     */
    public String getElements()
    {
        return _sElements;
    }

    /**
     * Returns true if the request has an "element" attribute
     * 
     * @return
     */
    public boolean hasElementsAttribute()
    {
        return (_sElements != null ? true : false);
    }

    /**
     * Returns an array of elements name containing each element of the "elements" attribute
     * 
     * @return
     */
    public String[] getElementsArray()
    {
        if (_sElementsArray == null)
        {
            //--- Array not yet build => build the array
            //_sElementsArray = buildElements();
            _sElementsArray = buildArray(_sElements);
        }
        //--- Return a copy of the existing array
        return (String[]) _sElementsArray.clone();
    }

    /**
     * Returns true if the "elements" attribute of the request contains the given element name
     * 
     * @param sArgElementName
     * @return true if the element is in the list (else false)
     */
    public boolean elementsAttributeContains(String sArgElementName)
    {
        if (sArgElementName != null)
        {
            String sElementName = sArgElementName.trim();
            String[] elements = getElementsArray();
            for ( int i = 0 ; i < elements.length ; i++ )
            {
                if (sElementName.equals(elements[i]))
                {
                    return true;
                }
            }
        }
        return false;
    }

    //---------------------------------------------------------------------
    // VIEWS
    //---------------------------------------------------------------------
    /**
     * Adds a view to the request
     * 
     * @param v
     */
    public void addView(ScreenRequestView v)
    {
        trace("addView(v)...");
        if (_alViews == null)
        {
            _alViews = new ArrayList(8);
        }
        _alViews.add(v);
    }

    /**
     * Returns the "views" of the request
     * 
     * @return
     */
    public ScreenRequestView[] getViews()
    {
        if (_alViews != null)
        {
            return (ScreenRequestView[]) _alViews.toArray( VOID_VIEWS_ARRAY );
        }
        else
        {
            return null ;
            //return VOID_VIEWS_ARRAY ;
        }
    }

    /**
     * Returns true if the request has one or more "view"
     * 
     * @return
     */
    public boolean hasViews()
    {
        if (_alViews != null)
        {
            if (_alViews.size() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the Request Parameters .
     */
    public ScreenRequestParameters getParameters()
    {
        return _parameters;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the status of the "SET" flag
     * @return
     */
    public boolean getFlagSetData()
    {
        return _bFlagSet;
    }

    /**
     * Sets the status of the "SET" flag
     * @param v
     */
    public void setFlagSetData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagSet = true;
        }
        else
        {
            _bFlagSet = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the status of the "GET" flag
     * @return
     */
    public boolean getFlagGetData()
    {
        return _bFlagGet;
    }

    /**
     * Sets the status of the "GET" flag
     * @param v
     */
    public void setFlagGetData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagGet = true;
        }
        else
        {
            _bFlagGet = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the status of the "clear" flag
     * @return
     */
    public boolean getFlagClearData()
    {
        return _bFlagClear;
    }

    /**
     * Sets the status of the "clear" flag
     * @param v
     */
    public void setFlagClearData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagClear = true;
        }
        else
        {
            _bFlagClear = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    // PARAMETERS
    //-----------------------------------------------------------------------------------------------
    /**
     * Sets a parameter in the request
     * @param sParamName parameter name
     * @param sParamValue parameter value
     */
    public void setParameter(String sParamName, String sParamValue)
    {
        _parameters.setParameter(sParamName, sParamValue);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value of the given parameter name, or NULL if not found
     * @param sParamName
     * @return
     */
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParameter(java.lang.String)
     */
    public String getParameter(String sParamName)
    {
        return _parameters.getParameter(sParamName);
    }

    public String getParameter(String paramName, String defaultValue)
    {
        return _parameters.getParameter(paramName, defaultValue);
    }

//    /**
//     * Returns the value of the given parameter name, 
//     * throws TelosysRuntimeException if the parameter is not found.
//     * @param sParamName
//     * @return
//     */
//    private String getParamOrThrowException(String sParamName) 
//    {
//        String s = _parameters.getParameter(sParamName);
//        if ( s != null )
//        {
//            return s ;
//        }
//        else
//        {
//            throw new TelosysRuntimeException("No '" + sParamName + "' parameter");
//        }
//    }
    
    /**
     * Returns the int value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public int getParamAsInt(String paramName) 
    {
        return _parameters.getParamAsInt(paramName);
//        String s = getParamOrThrowException(sParamName);
//        try
//        {
//            return Integer.parseInt( s.trim() );
//        }
//        catch (NumberFormatException ex)
//        {
//            throw new TelosysRuntimeException("Cannot convert parameter '" + sParamName 
//                    + "' to int (value='"+s+"')");
//        }        
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsInt(java.lang.String, int)
     */
    public int getParamAsInt(String paramName, int defaultValue) 
    {
        return _parameters.getParamAsInt(paramName, defaultValue);
    }

    /**
     * Returns the long value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public long getParamAsLong(String paramName) 
    {
        return _parameters.getParamAsShort(paramName);
//        String s = getParamOrThrowException(sParamName);
//        try
//        {
//            return Long.parseLong( s.trim() );
//        }
//        catch (NumberFormatException ex)
//        {
//            throw new TelosysRuntimeException("Cannot convert parameter '" + sParamName 
//                    + "' to long (value='"+s+"')");
//        }        
    }
    public long getParamAsLong(String paramName, long defaultValue) 
    {
        return _parameters.getParamAsLong(paramName, defaultValue);
    }
    
    /**
     * Returns the short value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public short getParamAsShort(String paramName) 
    {
        return _parameters.getParamAsShort(paramName);
//        String s = getParamOrThrowException(sParamName);
//        try
//        {
//            return Short.parseShort( s.trim() );
//        }
//        catch (NumberFormatException ex)
//        {
//            throw new TelosysRuntimeException("Cannot convert parameter '" + sParamName 
//                    + "' to short (value='"+s+"')");
//        }        
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsShort(java.lang.String, short)
     */
    public short getParamAsShort(String paramName, short defaultValue) 
    {
        return _parameters.getParamAsShort(paramName, defaultValue);
    }
    
    /**
     * Returns the float value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public float getParamAsFloat(String paramName) 
    {
        return _parameters.getParamAsFloat(paramName);
//        String s = getParamOrThrowException(sParamName);
//        try
//        {
//            return Float.parseFloat( s.trim() );
//        }
//        catch (NumberFormatException ex)
//        {
//            throw new TelosysRuntimeException("Cannot convert parameter '" + sParamName 
//                    + "' to float (value='"+s+"')");
//        }        
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsFloat(java.lang.String, float)
     */
    public float getParamAsFloat(String paramName, float defaultValue) 
    {
        return _parameters.getParamAsFloat(paramName, defaultValue);
    }
    
    /**
     * Returns the double value of the given parameter name.<br>
     * Throws an exception if the parameter is not found or cannot be converted.
     * @param paramName
     * @return
     * @throws TelosysRuntimeException if not found or invalid
     * @since 1.0.2
     */
    public double getParamAsDouble(String paramName) 
    {
        return _parameters.getParamAsDouble(paramName);
//        String s = getParamOrThrowException(sParamName);
//        try
//        {
//            return Double.parseDouble( s.trim() );
//        }
//        catch (NumberFormatException ex)
//        {
//            throw new TelosysRuntimeException("Cannot convert parameter '" + sParamName 
//                    + "' to double (value='"+s+"')");
//        }        
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDouble(java.lang.String, double)
     */
    public double getParamAsDouble(String paramName, double defaultValue) 
    {
        return _parameters.getParamAsDouble(paramName, defaultValue);
    }
    
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsByte(java.lang.String)
     */
    public byte getParamAsByte(String paramName ) 
    {
        return _parameters.getParamAsByte(paramName);
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsByte(java.lang.String, byte)
     */
    public byte getParamAsByte(String paramName, byte defaultValue) 
    {
        return _parameters.getParamAsByte(paramName, defaultValue);
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsChar(java.lang.String)
     */
    public char getParamAsChar(String paramName ) 
    {
        return _parameters.getParamAsChar(paramName);
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsChar(java.lang.String, char)
     */
    public char getParamAsChar(String paramName, char defaultValue) 
    {
        return _parameters.getParamAsChar(paramName, defaultValue);
    }
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsBoolean(java.lang.String)
     */
    public boolean getParamAsBoolean(String paramName ) 
    {
        return _parameters.getParamAsBoolean(paramName);
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsBoolean(java.lang.String, boolean)
     */
    public boolean getParamAsBoolean(String paramName, boolean defaultValue) 
    {
        return _parameters.getParamAsBoolean(paramName, defaultValue);
    }
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDate(java.lang.String)
     */
    public java.util.Date getParamAsDate(String paramName ) 
    {
        return _parameters.getParamAsDate(paramName);
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParamAsDate(java.lang.String, java.util.Date)
     */
    public java.util.Date getParamAsDate(String paramName, java.util.Date defaultValue) 
    {
        return _parameters.getParamAsDate(paramName, defaultValue);
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * @param sParamName
     * @return
     * @throws TelosysException
     * @deprecated use getParamAsInt 
     */
    public int getIntParameter(String sParamName) throws TelosysException
    {
        return _parameters.getIntParameter(sParamName);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the Screen Data Elements held by the request
     * @return
     */
    public ScreenDataElements getDataElements()
    {
        return _dataElements;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Adds a new element in the request elements list
     * 
     * @param sTagName
     * @param objData
     */
    public void storeDataElement(String sTagName, Object objData)
    {
        if (_dataElements == null)
        {
            _dataElements = new ScreenDataElements();
        }
        //ScreenDataElement oElement = new ScreenDataElement(sTagName, objData.getClass(), objData);
        //--- Creates a void element
        ScreenDataElement oElement = new ScreenDataElement(sTagName, objData.getClass());
        //--- Set the element instance
        oElement.setInstance(objData);
        //--- Add the element to the request elements list
        _dataElements.add(oElement);
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        StringBuffer sb = new StringBuffer(256);
        sb.append("ScreenRequest : " );
        sb.append("\n . "
                + "screen-name='" + _sScreenName + "' " 
                + "screen-id='" + _sScreenId + "' " 
                + "action='" + _sAction + "' " 
                + "proc='" + _sProcName + "' ");
        sb.append("\n . flags : set='" + _bFlagSet + "' get='" + _bFlagGet + "' clear='" + _bFlagClear + "' " );

        //--- Elements to retrieve
        sb.append("\n . elements : '" + _sElements + "' " );

        //--- Views to retrieve
        sb.append("\n . views : "  );
        if ( _alViews != null )
        {
            Iterator iter = _alViews.iterator();
            while ( iter.hasNext() )
            {
                ScreenRequestView v = (ScreenRequestView) iter.next();
                sb.append("\n  . " + v.toString() );            
            }
        }
        else
        {
            sb.append("\n   (no view) "  );            
        }
        
        //--- Parameters
        sb.append("\n . parameters : \n" );        
        if (_parameters != null)
        {
            sb.append( _parameters.toString() );
        }
        else
        {
            sb.append( "   (no parameter)" );            
        }

        //--- Data
        sb.append("\n . data elements : \n" ); 
        if ( _dataElements != null )
        {
            sb.append( _dataElements.toString() );
        }
        else
        {
            sb.append( "   (no data element)" );                        
        }

        return sb.toString() ;
    }
    //-----------------------------------------------------------------------------------------------
}