package org.objectweb.telosys.rpl;

import java.io.InputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.rpl.xml.ScreenRequestSaxHandler;
import org.objectweb.telosys.rpl.xml.XmlSaxParser;
import org.objectweb.telosys.screen.core.ScreenActions;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;

/**
 * @author Laurent GUERIN
 *  
 */
public final class ScreenRequestProcessor 
{
    private static final TelosysClassLogger log                   = new TelosysClassLogger(ScreenRequestProcessor.class);

    //--- Open/Close Screen
    public final static String              ACTION_OPEN           = "open";

    public final static String              ACTION_CLOSE          = "close";

    //--- Screen context Actions
    public final static String              ACTION_GET            = "get";

    public final static String              ACTION_SET            = "set";

    public final static String              ACTION_CLEAR          = "clear";

    //--- Data Access Layer Actions ( use DAO )
    public final static String              ACTION_LOAD           = "load";

    public final static String              ACTION_SAVE           = "save";

    public final static String              ACTION_SAVE_CURRENT   = "save_current";

    public final static String              ACTION_INSERT         = "insert";

    public final static String              ACTION_UPDATE         = "update";

    public final static String              ACTION_DELETE         = "delete";

    public final static String              ACTION_DELETE_CURRENT = "delete_current";

    //--- Call specific process
    public final static String              ACTION_EXEC           = "exec";

    //--- Test / Debug actions
    public final static String              ACTION_PING           = "ping";

    public final static String              ACTION_ECHO           = "echo";
    
    //--- List management actions
    public final static String              ACTION_LIST_SELECT    = "list_select";
    
    public final static String              ACTION_LIST_INSERT    = "list_insert";
    
    public final static String              ACTION_LIST_ADD       = "list_add";
    
    public final static String              ACTION_LIST_REMOVE    = "list_remove";
    
    public final static String              ACTION_LIST_CLEAR     = "list_clear";
    
    public final static String              ACTION_LIST_SORT      = "list_sort";
    
    /**
     * No constructor for this class
     */
    private ScreenRequestProcessor()
    {
    }

    //---------------------------------------------------------------------------
    /**
     * Parse the XML input stream and return the resulting ScreenRequest instance
     * 
     * @param input
     * @return
     * @throws TelosysException
     */
    //---------------------------------------------------------------------------
    protected static ScreenRequest parseInputStream(ScreenRequest screenRequest, InputStream input, String sCharset) throws TelosysException
    {
        log.trace("getScreenRequest()...");
        if (input == null)
        {
            throw new TelosysException("ScreenRequestProcessor.parseInputStream : InputStream parameter is null ! ");
        }
        //---------------------------------------------------------------------------
        // Prepare to process the XML input Stream 
        //---------------------------------------------------------------------------

        IXmlWrapperProvider wrapperProvider = Telosys.getXmlWrapperProvider() ;
        if (wrapperProvider == null)
        {
            throw new TelosysException("ScreenRequestProcessor.parseInputStream : XML wrapper provider is null ! ");
        }

        ScreenRequestSaxHandler handler = new ScreenRequestSaxHandler(screenRequest, wrapperProvider);
        XmlSaxParser parser = new XmlSaxParser(handler);

        //---------------------------------------------------------------------------
        // Parse the XML input stream ( load the ScreenData hold by the ScreenRequest )
        //---------------------------------------------------------------------------
        log.trace("start XML parsing()...");
        //--- Parsing the XML input stream
        parser.parse(input, sCharset);

        //---------------------------------------------------------------------------
        // Return the Screen Request
        //---------------------------------------------------------------------------

//        System.out.println("-----------------------------------------------------------");
//        System.out.println(" AFTER REQUEST XML PARSING ");
//        System.out.println("-----------------------------------------------------------");
//        System.out.println("ScreenRequest : ");
//        System.out.println(screenRequest.toString());
//        System.out.println("-----------------------------------------------------------");
        
        return screenRequest;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns a ScreenResponse instance created from the given ScreenRequest and HttpServletResponse
     * @param request
     * @param httpResponse
     * @return
     * @throws TelosysException
     */
    // v 0.9.9
    //public static ScreenResponse getScreenResponse(ScreenRequest request) throws TelosysException
    protected static ScreenResponse getScreenResponse(ScreenRequest request, HttpServletResponse httpResponse) throws TelosysException
    {
        log.trace("getScreenResponse()...");

        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenResponse : ScreenRequest parameter is null ! ");
        }

        //---------------------------------------------------------------------------
        //--- Create a Screen Response
        //---------------------------------------------------------------------------
        log.trace("getScreenResponse() : create a screen response");
        String sScreenName = request.getScreenName();
        String sScreenId = request.getScreenId();
        String sAction = request.getAction();
        String sProcName = request.getProcName();

        //ScreenResponse response = new ScreenResponse(sScreenName, sScreenId, sAction, sProcName);
        //ScreenResponse response = new ScreenResponse(httpResponse, sScreenName, sScreenId, sAction, sProcName); // V 0.9.9
        ScreenResponse response = new ScreenResponse(request.getHttpRequest(), httpResponse, sScreenName, sScreenId, sAction, sProcName); // V 0.9.9 bis
        
        
        return response;
    }

    /**
     * Return the current ScreenSession for the HttpSession, throws a TelosysException if the session is not opened.
     * 
     * @param httpRequest
     * @return
     * @throws TelosysException
     */
    protected static ScreenSession getScreenSession(HttpServletRequest httpRequest) throws TelosysException
    {
        log.trace("getScreenSession()...");
        if (httpRequest == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenSession : HttpServletRequest parameter is null ! ");
        }
        ScreenSession screenSession = ScreenSessionManager.getScreenSession(httpRequest);
        if (screenSession == null)
        {
            //--- No Screen Session ( no screen opened for this user http session )
            throw new TelosysException("ScreenRequestProcessor.getScreenSession : Screen session not found (session not opened) !");
        }
        //--- Screen Session found
        return screenSession;
    }

    /**
     * Return the integer value of Screen ID String
     * 
     * @param sScreenId
     * @return
     * @throws TelosysException
     */
    private static int getIntScreenId(String sScreenId) throws TelosysException
    {
        log.trace("getIntScreenId(" + sScreenId + ")...");

        //--- Get the Screen Id
        int iScreenId = 0;
        try
        {
            iScreenId = Integer.parseInt(sScreenId);
        } catch (Exception e)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " ) ! ");
        }
        if (iScreenId < 0)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " < 0 ) ! ");
        }
        if (iScreenId > ScreenSession.getMaxScreenId())
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " > " + ScreenSession.getMaxScreenId() + " ) ! ");
        }
        return iScreenId;
    }

    /**
     * Return the current Screen Context corresponding to the Screen Id and Screen Name of the request
     * 
     * @param request
     * @param screenSession
     * @return
     * @throws TelosysException
     */
    protected static ScreenContext getScreenContext(ScreenRequest request, ScreenSession screenSession) throws TelosysException
    {
        log.trace("getScreenContext()...");

        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenContext : ScreenRequest parameter is null ! ");
        }
        if (screenSession == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenContext : ScreenSession parameter is null ! ");
        }

        //--- Get the Screen Id ( Exception if invalid )
        int iScreenId = getIntScreenId(request.getScreenId());

        //--- Get the action
        String sAction = request.getAction();

        ScreenContext screenContext = screenSession.getScreenContext(iScreenId);
        if (screenContext == null)
        {
            //--- No current screen context for this ID
            if (sAction.equalsIgnoreCase(ACTION_OPEN))
            {
                //--- Ok, the screen will be created by the action
                return null;
            }
            else
            {
                throw new TelosysException("Screen context not found ( Id = " + request.getScreenId() + " ), Screen not initialized ( in ScreenRequestProcessor.getScreenContext() )");
            }
        }
        else
        {
            //--- There's a current screen context for this ID
            if (!sAction.equalsIgnoreCase(ACTION_OPEN))
            {
                //--- If NOT "open" action => check the context type
                //--- Get the Screen Name
                String sScreenName = request.getScreenName();
                if (sScreenName == null)
                {
                    throw new TelosysException("No Screen Name in the request !");
                }
                //--- Is it the expected Screen Context ?
                if (!sScreenName.equalsIgnoreCase(screenContext.getScreenName()))
                {
                    throw new TelosysException("The ScreenContext for id '" + iScreenId + "' is '" + screenContext.getScreenName() + "' ! ( '" + sScreenName + "' expected ) ");
                }
            }
        }

        return screenContext;
    }

    //---------------------------------------------------------------------------
    /**
     * Execute the ScreenRequest ACTION in a ScreenContext, and set the ScreenResponse.
     * 
     * @param screenContext
     * @param request
     * @param response
     */
    protected static void executeSynchronizedAction(ScreenSession screenSession, ScreenContext screenContext, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        log.trace("executeSynchronizedAction()...");
        //--- Lock the Screen Session Object
        synchronized (screenSession)
        {
            executeAction(screenSession, screenContext, request, response);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Execute the ScreenRequest ACTION in a ScreenContext, and set the ScreenResponse.
     * 
     * @param screenContext
     * @param request
     * @param response
     */
    protected static void executeAction(ScreenSession screenSession, ScreenContext screenContext, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        log.trace("executeAction()...");

        //--- Check the parameters (the ScreenContext can be null )
        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenRequest parameter is null ! ");
        }
        if (response == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenResponse parameter is null ! ");
        }

        String sAction = request.getAction();
        log.trace("  Action = " + sAction);
        if (sAction == null)
        {
            throw new TelosysException("ScreenRequestProcessor.executeAction : Request action is null ! ");
        }
        sAction = sAction.toLowerCase();
        if (screenContext == null)
        {
            if (!sAction.equals(ACTION_OPEN))
            {
                //--- No screen context and action != OPEN => ERROR
                throw new TelosysException("ScreenRequestProcessor.executeAction : ScreenContext parameter is null ! ");
            }
        }

        //--- Execute the action
        try
        {
            //----------------------------------------------------------
            if (sAction.equals(ACTION_OPEN))
            {
                openScreen(screenSession, request);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_CLOSE))
            {
                closeScreen(screenSession, request);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_PING))
            {
                ping(screenSession, request, response);
                return;
            }
            //----------------------------------------------------------
            if (sAction.equals(ACTION_ECHO))
            {
                echo(screenSession, request, response);
                return;
            }
            //----------------------------------------------------------

            // v 0.9.9
//            ScreenActions actions = screenContext.getScreenActions();
//            if (actions == null)
//            {
//                actions = new StandardScreenActions();
//            }
            
            //--- Retrieve the ScreenActions associated with the ScreenContext
            ScreenActions actions = screenContext.getOrCreateScreenActions(); // throws TelosysException if cannot create
            // if no 'exception' here the action is not null
            
            //----------------------------------------------------------
            if (sAction.equals(ACTION_GET))
            {
                actions.actionGet(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SET))
            {
                actions.actionSet(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_CLEAR))
            {
                actions.actionClear(screenContext, request, response);
            }
            //----------------------------------------------------------
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LOAD))
            {
                actions.actionLoad(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SAVE))
            {
                actions.actionSave(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_SAVE_CURRENT))
            {
                actions.actionSaveCurrent(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_DELETE))
            {
                actions.actionDelete(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_DELETE_CURRENT))
            {
                actions.actionDeleteCurrent(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_INSERT))
            {
                actions.actionInsert(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_UPDATE))
            {
                actions.actionUpdate(screenContext, request, response);
            }
            //----------------------------------------------------------
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_SELECT))
            {
                actions.actionListSelect(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_INSERT))
            {
                actions.actionListInsert(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_ADD))
            {
                actions.actionListAdd(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_REMOVE))
            {
                actions.actionListRemove(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_CLEAR)) 
            {
                actions.actionListClear(screenContext, request, response);
            }
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_LIST_SORT)) 
            {
                actions.actionListSort(screenContext, request, response);
            }
            //----------------------------------------------------------
            //----------------------------------------------------------
            else if (sAction.equals(ACTION_EXEC))
            {
                actions.actionExec(screenContext, request, response);
            }
            //----------------------------------------------------------
            else
            {
                String sMsg = "Invalid action : " + sAction;
                Telosys.error(sMsg);
                
                // response.setReturnCode(ScreenResponse.INVALID_ACTION); // v 1.0.2
                // response.setReturnMessage(sMsg); // v 1.0.2
                response.setActionReturn(ScreenResponse.INVALID_ACTION, sMsg ); // v 1.0.2
            }
        } catch (TelosysException tex)
        {
            Telosys.error(tex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(tex);
        } catch (Exception ex)
        {
            Telosys.error(ex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(ex);
        } catch (Throwable th)
        {
            Telosys.error(th);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(th);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Open the screen specified in the request (create a new ScreenContext)
     * 
     * @param screenSession
     * @param screenRequest
     * @throws TelosysException
     */
    private static void openScreen(ScreenSession screenSession, ScreenRequest screenRequest) throws TelosysException
    {
        log.trace("openScreen()...");
        //--- Get the screen NAME
        String sScreenName = screenRequest.getScreenName();
        //--- Get the screen ID
        int iScreenId = getIntScreenId(screenRequest.getScreenId());
        //--- Open (create) the screen context
        ScreenContextManager.openScreenContext(screenSession, sScreenName, iScreenId, screenRequest.getParameters() );
    }

    //---------------------------------------------------------------------------
    /**
     * Close the screen specified in the request (remove the ScreenContext)
     * 
     * @param screenSession
     * @param screenRequest
     * @throws TelosysException
     */
    private static void closeScreen(ScreenSession screenSession, ScreenRequest screenRequest) throws TelosysException
    {
        log.trace("closeScreen()...");
        //--- Get the screen ID
        int iScreenId = getIntScreenId(screenRequest.getScreenId());
        //--- Close = remove the screen context from the current session
        ScreenContextManager.closeScreenContext(screenSession, iScreenId);
    }

    //---------------------------------------------------------------------------
    private static void ping(ScreenSession screenSession, ScreenRequest screenRequest, ScreenResponse response) throws TelosysException
    {
        log.trace("ping()...");
    }
    //---------------------------------------------------------------------------
    private static void echo(ScreenSession screenSession, ScreenRequest screenRequest, ScreenResponse response) throws TelosysException
    {
        log.trace("echo()...");
        /**
        ScreenData screenData = screenRequest.getData();        
        response.setDataElements(screenData);
        **/
        response.setDataElements( screenRequest.getDataElements() );
    }
    //---------------------------------------------------------------------------
}