package org.objectweb.telosys.rpl.xml;

import java.util.HashMap;
import java.util.Properties;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.rpl.TelosysRPL;
import org.objectweb.telosys.rpl.xml.mapper.BigDecimalValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.BooleanValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.DateValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.DoubleValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.FloatValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.IntValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.LongValueMapper;
import org.objectweb.telosys.rpl.xml.mapper.StringValueMapper;
import org.objectweb.telosys.util.StrUtil;

/**
 * Standard XML Wrappers Registry ( designed to provide XML Wrappers )
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardXmlWrapperRegistry extends TelosysObject implements IXmlWrapperProvider
{
    //--- Property name for XML mapper class pattern
    private final static String       MAPPERCLASS = "mapperclass" ;
    
    //--- Wrapper instances accessible by bean class name
    private final static HashMap      $hmWrappersByBeanClass  = new HashMap(128);

    private static boolean            $bFlagGetPattern        = false ;
    
    private static String             $ClassNamePattern       = null ;
    
    //--- Static block ( class initialization )
    {
        //--- Init the registry with predefined mappers
        register( BigDecimalValueMapper.class );
        register( BooleanValueMapper.class );
        register( DateValueMapper.class );
        register( DoubleValueMapper.class );
        register( FloatValueMapper.class );
        register( IntValueMapper.class );
        register( LongValueMapper.class );
        register( StringValueMapper.class );
    }
    
//    /**
//     * Constructor ( supposed to be called once in TelosysFactory )
//     */
//    public StandardXmlWrapperRegistry() // v 1.0.0
//    {
//        if ( $ClassNamePattern == null ) // if not yet init ( must always be the case )
//        {
//            $ClassNamePattern = getClassNamePattern();
//        }
//    }

    /**
     * Returns the Class name pattern specified in "telosys.properties" <br>
     * e.g. "my.pacakge.vo.${BEANNAME}Xml"
     * @return
     */
    private String getClassNamePattern() // v 1.0.0
    {
        //--- Try to find a generic pattern in "telosys.properties" 
        Properties prop = Telosys.getProperties();
        if ( prop != null ) 
        {
            String s = prop.getProperty(MAPPERCLASS);
            if ( StrUtil.nullOrVoid(s) )
            {
                return null ;
            }
            else
            {
                return s.trim();
            }
        }
        else
        {
            error("Cannot get Telosys properties");
        }
        
        //--- 4) Not found => no pattern for this database
        return null ;
    }

    //-----------------------------------------------------------------------------------
    /**
     * Registers an XML wrapper using its class <br>
     * This method is desing to used in sub classes, in order to register specific XML wrappers
     * 
     * @param beanWrapperClass :
     *            the wrapper class
     */
    protected static void register(Class beanWrapperClass)
    {
        if ( beanWrapperClass == null )
        {
            throw new TelosysRuntimeException("register(null) : class argument is null ");
        }
        
        if (IXmlWrapper.class.isAssignableFrom(beanWrapperClass) != true)
        {
            throw new TelosysRuntimeException("register("+beanWrapperClass+") : does not implement IXmlWrapper !");
        }

        //--- Create the Wrapper instance 
        IXmlWrapper wrapperInstance = createWrapperInstance(beanWrapperClass);
        
        //--- Register the Wrapper instance 
        registerInstance(wrapperInstance);
    }
    
    /**
     * Registers an XML wrapper instance
     *  
     * @param wrapperInstance
     */
    private static void registerInstance(IXmlWrapper wrapperInstance)
    {
        if (wrapperInstance == null)
        {
            throw new TelosysRuntimeException("registerInstance(null) : no parameter (null)");
        }
        Class beanClass = wrapperInstance.getBeanClass();
        
        if (beanClass != null)
        {
            //--- Register the wrapper by bean class
            //$hmWrappersByBeanClass.put(beanClass.getName(), beanWrapperClass);
            $hmWrappersByBeanClass.put(beanClass.getName(), wrapperInstance);
        }
        else
        {
            //log.error("register(" + beanWrapperClass.getName() + ") : Cannot get bean class !");
            throw new TelosysRuntimeException("registerInstance(" + wrapperInstance.getClass().getName() + ") : Cannot get bean class");
        }
        return ;
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Creates a wrapper instance with the given class
     * 
     * @param objClass
     *            class to use to create a wrapper instance
     * @return the wrapper instance (never null)
     */
    private static IXmlWrapper createWrapperInstance(Class wrapperClass)
    {
        //log.trace("createWrapperInstance(" + wrapperClass + ")...");
        if (wrapperClass != null)
        {
            try
            {
                //--- Create the wrapper instance with the given class
                Object objWrapper = wrapperClass.newInstance();
                if (objWrapper instanceof IXmlWrapper)
                {
                    //log.trace("createWrapperInstance(" + wrapperClass + ") : OK, wrapper inctance created");
                    return (IXmlWrapper) objWrapper;
                }
                else
                {
                    throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : not a wrapper class ");
                }
            } catch (InstantiationException ex)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : InstantiationException ", ex);
                //log.error("createWrapperInstance(" + objClass + ") : InstantiationException ");
            } catch (IllegalAccessException ex)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : IllegalAccessException ", ex);
                //log.error("createWrapperInstance(" + objClass + ") : IllegalAccessException ");
            } catch (Throwable t)
            {
                throw new TelosysRuntimeException("createWrapperInstance(" + wrapperClass + ") : Throwable " + t.toString(), t);
                //log.error("createWrapperInstance(" + objClass + ") : Throwable " + t.toString());
            }
        }
        else
        {
            throw new TelosysRuntimeException("createWrapperInstance(null) : class is null");
        }
    }

    //-----------------------------------------------------------------------------------
    private Class tryToLoadWrapperClass(String sClassName) // v 1.0.0
    {
        trace("tryToLoadWrapperClass('" + sClassName + "')" );
        try
        {
            Class c = Class.forName(sClassName);
            return c ; // FOUND and LOADED
        } catch (ClassNotFoundException ex)
        {
            //--- Class not found : it's not an error ( there's no wrapper for this kind of object )
            return null ; // NOT FOUND
        } catch (Throwable t)
        {
            //--- Other error cases : abnormal => ERROR
            throw new TelosysRuntimeException("cannot load XML wrapper class '" + sClassName + "' : Throwable " + t.toString(), t);
        }
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Try to load a Wrapper class from the given bean class name, using the "old fashion" class description
     * ( wrapper class name = "BeanClass" + "WrapperSuffix" )
     * @param psBeanClassName
     * @return the class loaded or null if not found
     */
    private Class loadDefaultWrapperClassOldFashion(String psBeanClassName)
    {
        trace("loadDefaultWrapperClass(" + psBeanClassName + ")");
        
//        String sWrapperPackage = "demo.xmlwrapper" ;
//        String sWrapperSuffix  = "XmlWrapper" ;

        //--- Wrapper class package
        String sWrapperPackage = TelosysRPL.getXmlWrapperPackage() ;
        if ( sWrapperPackage == null )
        {
            return null ;  // No package = no default wrapper => NOT FOUND        
        }
        if ( ! sWrapperPackage.endsWith("."))
        {
            sWrapperPackage = sWrapperPackage + "." ;
        }

        //--- Wrapper class suffix
        String sWrapperSuffix  = TelosysRPL.getXmlWrapperSuffix() ;
        if ( sWrapperSuffix == null )
        {
            sWrapperSuffix = "" ;  // No suffix         
        }
        
        String[] r = psBeanClassName.split("\\.");
        int n = r.length ;
        if ( n > 0 )
        {
            String s = r[n-1];
            if ( s != null )
            {
                Class clWrapper = null ;
                String sWrapperClassName = sWrapperPackage + s + sWrapperSuffix ;          
                return tryToLoadWrapperClass(sWrapperClassName); // v 1.0.0
                /**
                trace("loadDefaultWrapperClass(" + psBeanClassName + ") : class name = '" + sWrapperClassName + "'" );
                try
                {
                    clWrapper = Class.forName(sWrapperClassName);
                } catch (ClassNotFoundException ex)
                {
                    //--- Class not found : it's not an error ( there's no wrapper for this kind of object )
                    clWrapper = null ; // NOT FOUND
	            } catch (Throwable t)
	            {
                    //--- Other error cases : abnormal => ERROR
	                throw new TelosysRuntimeException("cannot load XML wrapper class '" + sWrapperClassName + "' : Throwable " + t.toString(), t);
	            }
                return clWrapper ;
                **/
            }
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Try to load a wrapper class using the class name pattern
     * @param beanClass
     * @return the class or null
     * @since 1.0.0
     */
    private Class loadByClassNamePattern( Class beanClass ) // Class parameter cannot be null
    {
        if ( $ClassNamePattern == null && $bFlagGetPattern == false )
        {
            trace("loadByClassNamePattern(" + beanClass + ") : try to get pattern ...");
            //--- Get pattern for the first and last time 
            $ClassNamePattern = getClassNamePattern();
            $bFlagGetPattern = true ;
        }
        trace("loadByClassNamePattern(" + beanClass + ") : pattern = '" + $ClassNamePattern + "'");
        if ( $ClassNamePattern != null )
        {
            //--- Name of the bean
            String sBeanName = Tool.getBeanName(beanClass);
            
            //--- Wrapper class name for the given bean name ( by subtitution in pattern )
            String sWrapperClassName = Tool.buildClassName($ClassNamePattern, sBeanName);

            //--- 
            return tryToLoadWrapperClass(sWrapperClassName); // v 1.0.0
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------
    /**
     * Try to find the XML Wrapper for the given Java class 
     * Search in the wrapper registry, if not found try to load 
     * the default wrapper class  
     * @param beanClass 
     * @return the XML Wrapper ( or null if wrapper not found )
     */
    private IXmlWrapper findWrapper(Class beanClass)
    {
        if (beanClass == null)
        {
            error("findWrapper(Class) : class argument is null ");
            return null;
        }
        String sBeanClassName = beanClass.getName();
        trace("findWrapper(" + sBeanClassName + ")...");
        
        //--- 1) Search a wrapper in the registry 
        Object obj = $hmWrappersByBeanClass.get(sBeanClassName);
        if (obj != null)
        {
            //--- Wrapper FOUND in the registry => use it  
            return (IXmlWrapper) obj ; // v 1.0.0
//            trace("getSpecificWrapper(" + sClassName + ") : CLASS FOUND");   
//            if ( obj instanceof IXmlWrapper )
//            {
//                return (IXmlWrapper) obj ;
//            }
//            else
//            {
//                throw new TelosysRuntimeException("registry corrupted : wrapper for '"+sClassName+"' is not a IXmlWrapper ");
//            }
        }

        //--- Not found => try to load and register a Wrapper class based on the bean class name
        trace("findWrapper(" + sBeanClassName + ") : not found in registry => try to load default class...");            
        Class wrapperClass = null ;
        
        //--- 2) Using the "Class Name Pattern"
        wrapperClass = loadByClassNamePattern(beanClass);
        if (wrapperClass != null)
        {
            //--- Create the wrapper instance 
            IXmlWrapper wrapper = createWrapperInstance(wrapperClass);
            //--- Register it to reuse it in the future 
            registerInstance(wrapper);
            return wrapper ;
        }
        
        //--- 3) Using the "old fashion" Package name + Suffix
        wrapperClass = loadDefaultWrapperClassOldFashion(sBeanClassName);
        if (wrapperClass != null)
        {
            //--- Create the wrapper instance 
            IXmlWrapper wrapper = createWrapperInstance(wrapperClass);
            //--- Register it to reuse it in the future 
            registerInstance(wrapper);
            return wrapper ;
        }
        
        //--- NOT FOUND 
        return null;
    }

    //-----------------------------------------------------------------------------------
    // IXmlWrapperProvider implementation
    //-----------------------------------------------------------------------------------
    
    //-----------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.rpl.xml.IXmlWrapperProvider#getWrapper(java.lang.Class)
     */
    public IXmlWrapper getWrapper(Class beanClass) 
    {
        if (beanClass == null)
        {
            throw new TelosysRuntimeException("getWrapper(Class c) : class argument is null ");
        }
        trace("getWrapper( '" + beanClass.getName() + "' )");
        
        //--- Try to find a Wrapper for this bean 
        IXmlWrapper wrapper = findWrapper(beanClass);
        if (wrapper == null)
        {
            trace("wrapper not found");
        }
        else
        {
            trace("wrapper found : " + wrapper.getClass() );
        }
        return wrapper;
    }

    //-----------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.rpl.xml.IXmlWrapperProvider#getWrapper(java.lang.Object)
     */
    public IXmlWrapper getWrapper(Object beanInstance)
    {
        trace("getWrapper(beanInstance)");
        if (beanInstance == null)
        {
            throw new TelosysRuntimeException("getWrapper(Object o) : object argument is null ");
        }
        Class cl = beanInstance.getClass();
        return getWrapper(cl);
    }

    //-----------------------------------------------------------------------------------
}