package org.objectweb.telosys.rpl.xml;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * @author Laurent GUERIN
 *  
 */
public final class XmlSaxParser extends TelosysObject
{

    /** */
    private String         _sParserClass = null;

    /** */
    private ContentHandler _saxHandler   = null;

    //---------------------------------------------------------------------------
    /**
     * Constructor
     * @param handler :
     */
    public XmlSaxParser(final ContentHandler handler)
    {
        _saxHandler = handler;
        _sParserClass = Telosys.getSAXParserClassName();
    }

    //---------------------------------------------------------------------------
    /**
     * Constructor
     * @param handler :
     * @param sClass :
     */
    public XmlSaxParser(final ContentHandler handler, final String sClass)
    {
        _saxHandler = handler;
        _sParserClass = sClass;
    }

    //---------------------------------------------------------------------------
    /**
     * Parse the input stream using the given charset
     * @param stream : the stream to parse
     * @param sCharset : the charset to use ( utf-8, iso-8859-1, ... )
     * @throws TelosysException
     */
    public void parse(final InputStream stream, String sCharset) throws TelosysException
    {
        trace("parse(InputStream)...");
        try
        {
            //--- Try to create an XML reader (SAX 2 parser)
            // NB: SAX parser instances are not thread safe
            XMLReader myXmlReader = null;
            if (_sParserClass != null)
            {
                //--- Attempt to create an XML reader from the given class name
                myXmlReader = XMLReaderFactory.createXMLReader(_sParserClass);
            }
            else
            {
                //--- Attempt to create an XML reader from a system property
                myXmlReader = XMLReaderFactory.createXMLReader();
            }

            if (myXmlReader == null)
            {
                throw new TelosysException("Cannot get SAX parser : XMLReaderFactory.createXMLReader(" + _sParserClass + ")");
            }
            trace("XMLReader created.");

            if (_saxHandler == null)
            {
                throw new TelosysException("SAX handler is null ");
            }
            //--- Set the SAX handler SAX
            myXmlReader.setContentHandler(_saxHandler);
            trace("SAX handler set.");
            //myXmlReader.setErrorHandler(ErrorHandler h);
            //myXmlReader.setProperty("xx", obj);

            //--- Build an InputSource from the given XML InputStream
            InputSource source = new InputSource(stream);

            //--- Set the XML InputSource charset 
            source.setEncoding(sCharset); // "UTF-8" or "ISO-8859-1"
            
            //--- Parse the XML InputSource
            trace("XMLReader.parse(InputSource) ... ( encoding = '" + source.getEncoding() + "' ) ");            
            myXmlReader.parse(source);

        } catch (SAXException exSAX)
        {
            if (exSAX.getException() != null)
            {
                Exception embeddedException = exSAX.getException();
                if (embeddedException instanceof TelosysException)
                {
                    throw (TelosysException) embeddedException;
                }
                else if (embeddedException instanceof TelosysRuntimeException)
                {
                    throw (TelosysRuntimeException) embeddedException;
                }
                else
                {
                    throw new TelosysException("Error in parsing XML (SAXException) : " + exSAX.getMessage(), embeddedException);
                }
            }
            else
            {
                throw new TelosysException("Error in parsing XML (SAXException) : " + exSAX.getMessage(), exSAX);
            } 
        } catch (IOException e)
        {
            throw new TelosysException("Error in parsing XML (IOException) : " + e.getMessage(), e);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Parse the given string, using the given charset
     * @param stringXml : the XML string to parse 
     * @param sCharset : the charset to use 
     * @throws TelosysException
     */
    public void parseString(final String stringXml, String sCharset) throws TelosysException
    {
        try
        {
            // Encodes the given String into a sequence of bytes using the
            // named charset, storing the result into a new byte array            
            //byte[] bytesXml = stringXml.getBytes("UTF8");
            byte[] bytesXml = stringXml.getBytes(sCharset);
            if (bytesXml == null)
            {
                throw new TelosysException("Error in parseString() : stringXml.getBytes('UTF8') returns NULL ");
            }
            InputStream streamXml = new ByteArrayInputStream(bytesXml);
            this.parse(streamXml, sCharset);
        } catch (UnsupportedEncodingException e)
        {
            throw new TelosysException("Error in parseString() : UnsupportedEncodingException ", e);
        } catch (TelosysException e)
        {
            // Just re-throw
            throw e;
        } catch (Exception e)
        {
            throw new TelosysException("Error in parseString()", e);
        }
    }
    //---------------------------------------------------------------------------
}