package org.objectweb.telosys.screen.core;

import java.io.Serializable;

import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public interface ScreenContext extends Serializable
{
    //---------------------------------------------------------------------------
    // get the basic screen elements ( name, session, ... )
    //---------------------------------------------------------------------------
    /**
     * Returns the symbolic screen context name 
     * @return the screen context name
     */
    public String getScreenName();

    /**
     * Returns the ID of the screen context 
     * @return : screen context ID ( 0 to N )
     */
    public int getScreenId();

    //---------------------------------------------------------------------------
    // getters for Session & Application : do not use if session serialization 
    // => deprecated in v 0.9.9
    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication instance 
     * @return the ScreenApplication
     * @deprecated use ScreenApplicationManager.getScreenApplication() instead
     */
    public ScreenApplication getScreenApplication(); // Not used in Telosys framework
    
    /**
     * Returns the ScreenSession to which the ScreenContext belongs  
     * @return the ScreenSession
     * @deprecated use ScreenRequest|ServiceRequest.getScreenSession() instead
     */
    public ScreenSession getScreenSession(); // Not used in Telosys framework

    /**
     * Returns the standard HttpSession to which the ScreenContext belongs   
     * @return
     * @deprecated use ScreenRequest|ServiceRequest.getHttpSession() instead
     */
    public HttpSession getHttpSession(); // Not used in Telosys framework

    //---------------------------------------------------------------------------
    // ScreenData management 
    //---------------------------------------------------------------------------
    /**
     * Returns the current ScreenData of the context (can be null)
     * @return : the current context data or null
     */
    public ScreenData getCurrentData();

    /**
     * Creates a new ScreenData instance and returns it.<br>
     * NB: the ScreenData created is not set as the current ScreenData of the context 
     * @return : the new ScreenData created
     */
    public ScreenData createNewScreenData();

    /**
     * Returns the current ScreenData of the context if it's not null, <br>
     * if the current ScreenData is null : it is created, set as current and returned
     * @return : the current data (never null)
     */
    public ScreenData getOrCreateCurrentData();

    /**
     * Sets the given ScreenData as the new current ScreenData of the context 
     * @param data the new ScreenData to set
     */
    public void setCurrentData(ScreenData data);

    /**
     * Remove the current ScreenData of the context <br>
     * After this operation, getCurrentData() returns 'null' 
     */
    public void clearCurrentData();
    
    /**
     * Notify the screen context that an action is about to alter the current screen data<br>
     * The action needs the current ScreenData instance to work with<br>
     * The screen context keep an image of the current ScreenData<br>
     * to do a "cancel" if needed
     * @return the current screen data 
     */
    public ScreenData beginAction() ;
    
    /**
     * Notify the screen context that the current action must be canceled<br>
     * The screen context restore the original ScreenData (from the ScreenData image) <br>
     * 
     */
    public void cancelAction() ;

    /**
     * Returns a temporary ScreenData instance for the operations which are <br> 
     * working without modifying the current screen data 
     * @return : a temporary ScreenData instance
     */
    public ScreenData getTempData() ;

    /**
     * Creates and returns a new ScreenData instance for 'load' operations 
     * @return : ScreenData instance
     */
    public ScreenData getDataForLoad() ;
    
    //---------------------------------------------------------------------------
    // Screen Context attributes
    //---------------------------------------------------------------------------
    /**
     * Sets an object (attribute) in the 'screen context storage space', associated with a unique name <br>
     * If name is null : do nothing <br>
     * If value is null : remove the attribute associated with the given name
     * @param name : a string specifying the name of the object 
     * @param value : the value  
     * @return : the previous value of the specified key in this hashtable, or null if it did not 
	 have one
     */
    public Object setAttribute(String name, Object value);

    /**
     * Gets an object (attribute) from the 'screen context storage space' with the given name <br>
     * If name is null : return null
     * @param name : a string specifying the name of the object 
     * @return : value instance ( or null if key is null or not found ) 
     */
    public Object getAttribute(String name);

    /**
     * Removes an object (attribute) from the 'screen context storage space' <br>
     * If name is null : do nothing and return null
     * @param name : a string specifying the name of the object 
     * @return : the value removed, or null if not found
     */
    public Object removeAttribute(String name);

    //---------------------------------------------------------------------------

    /**
     * Returns the specific ScreenActions associated with the ScreenContext
     * @return : the ScreenActions (or null)
     */
    public ScreenActions getScreenActions();
    
    /**
     * Returns the ScreenManager associated with the ScreenContext  <br>
     * This method replaces the "getScreenDataAccessor" method,<br>
     * 
     * @deprecated just for backward compatibility (will be removed), do not use
     * @return
     */
    public ScreenManager getScreenManager();
    
    /**
     * Returns the current ScreenActions if not null, <br>
     * if null the current ScreenActions is created, set as current and returned
     * @return : the ScreenActions (never null)
     * @throws TelosysException if the ScreenActions cannot be created
     */
    public ScreenActions getOrCreateScreenActions() throws TelosysException ;

    //---------------------------------------------------------------------------
    /**
     * Returns the state of the "reused" flag
     * @return
     */
    public boolean hasBeenReused(); // Since v 0.9.9
    
    /**
     * ScreenContext 'create' event 
     * @param params
     * @throws TelosysException
     */
    //public void eventCreate(ScreenRequestParameters params) throws TelosysException ; // Since v 0.9.9
    public void eventCreate(ScreenSession screenSession, ScreenRequestParameters params) throws TelosysException ; // Since v 0.9.9
    
    /**
     * ScreenContext 'reuse' event 
     * @param params
     * @throws TelosysException
     */
    //public void eventReuse(ScreenRequestParameters params) throws TelosysException ; // Since v 0.9.9
    public void eventReuse(ScreenSession screenSession, ScreenRequestParameters params) throws TelosysException ; // Since v 0.9.9
}