package org.objectweb.telosys.screen.core;

import java.io.Serializable;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author L. Guerin
 *
 */
/* package */ class ScreenContextDefinition implements Serializable
{    
	private static final long serialVersionUID = 1L;

    private final static String ERR_SCREEN      = "ScreenProvider : cannot create screen ";

    private final static String ERR_SCREEN_DATA = "ScreenProvider : cannot create screen data ";
    
    //----------------------------------------------------------------------------------
    // Screen NAME 
    //----------------------------------------------------------------------------------
    private String              _screenName; // serializable (std in Java)

    //----------------------------------------------------------------------------------
    // Screen DATA 
    //----------------------------------------------------------------------------------
    private Class               _screenDataClass; // serializable (std in Java)

    //----------------------------------------------------------------------------------
    // Standard screen ACTIONS sub-components 
    //----------------------------------------------------------------------------------
    private Class               _screenManagerClass; // serializable (std in Java)

    private Class               _screenTriggersClass; // serializable (std in Java)

    private Class               _screenProceduresClass; // serializable (std in Java)

    //----------------------------------------------------------------------------------
    // Specific screen ACTIONS 
    //----------------------------------------------------------------------------------
    private Class               _specificScreenActionsClass; // serializable (std in Java)

    //----------------------------------------------------------------------------------
    /**
     * Constructor
     * 
     * @param screenName
     * @param screenDataClass
     * @param screenManagerClass
     * @param screenActionsClass
     * @param screenTriggersClass
     * @param screenProceduresClass
     */
    public ScreenContextDefinition(String screenName, Class screenDataClass, Class screenManagerClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass) throws TelosysException
    {
        final String sErrMsg = "Register Screen '" + screenName + "' : Invalid class for ";
      
        try
        {
            //--- Screen Name ------------------------------------
            _screenName = screenName;
            if (_screenName == null)
            {
                throw new TelosysException("Register Screen : Screen name is NULL ");
            }

            //--- ScreenData class ------------------------------------
            _screenDataClass = screenDataClass;
            if (_screenDataClass != null)
            {
                if (ScreenData.class.isAssignableFrom(_screenDataClass) != true)
                {
                    throw new TelosysException(sErrMsg + "ScreenData ");
                }
            }

            //--- ScreenDAO class ------------------------------------
            _screenManagerClass = screenManagerClass;
            if (_screenManagerClass != null)
            {
                if (ScreenManager.class.isAssignableFrom(_screenManagerClass) != true)
                {
                    throw new TelosysException(sErrMsg + "ScreenManager ");
                }
            }
            
            //--- ScreenActions class ------------------------------------
            _specificScreenActionsClass = screenActionsClass;
            if (_specificScreenActionsClass != null)
            {
                if (ScreenActions.class.isAssignableFrom(_specificScreenActionsClass) != true)
                {
                    throw new TelosysException(sErrMsg + "ScreenActions ");
                }
            }
            
            //--- ScreenTriggers class ------------------------------------                
            _screenTriggersClass = screenTriggersClass;
            if (_screenTriggersClass != null)
            {
                if (ScreenTriggers.class.isAssignableFrom(_screenTriggersClass) != true)
                {
                    throw new TelosysException(sErrMsg + "ScreenTriggers ");
                }
            }
            
            //--- ScreenProcedures class ------------------------------------                
            _screenProceduresClass = screenProceduresClass;
            if (_screenProceduresClass != null)
            {
                if (ScreenProcedures.class.isAssignableFrom(_screenProceduresClass) != true)
                {
                    throw new TelosysException(sErrMsg + "ScreenProcedures ");
                }
            }

        } catch (Throwable e)
        {
            throw new TelosysException("Register Screen '"+ _screenName + "' : Exception", e);
        }
    }

    /**
     * Create an instance of ScreenContext (without screen data)
     * 
     * @return
     * @throws TelosysException
     */
    protected ScreenContext createScreenContext(ScreenSession screenSession, int iScreenId ) throws TelosysException
    {
        createScreenActions() ;
        StandardScreenContext screen = new StandardScreenContext(_screenName, this, screenSession, iScreenId ) ;// v 0.9.9
        return screen ;
    }

    /**
     * Create an instance of ScreenData
     * 
     * @return
     * @throws TelosysException
     */
    protected ScreenData createScreenData() throws TelosysException
    {
        try
        {
            //--- Create the screen data instance with the class
            ScreenData screenData = (ScreenData) _screenDataClass.newInstance();
            if (screenData != null)
            {
                return screenData;
            }
            else
            {
                throw new TelosysException(ERR_SCREEN_DATA + _screenName + " (class.newInstance return null)");
            }
        } catch (InstantiationException e)
        {
            throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
        } catch (IllegalAccessException e)
        {
            throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
        } catch (Throwable e)
        {
            throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
        }
    }

    protected ScreenActions createScreenActions() throws TelosysException
    {
        ScreenActions screenActions = null ;
        try
        {
	        if ( _specificScreenActionsClass != null )
	        {
	            //--- SPECIFIC ScreenActions implementation 
	            screenActions = (ScreenActions) _specificScreenActionsClass.newInstance();
	        }
	        else
	        {
	            //--- STANDARD ScreenActions with specific sub-components
	            ScreenManager manager = null ;
	            ScreenTriggers triggers = null ;
	            ScreenProcedures procedures = null ;
	            
	            //--- Create the screen DATA ACCESSOR if any
	            if (_screenManagerClass != null)
	            {
	                manager = (ScreenManager) _screenManagerClass.newInstance() ;
	            }
	            //--- Create the screen TRIGGERS if any
	            if (_screenTriggersClass != null)
	            {
	                triggers = (ScreenTriggers) _screenTriggersClass.newInstance();
	            }
	            //--- Create the screen PROCEDURES if any
	            if (_screenProceduresClass != null)
	            {
	                procedures = (ScreenProcedures) _screenProceduresClass.newInstance();
	            }
	            
	            //--- Create the STANDARD ScreenActions instance with its sub-components
	            screenActions = new StandardScreenActions(manager, triggers, procedures);
	        }
	    } catch (InstantiationException e)
	    {
	        throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
	    } catch (IllegalAccessException e)
	    {
	        throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
	    } catch (Throwable e)
	    {
	        throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
	    }
	    
	    return screenActions ;
    }
}

