package org.objectweb.telosys.screen.core;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public interface ScreenManager
{
    /**
     * Returns a new DatabaseSession instance corresponding to  <br>
     * the DatabaseId associated with the current screen context.<br>
     * NB : <br> 
     * . each call returns a NEW INSTANCE <br>
     * This instance is not the same as the DatabaseSession parameter used in<br>
     * the ScreenManager methods, and it is not stored as thread-local variable.<br>
     * . it's the programmer's reponsability to managed the DatabaseSession return <br>
     * e.g. : this DatabaseSession must be closed after use.
     * 
     * In classical cases, this method is not supposed to be used 
     *  
     * @return
     * @throws TelosysException
     */
    public DatabaseSession getDatabaseSession() throws TelosysException;

    /**
     * Returns the Database Id used by the ScreenManager 
     * . 0 to N : the Databse Id specified
     * . DEFAULT_DATABASE : the default database id defined in the Data Access Layer
     * . NO_DATABASE : the ScreenManager is not supposed to use a database
     * @return
     * @since v 1.0.2
     */
    public int getDatabaseId() ;
    
    //--- New method ( v 0.9.7 )
    /**
     * @param screenData
     * @param screenContext
     * @param param
     * @param dbSession
     * @throws TelosysException
     * @deprecated no longer used by the framework, do not use
     */
    public void doInit(ScreenData screenData, ScreenContext screenContext, Map param, DatabaseSession dbSession) throws TelosysException ;

    /**
     * Entry point for "load" action  
     * @param screenData
     * @param screenContext
     * @param request
     * @param response
     * @param session
     * @throws TelosysException
     */
    public void doLoad(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    /**
     * Entry point for "save" action  
     * @param screenData
     * @param screenContext
     * @param request
     * @param response
     * @param session
     * @throws TelosysException
     */
    public void doSave(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    /**
     * Entry point for "insert" action  
     * @param screenData
     * @param screenContext
     * @param request
     * @param response
     * @param session
     * @throws TelosysException
     */
    public void doInsert(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    /**
     * Entry point for "update" action  
     * @param screenData
     * @param screenContext
     * @param request
     * @param response
     * @param session
     * @throws TelosysException
     */
    public void doUpdate(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    /**
     * Entry point for "delete" action 
     * @param screenData
     * @param screenContext
     * @param request
     * @param response
     * @param session
     * @throws TelosysException
     */
    public void doDelete(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;
    
    
    //--- New method ( v 0.9.9 )
    //public void eventCreate(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession ) throws TelosysException ;
    //-- New parameter ( v 1.0.2 )
    /**
     * Entry point for a "create event", origin : ScreenContextManager ("open" or "use" and not found)
     * @param screenContext
     * @param params
     * @param screenSession
     * @param session
     * @throws TelosysException
     */
    public void eventCreate(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession, DatabaseSession session ) throws TelosysException ;

    //--- New method ( v 0.9.9 )
    //public void eventReuse(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession ) throws TelosysException ;
    //-- New parameter ( v 1.0.2 )
    /**
     * Entry point for a "reuse event", origin : ScreenContextManager ("use" and found)
     * @param screenContext
     * @param params
     * @param screenSession
     * @param session
     * @throws TelosysException
     */
    public void eventReuse(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession, DatabaseSession session ) throws TelosysException ;
    
}