package org.objectweb.telosys.screen.core;

import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.dao.DatabaseSessionManager;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 * 
 * This class is the default implementation of the ScreenActions interface. <br>
 * This is the standard behavior of a screen context. <br>
 * 
 * This is a "composite" using external "components" : <br>
 * . ScreenManager <br>
 * . ScreenTriggers <br>
 * . ScreenProcedures <br>
 *  
 */
public class StandardScreenActions extends TelosysObject implements ScreenActions
{
    //-------------------------------------------------------------------------------------
    // The 3 components used by the standard actions  
    //-------------------------------------------------------------------------------------
    private ScreenManager      _screenManager = null ;
    
    private ScreenTriggers     _screenTriggers = null ;
    
    private ScreenProcedures   _screenProcedures = null ;
    
    
    //-------------------------------------------------------------------------------------
    // List actions extensions   
    //-------------------------------------------------------------------------------------
    private StandardScreenActionsList _actionsList = new StandardScreenActionsList();

    
    //-------------------------------------------------------------------------------------
    /**
     * 
     */
    public StandardScreenActions( ScreenManager manager, ScreenTriggers triggers, ScreenProcedures procedures )
    {
        super();
        _screenManager      = manager ;
        _screenTriggers     = triggers ;
        _screenProcedures   = procedures ;
        trace("CONSTRUCTOR");
    }
    
    //-------------------------------------------------------------------------------------
    /**
     * Provide a ScreenTriggers implementation <br>
     * Return the specific Screen Triggers if it exists or a void implementation
     * @return
     */
    private ScreenTriggers getScreenTriggers()
    {
        trace("getScreenTriggers()");
        if ( _screenTriggers != null )
        {
            return _screenTriggers ;
        }
        else
        {
            _screenTriggers = new StandardScreenTriggers();
            return _screenTriggers ;
        }
    }


    //-------------------------------------------------------------------------------------
    /**
     * Returns the ScreenManager if any
     * NB : just for backward compatibility
     * In future version this method will become 'private'
     * @return
     * @throws TelosysException
     */
    protected ScreenManager getScreenManager() throws TelosysException
    {
        trace("getScreenManager()");
        if ( _screenManager != null)
        {
            return _screenManager;
        }
        else
        {
            // If we try to get it => we need it ! 
            throw new TelosysException("StandardScreenActions : No ScreenManager for this screen");
        }
    }

    //-------------------------------------------------------------------------------------
    private ScreenProcedures getScreenProcedures() throws TelosysException
    {
        trace("getScreenProcedures()");
        if ( _screenProcedures != null)
        {
            return _screenProcedures;
        }
        else
        {
            // If we try to get it => we need it ! 
            throw new TelosysException("StandardScreenActions : No ScreenProcedures for this screen");
        }
    }

    //-------------------------------------------------------------------------------------
    private DatabaseSession getDatabaseSession( ScreenManager screenManager ) throws TelosysException
    {
        int iDbId = screenManager.getDatabaseId();
        return DatabaseSessionManager.getThreadLocalDatabaseSession(iDbId);
    }
    

    //-------------------------------------------------------------------------------------
    private void close(DatabaseSession dbSession)
    {
        if (dbSession != null)
        {
            try
            {
                dbSession.close();
            } catch (TelosysException e)
            {
                error("close(DatabaseSession)", e);
            }
        }
    }

    //-------------------------------------------------------------------------------------
    private void rollback(DatabaseSession dbSession)
    {
        if (dbSession != null)
        {
            try
            {
                dbSession.rollback();
            } catch (TelosysException e)
            {
                error("rollback(DatabaseSession)", e);
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the given element instance and apply its binding if any
     * 
     * @param screenData
     * @param sElementName
     * @param oInstance
     */
    private void setElementInstance(ScreenData screenData, String sElementName, Object oInstance)
    {
        trace("setElementInstance( ScreenData, '" + sElementName + "', Object )");
        //--- Set the element instance
        screenData.setElement(sElementName, oInstance);

        //--- Retrieve the element
        ScreenDataElement element = screenData.getScreenDataElement(sElementName);
        if (element != null)
        {
            //--- Apply binding if any
            element.applyBinding();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the given ScreenData Elements from the Request DataElements and apply binding for each
     * Called by actions which set the screen context elements ( SET, SAVE, INSERT, UPDATE, ... )
     * 
     * @param screenData
     * @param request
     */
    private void setElements(ScreenData screenData, ScreenRequest request) throws TelosysException
    {
        trace("setElements(ScreenData, ScreenRequest)");
        ScreenDataElements requestDataElements = request.getDataElements();
        if ( requestDataElements != null )
        {            
	        int n = requestDataElements.size();
	        for ( int i = 0 ; i < n ; i++ ) // for each data element of the request
	        {
	            ScreenDataElement requestElement = requestDataElements.get(i);
//	            if (request.hasElementsAttribute()) // the request has an "elements" attribute
//	            {
//	                //--- Set the element only if present in the "elements" list
//	                if (request.elementsAttributeContains(requestElement.getName()))
//	                {
//	                    setElementInstance(screenData, requestElement.getName(), requestElement.getInstance());
//	                }
//	            }
//	            else
//	            {
//	                //--- No "elements" attribute => always set the element instance
//	                setElementInstance(screenData, requestElement.getName(), requestElement.getInstance());
//	            }
                //--- Always set the element instance
                setElementInstance(screenData, requestElement.getName(), requestElement.getInstance());
	        }
        }
        else
        {
            //--- No DATA ELEMENTS in the REQUEST 
            throw new TelosysException("No Data Elements in the request : Cannot set elements in the screen context !");
        }
    }

    /**
     * Set the response data elements from the given screen context<br>
     * 
     * Set only the required elements if there is a "elements" attribute in the request<br>
     * @param screenContext
     * @param request
     * @param response
     */
    private void setResponseDataElements(ScreenContext screenContext, ScreenRequest request, ScreenResponse response)
    {
        ScreenData currentScreenData = screenContext.getCurrentData();
        if (request.getElements() != null) 
        {
            //--- The request has an "elements" attribute to get a specific set of elements
            if (currentScreenData != null)
            {
                //--- Attribute "elements" in request => get only those elements
                String[] reqElements = request.getElementsArray(); // Requested elements names
                ScreenDataElements responseElements = new ScreenDataElements(); // Void list
                if ( reqElements != null )
                {                    
	                for ( int i = 0 ; i < reqElements.length ; i++ )
	                {
	                    //--- Get the element name (from request parameters)
	                    String sElementName = reqElements[i];
	                    //--- Get the screen data element associated with this name
	                    ScreenDataElement oScreenDataElement = currentScreenData.getScreenDataElement(sElementName);
	                    if (oScreenDataElement != null)
	                    {
	                        //--- Add the screen data element to the response elements list
	                        responseElements.add(oScreenDataElement);
	                    }
	                }
                }
                //--- Set the selected elements as the response elements list
                response.setDataElements(responseElements);
            }
        }
        // NEW in v 0.9.8 : No "elements" attribute => no elements returned in the response 
        /***
        else
        {
            //--- No "elements" in request => get all elements
            response.setDataElements(currentScreenData);
        }
        ***/
        
    }
    
    //===========================================================================
    // ScreenContext events
    //===========================================================================    
    public void eventCreate(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession) throws TelosysException
    {
        // Delegates the event to the ScreenData Manager if any
        // ScreenData Manager not required ( if no Manager => no event ) 
        if ( _screenManager != null )
        {
            //_screenManager.eventCreate(screenContext, params, screenSession);
            //--- New in v 1.0.2
            DatabaseSession dbSession = getDatabaseSession( _screenManager ); // v 1.0.2
            try {
                _screenManager.eventCreate(screenContext, params, screenSession, dbSession); // v 1.0.2
            }
            catch ( TelosysException ex) {
                throw ex ; // re-throw 
            }
            catch ( Throwable t) {
                String s = "ScreenContext : "+screenContext.getScreenId()+"-"+screenContext.getScreenName();
                throw new TelosysException("Error in 'onCreate' event (" + s + ")", t);
            }
            finally {
                close(dbSession);
            }
        }
    }
    
    public void eventReuse(ScreenContext screenContext, ScreenRequestParameters params, ScreenSession screenSession) throws TelosysException
    {
        // Delegates the event to the ScreenData Manager if any
        // ScreenData Manager not required ( if no Manager => no event ) 
        if ( _screenManager != null )
        {
            //_screenManager.eventReuse(screenContext, params, screenSession);
            //--- New in v 1.0.2
            DatabaseSession dbSession = getDatabaseSession( _screenManager ); // v 1.0.2
            try {
                _screenManager.eventReuse(screenContext, params, screenSession, dbSession); // v 1.0.2
            }
            catch ( TelosysException ex) {
                throw ex ; // re-throw 
            }
            catch ( Throwable t) {
                String s = "ScreenContext : "+screenContext.getScreenId()+"-"+screenContext.getScreenName();
                throw new TelosysException("Error in 'onReuse' event (" + s + ")", t);
            }
            finally {
                close(dbSession);
            }
        }
    }
    
    //===========================================================================
    // Actions without persistence
    //===========================================================================

    //---------------------------------------------------------------------------
    // Perform action "SET"
    //---------------------------------------------------------------------------
    public void actionSet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionSet()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers();

        /**
         * //--- Get the data received in the request to do the SET operation (with parameters set) ScreenData newData = request.getData();
         */
        //ScreenData newData = screen.getDataForAction();
        //ScreenData oldData = screen.getCurrentData();
        ScreenData newData = screen.beginAction();

        //--- Populate elements instances from the request
        setElements(newData, request);

        boolean bCanceled = false;
        //--- Check Business RULES "before SET "
        if (triggers.beforeSet(newData, screen, request, response) == false)
        {
            //--- Action canceled by Trigger
            bCanceled = true;
        }
        else
        {
            //--- Set the new data as the current screen data
            screen.setCurrentData(newData);

            //--- Check Business RULES "after SET"
            if (triggers.afterSet(newData, screen, request, response) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
                //--- Restore the original data in the screen context
                /*
                 * screen.setCurrentData(oldData);
                 */
                screen.cancelAction();
            }
        }

        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if (bCanceled)
        {
            //--- Action RETURN : "canceled" (by Trigger)
            response.setActionReturn(ScreenResponse.ACTION_CANCELED);
        }
        else
        {
            //--- Action RETURN : OK
            response.setActionReturn(ScreenResponse.ACTION_OK);
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "GET"
    //---------------------------------------------------------------------------
    public void actionGet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionGet()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers();

        boolean bCanceled = false;
        //--- Check Business RULES "before GET"
        if (triggers.beforeGet(screen, request, response) == false)
        {
            //--- Action canceled by Trigger
            bCanceled = true;
        }
        else
        {
            //--- Get the current screen data and put it in the response
            //response.setData(screen.getCurrentData());
            //--- Set the "elements to retrieve" in the response
            setResponseDataElements(screen, request, response);
            
//            ScreenData currentScreenData = screen.getCurrentData();
//            if (request.getElements() != null) // the request has an "element" attribute
//            {
//                if (currentScreenData != null)
//                {
//                    //--- Attribute "elements" in request => get only those elements
//                    String[] reqElements = request.getElementsArray(); // Requested elements names
//                    ScreenDataElements responseElements = new ScreenDataElements(); // Void list
//                    for ( int i = 0 ; i < reqElements.length ; i++ )
//                    {
//                        //--- Get the element name (from request parameters)
//                        String sElementName = reqElements[i];
//                        //--- Get the screen data element associated with this name
//                        ScreenDataElement oScreenDataElement = currentScreenData.getScreenDataElement(sElementName);
//                        if (oScreenDataElement != null)
//                        {
//                            //--- Add the screen data element to the response elements list
//                            responseElements.add(oScreenDataElement);
//                        }
//                    }
//                    //--- Set the selected elements as the response elements list
//                    response.setDataElements(responseElements);
//                }
//            }
//            else
//            {
//                //--- No "elements" in request => get all elements
//                response.setDataElements(currentScreenData);
//            }

            //--- Check Business RULES "after GET"
            if (triggers.afterGet(screen, request, response) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
                //--- No data returned in the response
                //response.setData(null);
                response.setDataElements((ScreenDataElements) null);
            }
        }

        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if (bCanceled)
        {
            //--- Action RETURN : "canceled" (by Trigger)
            response.setActionReturn(ScreenResponse.ACTION_CANCELED);
        }
        else
        {
            //--- Action RETURN : OK
            response.setActionReturn(ScreenResponse.ACTION_OK);
        }

    }

    //---------------------------------------------------------------------------
    // Perform action "CLEAR"
    //---------------------------------------------------------------------------
    public void actionClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionClear()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers();

        //--- Keep a reference on the original screen data (for restore if necesary)
        ScreenData originalData = screen.getCurrentData();

        boolean bCanceled = false;
        //--- Check Business RULES "before CLEAR"
        if (triggers.beforeClear(screen, request, response) == false)
        {
            //--- Action canceled by Trigger
            bCanceled = true;
        }
        else
        {
            //--- CLEAR the screnn context data
            screen.clearCurrentData();

            //--- Check Business RULES "after CLEAR"
            if (triggers.afterClear(screen, request, response) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
                //--- Restore the original data in the screen context
                screen.setCurrentData(originalData);
            }
        }

        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if (bCanceled)
        {
            //--- Action RETURN : "canceled" (by Trigger)
            response.setActionReturn(ScreenResponse.ACTION_CANCELED);
        }
        else
        {
            //--- Action RETURN : OK
            response.setActionReturn(ScreenResponse.ACTION_OK);
        }
    }

    //===========================================================================
    // Actions with persistence
    //===========================================================================
    //---------------------------------------------------------------------------
    // Perform action "LOAD"
    //---------------------------------------------------------------------------
    public void actionLoad(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionLoad()");
        //--- Get a Database session
        DatabaseSession dbSession = null;
        try
        {
            boolean bCanceled = false;
            //--------------------------------------------------
            //--- Try to LOAD
            //--------------------------------------------------
            //--- Get the DAO for this screen
            ScreenManager screenManager = getScreenManager();
            //--- Get a Database session
            //dbSession = screenManager.getDatabaseSession();
            dbSession = getDatabaseSession( screenManager ); // v 1.0.2
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers();
            //--- Get the data to use for load operation ( initialized by copy from current data )
            ScreenData newData = screen.getDataForLoad();

            //--- Check Business RULES "before LOAD"
            if (triggers.beforeLoad(newData, screen, request, response, dbSession) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
            }
            else
            {
                //--- LOAD OPERATION (use the DAO to load data from the Database)
                screenManager.doLoad(newData, screen, request, response, dbSession);

                //--- Check Business RULES "after LOAD"
                if (triggers.afterLoad(newData, screen, request, response, dbSession) == false)
                {
                    //--- Action canceled by Trigger
                    bCanceled = true;
                }
            }

            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if (bCanceled)
            {
                //--- Action RETURN : "canceled" (by Trigger)
                response.setActionReturn(ScreenResponse.ACTION_CANCELED);
            }
            else
            {
                //--- Action RETURN : OK
                response.setActionReturn(ScreenResponse.ACTION_OK);

                //--- Use the data loaded (if found)
                if (newData.isFound())
                {
                    //--- Set the current screen data if the request flag SET is TRUE
                    if (request.getFlagSetData() == true) //-- SET="true"
                    {
                        //--- Set the data loaded as the current screen
                        screen.setCurrentData(newData);
                    }
                    //--- Set the response flag "FOUND"
                    response.setFlagFound();
                    //--- Return the data in the response if flag "GET" is TRUE
                    if (request.getFlagGetData() == true) //-- GET="true"
                    {
                        //--- Return the data loaded in the response
                        setResponseDataElements(screen, request, response); 
                    }
                }
                else
                {
                    //--- Set the response flag "NOT FOUND"
                    response.setFlagNotFound();
                }
            }

        } catch (TelosysException tex)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : TelosysException ", tex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(tex);
        } catch (Exception ex)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : Exception ", ex);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(ex);
        } catch (Throwable th)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : Throwable ", th);
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            //dbSession.close();
            close(dbSession); // v 0.9.4
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "SAVE"
    //---------------------------------------------------------------------------
    public void actionSave(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse) throws TelosysException
    {
        trace("actionSave()");
        
        if (screenRequest.getFlagSetData() == true)
        {
            //--- SET and SAVE
            //--- 0) Get the "current ScreenData" to use ( after a copy in a "ScreenData image" for cancel capacity )
            ScreenData currentScreenData = screenContext.beginAction(); 
            //--- 1) SET the received data in the ScreenContext DATA
            setElements(currentScreenData, screenRequest);
            // USELESS : screenContext.setCurrentData(currentScreenData);
	        //--- 2) SAVE the Current Data 
	        boolean bOK = save(screenContext, screenRequest, screenResponse, currentScreenData);
	        if (bOK != true)
	        {
	            //--- Restore the original "current ScreenData" ( from the "ScreenData image" )
	            screenContext.cancelAction(); 
	        }
        }
        else
        {
            //--- DO NOT SET, only SAVE
            //--- 0) Get a temporary working data  
            ScreenData tempData = screenContext.getTempData(); 
            //--- 1) SET the received data in the temporary DATA
            setElements(tempData, screenRequest);
	        //--- 2) SAVE the temporary DATA
	        save(screenContext, screenRequest, screenResponse, tempData);
	        // NOTHING to CANCEL if NOT OK
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "SAVE_CURRENT"
    //---------------------------------------------------------------------------
    public void actionSaveCurrent(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse) throws TelosysException
    {
        trace("actionSaveCurrent()");
        // No request flag "SET" to managed
        //--- Save the current data of the screen
        ScreenData screenDataToSave = screenContext.getCurrentData();
        save(screenContext, screenRequest, screenResponse, screenDataToSave);
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "SAVE and "SAVE_CURRENT"
    //---------------------------------------------------------------------------
    private boolean save(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse, ScreenData screenDataToSave) throws TelosysException
    {
        trace("save()");
        DatabaseSession dbSession = null; 
        boolean bCanceled = false;
        try
        {
            //--------------------------------------------------
            //--- Try to SAVE
            //--------------------------------------------------
            //--- Get the DA for this screen
            ScreenManager screenManager = getScreenManager();
            //--- Get a Database session
            //dbSession = screenManager.getDatabaseSession();
            dbSession = getDatabaseSession( screenManager ); // v 1.0.2
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers();
            //--- Check Business RULES "before SAVE"
            if (triggers.beforeSave(screenDataToSave, screenContext, screenRequest, screenResponse, dbSession) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
            }
            else
            {
                //--- SAVE OPERATION (use the DAO to save the data in database)
                screenManager.doSave(screenDataToSave, screenContext, screenRequest, screenResponse, dbSession);
                //--- Check Business RULES "after SAVE"
                if (triggers.afterSave(screenDataToSave, screenContext, screenRequest, screenResponse, dbSession) == false)
                {
                    //--- Action canceled by Trigger
                    bCanceled = true;
                }
            }

            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if (bCanceled)
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();
                //--- Action RETURN : "canceled" (by Trigger)
                screenResponse.setActionReturn(ScreenResponse.ACTION_CANCELED);
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();
                //--- Action RETURN : OK
                screenResponse.setActionReturn(ScreenResponse.ACTION_OK);
                //--- Set the requested elements in the response ( if any )
                setResponseDataElements(screenContext, screenRequest, screenResponse); // NEW ( v 0.9.8 )
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession);
            error("save() : TelosysException", tex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(tex);
        } catch (Exception ex)
        {
            rollback(dbSession);
            error("save() : Exception", ex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession);
            error("save() : Throwable", th);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession);
        }
        return (!bCanceled);
    }

    //---------------------------------------------------------------------------
    // Perform action "INSERT"
    //---------------------------------------------------------------------------
    public void actionInsert(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse) throws TelosysException
    {
        trace("actionInsert()");
        if ( screenRequest.getFlagSetData() == true )
        {
            //--- SET and INSERT
            //--- 0) Get the "current ScreenData" to use ( after a copy in a "ScreenData image" for cancel capacity )
            ScreenData currentScreenData = screenContext.beginAction(); 
            //--- 1) SET the received data in the ScreenContext DATA
            setElements(currentScreenData, screenRequest);
            // USELESS : screenContext.setCurrentData(currentScreenData);
	        //--- 2) INSERT the Current Data 
	        boolean bOK = insert(screenContext, screenRequest, screenResponse, currentScreenData);
	        if (bOK != true)
	        {
	            //--- Restore the original "current ScreenData" ( from the "ScreenData image" )
	            screenContext.cancelAction(); 
	        }
        }
        else
        {
            //--- DO NOT SET, only SAVE
            //--- 0) Get a temporary working data  
            ScreenData tempData = screenContext.getTempData(); 
            //--- 1) SET the received data in the temporary DATA
            setElements(tempData, screenRequest);
	        //--- 2) INSERT the temporary DATA
            insert(screenContext, screenRequest, screenResponse, tempData);
	        // NOTHING to CANCEL if NOT OK
        }        
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "INSERT" 
    //---------------------------------------------------------------------------
    private boolean insert(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse, ScreenData screenDataToInsert) throws TelosysException
    {
        trace("insert()");
        DatabaseSession dbSession = null;
        boolean bCanceled = false;
        try
        {
            //--------------------------------------------------
            //--- Try to INSERT
            //--------------------------------------------------
            //--- Get the DAO for this screen
            ScreenManager screenManager = getScreenManager();
            //--- Get a Database session
            //dbSession = screenManager.getDatabaseSession();
            dbSession = getDatabaseSession( screenManager ); // v 1.0.2
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers();
            //--- Check Business RULES "before INSERT"
            if (triggers.beforeInsert(screenDataToInsert, screenContext, screenRequest, screenResponse, dbSession) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
            }
            else
            {
                //--- INSERT OPERATION (use the DA to insert the data in database)
                screenManager.doInsert(screenDataToInsert, screenContext, screenRequest, screenResponse, dbSession);
                //--- Check Business RULES "after INSERT"
                if (triggers.afterInsert(screenDataToInsert, screenContext, screenRequest, screenResponse, dbSession) == false)
                {
                    //--- Action canceled by Trigger
                    bCanceled = true;
                }
            }

            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if (bCanceled)
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();
                //--- Action RETURN : "canceled" (by Trigger)
                screenResponse.setActionReturn(ScreenResponse.ACTION_CANCELED);
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();
                //--- Action RETURN : OK
                screenResponse.setActionReturn(ScreenResponse.ACTION_OK);
                //--- Set the requested elements in the response ( if any )
                setResponseDataElements(screenContext, screenRequest, screenResponse); // NEW ( v 0.9.8 )                
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession);
            error("insert() : TelosysException ", tex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(tex);
            if (tex.getCause() != null)
            {
                Throwable t = tex.getCause();
                screenResponse.addError(t);
                if (t instanceof SQLException)
                {
                    SQLException sqlEx = (SQLException) t;
                    String sMsg = "SQL Error : SQL State = " + sqlEx.getSQLState() + " - Vendor-specific error code = " + sqlEx.getErrorCode();
                    ScreenError error = new ScreenError(0, sMsg);
                    screenResponse.addError(error);
                }
            }
        } catch (Exception ex)
        {
            rollback(dbSession);
            error("insert() : Exception ", ex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession);
            error("insert() : Throwable ", th);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession); 
        }
        return (!bCanceled);
    }

    //---------------------------------------------------------------------------
    // Perform action "UPDATE"
    //---------------------------------------------------------------------------
    public void actionUpdate(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse) throws TelosysException
    {
        trace("actionUpdate()");
        
        if (screenRequest.getFlagSetData() == true)
        {
            //--- SET and UPDATE
            //--- 0) Get the "current ScreenData" to use ( after a copy in a "ScreenData image" for cancel capacity )
            ScreenData currentScreenData = screenContext.beginAction(); 
            //--- 1) SET the received data in the ScreenContext DATA
            setElements(currentScreenData, screenRequest);
            // USELESS : screenContext.setCurrentData(currentScreenData);
	        //--- 2) UPDATE the Current Data 
	        boolean bOK = update(screenContext, screenRequest, screenResponse, currentScreenData);
	        if (bOK != true)
	        {
	            //--- Restore the original "current ScreenData" ( from the "ScreenData image" )
	            screenContext.cancelAction(); 
	        }
        }
        else
        {
            //--- DO NOT SET, only UPDATE
            //--- 0) Get a temporary working data  
            ScreenData tempData = screenContext.getTempData(); 
            //--- 1) SET the received data in the temporary DATA
            setElements(tempData, screenRequest);
	        //--- 2) UPDATE the temporary DATA
            update(screenContext, screenRequest, screenResponse, tempData);
	        // NOTHING to CANCEL if NOT OK
        }        
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "UPDATE" 
    //---------------------------------------------------------------------------
    private boolean update(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse, ScreenData screenDataToUpdate) throws TelosysException
    {
        trace("update()");
        DatabaseSession dbSession = null;
        boolean bCanceled = false;
        try
        {
            //--------------------------------------------------
            //--- Try to UPDATE
            //--------------------------------------------------
            //--- Get the DAO for this screen
            ScreenManager screenManager = getScreenManager(); 
            //--- Get a Database session
            //dbSession = screenManager.getDatabaseSession();
            dbSession = getDatabaseSession( screenManager ); // v 1.0.2
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers();
            //--- Check Business RULES "before UPDATE"
            if (triggers.beforeUpdate(screenDataToUpdate, screenContext, screenRequest, screenResponse, dbSession) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
            }
            else
            {
                //--- INSERT OPERATION (use the DAO to insert the data in database)
                screenManager.doUpdate(screenDataToUpdate, screenContext, screenRequest, screenResponse, dbSession);
                //--- Check Business RULES "after UPDATE"
                if (triggers.afterUpdate(screenDataToUpdate, screenContext, screenRequest, screenResponse, dbSession) == false)
                {
                    //--- Action canceled by Trigger
                    bCanceled = true;
                }
            }

            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if (bCanceled)
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();
                //--- Action RETURN : "canceled" (by Trigger)
                screenResponse.setActionReturn(ScreenResponse.ACTION_CANCELED);
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();
                //--- Action RETURN : OK
                screenResponse.setActionReturn(ScreenResponse.ACTION_OK);
                //--- Set the requested elements in the response ( if any )
                setResponseDataElements(screenContext, screenRequest, screenResponse); // NEW ( v 0.9.8 )                
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession); // v 0.9.4
            error("update() : TelosysException", tex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(tex);
            if (tex.getCause() != null)
            {
                Throwable t = tex.getCause();
                screenResponse.addError(t);
                if (t instanceof SQLException)
                {
                    SQLException sqlEx = (SQLException) t;
                    String sMsg = "SQL Error : SQL State = " + sqlEx.getSQLState() + " - Vendor-specific error code = " + sqlEx.getErrorCode();
                    ScreenError error = new ScreenError(0, sMsg);
                    screenResponse.addError(error);
                }
            }
        } catch (Exception ex)
        {
            rollback(dbSession); // v 0.9.4
            error("update() : Exception", ex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession); // v 0.9.4
            error("update() : Throwable", th);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession); // v 0.9.4
        }
        return (!bCanceled);
    }

    //---------------------------------------------------------------------------
    // Perform action "DELETE"
    //---------------------------------------------------------------------------
    public void actionDelete(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionDelete()");
        //--- Work on Temporay Data for DELETE operation
        ScreenData dataToDelete = screen.getTempData(); 
        //--- Call the common DELETE operation
        delete(screen, request, response, dataToDelete);
    }

    //---------------------------------------------------------------------------
    // Perform action "DELETE_CURRENT"
    //---------------------------------------------------------------------------
    public void actionDeleteCurrent(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionDeleteCurrent()");
        ScreenData dataToDelete = screen.getCurrentData();
        //--- Call the common DELETE operation
        delete(screen, request, response, dataToDelete);
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "DELETE and "DELETE_CURRENT"
    //---------------------------------------------------------------------------
    private void delete(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse, ScreenData screenDataToDelete) throws TelosysException
    {
        trace("delete()");
        DatabaseSession dbSession = null;

        try
        {
            boolean bCanceled = false;
            //--------------------------------------------------
            //--- Try to DELETE
            //--------------------------------------------------
            //--- Get the DAO for this screen
            ScreenManager screenManager = getScreenManager();
            //--- Get a Database session
            //dbSession = screenManager.getDatabaseSession();
            dbSession = getDatabaseSession( screenManager ); // v 1.0.2
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers();

            //--- Check Business RULES "before DELETE"
            if (triggers.beforeDelete(screenDataToDelete, screenContext, screenRequest, screenResponse, dbSession) == false)
            {
                //--- Action canceled by Trigger
                bCanceled = true;
            }
            else
            {
                //--- DELETE OPERATION (use the DA to delete data in the Database)
                screenManager.doDelete(screenDataToDelete, screenContext, screenRequest, screenResponse, dbSession);
                //--- Check Business RULES "after DELETE"
                if (triggers.afterDelete(screenDataToDelete, screenContext, screenRequest, screenResponse, dbSession) == false)
                {
                    //--- Action canceled by Trigger
                    bCanceled = true;
                }
            }

            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if (bCanceled)
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();
                //--- Action RETURN : "canceled" (by Trigger)
                screenResponse.setActionReturn(ScreenResponse.ACTION_CANCELED);
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();
                //--- Request Flag "clear=true/false"
                if (screenRequest.getFlagClearData() == true)
                {
                    //--- "CLEAR Flag" : clear the current screen data
                    screenContext.clearCurrentData();
                }
                //--- Action RETURN : OK
                screenResponse.setActionReturn(ScreenResponse.ACTION_OK);
                //--- Set the requested elements in the response ( if any )
                setResponseDataElements(screenContext, screenRequest, screenResponse); // NEW ( v 0.9.8 )                
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession);
            error("delete() : TelosysException ", tex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(tex);
        } catch (Exception ex)
        {
            rollback(dbSession);
            error("delete() : Exception ", ex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession);
            error("delete() : Throwable ", th);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession);
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "LIST_SELECT"
    //---------------------------------------------------------------------------
    public void actionListSelect(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListSelect(screen, request, response);
    }

    public void actionListInsert(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListInsertOrAdd(screen, request, response, StandardScreenActionsList.ACTION_LIST_INSERT);
    }

    public void actionListAdd(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListInsertOrAdd(screen, request, response, StandardScreenActionsList.ACTION_LIST_ADD);
    }

    public void actionListRemove(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListRemove(screen, request, response);
    }

    public void actionListClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListClear(screen, request, response);
    }

    public void actionListSort(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        _actionsList.actionListSort(screen, request, response);
    }

    //---------------------------------------------------------------------------
    // Perform action "EXEC"
    //---------------------------------------------------------------------------
    public void actionExec(ScreenContext screenContext, ScreenRequest screenRequest, ScreenResponse screenResponse) throws TelosysException
    {
        //--- Retrieve the Procedure Name from the request
        String sProcedureName = screenRequest.getProcName();
        trace("exec() : " + sProcedureName);

        if (sProcedureName == null)
        {
            //--- Action RETURN : ERROR
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : No procedure name (ProcName is null)", ScreenMessage.ERROR);
            screenResponse.addMessage(msg);
            return;
        }

        try
        {
            //--------------------------------------------------
            //--- Try to EXECUTE the procedure
            //--------------------------------------------------
            //--- Get the procedures for this screen
            //ScreenProcedures procedures = screenContext.getScreenProcedures();
            ScreenProcedures procedures = getScreenProcedures();            
            if (procedures != null)
            {
                boolean bProcRet = procedures.execute(sProcedureName, screenContext, screenRequest, screenResponse);
                if (bProcRet == true)
                {
                    //--- Action RETURN : OK
                    screenResponse.setActionReturn(ScreenResponse.ACTION_OK);
                    //--- Set the requested elements in the response ( if any )
                    setResponseDataElements(screenContext, screenRequest, screenResponse); // NEW ( v 0.9.9 )                
                    
                }
                else
                {
                    //--- Action RETURN : ERROR
                    screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
                    ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : Procedure execution return false ", ScreenMessage.ERROR);
                    screenResponse.addMessage(msg);
                }
            }
            else // never happends ( throw Ex if null )
            {
                //--- Action RETURN : ERROR
                screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
                ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : Procedures not found for this screen (Screen class = " + screenContext.getClass().getName(), ScreenMessage.ERROR);
                screenResponse.addMessage(msg);
            }
        } catch (TelosysException tex)
        {
            error("actionExec() : TelosysException ", tex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(tex);
        } catch (Exception ex)
        {
            error("actionExec() : Exception ", ex);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(ex);
        } catch (Throwable th)
        {
            error("actionExec() : Throwable ", th);
            screenResponse.setActionReturn(ScreenResponse.ACTION_ERROR);
            screenResponse.addError(th);
        } finally
        {
            //--- Nothing to do
            //dbSession.close();
        }

    }

}