package org.objectweb.telosys.screen.core;

import java.util.List;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.util.StrUtil;

/**
 * This class is the default implementation of the List Management Actions
 * 
 * @author Laurent GUERIN
 * 
 *  
 */
public class StandardScreenActionsList extends TelosysObject //implements ScreenActions
{
    public final static int    ACTION_LIST_INSERT  = 1;

    public final static int    ACTION_LIST_ADD     = 2;

    public final static String VALUE_CURRENT_INDEX = "current-index";

    public final static String VALUE_LIST_SIZE     = "list-size";

    /**
     * Returns the list name from the request ( parameter "list" )
     * 
     * @param request
     * @return
     * @throws TelosysException
     */
    private String getListName(ScreenRequest request) throws TelosysException
    {
        String s = request.getParameter("list");
        if (s == null)
        {
            throw new TelosysException("No 'list' parameter in the request");
        }
        return s;
    }

    /**
     * Returns the item name from the request ( parameter "item" )
     * 
     * @param request
     * @return
     * @throws TelosysException
     */
    private String getItemName(ScreenRequest request) throws TelosysException
    {
        String s = request.getParameter("item");
        if (s == null)
        {
            throw new TelosysException("No 'item' parameter in the request");
        }
        return s;
    }

    /**
     * Returns the VOList for the list name send in the request
     * 
     * @param screen
     * @param request
     * @return
     * @throws TelosysException
     */
    private GenericVOList getVOList(ScreenContext screen, ScreenRequest request) throws TelosysException
    {
        ScreenData screenData = screen.getCurrentData();
        String sListName = getListName(request);
        Object oList = screenData.getElement(sListName);
        if (oList == null)
        {
            throw new TelosysException("List '" + sListName + "' not found in the screen context ");
        }
        if (!(oList instanceof GenericVOList))
        {
            throw new TelosysException("Screen element '" + sListName + "' is not an instance of GenericVOList ");
        }
        return (GenericVOList) oList;
    }

    //    /**
    //     * Returns the standard Java List of the VOList for the list name send in the request
    //     *
    //     * @param screen
    //     * @param request
    //     * @return
    //     * @throws TelosysException
    //     */
    //    private List getList(ScreenContext screen, ScreenRequest request) throws TelosysException
    //    {
    //        GenericVOList voList = getVOList(screen, request);
    //        //--- Get the standard Java List
    //        List list = voList.getList();
    //        if (list == null)
    //        {
    //            throw new TelosysException("Cannot get standard list from 'VO-List' ");
    //        }
    //        return list;
    //    }

    /**
     * Returns the "old index" value from the request ( parameter "old-index" )
     * 
     * @param request
     * @param list
     * @return
     * @throws TelosysException
     */
    private int getOldIndex(ScreenRequest request, List list) throws TelosysException
    {
        int iIndex = request.getIntParameter("old-index");
        if (iIndex < -1)
        {
            throw new TelosysException("Invalid 'old index' : " + iIndex + " < -1 ");
        }
        if (iIndex >= list.size())
        {
            throw new TelosysException("Invalid 'old index' : " + iIndex + " >= " + list.size());
        }
        return iIndex;
    }

    /**
     * Returns the "new index" value from the request ( parameter "new-index" )
     * 
     * @param request
     * @param list
     * @return
     * @throws TelosysException
     */
    private int getNewIndex(ScreenRequest request, List list) throws TelosysException
    {
        int iIndex = request.getIntParameter("new-index");
        if (iIndex < 0)
        {
            throw new TelosysException("Invalid 'new index' : " + iIndex + " < 0 ");
        }
        if (iIndex >= list.size())
        {
            throw new TelosysException("Invalid 'new index' : " + iIndex + " >= " + list.size());
        }
        return iIndex;
    }

    private void updateListItem(ScreenRequest request, List list, String sItemName, int iOldIndex) // throws TelosysException
    {
        if (iOldIndex >= 0) // There was a current selected row before => update item
        {
            ScreenDataElements reqElements = request.getDataElements();
            int n = reqElements.size();
            for ( int i = 0 ; i < n ; i++ ) // for each data element of the request
            {
                ScreenDataElement e = reqElements.get(i);
                if (sItemName.equals(e.getName())) //--- This is the LIST ITEM
                {
                    //--- Set the old-index item instance
                    list.set(iOldIndex, e.getInstance());
                    break;
                }
            }
        }
    }

    private void returnItem(ScreenResponse response, Object oItem, String sItemName) // throws TelosysException
    {
        if (oItem != null)
        {
            //--- Response elements list
            ScreenDataElements elements = new ScreenDataElements();
            //--- Add the element in the response elements list
            elements.add(sItemName, oItem);
            response.setDataElements(elements);
        }
    }

    private Object newElementInstance(GenericVOList voList) throws TelosysException
    {
        //--- Get element type from the list ( element class )
        Class cl = voList.getElementType();
        if (cl == null)
        {
            throw new TelosysException("Cannot get element type from 'VO-List' " + voList.getClass().getName());
        }

        //--- Create a new instance of element
        Object oNewItem = null;
        try
        {
            oNewItem = cl.newInstance();
        } catch (InstantiationException e)
        {
            throw new TelosysException("Cannot create element : InstantiationException ( class = '" + cl.getName() + "'", e);
        } catch (IllegalAccessException e)
        {
            throw new TelosysException("Cannot create element : IllegalAccessException ( class = '" + cl.getName() + "'", e);
        }
        return oNewItem;
    }

    //---------------------------------------------------------------------------
    // Perform action "LIST_SELECT"
    //---------------------------------------------------------------------------
    public void actionListSelect(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionListSelect()");

        // NEW
        // List list = getList(screen, request);
        //--- Get the VO List
        GenericVOList voList = getVOList(screen, request);
        //--- Get the standard Java List
        List list = voList.getList();
        if (list == null)
        {
            throw new TelosysException("Cannot get standard list from 'VO-List' ");
        }

        String sItemName = getItemName(request);
        int iOldIndex = getOldIndex(request, list);
        int iNewIndex = getNewIndex(request, list);

        //--- Set the data of the old index
        updateListItem(request, list, sItemName, iOldIndex);

        //--- Return the data of the new index
        //returnItem(response, list.get(iNewIndex), sItemName);
        voList.select(iNewIndex); // NEW
        returnItem(response, voList.getSelectedElement(), sItemName);

        //--- Action RETURN : OK
        response.setActionReturn(ScreenResponse.ACTION_OK);
        response.setValue(VALUE_CURRENT_INDEX, iNewIndex);
        response.setValue(VALUE_LIST_SIZE, list.size());
    }

    //---------------------------------------------------------------------------
    public void actionListInsertOrAdd(ScreenContext screen, ScreenRequest request, ScreenResponse response, int iAction) throws TelosysException
    {
        trace("actionListInsertOrAdd(" + iAction + ")");
        //--- Get the VO List
        GenericVOList voList = getVOList(screen, request);
        //--- Get the standard Java List
        List list = voList.getList();
        if (list == null)
        {
            throw new TelosysException("Cannot get standard list from 'VO-List' ");
        }

        String sItemName = getItemName(request);
        int iOldIndex = getOldIndex(request, list);

        //--- Update "old-index" item if necessary
        updateListItem(request, list, sItemName, iOldIndex);

        //--- Create a new instance of element
        Object oNewItem = newElementInstance(voList);

        int iCurrentIndex = -1;
        if (iAction == ACTION_LIST_INSERT)
        {
            int iNewIndex = getNewIndex(request, list);
            //--- Insert the new item before the "new index"
            trace("list.add(" + iNewIndex + ", newItem)");
            list.add(iNewIndex, oNewItem);
            iCurrentIndex = iNewIndex;
        }
        else if (iAction == ACTION_LIST_ADD)
        {
            //--- Add the new item at the end of the list
            trace("list.add(newItem)");
            list.add(oNewItem);
            iCurrentIndex = list.size() - 1;
        }

        //--- Return the new void data
        returnItem(response, oNewItem, sItemName);
        
        voList.select(iCurrentIndex);
        
        //--- Action RETURN : OK
        response.setActionReturn(ScreenResponse.ACTION_OK);
        //response.setValue(VALUE_CURRENT_INDEX, iCurrentIndex);
        response.setValue(VALUE_CURRENT_INDEX, voList.getSelectedIndex() );
        response.setValue(VALUE_LIST_SIZE, list.size());
    }

    //---------------------------------------------------------------------------
    public void actionListRemove(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionListRemove()");
        //--- Get the VO List
        GenericVOList voList = getVOList(screen, request);
        //--- Get the standard Java List
        List list = voList.getList();
        if (list == null)
        {
            throw new TelosysException("Cannot get standard list from 'VO-List' ");
        }

        String sItemName = getItemName(request);
        int iOldIndex = getOldIndex(request, list);
        int iIndexToDelete = getNewIndex(request, list);
        int iCurrentIndex = -1;

        //--- Set the data of the old index
        updateListItem(request, list, sItemName, iOldIndex);

        //--- Remove the element at the given position
        list.remove(iIndexToDelete);

        if (iIndexToDelete == iOldIndex)
        {
            // The current line has been removed => find new current line
            //--- Same algo as the "client side"
            if (list.size() > 0)
            {
                if (iIndexToDelete < list.size())
                {
                    iCurrentIndex = iIndexToDelete;
                }
                else
                {
                    iCurrentIndex = list.size() - 1;
                }
                //--- Return the data of the new index
                returnItem(response, list.get(iCurrentIndex), sItemName);
            }
            else
            {
                //--- After the remove operation the list is void
                iCurrentIndex = -1; // No current index (list is void)
                //--- Create a new instance of element
                Object oVoidItem = newElementInstance(voList);
                //--- Return a void item
                returnItem(response, oVoidItem, sItemName);
            }
        }
        else
        {
            //--- Keep the same current row
            if (iIndexToDelete < iOldIndex) // before current index
            {
                iCurrentIndex = iOldIndex - 1; // The current row goes up => correct index
            }
            else
            {
                iCurrentIndex = iOldIndex; // The current stay at the same place => keep same index
            }
        }
        //--- Set the new selected index
        if (iCurrentIndex >= 0)
        {
            voList.select(iCurrentIndex);
        }
        else
        {
            voList.unselect(); // No current item
        }

        //--- Action RETURN : OK
        response.setActionReturn(ScreenResponse.ACTION_OK);
        //response.setValue(VALUE_CURRENT_INDEX, iCurrentIndex);
        response.setValue(VALUE_CURRENT_INDEX, voList.getSelectedIndex() );
        response.setValue(VALUE_LIST_SIZE, list.size());
    }

    //---------------------------------------------------------------------------
    public void actionListClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionListClear()");
        //--- Get the VO List
        GenericVOList voList = getVOList(screen, request);
        //--- Clear the list
        voList.clear();
        //--- Unselect the current list item
        voList.unselect();

        //--- Action RETURN : OK
        response.setActionReturn(ScreenResponse.ACTION_OK);
        response.setValue(VALUE_CURRENT_INDEX, -1); // No current index
        response.setValue(VALUE_LIST_SIZE, 0); // Void => size = 0
    }

    //---------------------------------------------------------------------------
    /**
     * Sort the VOList using the given 'field name' as criteria
     * @param request
     * @param voList
     * @param sFieldName 
     * @throws TelosysException
     */
    private void sortVOList(ScreenRequest request, GenericVOList voList, String sFieldName ) throws TelosysException
    {
        //--- Get the standard Java List
        List list = voList.getList();
        if (list == null)
        {
            throw new TelosysException("Cannot get standard list from 'VO-List' ");
        }
        
        //--- 2) Set the data of the old index
        String sItemName = getItemName(request);
        int iOldIndex = getOldIndex(request, list);
        updateListItem(request, list, sItemName, iOldIndex);
        //--- Unselect the current list item
        voList.unselect();
        
        //--- 3) Set the sort order parameter
        String sOrder = request.getParameter("order");
        if (sOrder == null)
        {
            sOrder = "asc"; // By default order is ASCENDING
            if (voList.isSortedBy(sFieldName))
            {
                //--- Allready sorted by this field => Reverse sort order
                if (voList.isSortedInAscendingOrder())
                {
                    sOrder = "desc";
                }
            }
        }

        //--- 4) Try to sort the list
        try
        {
            trace("sort list(" + sFieldName + "," + sOrder + ")");
            voList.sortIgnoreCase(sFieldName, sOrder); // Ignore case by default
        } catch (RuntimeException ex)
        {
            throw new TelosysException("Cannot sort VOList (field:" + sFieldName + ",order:" + sOrder + ")", ex);
        }
    }
    //---------------------------------------------------------------------------
    /**
     * Sort the DataSet unsing the given 'column' as criteria
     * @param request
     * @param ds
     * @param sColumn
     * @throws TelosysException
     */
    private void sortDataSet(ScreenRequest request, DataSet ds, String sColumn ) throws TelosysException
    {
        // DataSet => No current item refresh + unselect ( just SORT )
        //--- Get the column 
        int iCol = StrUtil.getInt(sColumn, -1);
        if ( iCol < 0 )
        {
            throw new TelosysException("Cannot sort DataSet : invalid column '" + sColumn + "' parameter");
        }
        //--- Set the sort order parameter
        String sOrder = request.getParameter("order");
        if (sOrder == null)
        {
            sOrder = "asc"; // By default order is ASCENDING
            if (ds.isSortedBy(iCol))
            {
                //--- Allready sorted by this field => Reverse sort order
                if (ds.isSortedInAscendingOrder())
                {
                    sOrder = "desc";
                }
            }
        }

        //--- Try to sort the DataSet
        try
        {
            trace("sort DataSet(" + iCol + "," + sOrder + ")");
            ds.sortIgnoreCase(iCol, sOrder); // Ignore case by default
        } catch (RuntimeException ex)
        {
            throw new TelosysException("Cannot sort DataSet (column:" + iCol + ",order:" + sOrder + ")", ex);
        }
    }
    //---------------------------------------------------------------------------
    //---------------------------------------------------------------------------
    public void actionListSort(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionListSort()");
        int iListSize = 0 ;

        //--- 1) Get the "list name" & "field name" parameters
        String sListName = getListName(request);
        String sColumn = request.getParameter("column");
        if (sColumn == null)
        {
            throw new TelosysException("LIST SORT : No 'column' parameter in the request");
        }

        //--- 2) Get the requested list from the screen context and sort it
        ScreenData screenData = screen.getCurrentData();
        Object oList = screenData.getElement(sListName);
        if (oList == null)
        {
            throw new TelosysException("List '" + sListName + "' not found in the screen context ");
        }
        if ( oList instanceof GenericVOList )
        {
            //--- Sort the VOList
            GenericVOList voList = (GenericVOList) oList ;
            sortVOList( request, voList, sColumn );
            iListSize = voList.size();
        }
        else if ( oList instanceof DataSet )
        {
            //--- Sort the DataSet
            DataSet ds = (DataSet)oList ;
            sortDataSet( request, ds, sColumn );
            iListSize = ds.getRowCount();
        }
        else
        {
            throw new TelosysException("Screen element '" + sListName + "' is not a VOList or DataSet");
        }

//-------------------------------------------------------------------------------------------
        // TODO : set the elements to retrieve in the response ( see StandardScreenActions : private method )
        //setResponseDataElements(screen, request, response);
//-------------------------------------------------------------------------------------------
        
        //--- Action RETURN : OK
        response.setActionReturn(ScreenResponse.ACTION_OK);
        response.setValue(VALUE_CURRENT_INDEX, -1); // No current index ( no selection after a sort )
        response.setValue(VALUE_LIST_SIZE, iListSize );
    }
    //---------------------------------------------------------------------------
}