package org.objectweb.telosys.screen.core;

import java.util.Hashtable;

import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * The standard ScreenContext implementation.
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenContext extends TelosysObject implements ScreenContext
{
	private static final long serialVersionUID = 1L;
    
    private String                  _sScreenName        = null; // serializable (std in Java)

    private int                     _iScreenId          = 0;    // serializable (std in Java)

    private ScreenContextDefinition _screenDefinition   = null ; // serializable ( v 0.9.9 )
    
    private ScreenSession           _screenSession      = null; // circular reference <!> => usage DEPRECATED

    private ScreenData              _currentScreenData  = null; // serializable ( v 0.9.9 )

    private ScreenData              _screenDataImage    = null; // serializable ( v 0.9.9 )

    private transient ScreenActions _screenActions      = null; // NOT DATA => TRANSIENT

    //--- Screen Data used for LOAD operation
    private transient ScreenData   _newDataLoaded      = null; // TEMP => TRANSIENT

    //--- Reusable temporary Screen Data (used only for DELETE operations)
    private transient ScreenData   _tempData           = null; // TEMP => TRANSIENT

    //--- Screen context attributes
    private Hashtable              _htAttributes       = new Hashtable(); // serializable (std in Java)
    
    private boolean                _bHasBeenReused     = false ;  // v 0.9.9 // serializable (std in Java)

    /**
     * StandardScreenContext constructor
     * @param sScreenName the symbolic screen name
     * @param screenDefinition the definition of the screen context
     * @param screenSession the screen session
     * @param iScreenId the screen ID ( 0 to N )
     */
    public StandardScreenContext(String sScreenName, ScreenContextDefinition screenDefinition, ScreenSession screenSession, int iScreenId) // v 0.9.4
    {
        _currentScreenData = null; // null : no ScreenData
        _sScreenName = sScreenName;
        //_screenDataClass = screenDataClass;
        _screenDefinition = screenDefinition ; // Cannot be null 
        //_screenActions = screenActions ;
        _screenSession = screenSession;
        //--- Set the Screen ID 
        if (_iScreenId >= 0)
        {
            _iScreenId = iScreenId;
        }
        else
        {
            error("ScreenContext creation : initScreen : invalid Screen ID : " + iScreenId);
            _iScreenId = 0;
        }
    }

    //---------------------------------------------------------------------------
    // get the screen name
    //---------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    public int getScreenId()
    {
        return _iScreenId;
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenApplication()
     * <!> DEPRECATED : Do not use _screenSession
     */
    public ScreenApplication getScreenApplication() // Not used in Telosys framework
    {
        return ScreenApplicationManager.getScreenApplication(); // v 1.0.0
        /***
        if (_screenSession != null)
        {
            return _screenSession.getScreenApplication();
        }
        else
        {
            error("getScreenApplication() : ScreenSession instance is null !");
            return null;
        }
        ***/
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenSession()
     * <!> DEPRECATED : Do not use _screenSession
     */
    public ScreenSession getScreenSession() // Not used in Telosys framework
    {
        return _screenSession;
    }

    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getHttpSession()
     * <!> DEPRECATED : Do not use _screenSession
     */
    public HttpSession getHttpSession() // Not used in Telosys framework
    {
        if (_screenSession != null)
        {
            return _screenSession.getHttpSession();
        }
        else
        {
            error("getHttpSession() : ScreenSession instance is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Set the "reused" flag to TRUE ( irreversible )
     * This method is not exposed in the interface
     */
    public void setReused()
    {
        _bHasBeenReused = true ;
    }

    public boolean hasBeenReused()
    {
        return _bHasBeenReused ;
    }
    
    //---------------------------------------------------------------------------
    // get/set/clear the current data of the screen
    //---------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getCurrentData()
     */
    public ScreenData getCurrentData()
    {
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getOrCreateCurrentData()
     */
    public ScreenData getOrCreateCurrentData()
    {
        if (_currentScreenData == null)
        {
            //--- No current data => create it
            _currentScreenData = createNewScreenData();
        }
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#setCurrentData(org.objectweb.telosys.screen.core.ScreenData)
     */
    public void setCurrentData(ScreenData data)
    {
        _currentScreenData = data;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#clearCurrentData()
     */
    public void clearCurrentData()
    {
        _currentScreenData = null;
    }



    /**
     * Injects ACTIONS dependency
     * 
     * @param screenActions
     */
    /* package */void setScreenActions(ScreenActions screenActions)
    {
        _screenActions = screenActions;
    }


    //==============================================================================
    // ABSTRACT METHODS IMPLEMENTATIONS
    //==============================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenActions()
     */
    public ScreenActions getScreenActions()
    {
        return _screenActions;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getOrCreateScreenActions()
     */
    public ScreenActions getOrCreateScreenActions() throws TelosysException
    {
        if ( _screenActions != null ) 
        {
            // The ACTIONS implementation is ready => use it
            return _screenActions ;
        }
        else
        {
            // The ACTIONS implementation is not yet set for this ScreenContext => create it
            _screenActions = _screenDefinition.createScreenActions();
            return _screenActions ;
        }
    }

    //public ScreenManager getScreenDataAccessor()
    public ScreenManager getScreenManager() // v 1.0.0
    {
        if ( _screenActions == null )
        {
            try
            {
                _screenActions = _screenDefinition.createScreenActions();
            } catch (TelosysException e)
            {
                throw new TelosysRuntimeException(e.getMessage(), e.getCause());
            }
        }
        if ( _screenActions instanceof StandardScreenActions )
        {
            StandardScreenActions actions = (StandardScreenActions) _screenActions ;
            ScreenManager da = null ;
            try
            {
                da = actions.getScreenManager();
            } catch (TelosysException e)
            {
                throw new TelosysRuntimeException(e.getMessage());
            }
            return da ;
        }
        else
        {
            throw new TelosysRuntimeException("The ScreenActions implementation is not a StandardScreenActions");
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#createNewScreenData()
     */
    public ScreenData createNewScreenData()
    {
        ScreenData screenData = null;
//        //--- If the class to build screen data is set...
//        if (_screenDataClass != null)
//        {
//            //--- Try to create a valid instance of ScreenData with the given class
//            try
//            {
//                //--- Create the screenData instance with the class
//                Object object = _screenDataClass.newInstance();
//                if (object instanceof ScreenData)
//                {
//                    screenData = (ScreenData) object;
//                }
//            } catch (InstantiationException e)
//            {
//                screenData = null;
//                error("createNewScreenData() : InstantiationException ! ", e);
//            } catch (IllegalAccessException e)
//            {
//                screenData = null;
//                error("createNewScreenData() : IllegalAccessException ! ", e);
//            }
//        }
//        return screenData;
        try
        {
            screenData = _screenDefinition.createScreenData();
        } catch (TelosysException e)
        {
            error("Cannot create ScreenData");
        }
        return screenData ;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#beginAction()
     */
    public ScreenData beginAction()
    {
        //--- Prepare a "ScreenData image" (copy of the "current ScreenData" )
        if (_currentScreenData == null)
        {
            //--- No "current ScreenData", so ...
            // No copy of the original ScreenData
            _screenDataImage = null;
            // New instance of "current ScreenData"
            _currentScreenData = createNewScreenData();
        }
        else
        {
            //--- The "current ScreenData" exists, so ...
            // Ensure that there's a "ScreenData image" ready to use
            if (_screenDataImage == null)
            {
                _screenDataImage = createNewScreenData();
            }
            // Copy the "current ScreenData" in the "ScreenData image"
            //_currentScreenData.copyElementsTo(_screenDataImage);
            _currentScreenData.copyTo(_screenDataImage); // v 0.9.8 ( 08/01/08 )
        }
        //--- Return the "current ScreenData"
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#cancelAction()
     */
    public void cancelAction()
    {
        // _currentScreenData = _screenDataImage; // v 0.9.8 ( 08/01/08 )
        //--- Restore the preserved "ScreenData image" by reverse copy // v 0.9.8 ( 08/01/08 )
        if (_screenDataImage == null)
        {
            //--- Restore the original "null value"
            _currentScreenData = null ; 
        }
        else
        {
            //--- Restore the original state
            _screenDataImage.copyTo(_currentScreenData);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getDataForLoad(org.objectweb.telosys.rpl.ScreenRequest)
     */
    public ScreenData getDataForLoad() //throws TelosysException
    {
        trace("getDataForLoad");
        
//      v 0.9.8 ( 08/01/08 ) : re-use and initialize Data For Load
        //_newDataLoaded = createNewScreenData();

        // Ensure that there's a "ScreenData instance" ready to use
        if (_newDataLoaded == null)
        {
            _newDataLoaded = createNewScreenData();
        }
        
        // Copy the "current ScreenData" in the "ScreenData for load"
        if (_currentScreenData != null)
        {
            _currentScreenData.copyTo(_newDataLoaded); 
        }
        // Reset the "found flag" to "not found"
        _newDataLoaded.setNotFound();
        
        //--- Return the data
        return _newDataLoaded;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getTempData()
     */
    public ScreenData getTempData() 
    {
        trace("getTempData");
        //--- Create or reuse a working ScreenData 
        if (_tempData == null)
        {
            _tempData = createNewScreenData();
        }
        else
        {
            //_tempData.clearElements();
            _tempData.clear(); // v 0.9.8 ( 08/01/08 )
        }

        //--- Return the data
        return _tempData;
    }

    //---------------------------------------------------------------------------
    // Screen Context attributes
    //---------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#setAttribute(java.lang.String, java.lang.Object)
     */
    public Object setAttribute(String name, Object value)
    {
        //--- Avoid NullPointerException if key is null
        //if (name != null && value != null)
        if ( name != null ) // v 1.0.0
        {
            if ( value != null ) // v 1.0.0
            {
                return _htAttributes.put(name, value);
            }
            else
            {
                return removeAttribute(name); // v 1.0.0
            }
        }
        else
        {
            error("setAttribute(name,value) : name is null !");
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getAttribute(java.lang.String)
     */
    public Object getAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.get(name);
        }
        else
        {
            error("getAttribute(name) : name is null !");
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#removeAttribute(java.lang.String)
     */
    public Object removeAttribute(String name)
    {
        //--- Avoid NullPointerException if key is null
        if (name != null)
        {
            return _htAttributes.remove(name);
        }
        else
        {
            error("removeAttribute(name) : name is null !");
            return null;
        }
    }


    //public void eventCreate(ScreenRequestParameters params) throws TelosysException
    //public void eventCreate(ScreenSession screenSession, ScreenRequestParameters params) throws TelosysException
    public void eventCreate(ScreenSession screenSession, ScreenRequestParameters params) throws TelosysException
    {
        ScreenActions actions = getOrCreateScreenActions();
        if ( actions != null )
        {
            //actions.eventCreate(this, params);
            actions.eventCreate(this, params, screenSession);
        }
    }
    
    //public void eventReuse(ScreenRequestParameters params) throws TelosysException
    public void eventReuse(ScreenSession screenSession, ScreenRequestParameters params) throws TelosysException
    {
        ScreenActions actions = getOrCreateScreenActions();
        if ( actions != null )
        {
            //actions.eventReuse(this, params);
            actions.eventReuse(this, params, screenSession);
            setReused();
        }
    }


}