package org.objectweb.telosys.service;

import java.util.Map;
import java.util.Set;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.RequestParameters;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.ScreenPostback;
import org.objectweb.telosys.uil.SessionPostback;

/**
 * Request object for Telosys Services <br>
 * 
 * An instance of this class is created for each request, <br> 
 * and populated with the http request informations 
 * 
 * @author Laurent GUERIN
 *  
 */
public class ServiceRequest extends RequestParameters // TelosysObject 
{
    //private static final String SCREEN_FORM_MARKER = "__TELOSYS_SCREEN_FORM_MARKER__";

    //private static final String POSTBACK_TYPE = "__TELOSYS_POSTBACK_TYPE__";

    private static final int   POSTBACK_NONE    = 0;

    private static final int   POSTBACK_REQUEST = 1;

    private static final int   POSTBACK_SESSION = 2;

    //-----------------------------------------------------------------------------------------------
    // Request parameters
    //-----------------------------------------------------------------------------------------------

    private String             _sServiceName    = null;

    private HttpServletRequest _httpRequest     = null;

    private ScreenSession      _screenSession   = null;

    //--- Optional attributes
    private int                _iPostBackType   = POSTBACK_NONE;

    private String             _sScreenName     = null;

    private String             _sScreenId       = null;

    private String             _sFormName       = null;

    private ScreenPostback     _screenPostback  = null;

    //-----------------------------------------------------------------------------------------------
    // Constructor(s)
    //-----------------------------------------------------------------------------------------------
    /**
     *  
     */
    public ServiceRequest(String sServiceName, HttpServletRequest httpRequest, ScreenSession screenSession)
    {
        trace("ServiceRequest constructor...");
        _sServiceName = sServiceName;
        _httpRequest = httpRequest;
        _screenSession = screenSession;

        if (_httpRequest != null)
        {
            String sPostBackType = null;
            //--- Try to find the "Screen Marker" ( special hidden field )
            String sScreenMarker = _httpRequest.getParameter(TelosysConst.SCREEN_FORM_MARKER);
            if (sScreenMarker != null)
            {
                trace("ServiceRequest constructor : Screen marker found : '" + sScreenMarker + "' ");
                //--- Split the marker ( "ScreenName;ScreenId;FormName;postback" )
                //String[] sParts = sScreenMarker.split(";");
                String[] sParts = parseMarker(sScreenMarker);
                if (sParts.length < 4)
                {
                    error("Invalid ScreenMarker '" + sScreenMarker + "' ( " + sParts.length + " parts, 4 expected )");
                    _sScreenName = null;
                    _sScreenId = null;
                    _sFormName = null;
                    sPostBackType = null;
                }
                else
                {
                    int i = 0;
                    _sScreenName = sParts[i++].trim();
                    _sScreenId = sParts[i++].trim();
                    _sFormName = sParts[i++].trim();
                    sPostBackType = sParts[i++].trim();
                }
            }
            else
            {
                trace("ServiceRequest constructor : Screen marker NOT found ");                
            }

            //--- Set the "PostBack Type" from the screen form marker ( hidden field )
            _iPostBackType = POSTBACK_NONE;
            if (sPostBackType != null)
            {
                sPostBackType = sPostBackType.trim();
                if ("request".equalsIgnoreCase(sPostBackType))
                {
                    _iPostBackType = POSTBACK_REQUEST;
                }
                else if ("session".equalsIgnoreCase(sPostBackType))
                {
                    _iPostBackType = POSTBACK_SESSION;
                }
                try
                {
                    if (_iPostBackType != POSTBACK_NONE)
                    {
                        managePostBack();
                    }
                } catch (TelosysException e)
                {
                    error("Cannot managed postback values - TelosysException : " + e.getMessage());
                }
            }
        }
        trace("ServiceRequest constructor : PostBack Type = " + _iPostBackType );
    }

    /**
     * Returns an array of 4 Strings (the 4 parts of the marker)
     * 
     * @param sMarker
     * @return : the 4 parts of the marker
     */
    private String[] parseMarker(String sMarker)
    {
        String sTokens[] = { "", "", "", "" };
        if (sMarker != null)
        {
            char chars[] = sMarker.toCharArray();
            int iToken = 0;
            for ( int i = 0 ; i < chars.length ; i++ )
            {
                if (chars[i] == ';')
                {
                    iToken++;
                }
                else
                {
                    if (iToken < sTokens.length)
                    {
                        sTokens[iToken] = sTokens[iToken] + chars[i];
                    }
                }
            }
        }
        return sTokens;
    }

    //-----------------------------------------------------------------------------------------------
    // Methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    public String getServiceName()
    {
        return _sServiceName;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard HTTP servlet request 
     * 
     * @return
     */
    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard HTTP session
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public HttpSession getHttpSession() // new in v 0.9.9
    {
        if ( _httpRequest != null )
        {
            return _httpRequest.getSession(false); // the session always exists
        }
        return null ;
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ID of the standard HTTP session <br>
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public String getHttpSessionId() // new in v 0.9.9
    {
        HttpSession httpSession = getHttpSession();
        if ( httpSession != null )
        {
            return httpSession.getId();
        }
        return null ;
    }
    
	//-----------------------------------------------------------------------------------------------
    /**
     * Returns the standard web app ServletContext <br>
     * or null if the Http Session doesn't exist
     * It cannot returns null, unless the HttpSession has been invalidate by an external way
     * @return
     */
    public ServletContext getServletContext() // new in v 0.9.9
    {
        HttpSession httpSession = getHttpSession();
        if (httpSession != null)
        {
            return httpSession.getServletContext();
        }
        return null;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the current ScreenSession associated with the request <br>
     * or null if the HttpSession doesn't exist or if it doesn't contain a ScreenSession attribute <br>
     * It cannot returns null, unless the HttpSession has been corrupted by an external way
     * 
     * @return
     */
    public ScreenSession getScreenSession() // new in v 0.9.9
    {
        return ScreenSessionManager.getCurrentScreenSession(_httpRequest);
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the ScreenApplication object associated with the current web app<br>
     * or null if the ScreenApplication has not yet been created (opened)<br>
     * It cannot returns null, unless the ServletContext has been corrupted by an external way
     * @return
     */
    public ScreenApplication getScreenApplication() // new in v 0.9.9
    {
        return ScreenApplicationManager.getScreenApplication(); // v 1.0.0
        /***
        HttpSession httpSession = getHttpSession();
        if ( httpSession != null )
        {
            return ScreenApplicationManager.getScreenApplication(httpSession); 
        }
        return null ;
        ***/
    }
    
    
    //-----------------------------------------------------------------------------------------------
    public int getPostBackType()
    {
        return _iPostBackType;
    }

    //-----------------------------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    //-----------------------------------------------------------------------------------------------
    public String getScreenId()
    {
        return _sScreenId;
    }

    //-----------------------------------------------------------------------------------------------
    //    public String getScreenKey()
    //    {
    //        return _sScreenKey;
    //    }

    //-----------------------------------------------------------------------------------------------
    public String getFormName()
    {
        return _sFormName;
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenPostback getScreenPostback()
    {
        return _screenPostback;
    }

    //-----------------------------------------------------------------------------------------------

    /**
     * Returns the standard parameters map of the HttpServletRequest object <br>
     * NB : each value of the Map is a String array ( cf HttpServletRequest : getParameterMap() )
     * 
     * @return
     */
    public Map getParameterMap()
    {
        return _httpRequest.getParameterMap();
    }

    /**
     * Return the number of parameters
     * 
     * @return
     */
    public int getParameterCount()
    {
        Map params = _httpRequest.getParameterMap();
        if (params != null)
        {
            return params.size();
        }
        return 0;
    }

    /**
     * Returns all the parameter names
     * 
     * @return : Names (never null : size = 0 if no parameters )
     */
    public String[] getParameterNames()
    {
        Map params = _httpRequest.getParameterMap();
        if (params != null)
        {
            Set keys = params.keySet();
            if (keys != null)
            {
                //Object[] arrayKeys = keys.toArray();
                try
                {
                    // NB: cannot cast directly
                    // ( ie: sKeys = (String[]) arrayKeys ; => Exception )
                    // => Cast from "Object[]" to "String[]" with "Collection.toArray(Object[])"
                    // ( returns a new String array with the good size )
                    String[] voidArrayModel = new String[0];
                    String[] sKeys = (String[]) keys.toArray(voidArrayModel);
                    return sKeys;
                } catch (ClassCastException e)
                {
                    error("Cannot cast parameters keys to String[] ");
                }
            }
        }
        return new String[0];
    }

    //-----------------------------------------------------------------------------------------------
//    /**
//     * Returns the value for the given parameter name
//     * 
//     * @param sParamName :
//     *            parameter name
//     * @return : parameter value (or null if not found)
//     */
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParameter(java.lang.String)
     */
    public String getParameter(String paramName)
    {
        return _httpRequest.getParameter(paramName);
    }
    /* (non-Javadoc)
     * @see org.objectweb.telosys.common.IRequestParameters#getParameter(java.lang.String, java.lang.String)
     */
    public String getParameter(String paramName, String defaultValue)
    {
        String s = _httpRequest.getParameter(paramName);
        return s != null ? s : defaultValue ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'nth' parameter value
     * @param iParamNum : parameter index ( 1 for "p1", 2 for "p2", ... )
     * @return : parameter value (or null if not found)
     */
    public String getParameter(int iParamNum)
    {
        return getParameter("p" + iParamNum);
    }
    /**
     * Returns the 'nth' parameter value
     * @param iParamNum  parameter index ( 1 for "p1", 2 for "p2", ... )
     * @param defaultValue default value (returned if the parameter cannot be found)
     * @return
     */
    public String getParameter(int iParamNum, String defaultValue)
    {
        return getParameter("p" + iParamNum, defaultValue);
    }
    
    
    public double getParamAsDouble(int paramNum)
    {
        return getParamAsDouble("p"+paramNum);
    }
    public double getParamAsDouble(int paramNum, double defaultValue)
    {
        return getParamAsDouble("p"+paramNum, defaultValue);
    }

    public float getParamAsFloat(int paramNum)
    {
        return getParamAsFloat("p"+paramNum);
    }
    public float getParamAsFloat(int paramNum, float defaultValue)
    {
        return getParamAsFloat("p"+paramNum, defaultValue);
    }
    
    public int getParamAsInt(int paramNum)
    {
        return getParamAsInt("p"+paramNum);
    }
    public int getParamAsInt(int paramNum, int defaultValue)
    {
        return getParamAsInt("p"+paramNum, defaultValue);
    }
    
    public long getParamAsLong(int paramNum)
    {
        return getParamAsLong("p"+paramNum);
    }
    public long getParamAsLong(int paramNum, long defaultValue)
    {
        return getParamAsLong("p"+paramNum, defaultValue);
    }
    
    public short getParamAsShort(int paramNum)
    {
        return getParamAsShort("p"+paramNum);
    }
    public short getParamAsShort(int paramNum, short defaultValue)
    {
        return getParamAsShort("p"+paramNum, defaultValue);
    }
    
    public byte getParamAsByte(int paramNum)
    {
        return getParamAsByte("p"+paramNum);
    }
    public byte getParamAsByte(int paramNum, byte defaultValue)
    {
        return getParamAsByte("p"+paramNum, defaultValue);
    }
    
    public char getParamAsChar(int paramNum)
    {
        return getParamAsChar("p"+paramNum);
    }
    public char getParamAsChar(int paramNum, char defaultValue)
    {
        return getParamAsChar("p"+paramNum, defaultValue);
    }
    
    public boolean getParamAsBoolean(int paramNum)
    {
        return getParamAsBoolean("p"+paramNum);
    }
    public boolean getParamAsBoolean(int paramNum, boolean defaultValue)
    {
        return getParamAsBoolean("p"+paramNum, defaultValue);
    }
    
    public java.util.Date getParamAsDate(int paramNum)
    {
        return getParamAsDate("p"+paramNum);
    }
    public java.util.Date getParamAsDate(int paramNum, java.util.Date defaultValue)
    {
        return getParamAsDate("p"+paramNum, defaultValue);
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'integer' value for the given parameter name
     * @param sParamName : parameter name
     * @return : parameter value
     * @throws TelosysException if parameter not found or invalid ( not 'integer' )
     * @deprecated use getParamAsInt
     */
    public int getIntParameter(String sParamName) throws TelosysException
    {
        int iVal = 0 ;
        String sVal = getParameter(sParamName) ;
        if (sVal == null)
        {
            throw new TelosysException("ServiceRequest parameter '" + sParamName + "' not found !");
        }
        else
        {
            try
            {
                iVal = Integer.parseInt(sVal);
            } catch (NumberFormatException e)
            {
                throw new TelosysException("Cannot convert ServiceRequest parameter '" + sParamName + "' to int (value=" + sVal + ") !");
            }
        }
        return iVal ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'long' value for the given parameter name
     * @param sParamName : parameter name
     * @return : parameter value
     * @throws TelosysException if parameter not found or invalid ( not 'long' )
     * @deprecated use getParamAsLong
     */
    public long getLongParameter(String sParamName) throws TelosysException
    {
        long lVal = 0 ;
        String sVal = getParameter(sParamName) ;
        if (sVal == null)
        {
            throw new TelosysException("ServiceRequest parameter '" + sParamName + "' not found !");
        }
        else
        {
            try
            {
                lVal = Long.parseLong(sVal);
            } catch (NumberFormatException e)
            {
                throw new TelosysException("Cannot convert ServiceRequest parameter '" + sParamName + "' to long (value=" + sVal + ") !");
            }
        }
        return lVal ;
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'integer' value of the 'nth' parameter 
     * @param iParamNum : parameter index ( 1 for "p1", 2 for "p2", ... )
     * @return : parameter value
     * @throws TelosysException if parameter not found or invalid ( not integer )
     * @deprecated use getParamAsInt
     */
    public int getIntParameter(int iParamNum) throws TelosysException
    {
        return getIntParameter("p" + iParamNum) ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'long' value of the 'nth' parameter 
     * @param iParamNum : parameter index ( 1 for "p1", 2 for "p2", ... )
     * @return : parameter value
     * @throws TelosysException if parameter not found or invalid ( not long )
     * @deprecated use getParamAsLong
     */
    public long getLongParameter(int iParamNum) throws TelosysException
    {
        return getLongParameter("p" + iParamNum) ;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns true if the submit element name is present in the request parameters <br>
     * ( useful for forms with multiple submit buttons and/or submit images )<br>
     * An HTML form can submit a request with <br>
     * a "button" ( input type="submit" name="abc" : parameter "abc" )<br>
     * an "image" ( input type="image" name="abc" : parameters "abc.x" and "abc.y" )<br>
     * 
     * @param sSubmitElementName
     * @return
     */
    public boolean submitBy(String sSubmitElementName)
    {
        //--- If the submit element is a standard button ( <input type="submit" name="" ... /> )
        if (getParameter(sSubmitElementName) != null)
        {
            return true;
        }
        //--- If the submit element is an image ( <input type="image" name="" ... /> )
        if (getParameter(sSubmitElementName + ".x") != null)
        {
            return true;
        }
        if (getParameter(sSubmitElementName + ".X") != null)
        {
            return true;
        }
        //--- Submit name not found
        return false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Manages the request input field values storage for future postback
     * 
     * @throws TelosysException
     */
    private void managePostBack() throws TelosysException
    {
        trace("managePostBack()...");
        Map requestParameters = this._httpRequest.getParameterMap();
        if (requestParameters == null)
        {
            return;
        }
        if (requestParameters.isEmpty())
        {
            return;
        }
        //--- There's field values posted in the current http request => store the field values
        if (_iPostBackType == POSTBACK_SESSION)
        {
            String sScreenKey = SessionPostback.composeScreenKey(_sScreenName, _sScreenId);
            trace("managePostBack() : type 'session' : screen key = '" + sScreenKey + "' ");
            //--- Store the field values in the ScreenSession
            if (sScreenKey == null)
            {
                error("Cannot managed postback values ( screen session key is null )");
                return;
            }
            else
            {
                ScreenPostback screenPostback = _screenSession.getScreenPostback(sScreenKey);
                screenPostback.initFieldValues(_sFormName, _httpRequest);
            }
        }
        else if (_iPostBackType == POSTBACK_REQUEST)
        {
            trace("managePostBack() : type 'request' ");
            //--- Store the field values in the current ServiceRequest
            _screenPostback = new ScreenPostback();
            _screenPostback.initFieldValues(_sFormName, _httpRequest);
            //--- Store the ScreenPostback in the http session (for taglib )
            _httpRequest.setAttribute(TelosysConst.POSTBACK_KEY, _screenPostback);
        }

        //OldScreensValues lOv = OldValuesManager.getScreenOldValue(screenSession, sScreenName, iScreenId);
        // TODO en fonction de l'id d'cran courant, du formulaire post etc ... on va stocker les donnes qui doivent l'tre
        //	        for (Iterator it=lParams.entrySet().iterator(); it.hasNext(); ) {
        //	            Map.Entry entrees = (Map.Entry)it.next();
        //	            Object cle = entrees.getKey();
        //	            Object valeur = entrees.getValue();
        //	            if (lOv.containsKey(cle)) {
        //	        		lOv.setFieldValue(cle, valeur);
        //	        	}
        //	        }
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        return "ServiceRequest : " + _sServiceName + " " + _httpRequest.getQueryString();
    }
    //-----------------------------------------------------------------------------------------------
}