package org.objectweb.telosys.service;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.util.web.GenericURI;

/**
 * The response of a ScreenService 
 * 
 * @author Laurent GUERIN
 *  
 */
public class ServiceResponse extends TelosysObject
{
    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------

    private String              _sServiceName    = null;

    private HttpServletRequest  _httpRequest     = null;

    private HttpServletResponse _httpResponse    = null; // since v 0.9.9
    
    private ScreenSession       _screenSession   = null;

    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------

    private Hashtable       _htValues         = null; // Values to return in the response

    private LinkedList      _listExceptions   = null; // List of all the exceptions

    private LinkedList      _listErrors       = null; // List of all the errors

    private LinkedList      _listMessages     = null; // List of all the app. messages

    private int             _iReturnCode      = 0;   // default = "0" ( = OK )

    private String          _sReturnMessage   = "Ok"; // default = "Ok"

    private Object          _serviceResult    = null;
    
    private String          _sRedirection     = null ; // v 0.9.9
    

    //-----------------------------------------------------------------------------------------------
    // Constructor(s)
    //-----------------------------------------------------------------------------------------------
    /**
     * Constructs a new ServiceResponse object
     * @param sServiceName
     * @param httpRequest
     * @param httpResponse
     * @param screenSession
     */
    // old constructor ( removed in v 0.9.9 )
    //public ServiceResponse(String sServiceName, HttpServletRequest httpRequest, ScreenSession screenSession)
    // new contructor : v 0.9.9
    public ServiceResponse(String sServiceName, HttpServletRequest httpRequest, HttpServletResponse httpResponse, ScreenSession screenSession)
    {
        _sServiceName  = sServiceName;
        _httpRequest   = httpRequest;
        _httpResponse  = httpResponse; // v 0.9.9
        _screenSession = screenSession;        
    }

    //-----------------------------------------------------------------------------------------------
    // Methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "service name"
     * @return
     */
    public String getServiceName()
    {
        return _sServiceName;
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "HttpServletRequest" object 
     * @return
     */
    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "ScreenSession" object 
     * @return
     */
    public ScreenSession getScreenSession()
    {
        return _screenSession;
    }
    //-----------------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------------
    /**
     * Set the "return code"
     * @param iCode
     */
    public void setReturnCode(int iCode)
    {
        _iReturnCode = iCode;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the "return code"
     * @return
     */
    public int getReturnCode()
    {
        return _iReturnCode;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the main "return message"
     * @param sMsg
     */
    public void setReturnMessage(String sMsg)
    {
        _sReturnMessage = sMsg;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the main "return message"
     * @return
     */
    public String getReturnMessage()
    {
        return _sReturnMessage;
    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    // Messages
    //-----------------------------------------------------------------------------------------------
    /**
     * Adds a message in the list
     * @param message
     */
    public void addMessage(ScreenMessage message)
    {
        if (_listMessages == null)
        {
            _listMessages = new LinkedList();
        }

        if (message != null)
        {
            _listMessages.add(message);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of messages 
     * @return
     */
    public int getNumberOfMessages()
    {
        if (_listMessages == null)
        {
            return 0;
        }
        else
        {
            return _listMessages.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the message associated with the given index
     * @param index
     * @return
     */
    public ScreenMessage getMessage(int index)
    {
        if (_listMessages == null)
        {
            return null;
        }
        else
        {
            Object obj = _listMessages.get(index);
            if (obj instanceof ScreenMessage)
            {
                return (ScreenMessage) obj;
            }
            else
            {
                error("getMessage(" + index + ") : not a ScreenMessage instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Errors
    //-----------------------------------------------------------------------------------------------
    /**
     * Adds an error in the list
     * @param error
     */
    public void addError(ScreenError error)
    {
        if (_listErrors == null)
        {
            _listErrors = new LinkedList();
        }

        if (error != null)
        {
            _listErrors.add(error);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of errors 
     * @return
     */
    public int getNumberOfErrors()
    {
        if (_listErrors == null)
        {
            return 0;
        }
        else
        {
            return _listErrors.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the error associated with the given index
     * @param index
     * @return
     */
    public ScreenError getError(int index)
    {
        if (_listErrors == null)
        {
            return null;
        }
        else
        {
            Object obj = _listErrors.get(index);
            if (obj instanceof ScreenError)
            {
                return (ScreenError) obj;
            }
            else
            {
                error("getError(" + index + ") : not a ScreenError instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Exceptions
    //-----------------------------------------------------------------------------------------------
    /**
     * Add an 'Exception' to the exceptions list
     * @param ex
     */
    public void addException(Exception ex)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (ex != null)
        {
            _listExceptions.add(ex);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Add a 'Throwable' to the exceptions list
     * @param th
     */
    public void addException(Throwable th)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (th != null)
        {
            _listExceptions.add(th);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of exception(s) 
     * @return
     */
    public int getNumberOfExceptions()
    {
        if (_listExceptions == null)
        {
            return 0;
        }
        else
        {
            return _listExceptions.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the exception associated with the given id 
     * @param index
     * @return
     */
    public Exception getException(int index)
    {
        if (_listExceptions == null)
        {
            return null;
        }
        else
        {
            Object obj = _listExceptions.get(index);
            if (obj instanceof Exception)
            {
                return (Exception) obj;
            }
            else
            {
                error("getException(" + index + ") : not an Exception instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Data
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the service result object 
     * @return
     */
    public Object getResult()
    {
        return _serviceResult;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Set the service result object 
     * @param v
     */
    public void setResult(Object v)
    {
        _serviceResult = v;
    }

    //-----------------------------------------------------------------------------------------------
    // Values
    //-----------------------------------------------------------------------------------------------
    /**
     * Set an 'String' value 
     * @param sName
     * @param sValue
     */
    public void setValue(String sName, String sValue)
    {
        if (_htValues == null)
        {
            _htValues = new Hashtable();
        }

        //--- NB : NullPointerException if the key or value is null.
        if (sName != null && sValue != null )
        {
            _htValues.put(sName, sValue);
        }
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set an 'int' value 
     * @param sName
     * @param iValue
     */
    public void setValue(String sName, int iValue)
    {
        this.setValue(sName, ""+iValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'long' value 
     * @param sName
     * @param lValue
     */
    public void setValue(String sName, long lValue)
    {
        this.setValue(sName, ""+lValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'short' value 
     * @param sName
     * @param shValue
     */
    public void setValue(String sName, short shValue)
    {
        this.setValue(sName, ""+shValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'byte' value 
     * @param sName
     * @param byValue
     */
    public void setValue(String sName, byte byValue)
    {
        this.setValue(sName, ""+byValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'float' value 
     * @param sName
     * @param fValue
     */
    public void setValue(String sName, float fValue)
    {
        this.setValue(sName, ""+fValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'double' value 
     * @param sName
     * @param dValue
     */
    public void setValue(String sName, double dValue)
    {
        this.setValue(sName, ""+dValue );
    }
    //-----------------------------------------------------------------------------------------------
    /**
     * Set a 'boolean' value 
     * @param sName
     * @param bValue
     */
    public void setValue(String sName, boolean bValue)
    {
        this.setValue(sName, ""+bValue );
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the response value associated with the given name
     * @param sName
     * @return
     */
    public String getValue(String sName)
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return (String) _htValues.get(sName);
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the number of values stored in the response
     * @return
     */
    public int getNumberOfValues()
    {
        if (_htValues == null)
        {
            return 0;
        }
        else
        {
            return _htValues.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the values names 
     * @return
     */
    public Enumeration getValueNames()
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return _htValues.keys();
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------
    /**
     * Exposes an object in the standard request scope for future use in a view <br>
     * ( the object is stored as an attribute in the HttpServletRequest )
     * @param sName 
     * @param obj
     */
    public void expose(String sName, Object obj)
    {
        if ( _httpRequest != null )
        {
            _httpRequest.setAttribute(sName, obj);            
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given URL <br>
     * ( replace the current screen by the given URL )
     * @param sURL
     */
    public void forceRedirect ( String sURL )
    {
        if ( sURL != null )
        {
            if ( _httpResponse != null )
            {
                _sRedirection = sURL ;
                _httpResponse.setHeader(TelosysConst.TELOSYS_REDIRECT, _sRedirection );
            }
            else
            {
                throw new TelosysRuntimeException("forceRedirect : HttpResponse is null");
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Force the client-side (the browser) to redirect to the given generic URI <br>
     * ( replace the current screen by the given object URL )
     * @param uri
     */
    public void forceRedirect ( GenericURI uri )
    {
        if ( uri != null )
        {
            forceRedirect( uri.getFullURL( _httpRequest ) );
        }
    }
    
    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the redirection URL set by 'forceRedirect' or null if none
     * @return
     */
    public String getRedirection()
    {
        return _sRedirection ;
    }


    //-----------------------------------------------------------------------------------------------
    // 
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        //--- Attributes
        //String sHeader = _sServiceName + "|" + _sServiceType;
        String sHeader = _sServiceName;
        /**
         * //--- Data String sData = "(no data)"; if (_Data != null) { sData = _Data.toString(); }
         */
        //return "ScreenResponse : " + sHeader + "\n Data = " + sData;
        return "ServiceResponse : " + sHeader;
    }
    //-----------------------------------------------------------------------------------------------
}