package org.objectweb.telosys.tools;

import java.io.CharArrayWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 * @author GOUGEON / GUERIN
 *  
 */
public class ScreenRequestLogger implements Filter
{
    private static final int   MODE_XML        = 1;

    private static final int   MODE_LOG        = 2;

    public static final String QUERY_PREFIX    = "QUERY_";

    public static final String RESPONSE_PREFIX = "RESPONSE_";

    //=========================================================================================
    // Sequence used to contruct file name
    //=========================================================================================
    private static int         _lastFileNumber = 0;

    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS : LoggingInputStream
    //=========================================================================================
    private static class LoggingInputStream extends ServletInputStream
    {
        private ServletInputStream in;

        // buffer contain body text data
        private StringBuffer       buffer = new StringBuffer();

        public LoggingInputStream(ServletInputStream inputStream)
        {
            super();
            in = inputStream;
        }

        public int read() throws IOException
        {
            int chr = in.read();
            if (chr != -1)
            {
                buffer.append((char) chr);
            }
            return chr;
        }

        public String getContent() throws IOException
        {
            while (read() != -1)
            {
            }
            return buffer.toString();
        }
    }

    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS : RequestWrapper
    //=========================================================================================
    private static class CharRequestWrapper extends HttpServletRequestWrapper
    {
        LoggingInputStream input = null;

        /**
         * Enforce the behavior of getInputStream method in PortletRequest class: return data stream for the first call, and null for all the subsequent calls
         */
        public ServletInputStream getInputStream() throws IOException
        {
            return input;
        }

        public CharRequestWrapper(HttpServletRequest request) throws IOException
        {
            super(request);
            input = new LoggingInputStream(request.getInputStream());
        }

        public String getBody() throws IOException
        {
            return input.getContent();
        }
    }

    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS : ResponseWrapper
    //=========================================================================================
    /**
     * Servlet Response Wrapper class Wrap the original servlet response and provide a "writer" which keeps the response output in a "CharArray"
     */
    private static class CharResponseWrapper extends HttpServletResponseWrapper
    {
        private int          iHttpStatusCode = SC_OK;        // 200 (by

        // default)

        private HashMap      hmHeaders       = new HashMap();

        private StringWriter output          = null;

        public CharResponseWrapper(HttpServletResponse response)
        {
            super(response);
            output = new StringWriter();
        }

        /**
         * Return the response status
         * 
         * @return
         */
        public String getStatus()
        {
            return "HTTP Status Code = " + iHttpStatusCode;
        }

        /**
         * Return the response headers
         * 
         * @return
         */
        public String getHeaders()
        {
            StringBuffer sb = new StringBuffer(400);
            //Collection values = hmHeaders.values();
            Set keys = hmHeaders.keySet();
            Iterator iter = keys.iterator();
            while (iter.hasNext())
            {
                Object objKey = iter.next();
                Object objVal = hmHeaders.get(objKey);
                sb.append(objKey.toString() + ":");
                if (objVal instanceof String)
                {
                    sb.append(objVal.toString() + "\r\n");
                }
                else
                {
                    if (objVal instanceof LinkedList)
                    {
                        sb.append("LinkedList" + "\r\n");
                    }
                }
            }
            return sb.toString();
        }

        /**
         * Return the response body
         * 
         * @return
         */
        public String getBody()
        {
            return output.toString();
        }

        //----------------------------------------------------------------------------
        // Override methods which change the STATUS ( to catch the response
        // status ).
        //----------------------------------------------------------------------------
        public void setStatus(int iStatus)
        {
            this.iHttpStatusCode = iStatus;
            super.setStatus(iStatus);
        }

        public void setStatus(int iStatus, String str)
        {
            this.iHttpStatusCode = iStatus;
            super.setStatus(iStatus, str);
        }

        public void sendError(int errorCode) throws IOException
        {
            this.iHttpStatusCode = errorCode;
            super.sendError(errorCode);
        }

        public void sendError(int errorCode, String errorMessage) throws IOException
        {
            this.iHttpStatusCode = errorCode;
            super.sendError(errorCode, errorMessage);
        }

        public void sendRedirect(String location) throws IOException
        {
            this.iHttpStatusCode = SC_MOVED_TEMPORARILY;
            super.sendRedirect(location);
        }

        //----------------------------------------------------------------------------
        // Override the response "writer provider".
        //----------------------------------------------------------------------------
        /*
         * Provide an specific writer instead of the standard writer
         * 
         * @see javax.servlet.ServletResponse#getWriter()
         */
        public PrintWriter getWriter()
        {
            return new PrintWriter(output);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponse#addHeader(java.lang.String, java.lang.String)
         */
        public void addHeader(String sName, String sValue)
        {
            Object obj = hmHeaders.get(sName);
            if (obj == null)
            {
                LinkedList list = new LinkedList();
                list.add(sValue);
                hmHeaders.put(sName, list);
            }
            else
            {
                if (obj instanceof LinkedList)
                {
                    LinkedList list = (LinkedList) obj;
                    list.add(sValue);
                }
                else
                {
                    if (obj instanceof String)
                    {
                        String sFirstValue = (String) obj;
                        LinkedList list = new LinkedList();
                        list.add(sFirstValue);
                        list.add(sValue);
                        hmHeaders.put(sName, list);
                    }
                    else
                    {
                        // ERROR !
                    }
                }
            }
            super.addHeader(sName, sValue);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponse#setHeader(java.lang.String, java.lang.String)
         */
        public void setHeader(String name, String value)
        {
            hmHeaders.put(name, value);
            super.addHeader(name, value);
        }
    }

    //=========================================================================================

    //private FilterConfig filterConfig = null;

    private String _sLogDirectory = null;

    private String _sFilterName   = null;

    private int    _iTraceMode    = MODE_XML;

    public void init(FilterConfig filterConfig) throws ServletException
    {
        //this.filterConfig = filterConfig;
        _sLogDirectory = filterConfig.getInitParameter("directory");
        if (_sLogDirectory == null)
        {
            throw new ServletException("filter parameter \"directory\" is mandatory for the log filter");
        }
        if (!_sLogDirectory.endsWith("/"))
        {
            _sLogDirectory += "/";
        }
        _sFilterName = filterConfig.getFilterName();

        String sMode = filterConfig.getInitParameter("mode");
        if ("XML".equals(sMode))
        {
            _iTraceMode = MODE_XML;
        }
        else
        {
            _iTraceMode = MODE_LOG;
        }
        initLastFileNumber();
        System.out.println("Filter " + _sFilterName + " ready, log directory = " + _sLogDirectory);
    }

    public void destroy()
    {
        //this.filterConfig = null;

    }

    /**
     * find the last file number
     */
    private void initLastFileNumber()
    {
        _lastFileNumber = 0;
        File file = new File(_sLogDirectory);
        File[] files = file.listFiles();
        if (files != null)
        {
            for ( int i = 0 ; i < files.length ; i++ )
            {
                File fileTemp = files[i];
                String sFileName = fileTemp.getName();
                if (sFileName.startsWith(QUERY_PREFIX))
                {
                    String sFileNumber = sFileName.substring(QUERY_PREFIX.length(), sFileName.length() - 4);
                    try
                    {
                        int iFileNumber = Integer.parseInt(sFileNumber);
                        if (iFileNumber > _lastFileNumber)
                        {
                            _lastFileNumber = iFileNumber;
                        }
                    } catch (NumberFormatException e)
                    {
                    }
                }
            }
        }
    }

    /**
     * @return a new file number
     */
    private synchronized String getNewFileNumber()
    {
        _lastFileNumber++;
        //  normalize sequence number
        StringBuffer sFileNumber = new StringBuffer(String.valueOf(_lastFileNumber));
        while (sFileNumber.length() < 10)
        {
            sFileNumber.insert(0, "0");
        }
        return sFileNumber.toString();
    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException
    {
        HttpServletRequest httpRequest = null;
        HttpServletResponse httpResponse = null;

        CharArrayWriter cawRequestQuery = new CharArrayWriter();
        CharArrayWriter cawRequestHeader = new CharArrayWriter();
        CharArrayWriter cawRequestBody = new CharArrayWriter();

        //--- Cast the request object
        if (request instanceof HttpServletRequest)
        {
            httpRequest = (HttpServletRequest) request;
            httpResponse = (HttpServletResponse) response;
        }

        //--- Keep the original reponse output
        PrintWriter out = response.getWriter();

        //--- Create a response wrapper for interceptions
        CharResponseWrapper responseWrapper = new CharResponseWrapper(httpResponse);

        //--- Create a request wrapper for interceptions
        CharRequestWrapper requestWrapper = new CharRequestWrapper(httpRequest);

        //--------------------------------------------------------------------------
        //--- Chain to next step ...
        chain.doFilter(requestWrapper, responseWrapper);
        //--------------------------------------------------------------------------

        //--- Keep the REQUEST informations in "CharArray" buffers
        writeRequestQuery(cawRequestQuery, requestWrapper);
        writeRequestHeader(cawRequestHeader, requestWrapper);
        writeRequestBody(cawRequestBody, requestWrapper);
        //--- Write the response body in the original servlet output
        out.write(responseWrapper.getBody());

        //--- After request processing : log the REQUEST & RESPONSE
        FileWriter fw = null;
        // construct le file name for request
        String sFileNumber = getNewFileNumber();
        StringBuffer sFileName = new StringBuffer();
        sFileName.append(_sLogDirectory);
        if (_iTraceMode != MODE_LOG)
        {
            // XML Trace request
            sFileName.append("QUERY_");
            sFileName.append(sFileNumber);
            sFileName.append(".xml");
            try
            {
                fw = new FileWriter(sFileName.toString(), false);
                fw.write(cawRequestBody.toString());
                fw.close();
            } catch (IOException ex)
            {
                ex.printStackTrace();
            } finally
            {
                //--- FERMETURE LOG
                try
                {
                    if (fw != null)
                    {
                        fw.close();
                    }
                } catch (IOException ex_close)
                {
                    ex_close.printStackTrace();
                }
            }
            // XML Trace response
            sFileName = new StringBuffer();
            sFileName.append(_sLogDirectory);
            sFileName.append("RESPONSE_");
            sFileName.append(sFileNumber);
            sFileName.append(".xml");
            try
            {
                fw = new FileWriter(sFileName.toString(), false);
                fw.write(responseWrapper.getBody());
                fw.close();
            } catch (IOException ex)
            {
                ex.printStackTrace();
            } finally
            {
                //--- FERMETURE LOG
                try
                {
                    if (fw != null)
                    {
                        fw.close();
                    }
                } catch (IOException ex_close)
                {
                    ex_close.printStackTrace();
                }
            }

        }
        else
        {
            // log trace
            sFileName.append("QUERY_");
            sFileName.append(sFileNumber);
            sFileName.append(".log");
            try
            {
                fw = new FileWriter(sFileName.toString(), false);

                fw.write("\r\n");
                fw.write("==============================================\r\n");
                //---------- REQUEST
                //--- Write the header log in the file
                fw.write(cawRequestQuery.toString());
                fw.write("----------------------------------------------\r\n");
                fw.write(cawRequestHeader.toString());
                fw.write(cawRequestBody.toString());

                fw.write("----------------------------------------------\r\n");
                //---------- RESPONSE
                //--- Write the response status code in the log file
                fw.write(responseWrapper.getStatus() + "\r\n");

                //--- Write the response headers in the log file
                fw.write(responseWrapper.getHeaders() + "\r\n");

                //--- Write the response output body in the log file
                fw.write(responseWrapper.getBody() + "\r\n");

                fw.write("==============================================\r\n");
                fw.flush();
            } catch (IOException ex)
            {
                ex.printStackTrace();
            } finally
            {
                //--- FERMETURE LOG
                try
                {
                    if (fw != null)
                    {
                        fw.close();
                    }
                } catch (IOException ex_close)
                {
                    ex_close.printStackTrace();
                }
            }
        }
    }

    /**
     * Write a string in the given "writer" (deal with null values)
     * 
     * @param writer
     * @param chaine
     * @throws IOException
     */
    private void writeLine(Writer writer, String chaine) throws IOException
    {
        if (chaine != null)
        {
            writer.write(chaine);
        }
        else
        {
            writer.write("");
        }
        writer.write("\r\n");
    }

    /**
     * Write the HTTP REQUEST HEADER in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestQuery(Writer writer, HttpServletRequest request) throws IOException
    {

        if (_iTraceMode != MODE_XML)
        {
            //String sProtocol = request.getProtocol();
            writeLine(writer, "Protocol : " + request.getProtocol());

            //String sMethod = request.getMethod();
            writeLine(writer, "Method   : " + request.getMethod());

            String sURL = request.getRequestURL().toString();
            if (sURL == null)
                sURL = "";
            String sParams = request.getQueryString();
            String sFullURL = null;
            if (sParams != null)
            {
                sFullURL = sURL + "?" + sParams;
            }
            else
            {
                sFullURL = sURL;
            }
            writeLine(writer, "URL      : " + sFullURL);
        }
    }

    /**
     * Write the HTTP REQUEST HEADER in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestHeader(Writer writer, HttpServletRequest request) throws IOException
    {
        if (_iTraceMode != MODE_XML)
        {
            Enumeration names = request.getHeaderNames();
            while (names != null && names.hasMoreElements())
            {
                String name = (String) names.nextElement();
                Enumeration values = request.getHeaders(name);
                while (values != null && values.hasMoreElements())
                {
                    String value = (String) values.nextElement();
                    writeLine(writer, name + ":" + value);
                }
            }

        }
    }

    /**
     * Write the HTTP REQUEST BODY in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestBody(Writer writer, CharRequestWrapper requestWrapper) throws IOException
    {
        writer.write(requestWrapper.getBody());
    }
}