package org.objectweb.telosys.uil.renderer;

import java.io.IOException;
import java.util.Hashtable;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.i18n.Translator;
import org.objectweb.telosys.uil.i18n.TranslatorLoader;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;

/**
 * The renderers registry. A renderer is identified by its JSP resource path.  
 * 
 * This registry is build dynamicaly  
 * 
 * @author Laurent GUERIN
 * 
 */
public final class Renderers 
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(Renderers.class);

    //--- The registry : 
    // KEY : the renderer JSP file ( e.g. : "/renderer/employees.jsp" )
    // VAL : the renderer instance 
    private final static Hashtable $htRenderers = new Hashtable();
    
    //------------------------------------------------------------------------------------
    public final static void generate(String sRenderer, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
    {
        $log.trace("generate(" + sRenderer + ", httpRequest, httpResponse)... ");
        
        //--- Check parameters
        if (sRenderer == null)
        {
            throw new TelosysRuntimeException("renderer is null ");
        }
        if (httpRequest == null)
        {
            throw new TelosysRuntimeException("httpRequest is null ");
        }
        if (httpResponse == null)
        {
            throw new TelosysRuntimeException("httpResponse is null ");
        }
        
        //--- Check the renderer registry 
        HttpSession httpSession = httpRequest.getSession();
        if (httpSession == null)
        {
            throw new TelosysRuntimeException("cannot get HttpSession ");
        }        
        ServletContext servletContext = httpSession.getServletContext();
        if (servletContext == null)
        {
            throw new TelosysRuntimeException("cannot get ServletContext ");
        }
        checkRegistry(servletContext, sRenderer);
        
        //--- Use a RequestDispatcher to include the renderer JSP
        RequestDispatcher rd = httpRequest.getRequestDispatcher(sRenderer);
        if (rd != null)
        {
            //--- 1) Set the renderer name in the request
            httpRequest.setAttribute( TelosysConst.CURRENT_RENDERER, sRenderer );
            //--- 2) Call the RENDERER (via "include")
            try
            {
                $log.trace("generate("+ sRenderer +") : rd.include(..,..)");
                rd.include(httpRequest, httpResponse);
            } catch (ServletException e)
            {
                throw new TelosysRuntimeException("generate("+ sRenderer +"): ServletException ", e);
            } catch (IOException e)
            {
                throw new TelosysRuntimeException("generate("+ sRenderer +"): IOException ", e);
            }
            //--- 3) Remove the renderer name from the Request
            httpRequest.removeAttribute( TelosysConst.CURRENT_RENDERER );
        }
        else
        {
            throw new TelosysRuntimeException("generate("+ sRenderer +"): cannot get RequestDispatcher for this path ");
        }
    }
    
    //------------------------------------------------------------------------------------
    /**
     * Checks that the given renderer is in the registry, if not => register it
     * This method must be called before each use of a renderer
     * @param servletContext
     * @param sRenderer
     * @return
     */
    public final static Renderer checkRegistry(ServletContext servletContext, String sRenderer)
    {
        Object obj = $htRenderers.get(sRenderer);
        if ( obj != null )
        {
            $log.trace("checkRegistry(servletContext," + sRenderer + ") --> Found in the registry ");
            return (Renderer) obj ;
        }
        else
        {
            $log.trace("checkRegistry(servletContext," + sRenderer + ") --> Not found in the registry ");
            //--- Unknown => create a new one
            Translator translator = loadRendererTranslator(servletContext, sRenderer);
            $log.trace("checkRegistry(servletContext," + sRenderer + ") --> translator : " 
                    + (translator != null ? "YES" : "NO") );
            Renderer renderer = new Renderer(sRenderer, translator);
            $log.trace("checkRegistry(servletContext," + sRenderer + ") --> ADD the renderer in the registry ");
            $htRenderers.put(sRenderer, renderer);
            return renderer ;
        }
    }
    
    //------------------------------------------------------------------------------------
    private final static Translator loadRendererTranslator(ServletContext servletContext, String sRenderer)
    {
        //--- Get the languages managed by the current application
        ScreenDefConfig config = TelosysUIL.getScreenDefConfig();
        String[] languages = config.getAllLanguagesArray();
        //--- Load the translator 
        Translator translator = TranslatorLoader.loadTranslations(servletContext, sRenderer, languages );
        return translator ;
    }
    
    //------------------------------------------------------------------------------------
    /**
     * Returns the renderer definition for the given renderer resource name
     * @param sRenderer the renderer resource name ( e.g. "/renderer/employees.jsp" )
     * @return
     */
    public final static Renderer getRenderer(String sRenderer)
    {
        Object obj = $htRenderers.get(sRenderer);
        if ( obj != null )
        {
            return (Renderer) obj ;
        }
        return null ;
    }
    
    //------------------------------------------------------------------------------------
    /**
     * Returns the renderer translator for the given renderer resource name
     * Returns null if the renderer is unknown in the registry or if it hasn't translator
     * @param sRenderer the renderer resource name ( e.g. "/renderer/employees.jsp" )
     * @return
     */
    public final static Translator getTranslator(String sRenderer)
    {
        Object obj = $htRenderers.get(sRenderer);
        if ( obj != null )
        {
            Renderer renderer = (Renderer) obj ;
            return renderer.getTranslator();
        }
        return null ;
    }
    //------------------------------------------------------------------------------------
}