package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.i18n.Translator;

/**
 * @author Laurent GUERIN 
 *  
 */
public class ScreenMap extends TelosysObject
{
    //--- The Screen itself 
    private String  _sScreenName           = null; // Screen name ( name defined in the "screens.xml" )

    private String  _sScreenType           = null; // Screen type : "html" or "xul"

    //--- The ScreenContext associated with this Screen ( if any ) 
    private int     _iScreenContextId      = 0;   // The screen context used by this Screen Map
    
    private String  _sScreenContextName    = null ; // since v 0.9.9 


    //--- The ressource names for the specific Screen Type ( HTML, XUL, ... )
    private String  _sTarget               = null; // Main target for this Screen ( template or page )

    private String  _sScreenBody           = null; // This screen Body

    private String  _sCss                  = null; // This screen CSS file

    private String  _sScript               = null; // This screen JS file (behavior)

    //--- The URLs for the specific Screen Type ( HTML, XUL, ... )
    private String  _sTargetPath           = null; // Main target path for this Screen ( template or page )

    private String  _sScreenBodyPath       = null; // The path for this screen Body

    private String  _sCssPath              = null; // The path for this screen CSS file

    private String  _sScriptPath           = null; // The path for this screen JS file (behavior)

    //--- 
    private String  _sImagesRootDir        = null;

    private HashMap _hmElements            = null;
    
    private Translator _translator         = null ;

    //---------------------------------------------------------------------------
    public ScreenMap(ScreenDefRootDir rootDir, ScreenDefinition def, String sScreenType, int iScreenId)
    {
        super();
        _sScreenType = sScreenType;
        _iScreenContextId = iScreenId;

        if (def != null)
        {
            _sScreenName = def.getScreenName();

            _sTarget = def.getScreenTarget(sScreenType);
            _sScreenBody = def.getScreenBody(sScreenType);
            _sCss = def.getScreenCss(sScreenType);
            _sScript = def.getScreenScript(sScreenType);

            _translator = def.getTranslator();
            
            //--- Target URL
            if (def.isBodyTarget(sScreenType))
            {
                //--- Target is "Body with template" => target URL = template URL
                _sTargetPath = rootDir.getTemplatePath(_sTarget);
            }
            else if (def.isPageTarget(sScreenType))
            {
                //--- Target is "Simple Page" => target URL = page URL
                _sTargetPath = rootDir.getPagePath(_sTarget);
            }
            else
            {
                _sTargetPath = _sTarget; // Target "as is"
            }

            //--- Screen Body path
            _sScreenBodyPath = rootDir.getBodyPath(_sScreenBody);

            //--- CSS path
            _sCssPath = rootDir.getCssPath(_sCss);

            //--- JavaScript path
            _sScriptPath = rootDir.getScriptPath(_sScript);

            //--- Images root directory
            _sImagesRootDir = rootDir.getImagesRootDir();

            //--- Templates elements ( 0 .. N )
            _hmElements = def.getElements();

            //--- By default the ScreenContext name is the same as the ScreenMap name
            _sScreenContextName = _sScreenName ; // V 0.9.9
        }
    }
    
    //---------------------------------------------------------------------------
    public ScreenMap(ScreenDefRootDir rootDir, ScreenDefinition def, String sScreenType, int iScreenContextId, String sScreenContextName)
    {
        this( rootDir, def, sScreenType, iScreenContextId );
        if ( sScreenContextName != null )
        {
            //--- There's a specific ScreenContext name => use it
            _sScreenContextName = sScreenContextName ;
        }
        else
        {
            //--- By default the ScreenContext name is the same as the ScreenMap name
            _sScreenContextName = _sScreenName ;
        }
    }
    
    //---------------------------------------------------------------------------
    /**
     * Returns the path to target the Screen 
     * @return the path to the Screen Template ( if ScreenBody ) or the Screen Page
     */
    public String getTargetPath()
    {
        return _sTargetPath;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the path to the ScreenBody if any 
     * @return
     */
    public String getScreenBodyPath()
    {
        return _sScreenBodyPath;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the ScreenBody if any 
     * @return
     */
    public String getScreenBody()
    {
        return _sScreenBody;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the path to the CSS file if any 
     * @return
     */
    public String getCssPath()
    {
        return _sCssPath;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the CSS file if any 
     * @return
     */
    public String getCss()
    {
        return _sCss;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the path to the JS script file if any 
     * @return
     */
    public String getScriptPath()
    {
        return _sScriptPath;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the JS script file if any 
     * @return
     */
    public String getScript()
    {
        return _sScript;
    }

    //---------------------------------------------------------------------------
    /**
     * @return the type of the Screen ( i.e. : "html" or "xul" )
     */
    public String getType()
    {
        return _sScreenType;
    }

    //---------------------------------------------------------------------------
    /**
     * @return the ID of the ScreenContext managed by the ScreenMap
     */
    public int getScreenContextId()
    {
        return _iScreenContextId;
    }

    //---------------------------------------------------------------------------
    /**
     * @return the name of ScreenContext managed by the ScreenMap ( the name defined in the ScreenContext registry )
     */
    public String getScreenContextName() // since v 0.9.9
    {
        return _sScreenContextName;
    }

    //---------------------------------------------------------------------------
    public boolean isHtmlType()
    {
        return (_sScreenType.equals(ScreenMapConst.HTML_SCREEN_TYPE));
    }

    //---------------------------------------------------------------------------
    public boolean isXulType()
    {
        return (_sScreenType.equals(ScreenMapConst.XUL_SCREEN_TYPE));
    }

    //---------------------------------------------------------------------------
    public String getImagesRootDir()
    {
        return _sImagesRootDir;
    }

    //---------------------------------------------------------------------------
    /**
     * @return the name of the ScreenMap ( name defined in the "screens.xml" file )
     */
    public String getName()
    {
        return _sScreenName;
    }

    //---------------------------------------------------------------------------
    public ScreenMapElement getElement(String sName)
    {
        if (_hmElements != null)
        {
            return (ScreenMapElement) _hmElements.get(sName);
        }
        else
        {
            return null;
        }
    }

    //---------------------------------------------------------------------------
    public int getElemCount()
    {
        if (_hmElements == null)
        {
            return 0;
        }
        else
        {
            return _hmElements.size();
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the screen map translator (if any)
     * @return
     */
    public Translator getTranslator()
    {
        return _translator ;
    }
    
    //---------------------------------------------------------------------------
    public String describe()
    {
        return "name=" + _sScreenName + " - type=" + _sScreenType + "\n" + "target : " + _sTarget + " -> URL : " + _sTargetPath + "\n" + "body   : " + _sScreenBody + " -> URL : " + _sScreenBodyPath + "\n" + "css    : " + _sCss + " -> URL : " + _sCssPath + "\n" + "script : " + _sScript + " -> URL : " + _sScriptPath + "\n" + "images dir = " + _sImagesRootDir + "\n" + "elements.size() = " + getElemCount() + "\n";
    }
    //---------------------------------------------------------------------------
    
    /**
     * Build a Web App context URI with the given ScreenMap parts <br>
     * ( e.g. : "/screenmap/name-type/id-ctx:action?params" )<br>
     * 
     * @param sScreenName : the ScreenMap name
     * @param sScreenType : the ScreenMap type ( e.g. "html" or "xul" ) null = default type
     * @param sContextId : the context id to use ( e.g. "1", "2", ... ) null = default context-id
     * @param sContextName : the name of the ScreenContext to use, or null = ScreenMap name
     * @param sContextAction : the action to apply on the ScreenContext ( "open", "use", "none" ) or null
     * @param sParams : the standard URL parameters ( e.g. "param1=aaa&param2=bbb" ) or null
     * @return : the relative URI in the current Web App ( e.g. "/screenmap/SCREEN01-html/1-MYCONTEXT:use" )
     * 
     */
    public static String getContextURI( String sScreenName, String sScreenType, 
            String sContextId, String sContextName, String sContextAction, String sParams )
    {
        String sScreenMapServletPath = TelosysUIL.getScreenMapServletPath();
        
        String s1 = "" ;
        String s2 = "" ;
        String s3 = "" ;
        String s4 = "" ;
                
        //--- Part 1 : "/name" or "/name-html" or "name-xul"
        if ( sScreenName != null )
        {
            s1 = "/" + sScreenName + ( sScreenType != null ? "-" + sScreenType : "" );
        }
        else
        {
            // s1 = "/ERROR_NO_SCREEN_NAME" ;
            return sScreenMapServletPath ; 
        }
        
        //--- Part 2 : "/id" or "/id-contextname" or "/contextname"
        if ( sContextId != null )
        {
            s2 = "/" + sContextId + ( sContextName != null ? "-" + sContextName : "" ) ;
        }
        else
        {
            if ( sContextName != null )
            {
                s2 = "/" + sContextName ;
            }
        }
        
        //--- Part 3 : ":action"
        if (sContextAction != null)
        {
            s3 = ":" + sContextAction;
        }
        
        //--- Part 4 : "?param1=xxx&param2=bbbb"
        if (sParams != null)
        {
            s4 = "?" + sParams ;
        }
        
        return sScreenMapServletPath + s1 + s2 + s3 + s4 ;
    }
}