package org.objectweb.telosys.uil.taglib;

import java.util.Collection;
import java.util.Iterator;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.RequestEnv;
import org.objectweb.telosys.util.BeanUtil;

/**
 * Static methods to find objects from their symbolic name
 * 
 * @author Laurent GUERIN
 *  
 */
public class Finder
{
    private static final TelosysClassLogger log = new TelosysClassLogger(Finder.class);
    
    //----------------------------------------------------------------------------------
    /**
     * Returns the HttpServletRequest object associated with the given PageContext
     * @param pageContext
     * @return the HttpServletRequest instance or null
     */
    private static HttpServletRequest getHttpServletRequest(PageContext pageContext)
    {        
	    if ( pageContext != null )
	    {
	        ServletRequest servletRequest = pageContext.getRequest();
	        if ( servletRequest != null )
	        {
		        if ( servletRequest instanceof HttpServletRequest )
		        {
		            return (HttpServletRequest) servletRequest ;
		        }
	        }
	    }
	    return null ;
    }
    
	//----------------------------------------------------------------------------------
	/**
     * Returns the ScreenSession object associated with the given PageContext request
	 * @param pageContext
	 * @return
	 */
	private static ScreenSession getScreenSession(PageContext pageContext)
	{
        HttpServletRequest httpServletRequest = getHttpServletRequest(pageContext);
	    if ( httpServletRequest != null )
	    {
	        return ScreenSessionManager.findScreenSession(httpServletRequest);
	    }
	    return null ;
	}

	//----------------------------------------------------------------------------------
// Suppr in v 1.0.1
//	/**
//     * Returns the ScreenApplication object from the given PageContext
//	 * @param pageContext
//	 * @return
//	 */
//	private static ScreenApplication getScreenApplication(PageContext pageContext)
//	{
//	    if ( pageContext != null )
//	    {
//	        HttpSession httpSession = pageContext.getSession();
//		    if ( httpSession != null )
//		    {
//		        return ScreenApplicationManager.getScreenApplication(httpSession); 
//		    }
//	    }
//	    return null ;
//	}

    //----------------------------------------------------------------------------------
    /**
     * Try to find an iterator from the given collection name located in the given scope
     * 
     * @param pageContext
     *            the page context
     * @param sName :
     *            the name of the list to search
     * @param sScope :
     *            the scope where to search (can be null for any scope )
     * @return the iterator if the given name exists and is an iterator provider (Collection, DataSet, VOList) <br>
     *         or null if the object cannot be found or is not an iterator provider 
     */
    public static Iterator findIterator(PageContext pageContext, String sName, String sScope) //throws TelosysException
    {
        Iterator oIterator = null;
        if (pageContext == null)
        {
            throw new TelosysRuntimeException("Finder.findIterator : page context is null ");
        }
        if (sName == null)
        {
            throw new TelosysRuntimeException("Finder.findIterator : list name is null ");
        }

        Object oList = Finder.findObject(pageContext, sName, sScope);
        if (oList != null)
        {
            if (oList instanceof GenericVOList)
            {
                oIterator = ((GenericVOList) oList).iterator();
            }
            else if (oList instanceof DataSet)
            {
                oIterator = ((DataSet) oList).iterator();
            }
            else if (oList instanceof Collection)
            {
                oIterator = ((Collection) oList).iterator();
            }
            else
            {
                //throw new TelosysException("Finder.findIterator : Invalid list type : class = " + oList.getClass().getName());
            }
        }
        else
        {
            //throw new TelosysException("Cannot find iterator : List '" + sName + "' not found");
        }
        return oIterator;
    }

    //----------------------------------------------------------------------------------
    /**
     * Try to find an object with the given name ( without scope => global search )
     * 
     * @param pageContext
     * @param sName
     *            the name of the object to search
     * @return
     */
    public static Object findObject(PageContext pageContext, String sName ) //throws TelosysException
    {
        return findObject( pageContext, sName, null );
    }
    
    /**
     * Try to find an object with the given name in the given scope
     * 
     * @param pageContext
     * @param sName
     *            the name of the object to search
     * @param sScope
     *            the scope where to search (can be null for any scope )
     * @return
     */
    public static Object findObject(PageContext pageContext, String sName, String sScope) 
    {
        if (pageContext == null)
        {
            throw new TelosysRuntimeException("Finder.findObject : page context is null ");
        }
        if (sName == null)
        {
            throw new TelosysRuntimeException("Finder.findObject : object name is null ");
        }

        String sBeanName = sName.trim();
        if (sBeanName.equalsIgnoreCase("RENDERER-BEAN") || sBeanName.equalsIgnoreCase("RENDERER_BEAN"))
        {
            return findInServletRequest(pageContext, TelosysConst.RENDERER_BEAN);
        }
        else
        {
            if (sScope != null)
            {
                return findWithScope(pageContext, sBeanName, sScope.trim());
            }
            else
            {
                return findWithoutScope(pageContext, sBeanName);
            }
        }
    }

    //----------------------------------------------------------------------------------
    /**
     * Try to find an object associated with the given name in the given scope<br>
     * @param pageContext : the current page context
     * @param sBeanName : the name of the bean to search
     * @param sScope : the scope where to search
     * @return : the object found (or null if not found)
     */
    private static Object findWithScope(PageContext pageContext, String sBeanName, String sScope) 
    {
        log.trace("findWithScope(pageContext,'"+sBeanName+"','"+ sScope+"')" ) ;
        
        if (sScope.equals("request"))
        {
            return findInServletRequest(pageContext, sBeanName);
        }
        else if (sScope.equals("session"))
        {
            return findInHttpSession(pageContext, sBeanName);
        }
        else if (sScope.equals("application"))
        {
            return findInServletContext(pageContext, sBeanName);
        }
        else if (sScope.equals("screen"))
        {
            //--- Search first in the "screen elements"
            Object obj = findElementInScreenData(pageContext, sBeanName);
            if ( obj != null )
            {
                return obj;
            }
            else
            {
                //--- If not found in the "screen elements" => search in the "screen attributes"
                return findAttributeInScreenContext(pageContext, sBeanName);
            }
        }
        else if (sScope.equals("screen-elements"))
        {
            return findElementInScreenData(pageContext, sBeanName);
        }
        else if (sScope.equals("screen-attributes"))
        {
            return findAttributeInScreenContext(pageContext, sBeanName);
        }
        else if (sScope.equals("screen-session"))
        {
            return findInScreenSession(pageContext, sBeanName);
        }
        else if (sScope.equals("screen-application"))
        {
            return findInScreenApplication(pageContext, sBeanName);
        }
        else if (sScope.equals("page"))
        {
            return findInPageContext(pageContext, sBeanName);
        }
        else
        {
            // Invalid scope
            //throw new TelosysException("Invalid scope '" + sScope + "'");
            return null ;
        }
    }

    //----------------------------------------------------------------------------------
    /**
     * Try to find an object associated with the given name<br>
     * The object is searched in all the request environment<br>
     * The the search is ordered by priory : <br>
     * 1 : ScreenData elements <br>
     * 2 : ScreenContext attributes <br> 
     * 3 : ServletRequest attributes <br> 
     * 4 : ScreenSession attributes <br> 
     * 5 : HttpSession attributes <br> 
     * 6 : ScreenApplication attributes <br> 
     * 7 : ServletContext attributes <br> 
     * 8 : JSP PageContext attributes ( since v 1.0.1 ) <br> 
     * @param pageContext : the current page context
     * @param sBeanName : the name of the bean to search
     * @return : the object found (or null if not found)
     */
    private static Object findWithoutScope(PageContext pageContext, String sBeanName)
    {
        log.trace("findWithoutScope(pageContext,'"+sBeanName+"')" ) ;
        Object oBean = null;

        //----------------------------------------------------------------------------------
        // SCREEN LEVEL 
        //----------------------------------------------------------------------------------
        //--- Search element in the "screen" (TELOSYS SCREEN-DATA ELEMENTS LEVEL)
        if ((oBean = findElementInScreenData(pageContext, sBeanName)) != null)
        {
            return oBean;
        }
        
        //--- Search attribute in the "screen" (TELOSYS SCREEN-CONTEXT ATTRIBUTES LEVEL)
        if ((oBean = findAttributeInScreenContext(pageContext, sBeanName)) != null)
        {
            return oBean;
        }
        
        //----------------------------------------------------------------------------------
        // REQUEST LEVEL 
        //----------------------------------------------------------------------------------
        //--- Search attribute in the "standard servlet request" (REQUEST LEVEL)
        if ((oBean = findInServletRequest(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //----------------------------------------------------------------------------------
        // SESSION LEVEL 
        //----------------------------------------------------------------------------------
        //--- Search attribute in the "screen session" (TELOSYS SESSION LEVEL)
        if ((oBean = findInScreenSession(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //--- Search attribute in the "standard http session" (SESSION LEVEL)
        if ((oBean = findInHttpSession(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //----------------------------------------------------------------------------------
        // APPLICATION LEVEL 
        //----------------------------------------------------------------------------------
        //--- Search attribute in the "screen application" (TELOSYS APPLICATION LEVEL)
        if ((oBean = findInScreenApplication(pageContext, sBeanName)) != null)
        {
            return oBean;
        }
        
        //--- Search attribute in the  "standard servlet context" (APPLICATION LEVEL)
        if ((oBean = findInServletContext(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //----------------------------------------------------------------------------------
        // JSP PAGE CONTEXT LEVEL ( since v 1.0.1 )
        //----------------------------------------------------------------------------------
        if ((oBean = findInPageContext(pageContext, sBeanName)) != null)
        {
            return oBean;
        }
        
        //--- Not found
        return null;
    }

    //----------------------------------------------------------------------------------
    // Find in JSP page context variables
    //----------------------------------------------------------------------------------
    private static Object findInPageContext(PageContext pageContext, String sBeanName) // v 1.0.1
    {
        return Page.getVar(pageContext, sBeanName);
    }

    //----------------------------------------------------------------------------------
    // Find in standard servlet environment 
    //----------------------------------------------------------------------------------
    private static Object findInServletRequest(PageContext pageContext, String sBeanName)
    {
        ServletRequest servletRequest = pageContext.getRequest();
        if (servletRequest != null)
        {
            return servletRequest.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (request) : cannot get ServletRequest");
        }
    }

    //----------------------------------------------------------------------------------
    private static Object findInHttpSession(PageContext pageContext, String sBeanName)
    {
        HttpSession httpSession = pageContext.getSession();
        if (httpSession != null)
        {
            return httpSession.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (session) : cannot get HttpSession");
        }
    }

    //----------------------------------------------------------------------------------
    private static Object findInServletContext(PageContext pageContext, String sBeanName)
    {
        ServletContext servletContext = pageContext.getServletContext();
        if (servletContext != null)
        {
            return servletContext.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (application) : cannot get ServletContext");
        }
    }

    //----------------------------------------------------------------------------------
    // Find in Telosys Environment 
    //----------------------------------------------------------------------------------
    private static Object findElementInScreenData(PageContext pageContext, String sBeanName)
    {
        log.trace("findElementInScreenData(pageContext,'"+sBeanName+"')" ) ;
        
        RequestEnv requestEnv = Page.getRequestEnv(pageContext);
        if (requestEnv != null)
        {
            log.trace("findElementInScreenData(pageContext,'"+sBeanName+"') : requestEnv FOUND " ) ;
            ScreenData screenData = requestEnv.getScreenData();
            if (screenData != null)
            {
                log.trace("findElementInScreenData(pageContext,'"+sBeanName+"') : screenData FOUND " ) ;
                if ( screenData.isElementDefined(sBeanName) ) // to avoid TelosysRuntimeException 
                {                    
//                    Object obj = null ;
//	                try
//	                {
//	                    obj = screenData.getElement(sBeanName);
//	                } catch ( TelosysRuntimeException e )
//	                {
//	                    // element not defined => considered as not found
//	                    obj = null ;
//	                }
//	                return obj ;
                    return screenData.getElement(sBeanName);
                }
                else
                {
                    return null ;
                }
            }
        }
        log.trace("findElementInScreenData(pageContext,'"+sBeanName+"') : requestEnv or screenData NOT FOUND " ) ;
        // if not found => it's not an error ( ScreenContext/ScreenData available only for ScreenAction request ) 
        return null ;
    }
    //----------------------------------------------------------------------------------
    private static Object findAttributeInScreenContext(PageContext pageContext, String sBeanName)
    {
        RequestEnv requestEnv = Page.getRequestEnv(pageContext);
        if (requestEnv != null)
        {
            ScreenContext screenContext = requestEnv.getScreenContext();
            if (screenContext != null)
            {
                return screenContext.getAttribute(sBeanName);
            }
        }
        // if not found => it's not an error ( ScreenContext available only for ScreenAction request ) 
        return null ;
    }
    //----------------------------------------------------------------------------------
    private static Object findInScreenSession(PageContext pageContext, String sBeanName)
    {
//        RequestEnv requestEnv = Page.getRequestEnv(pageContext);
//        if (requestEnv != null)
//        {
//            ScreenSession screenSession = requestEnv.getScreenSession();
//            if (screenSession != null)
//            {
//                return screenSession.getAttribute(sBeanName);
//            }
//            // if RequestEnv is set, the ScreenSession must exist
//            throw new TelosysRuntimeException("Finder : cannot get ScreenSession in RequestEnv");
//        }
//        return null ; // RequestEnv is not always set ( e.g. ScreenMap )
        
        ScreenSession screenSession = getScreenSession(pageContext);
        if (screenSession != null)
        {
            return screenSession.getAttribute(sBeanName);
        }        
        return null ; 
    }
    //----------------------------------------------------------------------------------
    private static Object findInScreenApplication(PageContext pageContext, String sBeanName)
    {
        //ScreenApplication screenApplication = getScreenApplication( pageContext );
        ScreenApplication screenApplication = ScreenApplicationManager.getScreenApplication(); // v 1.0.1
        if (screenApplication != null)
        {
            return screenApplication.getAttribute(sBeanName);
        }
        return null ;
    }

    //----------------------------------------------------------------------------------
    //----------------------------------------------------------------------------------
    /**
     * Try to find an object attribute with the given name ( without scope => global search )
     * 
     * @param pageContext
     *            the page context
     * @param sName
     *            the name of the attribute value to search ( 'bean.attr', 'attr', '.attr'
     * @return the attribute value or null if bean not found or attribute not found
     */
    public static Object findAttributeValue(PageContext pageContext, String sName)
    {
        return findAttributeValue(pageContext, sName, null);
    }

    //----------------------------------------------------------------------------------
    /**
     * Try to find an object attribute with the given name in the given scope
     * 
     * @param pageContext
     *            the page context
     * @param sName
     *            the name of the attribute value to search ( 'bean.attr', 'bean', '.attr' )
     * @param sScope
     *            the scope where to search (can be null for any scope )
     * @return the attribute value or null if bean not found or attribute not found
     */
    public static Object findAttributeValue(PageContext pageContext, String sName, String sScope)
    {
        String sBeanName = BeanUtil.getBeanName(sName);
        String sFieldName = BeanUtil.getAttributeName(sName);
        Object oBean = null;
        if (sBeanName != null)
        {
            //--- Try to find the object
            oBean = Finder.findObject(pageContext, sBeanName, sScope);
        }
        else
        {
            //--- Try to find the "current bean" object
            //oBean = Page.getCurrentBean(pageContext);
            oBean = CurrentBean.getBean(pageContext);
        }
        if ( oBean != null ) // Bean found
        {
            if ( sFieldName != null )
            {
                //--- Try to found the attribute
	            try
	            {
	                //--- Is there a "getFieldName()" or "isFieldName()" accessor ?
	                return BeanUtil.getFieldValue(oBean, sFieldName);
	            } catch (TelosysException e)
	            {
	                // Unknown field
	                return null;
	            }
            }
            else
            {
                //--- No attribute name => the object as is
                return oBean ;
            }
        }
        return null;
    }    
}