package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import org.objectweb.telosys.uil.taglib.TagConst;
import org.objectweb.telosys.uil.taglib.widget.Field;
import org.objectweb.telosys.uil.taglib.widget.GeneratorAncestor;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;

/**
 * Generic class for all HTML tags generators
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GenericHTML extends GeneratorAncestor
{

    //--------------------------------------------------------------------------------------------------
    // LABEL
    //--------------------------------------------------------------------------------------------------

    /**
     * Returns a "label" tag with "for" attribute referencing the current tag Id. <br>
     * The label has no style, no class. Its Id is tag_id + "_label"
     * 
     * @param sId
     * @param sTxt
     * @return
     */
    protected String getHtmlLabelFor(String sId, String sTxt)
    {
        return "<label for=\"" + sId + "\"" + " id=\"" + sId + "_label\"" + " >" + sTxt + "</label>";
    }

    //--------------------------------------------------------------------------------------------------
    // TITLE
    //--------------------------------------------------------------------------------------------------
    
    /**
     * Returns the html "title" attribute ( ie : 'title="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getTitleAttr(GenericTag tag)
    {
        String sTitle = tag.getTitle();
        if (sTitle != null)
        {
            return "title=\"" + sTitle + "\"";
        }
        else
        {
            return "";
        }
    }


//    /**
//     * Returns the html "id" attribute ( ie : 'id="xx"' )
//     * 
//     * @param tag
//     * @return 'id="xx"' or ''
//     */
//    protected String getIdAttr(GenericTag tag)
//    {
//        String sId = tag.getId();
//        if (sId != null)
//        {
//            return "id=\"" + sId + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }
    
//    /**
//     * Returns the html "id" attribute concat with index( ie : 'id="xx_nn"' )
//     * @param tag
//     * @param iIndex to concat 
//     * @return
//     */
//    protected String getIdAttr(GenericTag tag, int iIndex)
//    {
//        String sId = tag.getId();
//        if (sId != null)
//        {
//            return "id=\"" + sId + "_" + iIndex + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }

    //--------------------------------------------------------------------------------------------------
    // STYLE
    //--------------------------------------------------------------------------------------------------
    
    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' )
     * @param sStyle
     * @return 'style="xx"' ( or '' if style argument is null )
     */
    protected String getStyleAttr(String sStyle)
    {
        if (sStyle != null)
        {
            return "style=\"" + sStyle + "\"";
        }
        else
        {
            return "";
        }
    }
    
    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' )
     * including absolute position if X or Y is set and WIDTH/HEIGHT if set.
     * @param tag
     * @return
     */
    protected String getStyleAttr(GenericTag tag)
    {
        // return getStyleAttrWithPosition(tag); // v 1.0.2
        
        // v 1.0.2
        if ( tag.isPositionAbsolute() )
        {
            return getStyleAttrWithPosition(tag);
        }
        else
        {
            return getStyleAttrWithoutPosition(tag);
        }
    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' )
     * including user style if any, WIDTH/HEIGHT if set, absolute position if X/Y
     * plus a specific style fragment.
     * @param tag
     * @param sSpecificStyle
     * @return
     * @since v 1.0.2
     */
    protected String getStyleAttr(GenericTag tag, String sSpecificStyle)
    {
        String s1 = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + getStyleFragPosition(tag) ;
        String s2 = sSpecificStyle != null ? sSpecificStyle : "" ;
        
        String s = s1 + s2 ;
        if ( s.trim().length() > 0 )
        {
            return "style=\"" + s + "\"";                    
        }
        else
        {
            return "";                                
        }
    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' or '' )
     * 
     * @param tag
     * @return the style attribute string (never null)
     */
    protected String getStyleAttr(GenericTag tag, int iPosType)
    {
        if (iPosType == POS_XY)
        {
            return getStyleAttrWithPosition(tag);
        }
        else
        {
            return getStyleAttrWithoutPosition(tag);
        }
    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' or '' )
     * without the position 
     *  
     * @param tag
     * @return
     */
    protected String getStyleAttrWithoutPosition(GenericTag tag)
    {
        String s = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) ;
        if ( s.trim().length() > 0 )
        {
            return "style=\"" + s + "\"";                    
        }
        else
        {
            return "";                                
        }
    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' or '' )
     * with absolute position elements ( 'position:absolute;left:..px;top:..px;' ) 
     *  
     * @param tag
     * @return
     */
    private String getStyleAttrWithPosition(GenericTag tag)
    {
// v 1.0.2        
//        if ( tag.isPositionAbsolute() )
//        {
            String s = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + getStyleFragPosition(tag) ;
            if ( s.trim().length() > 0 )
            {
                return "style=\"" + s + "\"";                    
            }
            else
            {
                return "";                                
            }
// v 1.0.2        
//        }
//        else
//        {
//            return getStyleAttrWithoutPosition(tag);
//        }
    }

    /**
     * Returns the "style" attribute for "container tags" <br>
     * If the tag position is not absolute, the style attribute contains 'position:relative' 
     * @param tag
     * @return
     */
    protected String getContainerStyleAttr(GenericTag tag)
    {        
        if (tag.isPositionAbsolute())
        {
            return getStyleAttrWithPosition(tag);
        }
        else
        {
            //--- Style set by user + Width/Height + force "relative position"
            String sStyle = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + "position:relative;" ;
            return " style=\"" + sStyle + "\" " ;            
        }
    }

    /**
     * Returns the style fragment for user style ( ie : 'color:...; ' )
     * 
     * @param tag
     * @return the style frament (never null, ending by ';' if not void )
     */
    protected String getStyleFragUserStyle(GenericTag tag)
    {
        String sUserStyle = tag.getStyle();
        if (sUserStyle != null)
        {
            String s = sUserStyle.trim();
            if ( s.length() > 0 )
            {
                if ( ! s.endsWith(";") )
                {
                    s = s + ";" ;
                }
            }
            return s ;
        }
        else
        {
            return "";            
        }
    }

    /**
     * Returns the style fragment for "position:...;left:..px;top:..px;" <br>
     * ie : 
     * if at least x or y is set : 'position:absolute;left:..px;top:..px;' 
     * if x and y are not set for the tag : '' ( void string )
     * 
     * @param tag
     * @return
     */
    protected String getStyleFragPosition(GenericTag tag)
    {
        if ( tag.isPositionAbsolute() )
        {
            String sX = tag.getXValue(); // never null
            String sY = tag.getYValue(); // never null
            return "position:absolute;left:" + sX.trim() + "px;top:" + sY.trim() + "px; ";            
        }
        return "";
    }

    /**
     * Returns the style fragment for "width" and "height" ( ie : 'width:..px; height:..px;' )
     * 
     * @param tag
     * @return : "" or "width:..px;" or "height:..px;" or "width:..px; height:..px;"
     */
    protected String getStyleFragWidthHeight(GenericTag tag)
    {
        String sWidth = tag.getWidth();
        String sHeight = tag.getHeight();
        String s = "";
        if (sWidth != null)
        {
            s = s + "width:" + sWidth + "px;";
        }
        if (sHeight != null)
        {
            s = s + "height:" + sHeight + "px;";
        }
        return s;
    }
    
    /**
     * Returns the style fragment for "width" ( ie : 'width:..px;' )
     * @param iWidth
     * @return "width:..px;" ( or "" if the width is null or invalid )
     */
    protected String getStyleFragWidth(int iWidth)
    {
        if ( iWidth != TagConst.WIDTH_NULL && iWidth >= 0 )
        {
            return "width:" + iWidth + "px;";
        }
        return "";
    }
    
    /**
     * Returns the style fragment for "height" ( ie : 'height:..px;' )
     * @param iHeight
     * @return "height:..px;" ( or "" if the height is null or invalid )
     */
    protected String getStyleFragHeight(int iHeight)
    {
        if ( iHeight != TagConst.HEIGHT_NULL && iHeight >= 0)
        {
            return "height:" + iHeight + "px;";
        }
        return "";
    }
    
    //--------------------------------------------------------------------------------------------------
    // REGISTER
    //--------------------------------------------------------------------------------------------------
    
    protected boolean needRegister(GenericTag tag)
    {
        if (tag.hasId()) // Register only fields with an id
        {
            if (tag.hasZone() || tag.hasDataBinding())
            {
                return true ;
            }
        }
        return false ;
    }
    protected String firstArgsToRegister(GenericTag tag)
    {
        //--- 1 : Tag ID
        String sId = "\"" + tag.getId() + "\"";

        //--- 2 : Zone name
        String sZone = "null";
        if (tag.hasZone())
        {
            sZone = tag.getZone();
            if (sZone != null)
            {
                sZone = "\"" + sZone.trim() + "\"";
            }
        }

        //--- 3 & 4 : Data binding : XML element & XML attribute 
        String sElement = "null";
        String sAttribute = "null";
        if (tag.hasDataBinding())
        {
            sElement = tag.getDataElement();
            if (sElement != null)
            {
                sElement = "\"" + sElement.trim() + "\"";
            }
            sAttribute = tag.getDataAttribute();
            if (sAttribute != null)
            {
                sAttribute = "\"" + sAttribute.trim() + "\"";
            }
        }

        return sId + ", " + sZone + ", " + sElement + ", " + sAttribute ;
    }
    
    protected void registerField(Field tag) throws IOException
    {
        if (needRegister(tag))
        {
            String sFirstArgs = firstArgsToRegister(tag);
            
            String sJavascript = null ;
            if ( tag.getInternaType() == Field.TYPE_DATE )
            {
                String sFormat = tag.getFormat() ; // never null for a Date field                   
                if ( sFormat == null ) // never happens
                {
                    sFormat = Field.DEFAULT_DATE_FORMAT ;
                }
                sFormat = "\"" + sFormat + "\""; // no trim
                sJavascript = "RegisterFieldDate(" + sFirstArgs + ", " + sFormat + ")";
            }
            else
            {
                sJavascript = "RegisterField(" + sFirstArgs + ")";                    
            }
            tag.addJavascriptLine(sJavascript);
        }
    }

    //--------------------------------------------------------------------------------------------------
    
}