package org.objectweb.telosys.upload;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.util.DirUtil;
import org.objectweb.telosys.util.StrUtil;

/**
 * Standard servlet for processing single file upload from the tag "fileupload" ( cf. taglib )
 * 
 * @author Laurent GUERIN
 *
 */
public class FileUploadServlet extends TelosysServlet // extends HttpServlet
{
    //----------------------------------------------------------------------------------
    // CONSTANTS
    //----------------------------------------------------------------------------------
    private static final int    NOT_INIT                       = 0;

    private static final int    INIT_OK                        = 1;

    private static final int    INIT_ERR                       = -1;

    private static final long   serialVersionUID               = 1L;

    private static final String RESPONSE_CONTENT_TYPE          = "text/html; charset=UTF-8";

    //--- Request parameters ( set by the tag "fileupload" )
    private static final String PARAM_MAX_SIZE                 = "max_size";

    private static final String PARAM_DEST_DIR                 = "dest_dir";

    private static final String PARAM_CONTEXT_ID               = "context_id";

    private static final String PARAM_CONTEXT_ATTRIBUTE        = "context_attribute";

    private static final String PARAM_SESSION_KEY              = "session_key";

    //--- The HTML "name" attribute : always the same ( set by the tag "fileupload" ) 
    //    ( <input type="file" name="file" )> </input> )
    private static final String UPLOADED_FILE_NAME             = "file"; 

    //--- Error messages
    private static final String ERROR_NOT_MULTIPART            = "Not Multipart";

    private static final String ERROR_NOT_DIRECTORY            = "Not a directory : ";

    private static final String ERROR_NO_FILE                  = "No file found";

    private static final String ERROR_SCREEN_NOT_FOUND         = "Screen context not found";

//    private static final String ERROR_ATTRIBUTE_NOT_BYTE_ARRAY = "Screen attribute not byte array";

    private static final String ERROR_UPLOAD_DIR_NOT_INIT      = "The upload directory is not defined, check server configuration.";

    //----------------------------------------------------------------------------------
    // ATTRIBUTES
    //----------------------------------------------------------------------------------
    private int                 _iInitStatus                   = NOT_INIT;

    private String              _sUploadDirectory              = null;

    /**
     * DoGet
     */
    protected void doGet(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("doGet()...");
        sendResponse(httpResponse, 901, "Invalid http method : 'GET' ('POST' method is required for an UPLOAD request)");
    }

    /**
     * DoPost
     */
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
    {
        trace("doPost()...");
        process(req, resp);
    }

    /**
     * Process
     */
    private void process(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException
    {
        boolean bFileSaved = false ;
        //--- Response text 
        String sResponseText = "OK" ;

        //--- Check the request : Is it really multipart ?
        if ( MultipartRequest.isMultipartContent(req) )
        {
            try
            {
                //--- Get the file from the request ( one single file )
                FileUploaded fileUploaded = getFileUploaded(req);
                if ( fileUploaded == null )
                {
                    throw new TelosysException(ERROR_NO_FILE);                                        
                }
                
                sResponseText += "\nFile uploaded ( name='" + fileUploaded.getFileName()
                		+ "', type='" + fileUploaded.getFileType() 
                		+ "', size=" + fileUploaded.getDataSize()+ ")" ;
                    
                //--- Store the file in the filesystem ?
                String sDestDir = fileUploaded.getParamDestDir() ;
                String sFile = fileUploaded.getFileName();
                if (notVoid(sDestDir))
                {
                    saveInFileSystem(fileUploaded, sDestDir, sFile);
                    sResponseText += "\nFile saved in filesystem ( dir : " + sDestDir + ", file : " + sFile + " ). ";
                    bFileSaved = true ;
                }

                //--- Keep the file in memory in the Screen Context ?
                int    iContextId = fileUploaded.getParamContextId();
                String sContextAttribute = fileUploaded.getParamContextAttribute();
                if ( iContextId >= 0 && notVoid(sContextAttribute))
                {
                    saveInScreenContext(fileUploaded, iContextId, sContextAttribute, req);
                    sResponseText += "\nFile saved in screen context ( id : " + iContextId + ", attribute : " + sContextAttribute + " ). ";
                    bFileSaved = true ;
                }

                //--- Keep the file in memory in the Session ?
                String sSessionKey = fileUploaded.getParamSessionKey();
                if (notVoid(sSessionKey))
                {
                    saveInHttpSession(fileUploaded, sSessionKey, req);
                    sResponseText += "\nFile saved in HTTP session ( key : " + sSessionKey + " ). ";
                    bFileSaved = true ;
                }

                if ( bFileSaved != true )
                {
                    sResponseText += "\nWARNING : File not saved ( no save parameter ) ! ";
                }
                
            } catch (IOException ioe)
            {
                sendResponse(resp, 902, ioe.getMessage());
            } catch (TelosysException te)
            {
                sendResponse(resp, 903, te.getMessage());
            }
        }
        else
        {
            // Not multipart => error
            sendResponse(resp, 904, ERROR_NOT_MULTIPART);
        }
        if ( sResponseText != null )
        {
            sendResponse(resp, 0, sResponseText);
        }
    }

    private int checkInit() throws TelosysException
    {
        String sErrorMsg = null ;
        if (_iInitStatus == NOT_INIT) // not yet init
        {
            //--- Try to init
            Telosys.info("File Upload Servlet : INIT upload directory ...");

            //--- Get the Telosys properties ...
            Properties prop = Telosys.getProperties();
            if (prop == null)
            {
                sErrorMsg = "File Upload Servlet : cannot init upload directory ( cannot get Telosys properties ) !" ;
                
                //Telosys.error("File Upload Servlet : cannot get Telosys properties !");
            }
            else
            {
                //--- Fichier de configuration des bases de donnes
                String sDir = prop.getProperty("UploadDir");
                if (sDir == null)
                {
                    Telosys.info("File Upload Servlet : No upload directory => use default directory");
                }
                else
                {
                    sDir = sDir.trim();
                    //--- Build full path from relative path (if any)
                    //sDir = Telosys.transformFileName(sDir); // "$ROOT" substitution removed 18/12/2007
                    if (DirUtil.exists(sDir))
                    {
                        if (!sDir.endsWith("/"))
                        {
                            sDir = sDir + "/";
                        }
                        Telosys.info("File Upload Servlet : Upload directory = '" + sDir + "'");
                        _sUploadDirectory = sDir;
                        _iInitStatus = INIT_OK;
                        return _iInitStatus;
                    }
                    else
                    {
                        //Telosys.error("File Upload Servlet : Upload directory = '" + sDir + "' doesn't exist !");
                        sErrorMsg = "File Upload Servlet : Upload directory = '" + sDir + "' doesn't exist !" ;
                    }
                }
            }
            _iInitStatus = INIT_ERR;
            throw new TelosysException( sErrorMsg );
        }
        return _iInitStatus;
    }

    /**
     * Save in filesystem
     * 
     * @throws TelosysException
     */
    private void saveInFileSystem(FileUploaded fileUploaded, String psDir, String psFileName) throws IOException, TelosysException
    {
        trace("saveInFileSystem( dir = " + psDir + " , file = '" + psFileName + "' ) ...");
        if ( psFileName == null )
        {
            throw new TelosysException("File Upload Servlet : Cannot save file (file parameter is null)");
        }
        String sDirectory = ( psDir != null ? psDir : "" ) ;
        sDirectory = sDirectory.trim();
        if (checkInit() == INIT_OK)
        {
            //--- Build the full directory
            if (sDirectory.startsWith("/"))
            {
                sDirectory = sDirectory.substring(1); // remove 1st char
            }
            if ( sDirectory.equals(".") ) // "." = save in the standard upload dir
            {
                sDirectory = ""; // no subdirectory                 
            }
            sDirectory = _sUploadDirectory + sDirectory;
            if (!DirUtil.exists(sDirectory))
            {
                throw new TelosysException(ERROR_NOT_DIRECTORY + sDirectory);
            }
            if (!sDirectory.endsWith("/"))
            {
                sDirectory += "/";
            }

            //--- Save the file in the full directory
            FileOutputStream fosSave = null;
            try
            {
                fosSave = new FileOutputStream(sDirectory + psFileName);
                fosSave.write(fileUploaded.getData());
            } finally
            {
                if (fosSave != null)
                {
                    fosSave.close();
                }
            }
        }
        else
        {
            throw new TelosysException(ERROR_UPLOAD_DIR_NOT_INIT);
        }
    }

    /**
     * Save in screen context
     */
    private void saveInScreenContext(FileUploaded fileUploaded, int iContextId, String sContextAttribute, HttpServletRequest req) throws TelosysException
    {
        trace("saveInScreenContext( id = " + iContextId + " , attribute = '" + sContextAttribute + "' ) ...");
        ScreenContext screenContext = ScreenContextManager.getScreenContext(req, iContextId);
        if (screenContext == null)
        {
            throw new TelosysException(ERROR_SCREEN_NOT_FOUND + "( id = " + iContextId + ")");
        }
        screenContext.setAttribute(sContextAttribute, fileUploaded);
    }

    /**
     * Save in session
     */
    private void saveInHttpSession(FileUploaded fileUploaded, String sessionKey, HttpServletRequest req)
    {
        trace("saveInHttpSession( Key = '" + sessionKey + "' ) ...");
        HttpSession session = req.getSession();
        session.setAttribute(sessionKey, fileUploaded);
    }

    /**
     * Returns the file uploaded.<br>
     * Returns the file associated with the standard fixed file name ( cf const UPLOADED_FILE_NAME )
     * @param req
     * @return
     * @throws IOException
     */
    private FileUploaded getFileUploaded(HttpServletRequest req) throws IOException
    {
        trace("getFileUploaded()...");
        long lMaxSize = -1;
        String sDestDir = null ;
        
        //--- Request parameters
        
        //---------------------------------------------------------------------------
        // GET REQUEST PARAMETERS
        //---------------------------------------------------------------------------
        long lParamMaxSize = -1;
        String sParamDestDir = null ;
        UploadParameters param = new UploadParameters();
        if (notVoid(req.getParameter(PARAM_MAX_SIZE)))
        {
            lParamMaxSize = StrUtil.getInt(req.getParameter(PARAM_MAX_SIZE), 0);
            param.setMaxSize( lParamMaxSize );
        }

        if (notVoid(req.getParameter(PARAM_DEST_DIR)))
        {
            sParamDestDir = (String) req.getParameter(PARAM_DEST_DIR);
            param.setDestDir( sParamDestDir );
        }

        if (notVoid(req.getParameter(PARAM_CONTEXT_ID)))
        {
            param.setContextId( StrUtil.getInt(req.getParameter(PARAM_CONTEXT_ID), 0) );
        }

        if (notVoid(req.getParameter(PARAM_CONTEXT_ATTRIBUTE)))
        {
            param.setContextAttribute( req.getParameter(PARAM_CONTEXT_ATTRIBUTE) );
        }

        if (notVoid(req.getParameter(PARAM_SESSION_KEY)))
        {
            param.setSessionKey( req.getParameter(PARAM_SESSION_KEY) );
        }

        //---------------------------------------------------------------------------
        // SET DEFAULT VALUES
        //---------------------------------------------------------------------------
        //--- Set the maximum size for the file data ( in bytes )
        if ( lParamMaxSize > 0 )
        {
            lMaxSize = 1024 * lParamMaxSize ; // Use param size ( in kb )
        }
        else
        {
            lMaxSize = -1; // No maximum size
        }
        
        //--- Set the destination directory ( if the request parameter is set )
        if ( sParamDestDir != null )
        {
            sDestDir = sParamDestDir.trim() ;
            if (!sDestDir.endsWith("/"))
            {
                sDestDir += "/";
            }            
        }
        
        //---------------------------------------------------------------------------
        // PARSE THE "MULTIPART REQUEST"
        //---------------------------------------------------------------------------
        //--- Multipart request creation : http request parsing ( with or without maximum size )
        MultipartRequest multipartRequest = null;
        if ( lMaxSize < 0 )
        {
            //--- No maximum size 
            multipartRequest = new MultipartRequest(req);
        }
        else
        {
            //--- Limited size 
            multipartRequest = new MultipartRequest(req, lMaxSize);
        }

        //--- Get the single "file part" from the MultipartRequest
        Part filePart = multipartRequest.getFile( UPLOADED_FILE_NAME ); // fixed file name 
        if (filePart != null)
        {
            String sFileName = filePart.getFileName() ;
            byte[] fileData  = filePart.getValue();
            String sFileType = filePart.getContentType() ;
            
            return new FileUploaded(sFileName, fileData, sFileType, lMaxSize, sDestDir, param );
        }
        else
        {
            return null ;            
        }
    }

    /**
     * Send the response code and message
     */
    private void sendResponse(HttpServletResponse resp, int returnCode, String message)
    {
        try
        {
            resp.setContentType(RESPONSE_CONTENT_TYPE);

            PrintWriter writer = resp.getWriter();
            writer.print(returnCode + " " + message);
            writer.close();
        } catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }

    /**
     * Not void
     */
    private boolean notVoid(String s)
    {
        return (s != null && s.length() > 0);
    }

}