package org.objectweb.telosys.upload;

import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.ServletInputStream;

/**
 * Multipart request parser
 */
public class MultipartParser
{

    /** input stream to read parts from */
    private ServletInputStream _in = null ;

    /** MIME boundary that delimits parts */
    private String             _sBoundary = null ;

    /** reference to the last file part we returned */
    private Part               _lastFilePart = null;

    /** buffer for readLine method */
    private byte[]             _buffer = new byte[8 * 1024];

    /**
     * Constructor without limit length
     */
    public MultipartParser(HttpServletRequest req, long maxSize) throws IOException
    {
        this(req, maxSize, true, true);
    }

    /**
     * Constructor with specific limit length
     */
    public MultipartParser(HttpServletRequest req, long maxSize, boolean buffer, boolean limitLength) throws IOException
    {
        // Check the content type to make sure it's "multipart/form-data"
        // Access header two ways to work around WebSphere oddities
        String type = null;
        String type1 = req.getHeader("Content-Type");
        String type2 = req.getContentType();
        // If one value is null, choose the other value
        if (type1 == null && type2 != null)
        {
            type = type2;
        }
        else if (type2 == null && type1 != null)
        {
            type = type1;
        }
        // If neither value is null, choose the longer value
        else if (type1 != null && type2 != null)
        {
            type = (type1.length() > type2.length() ? type1 : type2);
        }
        if (type == null || !type.toLowerCase().startsWith("multipart/form-data"))
        {
            throw new IOException("Posted content type isn't multipart/form-data");
        }
        // Check the content length to prevent denial of service attacks
        int length = req.getContentLength();
        if (length > maxSize)
        {
            throw new IOException("File content too large (" + length + ")");
        }
        // Get the boundary string; it's included in the content type.
        // Should look something like "------------------------12012133613061"
        String boundary = extractBoundary(type);
        if (boundary == null)
        {
            throw new IOException("Separation boundary was not specified");
        }
        // Save our values for later
        this._in = req.getInputStream();
        this._sBoundary = boundary;
        // Read the first line, should be the first boundary
        String line = readLine();
        if (line == null)
        {
            throw new IOException("Corrupt form data: premature ending");
        }
        // Verify that the line is the boundary
        if (!line.startsWith(boundary))
        {
            throw new IOException("Corrupt form data: no leading boundary");
        }
    }

    /**
     * Reads and returns the next part in the stream
     * @return : the next 'Part' or null at the end
     * @throws IOException
     */
    public Part readNextPart() throws IOException
    {
        // Read the first line, should look like this:
        // content-disposition: form-data; name="field1"; filename="file1.txt"
        String line = readLine();
        if (line == null)
        {
            // No parts left, we're done
            return null;
        }
        else if (line.length() == 0)
        {
            // IE4 on Mac sends an empty line at the end; treat that as the end.
            return null;
        }
        
        //--- Parse the content-disposition line
        String[] dispInfo = extractDispositionInfo(line);
        // String disposition = dispInfo[0]; // not used
        String name = dispInfo[1];
        String filename = dispInfo[2];
        // Now onto the next line. This will either be empty
        // or contain a Content-Type and then an empty line.
        line = readLine();
        if (line == null)
        {
            // No parts left, we're done
            return null;
        }
        
        //--- Get the content type, or null if none specified
        String contentType = extractContentType(line);
        if (contentType != null)
        {
            // Eat the empty line
            line = readLine();
            if (line == null || line.length() > 0)
            { // line should be empty
                throw new IOException("Malformed line after content type: " + line);
            }
        }
        else
        {
            // The default content type is "text/plain" ( see rfc1867 )
            contentType = "text/plain";
        }
        
        // Now, finally, we read the content (end after reading the boundary)
        _lastFilePart = new Part(name, _in, _sBoundary, contentType, filename);
        return _lastFilePart;
    }

    /**
     * Extracts and returns the 'boundary token' from the given line
     * @param line
     * @return
     */
    private String extractBoundary(String line)
    {
        // Use lastIndexOf() beacuse old browsers like IE 4 send the "boundary=" string multiple times.
        int index = line.lastIndexOf("boundary=");
        if (index == -1)
        {
            return null;
        }
        String boundary = line.substring(index + 9); // 9 for "boundary="
        // The real boundary is always preceeded by an extra "--"
        boundary = "--" + boundary;
        return boundary;
    }

    /**
     * Extracts and returns disposition info from a line <br>
     * Line example : content-disposition: form-data; name="field1"; filename="file1.txt"
     * @param line
     * @return : the line's disposition data as a String array : 0:disposition, 1:name, 2:filename
     * @throws IOException
     */
    private String[] extractDispositionInfo(String line) throws IOException
    {
        // Convert the line to a lowercase string without the ending \r\n
        // Keep the original line for error messages and for variable names.
        String origline = line;
        line = origline.toLowerCase();
        
        //--- Get the content disposition, should be "form-data"
        int start = line.indexOf("content-disposition: ");
        int end = line.indexOf(";");
        if (start == -1 || end == -1)
        {
            throw new IOException("Content disposition corrupt: " + origline);
        }
        String disposition = line.substring(start + 21, end);
        if (!disposition.equals("form-data"))
        {
            throw new IOException("Invalid content disposition: " + disposition);
        }
        
        //--- Get the field name
        start = line.indexOf("name=\"", end); // start at last semicolon
        end = line.indexOf("\"", start + 7); // skip name=\"
        if (start == -1 || end == -1)
        {
            throw new IOException("Content disposition corrupt: " + origline);
        }
        String name = origline.substring(start + 6, end);
        
        //--- Get the filename, if given
        String filename = null;
        start = line.indexOf("filename=\"", end + 2); // start after name
        end = line.indexOf("\"", start + 10); // skip filename=\"
        if (start != -1 && end != -1)
        { // note the !=
            filename = origline.substring(start + 10, end);
            // The filename may contain a full path. Cut to just the filename.
            int slash = Math.max(filename.lastIndexOf('/'), filename.lastIndexOf('\\'));
            if (slash > -1)
            {
                filename = filename.substring(slash + 1); // past last slash
            }
        }
        
        //--- Return a String array: disposition, name, filename ( empty filename = no file posted )
        String[] retval = new String[3];
        retval[0] = disposition;
        retval[1] = name;
        retval[2] = filename;
        return retval;
    }

    /**
     * Extracts and returns the content type from the given line
     * @param sLine
     * @return : the content type ( i.e. "text/plain", ... )
     * @throws IOException
     */
    private String extractContentType(String sLine) throws IOException
    {
        String contentType = null;
        // Convert the line to a lowercase string
        String origline = sLine;
        sLine = origline.toLowerCase();
        // Get the content type, if any
        if (sLine.startsWith("content-type"))
        {
            int start = sLine.indexOf(" ");
            if (start == -1)
            {
                throw new IOException("Content type corrupt: " + origline);
            }
            contentType = sLine.substring(start + 1);
        }
        else if (sLine.length() != 0)
        { // no content type, so should be empty
            throw new IOException("Malformed line after disposition: " + origline);
        }
        return contentType;
    }

    /**
     * Reads the next input line.
     * @return 
     * @throws IOException
     */
    private String readLine() throws IOException
    {
        StringBuffer sbuf = new StringBuffer();
        int result;
        do
        {
            result = _in.readLine(_buffer, 0, _buffer.length); // does +=
            if (result != -1)
            {
                sbuf.append(new String(_buffer, 0, result, "ISO-8859-1"));
            }
        } while (result == _buffer.length); // loop only if the buffer was filled

        if (sbuf.length() == 0)
        {
            return null; // nothing read, must be at the end of stream
        }
        sbuf.setLength(sbuf.length() - 2); // cut off the trailing \r\n
        return sbuf.toString();
    }

}

