package org.objectweb.telosys.upload;

import java.io.FilterInputStream;
import java.io.IOException;
import javax.servlet.ServletInputStream;

/**
 * A PartInputStream filters a ServletInputStream
 */
public class PartInputStream extends FilterInputStream
{

    /** boundary which "ends" the stream */
    private String  _sBoundary = null ;

    /** our buffer */
    private byte[]  _buffer = new byte[64 * 1024]; // 64k

    /** number of bytes we've read into the buffer */
    private int     _iCount = 0 ;

    /** current position in the buffer */
    private int     _iPosition = 0 ;

    /** flag that indicates if we have encountered the boundary */
    private boolean _bEOF = false ;

    /**
     * Constructor
     */
    PartInputStream(ServletInputStream in, String boundary) throws IOException
    {
        super(in);
        this._sBoundary = boundary;
    }

    /**
     * Fill up the buffer from the underlying input stream
     */
    private void fill() throws IOException
    {
        if (_bEOF)
        {
            return;            
        }
        // as long as we are not just starting up
        if (_iCount > 0)
        {
            // if the caller left the requisite amount spare in the buffer
            if (_iCount - _iPosition == 2)
            {
                // copy it back to the start of the buffer
                System.arraycopy(_buffer, _iPosition, _buffer, 0, _iCount - _iPosition);
                _iCount -= _iPosition;
                _iPosition = 0;
            }
            else
            {
                // should never happen, but just in case
                throw new IllegalStateException("fill() detected illegal buffer state");
            }
        }
        // Try and fill the entire buffer, starting at count, line by line
        int read = 0;
        int boundaryLength = _sBoundary.length();
        int maxRead = _buffer.length - boundaryLength - 2; // -2 is for /r/n
        while (_iCount < maxRead)
        {
            // read a line
            read = ((ServletInputStream) in).readLine(_buffer, _iCount, _buffer.length - _iCount);
            // check for eof and boundary
            if (read == -1)
            {
                throw new IOException("unexpected end of part");
            }
            else
            {
                if (read >= boundaryLength)
                {
                    _bEOF = true;
                    for ( int i = 0 ; i < boundaryLength ; i++ )
                    {
                        if (_sBoundary.charAt(i) != _buffer[_iCount + i])
                        {
                            // Not the boundary!
                            _bEOF = false;
                            break;
                        }
                    }
                    if (_bEOF)
                    {
                        break;
                    }
                }
            }
            // success
            _iCount += read;
        }
    }

    /**
     * Read method
     */
    public int read() throws IOException
    {
        if (_iCount - _iPosition <= 2)
        {
            fill();
            if (_iCount - _iPosition <= 2)
            {
                return -1;
            }
        }
        return _buffer[_iPosition++] & 0xff;
    }

    /**
     * Read method
     */
    public int read(byte b[]) throws IOException
    {
        return read(b, 0, b.length);
    }

    /**
     * Read method
     */
    public int read(byte b[], int off, int len) throws IOException
    {
        int total = 0;
        if (len == 0)
        {
            return 0;
        }
        int avail = _iCount - _iPosition - 2;
        if (avail <= 0)
        {
            fill();
            avail = _iCount - _iPosition - 2;
            if (avail <= 0)
            {
                return -1;
            }
        }
        int copy = Math.min(len, avail);
        System.arraycopy(_buffer, _iPosition, b, off, copy);
        _iPosition += copy;
        total += copy;
        while (total < len)
        {
            fill();
            avail = _iCount - _iPosition - 2;
            if (avail <= 0)
            {
                return total;
            }
            copy = Math.min(len - total, avail);
            System.arraycopy(_buffer, _iPosition, b, off + total, copy);
            _iPosition += copy;
            total += copy;
        }
        return total;
    }

    /**
     * Returns the number of bytes that can be read from this input stream
     */
    public int available() throws IOException
    {
        int avail = (_iCount - _iPosition - 2) + in.available();
        // Never return a negative value
        return (avail < 0 ? 0 : avail);
    }

    /**
     * Closes this input stream and releases any system resources associated
     */
    public void close() throws IOException
    {
        if (!_bEOF)
        {
            while (read(_buffer, 0, _buffer.length) != -1)
                ; // do nothing
        }
    }
}