package org.objectweb.telosys.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.objectweb.telosys.common.TelosysException;

/**
 */
public class FileUtil
{
    private static final int BUFFER_SIZE = 1024 ; // 1 kb            

    //----------------------------------------------------------------------------
    /**
     * Save a memory content in a file
     * @param sFileName 
     * @param content
     * @throws TelosysException
     */
    public static void save(String sFileName, byte[] content) throws TelosysException
    {
        FileOutputStream fos = null ;
        try
        {
            fos = new FileOutputStream(sFileName);
        } catch (FileNotFoundException ex)
        {
            throw new TelosysException("save : cannot open file.", ex);
        }
        if ( fos != null )
        {
            try
            {
                fos.write(content, 0, content.length );
                fos.close();
            } catch (IOException ioex)
            {
                throw new TelosysException("save : write error.", ioex);
            }
        }
    }
    
    //----------------------------------------------------------------------------
    /**
     * Load a file content in memory
     * @param sFileName
     * @return file content or null if the file doesn't exist
     * @throws TelosysException
     */
    public static byte[] load(String sFileName) throws TelosysException
    {
        FileInputStream fis = null ;
        try
        {
            fis = new FileInputStream(sFileName);
        } catch (FileNotFoundException ex)
        {
            throw new TelosysException("load : cannot open file.", ex);
        }
        if ( fis != null )
        {
            ByteArrayOutputStream os = new ByteArrayOutputStream(BUFFER_SIZE);
            byte [] buffer = new byte [BUFFER_SIZE] ; 
            int i = 0;
            try
            {
                while ( ( i = fis.read(buffer) ) > 0 )
                {
                    os.write(buffer, 0, i);                
                }
                fis.close();
                //os.close(); // no effect on a ByteArrayOutputStream
            } catch (IOException ioex)
            {
                throw new TelosysException("load : read error.", ioex);
            }
            return os.toByteArray();
        }
        else
        {
            return null ;
        }
    }
    
    //----------------------------------------------------------------------------
    /**
     * Copy a file into another one
     * @param sInputFileName
     * @param sOutputFileName
     * @throws TelosysException
     */
    public static void copy(String sInputFileName, String sOutputFileName) throws TelosysException
    {
        //--- Open input file
		FileInputStream fis = null;
        try
        {
            fis = new FileInputStream(sInputFileName);
        } catch (FileNotFoundException ex)
        {
            throw new TelosysException("copy : cannot open input file.", ex);
        }
        
        //--- Open output file
		FileOutputStream fos = null;
        try
        {
            fos = new FileOutputStream(sOutputFileName);
        } catch (FileNotFoundException ex)
        {
            throw new TelosysException("copy : cannot open output file.", ex);
        }
        
        //--- Copy and close
        if ( fis != null && fos != null )
        {
			byte buffer[] = new byte[BUFFER_SIZE];
			int len = 0;
			
			try
            {
                while ((len = fis.read(buffer)) > 0)
                {
                    fos.write(buffer, 0, len);
                }
                fis.close();
                fos.close();
            } catch (IOException ioex)
            {
                throw new TelosysException("copy : IO error.", ioex);
            }
		}
    }
    
    //----------------------------------------------------------------------------
    /**
     * Delete a file (and only a file)
     * @param sFileName the full name of the file to delete
     * @return true if and only if the file or directory is successfully deleted; false otherwise
     * @throws TelosysException if the given filename is not a 'normal file' ( directory or other )
     */
    public static boolean delete(String sFileName) throws TelosysException
    {
        File file = new File(sFileName);
        if ( file.exists() )
        {
	        if ( file.isFile() )
	        {
	            return file.delete();
	        }
	        else
	        {
	            throw new TelosysException("delete : " + sFileName + " is not a file.");
	        }
        }
        else
        {
            return false ;
        }
    }
    
    //----------------------------------------------------------------------------
    /**
     * Test if the given filename exist and is realy a 'normal file' 
     * @param sFileName
     * @return true if the file exists, false if it is a directory or if it doesn't exist
     */
    public static boolean isFile(String sFileName) 
    {
        File file = new File(sFileName);
        return file.isFile() ;
    }
}
