package org.objectweb.telosys.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 *  
 */
public class JarUtil
{
    public static final boolean             KEEP_PARENT_DIR   = true;

    public static final boolean             REMOVE_PARENT_DIR = false;

    private static final TelosysClassLogger $log              = new TelosysClassLogger(JarUtil.class);

    /**
     * Open a JAR file and return a JarFile instance
     * 
     * @param sJarFile
     * @return JarFile instance
     * @throws TelosysException
     */
    private static JarFile openJarFile(String sJarFile) throws TelosysException
    {
        JarFile jarFile = null;
        try
        {
            jarFile = new JarFile(sJarFile);
        } catch (IOException e)
        {
            throw new TelosysException("Cannot open JAR file '" + sJarFile + "'.", e);
        }
        return jarFile;
    }

    /**
     * Copy an input stream in an output stream
     * 
     * @param inStream
     * @param outStream
     * @throws IOException
     */
    private static void copyStream(InputStream inStream, OutputStream outStream) throws IOException
    {
        byte[] buffer = new byte[1024];
        int bytesRead;

        //--- Read input data and write it to the output stream
        while ((bytesRead = inStream.read(buffer)) != -1)
        {
            outStream.write(buffer, 0, bytesRead);
        }
    }

    /**
     * Extract a file from a JAR file (replace the extracted file if it exists)
     * 
     * @param sJarFile
     *            jar file name
     * @param sJarEntry
     *            file name to extract
     * @param sExtractedFile
     *            file name of the extracted file
     * @throws TelosysException
     */
    public static void extract(String sJarFile, String sJarEntry, String sExtractedFile) throws TelosysException
    {
        JarFile jarFile = null;
//        InputStream inputStream = null;
//        FileOutputStream outputStream = null;

        if (sJarFile == null)
        {
            throw new TelosysException("Jar file name is null");
        }
        if (sJarEntry == null)
        {
            throw new TelosysException("Jar entry name is null");
        }
        if (sExtractedFile == null)
        {
            throw new TelosysException("Extracted file name is null");
        }

        try
        {
            jarFile = openJarFile(sJarFile);
            //System.out.println("JAR file " + sJarFile + " opened.");

            JarEntry jarEntry = jarFile.getJarEntry(sJarEntry);

            if (jarEntry != null)
            {
// v 0.9.9 : use generic "extractEntry" method
//                //--- Get an input stream for the entry.
//                try
//                {
//                    inputStream = jarFile.getInputStream(jarEntry);
//                } catch (IOException e)
//                {
//                    throw new TelosysException("Cannot get Input Stream for entry '" + sJarEntry + "' not found in JAR file '" + sJarFile + "'.");
//                }
//
//                //--- Create the output file (clobbering the file if it exists).
//                try
//                {
//                    outputStream = new FileOutputStream(sExtractedFile);
//                } catch (FileNotFoundException e1)
//                {
//                    throw new TelosysException("Cannot create output file '" + sExtractedFile + "'.");
//                }
//
//                //--- Create output file content (copy)
//                try
//                {
//                    copyStream(inputStream, outputStream);
//                } catch (IOException e2)
//                {
//                    throw new TelosysException("Cannot copy stream.", e2);
//                }
                extractEntry(jarFile, jarEntry, sExtractedFile);
            }
            else
            {
                throw new TelosysException("Entry '" + sJarEntry + "' not found in JAR file '" + sJarFile + "'.");
            }
        } finally
        {
//            if (outputStream != null)
//            {
//                //System.out.println("Close outputStream.");
//                try
//                {
//                    outputStream.close();
//                } catch (IOException e)
//                {
//                    // log and continue
//                    $log.error("Cannot close output stream ('" + sExtractedFile + "').");
//                }
//            }
//            if (inputStream != null)
//            {
//                //System.out.println("Close inputStream.");
//                try
//                {
//                    inputStream.close();
//                } catch (IOException e)
//                {
//                    // log and continue
//                    $log.error("Cannot close input stream ('" + sJarEntry + "').");
//                }
//            }
            if (jarFile != null)
            {
                //System.out.println("Close jar file.");
                try
                {
                    jarFile.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close jar file ('" + sJarFile + "').");
                }
            }
        }
    }

    /**
     * Extract the given entry from the given JAR file
     * The destination directories are created if necessary.
     * 
     * @param jarFile
     * @param jarEntry
     * @param sExtractedFile
     * @throws TelosysException
     */
    public static void extractEntry(JarFile jarFile, JarEntry jarEntry, String sExtractedFile) throws TelosysException
    {
        InputStream inputStream = null;
        FileOutputStream outputStream = null;

        if (jarFile == null)
        {
            throw new TelosysException("JarFile argument is null");
        }
        if (jarEntry == null)
        {
            throw new TelosysException("JarEntry argument is null");
        }
        if (jarEntry.isDirectory())
        {
            throw new TelosysException("JarEntry '" + jarEntry.getName() + "' is a directory");
        }
        if (sExtractedFile == null)
        {
            throw new TelosysException("Extracted file argument is null");
        }

        //--- Creates the directory of the file (if necessary)
        DirUtil.createFileDir(sExtractedFile);

        try
        {
            //--- Get an input stream for the entry.
            try
            {
                inputStream = jarFile.getInputStream(jarEntry);
            } catch (IOException e)
            {
                throw new TelosysException("Cannot get Input Stream for entry '" + jarEntry.getName() + "' not found in JAR file.");
            }

            //--- Create the output file (clobbering the file if it exists).
            try
            {
                outputStream = new FileOutputStream(sExtractedFile);
            } catch (FileNotFoundException e1)
            {
                throw new TelosysException("Cannot create output file '" + sExtractedFile + "'.");
            }

            //--- Create output file content (copy)
            try
            {
                copyStream(inputStream, outputStream);
            } catch (IOException e2)
            {
                throw new TelosysException("Cannot copy stream.", e2);
            }
        } finally
        {
            if (outputStream != null)
            {
                //System.out.println("Close outputStream.");
                try
                {
                    outputStream.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close output stream ('" + sExtractedFile + "').");
                }
            }
            if (inputStream != null)
            {
                //System.out.println("Close inputStream.");
                try
                {
                    inputStream.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close input stream ( JarEntry = '" + jarEntry.getName() + "' ).");
                }
            }
        }
    }

    /**
     * Extract all the files corresponding to the given beginning
     * 
     * @param sJarFile
     *            the JAR file name
     * @param sBeginning
     *            the beginning of the file names to extract
     * @param sArgDestDir
     *            the destination directory
     * @param bKeepParentDir
     *            KEEP_PARENT_DIR (true) : to keep the parent directory of the JAR entry, <br>
     *            REMOVE_PARENT_DIR (false) : to remove the parent directory
     * @throws TelosysException
     */
    public static void extractFiles(String sJarFile, String sBeginning, String sArgDestDir, boolean bKeepParentDir) throws TelosysException
    {
        if (sJarFile == null)
        {
            throw new TelosysException("JAR file argument is null");
        }
        if (sBeginning == null)
        {
            throw new TelosysException("Beginning argument is null");
        }
        if (sArgDestDir == null)
        {
            throw new TelosysException("Destination directory argument is null");
        }
        JarFile jarFile = null;
        //--- Destination directory ( without "/" at the end )
        String sDestDir = sArgDestDir;
        if (sDestDir.endsWith("/"))
        {
            //--- Remove last char
            sDestDir = sDestDir.substring(0, sDestDir.length() - 1);
        }

        //        //--- Create the destination directory if necessary
        //        File destDir = new File(sDestDir);
        //        if ( ! destDir.exists() )
        //        {
        //            boolean bRet = destDir.mkdirs();
        //            if ( bRet != true )
        //            {
        //                throw new TelosysException("extractFiles() : Cannot creat destination directory '" + sDestDir + "'." );
        //            }
        //        }

        try
        {
            //--- Open the JAR file
            jarFile = new JarFile(sJarFile);

            //--- For each entry ...
            Enumeration jarFileEntriesEnum = jarFile.entries();
            JarEntry currentJarEntry = null;
            while (jarFileEntriesEnum.hasMoreElements())
            {
                // get a JarEntry
                currentJarEntry = (JarEntry) jarFileEntriesEnum.nextElement();
                if (!currentJarEntry.isDirectory())
                {
                    String sEntryName = currentJarEntry.getName();
                    String sDestFile = null;

                    if (sEntryName.startsWith(sBeginning)) // This file is to extract
                    {
                        String sFileName = sEntryName;
                        if (bKeepParentDir == REMOVE_PARENT_DIR)
                        {
                            //--- Remove the parent directory
                            File f = new File(sEntryName);
                            sFileName = f.getName();
                        }

                        //--- Build the "Full Path" Destination File
                        if (sFileName.startsWith("/"))
                        {
                            sDestFile = sDestDir + sFileName;
                        }
                        else
                        {
                            sDestFile = sDestDir + "/" + sFileName;
                        }
                        //--- Extract the entry from the JAR file
                        //System.out.println(" . " + sEntryName + " --> " + sDestFile );
                        extractEntry(jarFile, currentJarEntry, sDestFile);
                    }
                }
            }
        } catch (IOException e)
        {
            throw new TelosysException("Cannot use JAR file '" + sJarFile + "'.", e);
        } finally
        {
            if (jarFile != null)
            {
                try
                {
                    jarFile.close();
                } catch (IOException e)
                {
                    // log and continue
                    $log.error("Cannot close jar file ('" + sJarFile + "').");
                }
            }
        }
    }
}