package org.objectweb.telosys.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author Laurent GUERIN
 *
 */
public final class XmlUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(XmlUtil.class);

	/**
	 * Comment for <code>$dateFormatter</code>
	 * The formatter to convert a Java Date to Xml String 
	 * International standard : ISO 8601 ( "yyyy-MM-dd" )
	 */
	private static SimpleDateFormat     $dateISOFormatter  = new SimpleDateFormat ("yyyy-MM-dd" ) ;
    
    /** */
    private XmlUtil()
    {
    }
    
    /**
     *  Convert "Standard String" to "Xml String"
     * 
     * @param sStdString :
     * @return String :
     */ 
    public static String xmlString(final String sStdString)
    {
        if (sStdString == null)
        {
            return "";
        }
        
        char c;
        StringBuffer sb = new StringBuffer(sStdString.length() + 40);
        for (int i = 0; i < sStdString.length(); i++)
        {
            c = sStdString.charAt(i);
            switch (c)
            {
                case '<':
                    sb.append("&lt;");
                    break;
                case '>':
                    sb.append("&gt;");
                    break;
                case '&':
                    sb.append("&amp;");
                    break;
                case '\"':
                    sb.append("&quot;");
                    break;
                case '\'':
                    sb.append("&apos;");
                    break; // Utilit  vrifier
                default:
                    sb.append(c);
                    break;
            }
        }
        return sb.toString();
    }

    public static String xmlDate(final Date date)
    {
        if (date == null)
        {
            return "";
        }
        return $dateISOFormatter.format(date);
    }

    public static String xmlBool(final boolean b)
    {
        if ( b )
        {
            return "1";
        }
        else
        {
            return "0";            
        }
    }
    
    //----------------------------------------------------------------------------------------------
    // XML DOM PARSER
    //----------------------------------------------------------------------------------------------
    
    //----------------------------------------------------------------------------------------------
    /**
     * Load an XML file as a "DOM document" with the default JAXP DOM Parser
     * @param sFileName : the XML file to load
     * @return : reference on the document loaded (or null if an error occurs)
     * @deprecated 
     */
    public static Document loadFile(String sFileName )
    {
        //--- Create a new factory instance via JAXP
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        
        //--- Configuring the Factory ...
        //factory.setValidating(true);
        //factory.setNamespaceAware(true);        

        //System.out.println("JAXP factory class = " + factory.getClass().getName() );
        
        return loadFile(sFileName, factory );
    }
    
    /**
     * Load an XML file as a "DOM document" using a JAXP factory
     * @param sFileName : the XML file to load
     * @param factory : the JAXP Document Builder Factory
     * @return : reference on the document loaded (or null if an error occurs)
     * 
     */
    private static Document loadFile(String sFileName, DocumentBuilderFactory factory )
    {
        Document document = null;
        try
        {
            //--- Create a DOM parser ( instance of a builder )
            DocumentBuilder builder = factory.newDocumentBuilder();
            //--- Parse the file
            document = builder.parse(new File(sFileName));
        } catch (SAXException ex)
        {
            //--- Error generated during parsing
            ex.printStackTrace();
            document = null ;
        } catch (ParserConfigurationException ex)
        {
            //--- Parser with specified options can't be built
            ex.printStackTrace();
            document = null ;
        } catch (IOException ex)
        {
            //--- I/O error  ( FileNotFoundException, etc ... )
            ex.printStackTrace();
            document = null ;
        }
        return document ;
    }

    //----------------------------------------------------------------------------------------------
    
    private static DocumentBuilder getDocumentBuilder() throws TelosysException
    {
        DocumentBuilderFactory factory = null ;
        DocumentBuilder builder = null ;
        //--- Create a new factory instance via JAXP
        try
        {
            factory = DocumentBuilderFactory.newInstance();
        } catch (FactoryConfigurationError e)
        {
            throw new TelosysException("XmlUtil.parse() : Cannot create DocumentBuilderFactory " , e);
	    } catch (Throwable t)
	    {
            throw new TelosysException("XmlUtil.parse() : Cannot create DocumentBuilderFactory " , t);
	    }
	    if ( factory != null )
	    {
	        //--- Create a DOM parser ( instance of a builder )
		    try
	        {
	            builder = factory.newDocumentBuilder();
	        } catch (ParserConfigurationException e)
	        {
	            throw new TelosysException("XmlUtil.parse() : Cannot create DocumentBuilder " , e);
		    } catch (Throwable t)
		    {
	            throw new TelosysException("XmlUtil.parse() : Cannot create DocumentBuilder " , t);
		    }
	    }
	    else
	    {
            throw new TelosysException("XmlUtil.parse() : DocumentBuilderFactory instance is null" );
	    }
	    
	    if ( builder == null )
	    {
            throw new TelosysException("XmlUtil.parse() : DocumentBuilder instance is null" );
	    }
	    return builder ;
    }
    
    //----------------------------------------------------------------------------------------------
    /**
     * Parse the given File name
     * @param sFileName
     * @return DOM document object
     * @throws TelosysException
     */
    public static Document parse( String sFileName ) throws TelosysException
    {
        File file = new File(sFileName) ;
        if ( file.exists() != true )
        {
            throw new TelosysException("XmlUtil.parse() : file '" + sFileName + "' not found ! ");
        }
        return parse(file);
    }
    
    //----------------------------------------------------------------------------------------------
    /**
     * Parse the given File object
     * @param file
     * @return DOM document object
     * @throws TelosysException
     */
    public static Document parse( File file ) throws TelosysException
    {
        Document document = null;
        DocumentBuilder builder = getDocumentBuilder();
        if ( builder != null )
        {
            try
            {
                //--- Parse the file
                document = builder.parse(file);
            } catch (SAXException e)
            {
                throw new TelosysException("XmlUtil.parse() : SAXException " , e);
            } catch (IOException e)
            {
                throw new TelosysException("XmlUtil.parse() : IOException " , e);
		    } catch (Throwable t)
		    {
                throw new TelosysException("XmlUtil.parse() : Throwable " , t);
		    }
        }
        return document ;
    }
    
    //----------------------------------------------------------------------------------------------
    /**
     * Parse the given InputStream
     * @param is
     * @return DOM document object
     * @throws TelosysException
     */
    public static Document parse( InputStream is ) throws TelosysException
    {
        Document document = null;
        DocumentBuilder builder = getDocumentBuilder();
        if ( builder != null )
        {
            try
            {
                //--- Parse the file
                document = builder.parse(is);
            } catch (SAXException e)
            {
                throw new TelosysException("XmlUtil.parse() : SAXException " , e);
            } catch (IOException e)
            {
                throw new TelosysException("XmlUtil.parse() : IOException " , e);
		    } catch (Throwable t)
		    {
                throw new TelosysException("XmlUtil.parse() : Throwable " , t);
		    }
        }
        return document ;
    }
    
    //----------------------------------------------------------------------------------------------
    
}